/*******************************************************************************
* FILE NAME: ibase.cpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class IBase.                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#ifdef __MVS__
#pragma priority( -2147482621 ) // Initialize before everything but iexcbase
#else
#pragma priority( -2147482623 ) // Initialize before everything but iexcbase
#endif

#include <new.h>
#include <istream.hpp>
#include <ibase.hpp>

#ifdef IC_PMWIN
#define INCL_DOSMODULEMGR
#endif

extern "C" {
#ifdef IC_PMWIN
  #include <iwindefs.h>
#endif
  #include <stdlib.h>
  #include <stdio.h>
}

#include <istring.hpp>

#include <icconst.h>

#include <itrace.hpp>

#include <iexcept.hpp>

#ifdef IC_MVS
  #include <asclocal.h>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IBASE_CPP_
  #include <ipagetun.h>
#endif



#ifndef IC_MVS

 // for MVS - this struct is defined in  ASCLOCAL.H
// Local struct for library initialization.

struct ICLUI_Init : IException::TraceFn
{
  ICLUI_Init();
 ~ICLUI_Init();
virtual void
  write ( const char *exceptionText );
static void
  newHandler ( );
};

#endif


// Dump Memory statistics
#ifdef __DEBUG_ALLOC__
  class IDumpAllRemainingStorage
  {
  public:
    ~IDumpAllRemainingStorage()
     {
        _dump_allocated(20);
     }
  };


#pragma data_seg(ICLNonConst)
  static IDumpAllRemainingStorage dumpIt;
#pragma data_seg()
#endif

#pragma data_seg(ICLNonConst)
#ifndef IC_MVS
 // for MVS it is moved to IEXCBASE.CPP
static ICLUI_Init initializer;
#endif

#pragma data_seg()

#pragma data_seg(ICLConst)
  //   initialize synonyms for IException::Severity
const IException::Severity
    IBase::unrecoverable = IException::unrecoverable,
    IBase::recoverable   = IException::recoverable;
#pragma data_seg()




ICLUI_Init ::   ICLUI_Init()
  {
 // printf ("ICLUI_Init Constructor Called \n");

  // Add this object as exception trace function:
  IException::setTraceFunction( *this );
  // Register newHandler function:

#if !defined (__RRBC_LIB__)     // SOM Version uses different linkages
  set_new_handler( ICLUI_Init::newHandler );
#endif

  }


ICLUI_Init :: ~ICLUI_Init()
{
 // printf ("ICLUI_Init Destructor Called \n");
}


void ICLUI_Init :: write ( const char *exceptionText )
  {
#ifdef IC_MVS
  fprintf (stderr, "%s\n", exceptionText);
#else
  ITrace log;
  log.write( exceptionText );
#endif
  }

void ICLUI_Init :: newHandler ( )
  {
#ifdef IC_MOTIF
  IOutOfMemory exception(IBase::messageText(IC_HEAP_EXHAUSTED),
                           IC_HEAP_EXHAUSTED, IException::unrecoverable);
  ITHROW( exception );
#else
  ITHROWLIBRARYERROR(IC_HEAP_EXHAUSTED,
                     IBaseErrorInfo::outOfMemory,
                     IException::unrecoverable);
#endif
  }


/*------------------------------------------------------------------------------
| Statics & functions related to message file handling                         |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
char
 *IBase::msgFile = 0;

static const char

#if ( defined(IC_MVS) || defined(IC_MOTIF)  )
 *defaultFile = "ibmcl.cat",
#endif

#ifdef IC_400
// must keep it as 10 character long for OS400
 defaultFile[] = "QCPPMSGF  ",
#endif

#ifdef IC_PM
 defaultFile[] = "CPPOOC3U.MSG",
#endif

#ifdef IC_WIN
 defaultFile[] = "CPPWOC3U.MSG",
#endif

 msgFileVar[]  = "ICLUI MSGFILE";
#pragma data_seg()

char* IBase :: messageFile ( )
  {
  if ( !IBase::msgFile )
    {
    const char *pFileName = getenv( msgFileVar );
    if ( !pFileName )
      pFileName = defaultFile;
    IBase::setMessageFile( pFileName );
    }
  return msgFile;
  }

void IBase :: setMessageFile ( const char *msgFileName )
  {
  if ( msgFileName )
    {
    if (IBase::msgFile)
       delete [] IBase::msgFile;
    IBase::msgFile = new char[strlen(msgFileName) + 1];
    strcpy(IBase::msgFile, msgFileName);
    }
  }

/*-----------------------------------------------------------------------------
| messageText - returns the message specified by "messageId",                 |
| with the specified text strings inserted at the proper places.              |
-----------------------------------------------------------------------------*/
IMessageText IBase :: messageText ( unsigned long messageId,
                                    const char*   textInsert1,
                                    const char*   textInsert2,
                                    const char*   textInsert3,
                                    const char*   textInsert4,
                                    const char*   textInsert5,
                                    const char*   textInsert6,
                                    const char*   textInsert7,
                                    const char*   textInsert8,
                                    const char*   textInsert9 )
  {
  IMessageText msg(messageId, IBase::messageFile(),
                   textInsert1, textInsert2, textInsert3,
                   textInsert4, textInsert5, textInsert6,
                   textInsert7, textInsert8, textInsert9);
  return msg;
  }

ostream &operator << ( ostream     &aStream,
                       const IBase &anObject )
  {
  return aStream << anObject.asString();
  }

IString IBase :: asString ( ) const
  {
  return "IBase object";
  }

IString IBase :: asDebugInfo ( ) const
  {
  IString result = "IBase(@";
  result += IString( (unsigned long)this ).d2x();
  result += ")";
  return result;
  }

IBase::Version IBase :: version ( )
  {

#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)
   Version   current;
      current.major = IC_MAJOR_VERSION;
      current.minor = IC_MINOR_VERSION;
#else
   Version
      current = { IC_MAJOR_VERSION, IC_MINOR_VERSION };
#endif    // SOM_ENABLED & RRBC_LIB


  return current;
  }
