/*******************************************************************************
* FILE NAME: iexcbase.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iexcbase.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifdef __MVS__
#pragma priority( -2147482622 ) // one lower (initialized before) IBase
#else
#pragma priority( -2147482624 ) // one lower (initialized before) IBase
#endif

#include <iexcbase.hpp>

#ifdef __WINDOWS__
#include <iplatfrm.hpp>
#endif

#include <new.h>

#if defined ( __BORLANDC__ )
  #include <except.h>
#elif defined ( __ZTC__ )
  #include <terminat.h>
#elif defined ( __IBMCPP__ )
  #include <terminat.h>
#else
  #include <terminate.h>
#endif

extern "C"
{
   #include <stdio.h>
   #include <string.h>
}

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IEXCBASE_CPP_
  #include <ipagetun.h>
#endif

#ifdef __MVS__
  #include <asclocal.h>
#endif

// Initialize ErrorCodeGroup strings
IException::ErrorCodeGroup const
                 IException::baseLibrary = "IBM Class Library";
IException::ErrorCodeGroup const
                 IException::CLibrary = "Standard C Library";
IException::ErrorCodeGroup const
                 IException::operatingSystem = "Operating System";
IException::ErrorCodeGroup const
                 IException::presentationSystem = "Presentation System";
IException::ErrorCodeGroup const
                 IException::other = "Other Error Group";

static const char* headerText =
         "%s exception thrown.\n   function: %s\n   file: %s\n   line: %d";


#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif


class IExceptStaticPtr
{
public:

IExceptStaticPtr();
~IExceptStaticPtr();

IException::TraceFn
 &traceFunction();

IException::TraceFn
 *pTraceFn,
 *pUserTraceFn;

#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)
#pragma SOMClassVersion  (*, 3, 0)
#pragma SOMAttribute (pTraceFn)
#pragma SOMAttribute (pUserTraceFn)
#endif    // SOM_ENABLED & RRBC_LIB

}; // IExceptStaticPtr

class IExceptTraceFn : public IException::TraceFn
{  }; // IExceptTraceFn


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMDefine (IExceptTraceFn)

#endif    // SOM_ENABLED & RRBC_LIB

IExceptTraceFn defaultTraceFn;
static IExceptStaticPtr IExceptStatics;

#ifdef __MVS__    // moved from ibase.cpp
#pragma data_seg(ICLNonConst)
static ICLUI_Init   initializer;
#pragma data_seg
#endif


IException::TraceFn& IExceptStaticPtr::traceFunction()
{
  if(pUserTraceFn)
    return *pUserTraceFn;
  if(!pTraceFn)
    pTraceFn = &defaultTraceFn;
  return *pTraceFn;
}

class IExcText {
public:
  IExcText  ( const char* errText,
              const IExcText* msgtxtOld );
  IExcText  ( const IExcText& msg );

  ~IExcText ( ) {
       if (pszClMsg)
          delete [] pszClMsg;
       if (msgtxtClNext)
          delete msgtxtClNext; }

const char
 *text ( ) const {
   return pszClMsg; }

IExcText
 *next ( ) const {
   return msgtxtClNext; }

void
  appendText ( const char* errText );

char
 *pszClMsg;
IExcText
 *msgtxtClNext;

#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)
#pragma SOMClassVersion  (*, 3, 0)
#pragma SOMAttribute (pszClMsg)
#pragma SOMAttribute (msgtxtClNext)
#endif    // SOM_ENABLED & RRBC_LIB

}; // IExcText


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif



// SOM Related changes
#if 0
IExceptTraceFn :: IExceptTraceFn()
{
  printf ("IExceptTraceFn Constructor called \n");
}
IExceptTraceFn :: ~IExceptTraceFn()
{
  printf ("IExceptTraceFn Destructor called \n");
}
#endif

IExceptStaticPtr :: IExceptStaticPtr()
   : pTraceFn(0), pUserTraceFn(0)
{
 // printf ("IExceptStaticPtr Constructor called \n");
}

IExceptStaticPtr :: ~IExceptStaticPtr()
{
 // printf ("IExceptStaticPtr Destructor called \n");
}

// -- End SOM Related changes

/****************************************************************/
/*  IExcText Member Functions                                   */
/****************************************************************/

IExcText :: IExcText ( const char* errText,
                       const IExcText* msgtxtOld )
/****************************************************************/
/*                                                              */
/****************************************************************/
{
    void(*poldnh)() = set_new_handler(0);
    pszClMsg = new char[strlen(errText)+1];
    if (pszClMsg)
      strcpy(pszClMsg, errText);
    msgtxtClNext = (IExcText*)msgtxtOld;
    set_new_handler(poldnh);
}

IExcText :: IExcText ( const IExcText& msg )
/****************************************************************/
/* Copy Constructor                                             */
/****************************************************************/
{
    void(*poldnh)() = set_new_handler(0);
    pszClMsg = new char[strlen(msg.pszClMsg)+1];
    if (pszClMsg)
    {
       strcpy(pszClMsg, msg.pszClMsg);
       if (msg.msgtxtClNext)
       {
          msgtxtClNext = new IExcText(*(msg.msgtxtClNext));
       }
       else msgtxtClNext = 0;
    }
    else
    {
       pszClMsg = msg.pszClMsg;
       msgtxtClNext = msg.msgtxtClNext;
    }
    set_new_handler(poldnh);
}

void IExcText :: appendText ( const char* errText )
/****************************************************************/
/*                                                              */
/****************************************************************/
{
    void(*poldnh)() = set_new_handler(0);
    char* pszText;
    if (pszClMsg)
       pszText = new char[strlen(pszClMsg)+strlen(errText)+1];
    else
       pszText = new char[strlen(errText)+1];
    if (pszText)
    {
      if (pszClMsg)
      {
         strcpy(pszText, pszClMsg);
         delete [] pszClMsg;
         strcat(pszText, errText);
         pszClMsg = pszText;
      }
      else
      {
         strcpy(pszText, errText);
         pszClMsg = pszText;
      }
    }
    set_new_handler(poldnh);
}

/****************************************************************/
/*  IExceptionLocation Member Functions                         */
/****************************************************************/


IExceptionLocation :: IExceptionLocation ( const char* fileName,
                                           const char* functionName,
                                           unsigned long lineNumber)
                      : pClFile(fileName), pClFunction(functionName),
                        ulClLineNo(lineNumber)
/****************************************************************/
/* Constructor for IExceptionLocation.                          */
/****************************************************************/
{
   ;
}

const char* IExceptionLocation :: fileName ( ) const
/****************************************************************/
/* Returns the path qualified file name where an exception was  */
/* thrown or re-thrown via the ITHROW macro.                    */
/****************************************************************/
{
  return pClFile;
}

const char* IExceptionLocation :: functionName ( ) const
/****************************************************************/
/* Returns the name of the function from which an exception was */
/* thrown or rethrown via the ITHROW macro.                     */
/****************************************************************/
{
   if (pClFunction)
      return pClFunction;
   return "Unknown";
}

unsigned long IExceptionLocation :: lineNumber ( ) const
/****************************************************************/
/* Returns the line number in the source file where an          */
/* exception was thrown or re-thrown via the ITHROW macro.      */
/****************************************************************/
{
  return ulClLineNo;
}

/****************************************************************/
/*  IException Member Functions                                 */
/****************************************************************/

IException::IException(const char* exMsgText,
                       unsigned long errorId,
                       Severity exSev)
                 : ulClErrorId(errorId), exsevCl(exSev),
                   ulexlocClCount(0), fErrorGroup(baseLibrary)
/****************************************************************/
/* Constructor. The exception text is passed in.                */
/****************************************************************/
{
   if (exMsgText != 0)
   {
      void(*poldnh)() = set_new_handler(0);
      msgtxtClTop = new IExcText(exMsgText, 0);
      if (msgtxtClTop)
        ulClTxtLvlCount = 1;
      else
        ulClTxtLvlCount = 0;
      set_new_handler(poldnh);
   }
   else         // null pointer to msg passed
   {
      ulClTxtLvlCount = 0;
      msgtxtClTop = 0;
   }
}

IException::IException(const IException& exc)
/****************************************************************/
/* Copy constructor                                             */
/****************************************************************/
{
   fErrorGroup = exc.fErrorGroup;
   exsevCl = exc.exsevCl;
   ulClErrorId = exc.ulClErrorId;
   ulexlocClCount = exc.ulexlocClCount;
   ulClTxtLvlCount = exc.ulClTxtLvlCount;
   if (exc.msgtxtClTop != 0)
     msgtxtClTop = new IExcText(*(exc.msgtxtClTop));
   else msgtxtClTop = 0;
   for (int i=0; i<exc.locationCount(); i++ )
   {
      exlocClArray[i] = exc.exlocClArray[i];
   } /* endfor */
}

IException::~IException()
/****************************************************************/
/* Destructor.                                                  */
/****************************************************************/
{
   if (msgtxtClTop)
     delete msgtxtClTop;
}

IException& IException::setErrorCodeGroup ( ErrorCodeGroup errorGroup )
/****************************************************************/
/* Sets the error code group.                                   */
/****************************************************************/
{
   fErrorGroup = errorGroup;
   return *this;
}

IException::ErrorCodeGroup IException::errorCodeGroup ( ) const
/****************************************************************/
/* Returns the error code group.                                */
/****************************************************************/
{
   return fErrorGroup;
}

IException& IException::appendText(const char* exMsgText)
/****************************************************************/
/* Concatenates a string to the exception text.                 */
/****************************************************************/
{
   if (exMsgText != 0)
   {
      if (msgtxtClTop)
        msgtxtClTop->appendText(exMsgText);
      else
        setText(exMsgText);
   }
   return *this;
}

IException& IException::setText(const char* exMsgText)
/****************************************************************/
/* Adds a new IExcText to the top of the message "stack".       */
/****************************************************************/
{
   if (exMsgText != 0)
   {
      void(*poldnh)() = set_new_handler(0);
      IExcText* msgtxtOld = msgtxtClTop;
      msgtxtClTop = new IExcText(exMsgText, msgtxtOld);
      if (msgtxtClTop)
         ulClTxtLvlCount++;
      else msgtxtClTop= msgtxtOld;
      set_new_handler(poldnh);
   }
    return *this;
}

IException& IException::setSeverity(Severity exsev)
/****************************************************************/
/* Sets the exception severity level.                           */
/****************************************************************/
{
   exsevCl = exsev;
   return *this;
}

IException& IException::setErrorId(unsigned long errorId)
/****************************************************************/
/* Sets the Error Id.                                           */
/****************************************************************/
{
   ulClErrorId = errorId;
   return *this;
}

IException& IException::addLocation(const IExceptionLocation& exloc)
/****************************************************************/
/* Adds the location information to the instance data of the    */
/* exception class.  Used by the ITHROW macro.                  */
/****************************************************************/
{
   if(ulexlocClCount == 5)
      ulexlocClCount--;          // reset to fill in last entry
   exlocClArray[ulexlocClCount] = exloc;
   ulexlocClCount++;
   return *this;
}

int IException::isRecoverable() const
/****************************************************************/
/* Return true if severity is recoverable.                      */
/****************************************************************/
{
   if(exsevCl)
      return 1;

   return 0;
}

const char* IException::name() const
/****************************************************************/
/* Returns the name of the exception class.                     */
/****************************************************************/
{
   return "IException";
}

const char* IException::text(unsigned long indexFromTop) const
/****************************************************************/
/* Returns the exception text from an entry in the stack.       */
/****************************************************************/
{
  IExcText* tmp = msgtxtClTop;
  if (textCount() > indexFromTop)
  {
     for (int i = 0; i < indexFromTop; ++i)
     {
        tmp = tmp->next();
     } /* endfor */
     return tmp->text();
  }
  else
     return 0;
}

unsigned long  IException::textCount() const
/****************************************************************/
/* Returns the number of levels of message text.                */
/****************************************************************/
{
   return ulClTxtLvlCount;
}

unsigned long IException::errorId() const
/****************************************************************/
/* Returns the error Id.                                        */
/****************************************************************/
{
  return ulClErrorId;
}

unsigned long IException :: locationCount() const
/****************************************************************/
/* Return the number of locations in the array.                 */
/****************************************************************/
{
   return ulexlocClCount;
}

const IExceptionLocation*
              IException :: locationAtIndex(unsigned long locIndex) const
/****************************************************************/
/* Return a pointer to the exception location information at    */
/* the index specified.  Index is from 0 to max - 1             */
/****************************************************************/
{
   if (locIndex < ulexlocClCount)
   {
     return &(exlocClArray[locIndex]);
   }
   return 0;
}

IException& IException :: logExceptionData ( )
/****************************************************************/
/* Log out error information                                    */
/****************************************************************/
{
   IExceptStatics.traceFunction().logData(*this);
  return *this;
}

IException::TraceFn* IException :: setTraceFunction (
                                   IException::TraceFn& traceFunction )
/****************************************************************/
/*                                                              */
/****************************************************************/
{
  IException::TraceFn* oldFunction = &(IExceptStatics.traceFunction());
  IExceptStatics.pUserTraceFn = &traceFunction;
  return oldFunction;
}

void IException :: terminate ( )
/****************************************************************/
/* Terminate the application, used when the termination model   */
/* is in effect.                                                */
/****************************************************************/
{
  ::terminate();
}

void IException :: assertParameter ( const char* exceptionText,
                                     IExceptionLocation location )
/****************************************************************/
/****************************************************************/
{
   IAssertionFailure invParm(exceptionText);
   invParm.addLocation(location);
   invParm.setErrorCodeGroup(other);
   invParm.logExceptionData();
   throw invParm;
}

  /****************************************************************/
  /* IException::TraceFn functions.                               */
  /****************************************************************/

IException::TraceFn::TraceFn ( )
{ }

void IException::TraceFn::exceptionLogged ( )
/****************************************************************/
/* Default completion function.                                 */
/****************************************************************/
{ }

void IException::TraceFn::write ( const char* buffer )
/****************************************************************/
/* Default write function.                                      */
/****************************************************************/
{
#ifdef __WINDOWS__
   if (!IPlatform::isWinNT())
      fprintf(stdout, "%s\n", buffer);
   else
#endif
      fprintf(stderr,"%s\n", buffer);
}

void IException::TraceFn::logData ( IException& exception )
/****************************************************************/
/* Default logData function.                                    */
/****************************************************************/
{
  void(*poldnh)() = set_new_handler(0);

  // Determine the largest buffer we will need.
  unsigned long maxLength = 200;
  unsigned long length = 0;
  if (exception.ulexlocClCount != 0)
     length = strlen(headerText) + 1 +
              strlen(exception.name()) +
 strlen(exception.exlocClArray[exception.ulexlocClCount - 1].functionName()) +
 strlen(exception.exlocClArray[exception.ulexlocClCount - 1].fileName()) + 8;
  if (length > maxLength)
     maxLength = length;
  if (exception.msgtxtClTop)
  {
     for (int i = (exception.textCount()-1); i >= 0 ;i-- )
       {
          length = strlen(exception.text(i)) + 6;
          if (length > maxLength)
             maxLength = length;
       } /* endfor */
  }

  char * pbuf = new char[maxLength + 1];

  if (pbuf)
  {
    if (exception.ulexlocClCount != 0)
    {
      sprintf(pbuf,headerText,
        exception.name(),
        exception.exlocClArray[exception.ulexlocClCount - 1].functionName(),
        exception.exlocClArray[exception.ulexlocClCount - 1].fileName(),
        exception.exlocClArray[exception.ulexlocClCount - 1].lineNumber());
    } else
      sprintf(pbuf,
           "\n%s exception thrown.\n   Location information is unavailable.",
           exception.name());

    IExceptStatics.traceFunction().write(pbuf);
    if (exception.errorId())
      sprintf(pbuf,"   Error Id is %d", exception.errorId());
    else
      sprintf(pbuf,"   Error Id is unavailable.");
    IExceptStatics.traceFunction().write(pbuf);
    if (exception.errorCodeGroup())
       sprintf(pbuf,"   Error Code group is %s", exception.errorCodeGroup());
    else
      sprintf(pbuf,"    Error Code group is unavailable.");
    IExceptStatics.traceFunction().write(pbuf);
    if (exception.msgtxtClTop)
    {
       sprintf(pbuf,"   Exception text is:");
       IExceptStatics.traceFunction().write(pbuf);
       for (int i = (exception.textCount()-1); i >= 0 ;i-- )
         {
            sprintf(pbuf,"      %s", exception.text(i));
            IExceptStatics.traceFunction().write(pbuf);
         } /* endfor */
    }
    else
    {
       sprintf(pbuf,"   Exception text is unavailable.");
       IExceptStatics.traceFunction().write(pbuf);
    }
    delete [] pbuf;
    exceptionLogged();
  } else                       // new failed, no buffer available
  {
#ifdef __WINDOWS__
    if (!IPlatform::isWinNT()) {
    if (exception.ulexlocClCount != 0)
    {
      fprintf(stdout,
        "%s exception thrown.\n   function: %s\n   file: %s\n   line: %d.\n",
        exception.name(),
        exception.exlocClArray[exception.ulexlocClCount - 1].functionName(),
        exception.exlocClArray[exception.ulexlocClCount - 1].fileName(),
        exception.exlocClArray[exception.ulexlocClCount - 1].lineNumber());
    } else
      fprintf(stdout,
         "\n%s exception thrown.\n   Location information is unavailable.\n",
         exception.name());
    if (exception.errorId())
      fprintf(stdout,"   Error ID is %d\n", exception.errorId());
    else
      fprintf(stdout,"   Error ID is unavailable.\n");
    if (exception.errorCodeGroup())
       fprintf(stdout,
                " Error Code group is %s\n", exception.errorCodeGroup());
    else
      fprintf(stdout,"   Error Code group is unavailable.\n");
    if (exception.msgtxtClTop)
    {
       fprintf(stdout,"   Exception text is:\n");
       for (int i = (exception.textCount()-1); i >= 0 ;i-- )
         {
            fprintf(stdout,"      %s\n", exception.text(i));
         } /* endfor */
    }
    else
       fprintf(stdout,"   Exception text is unavailable.\n");
    } else {
#endif
    if (exception.ulexlocClCount != 0)
    {
      fprintf(stderr,
        "%s exception thrown.\n   function: %s\n   file: %s\n   line: %d.\n",
        exception.name(),
        exception.exlocClArray[exception.ulexlocClCount - 1].functionName(),
        exception.exlocClArray[exception.ulexlocClCount - 1].fileName(),
        exception.exlocClArray[exception.ulexlocClCount - 1].lineNumber());
    } else
      fprintf(stderr,
         "\n%s exception thrown.\n   Location information is unavailable.\n",
         exception.name());
    if (exception.errorId())
      fprintf(stderr,"   Error ID is %d\n", exception.errorId());
    else
      fprintf(stderr,"   Error ID is unavailable.\n");
    if (exception.errorCodeGroup())
       fprintf(stderr,
                " Error Code group is %s\n", exception.errorCodeGroup());
    else
      fprintf(stderr,"   Error Code group is unavailable.\n");
    if (exception.msgtxtClTop)
    {
       fprintf(stderr,"   Exception text is:\n");
       for (int i = (exception.textCount()-1); i >= 0 ;i-- )
         {
            fprintf(stderr,"      %s\n", exception.text(i));
         } /* endfor */
    }
    else
       fprintf(stderr,"   Exception text is unavailable.\n");
#ifdef __WINDOWS__
    }
#endif
  } /* endif */


  set_new_handler(poldnh);
}

  /****************************************************************/
  /* Definition of IException subclasses using macro.             */
  /****************************************************************/
IEXCLASSIMPLEMENT(IAccessError,IException);
IEXCLASSIMPLEMENT(IAssertionFailure,IException);
IEXCLASSIMPLEMENT(IDeviceError,IException);
IEXCLASSIMPLEMENT(IInvalidParameter,IException);
IEXCLASSIMPLEMENT(IInvalidRequest,IException);
IEXCLASSIMPLEMENT(IResourceExhausted,IException);
  IEXCLASSIMPLEMENT(IOutOfMemory,IResourceExhausted);
  IEXCLASSIMPLEMENT(IOutOfSystemResource,IResourceExhausted);
  IEXCLASSIMPLEMENT(IOutOfWindowResource,IResourceExhausted);




#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

  /****************************************************************/
  /* Hidden Functions in IException and its subclasses            */
  /*  -- Their implementation is required for RRBC.               */
  /****************************************************************/

  /****************************************************************/
  /*  -- Issue a runtime error message if user calls any          */
  /*  -- of these these functions                                 */
  /****************************************************************/

IException&
       IException :: operator = (const IException& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}

IAccessError&
       IAccessError :: operator = (const IAccessError& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IAssertionFailure&
       IAssertionFailure :: operator = (const IAssertionFailure& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IDeviceError&
       IDeviceError :: operator = (const IDeviceError& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IInvalidParameter&
       IInvalidParameter :: operator = (const IInvalidParameter& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IInvalidRequest&
       IInvalidRequest :: operator = (const IInvalidRequest& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IResourceExhausted&
       IResourceExhausted :: operator = (const IResourceExhausted& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IOutOfMemory&
       IOutOfMemory :: operator = (const IOutOfMemory& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IOutOfSystemResource&
       IOutOfSystemResource :: operator = (const IOutOfSystemResource& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


IOutOfWindowResource&
       IOutOfWindowResource :: operator = (const IOutOfWindowResource& exc )
{

#if 0
  // log a runtime error message
  // find out from SOM folks a standard runtime error message
#endif

  return *this;
}


#endif    // SOM_ENABLED & RRBC_LIB
