/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiarel.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)


// ---
// IARelationImpl
// ---

// public members

IARelationImpl::
~IARelationImpl ()
{
}

void
IARelationImpl::
AddAntivalence (IARelationImpl const& collection1,
                IARelationImpl const& collection2)
{
}

void
IARelationImpl::
AddDifference (IARelationImpl const& collection1,
               IARelationImpl const& collection2)
{
  if (this == &collection1)
    return;
  else if (this == &collection2)
    Union (collection1);
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (! collection2.Contains (element))
        Add (collection1.ElementAt (cursor1), CursorOf (*this));
    }
  }
}

void
IARelationImpl::
AddIntersection (IARelationImpl const& collection1,
                 IARelationImpl const& collection2)
{
  if (this != &collection1 && this != &collection2) {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (collection2.Contains (element)) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IARelationImpl::
AddUnion (IARelationImpl const& collection1,
          IARelationImpl const& collection2)
{
  Union (collection1);
  Union (collection2);
}

void
IARelationImpl::
Antivalence (IARelationImpl const& collection)
{
}

void
IARelationImpl::
Difference (IARelationImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IARelationImpl::
Intersection (IARelationImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IARelationImpl::
Union (IARelationImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
}

IBoolean
IARelationImpl::
operator!= (IARelationImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IARelationImpl::
IARelationImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

IBoolean
IARelationImpl::
LocateNext (void const*, ICursorImpl& cursor) const
{ cursor.Invalidate ();
  return false;
}

INumber
IARelationImpl::
NumberOfOccurrences (void const* element) const
{ if (Contains (element))
    return 1;
  else
    return 0;
}

INumber
IARelationImpl::
RemoveAllOccurrences (void const* element)
{ if (Remove (element))
    return 1;
  else
    return 0;
}

#pragma info (restore)
#pragma pack ()
