/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiass.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)


// ---
// IASortedSetImpl
// ---

// public members

IASortedSetImpl::
~IASortedSetImpl ()
{
}

void
IASortedSetImpl::
AddAntivalence (IASortedSetImpl const& collection1,
                IASortedSetImpl const& collection2)
{
}

void
IASortedSetImpl::
AddDifference (IASortedSetImpl const& collection1,
               IASortedSetImpl const& collection2)
{
  if (this == &collection1)
    return;
  else if (this == &collection2)
    Union (collection1);
  else {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (! collection2.Contains (element))
        Add (collection1.ElementAt (cursor1), CursorOf (*this));
    }
  }
}

void
IASortedSetImpl::
AddIntersection (IASortedSetImpl const& collection1,
                 IASortedSetImpl const& collection2)
{
  if (this != &collection1 && this != &collection2) {
    ICursorImpl& cursor1 = CursorOf (collection1);
    IBoolean hasNext;
    for (hasNext = collection1.SetToFirst (cursor1);
         hasNext;
         hasNext = collection1.SetToNext (cursor1)) {
      void const* element = collection1.ElementAt (cursor1);
      if (collection2.Contains (element)) {
        Add (element, CursorOf (*this));
      }
    }
  }
}

void
IASortedSetImpl::
AddUnion (IASortedSetImpl const& collection1,
          IASortedSetImpl const& collection2)
{
  Union (collection1);
  Union (collection2);
}

void
IASortedSetImpl::
Antivalence (IASortedSetImpl const& collection)
{
}

long
IASortedSetImpl::
Compare (IASortedSetImpl const& collection,
         ICompFunc compFunc, void* addArg) const
{ long result = 0;

  if (this != &collection) {
    ICursorImpl& thisCursor = CursorOf (*this),
               & thatCursor = CursorOf (collection);
    IBoolean thisHasNext,
             thatHasNext;

    for (thisHasNext = SetToFirst (thisCursor),
         thatHasNext = collection.SetToFirst (thatCursor);
         result == 0 && thisHasNext && thatHasNext;
         thisHasNext = SetToNext (thisCursor),
         thatHasNext = collection.SetToNext (thatCursor)) {
      result = (*compFunc) (ElementAt (thisCursor),
                            collection.ElementAt (thatCursor), addArg);
    }

    if (result == 0) {
      if (thisHasNext && ! thatHasNext)
        result = 1;
      else if (! thisHasNext && thatHasNext)
        result = -1;
    }
  }
  return result;
}

void
IASortedSetImpl::
Difference (IASortedSetImpl const& collection)
{
  if (this != &collection) {
    ICursorImpl& cursor = CursorOf (collection);
    for (collection.SetToFirst (cursor);
         cursor.IsValid ();
         collection.SetToNext (cursor)) {
      Remove (collection.ElementAt (cursor));
    }
  }
  else
    RemoveAll ();
}

void
IASortedSetImpl::
Intersection (IASortedSetImpl const& collection)
{ if (this != &collection) {
    IACollectionImpl const* eq = &collection;
    RemoveAll (IACollectionImpl::Predicate::IsNotContained, (void*)eq);
  }
}

void
IASortedSetImpl::
Union (IASortedSetImpl const& collection)
{ if (this != &collection) {
    AddAllFrom (collection);
  }
}

IBoolean
IASortedSetImpl::
operator!= (IASortedSetImpl const& collection) const
{ return ! operator== (collection);
}

// protected members

IASortedSetImpl::
IASortedSetImpl (Ops& ops)
: IACollectionImpl (ops)
{
}

INumber
IASortedSetImpl::
NumberOfOccurrences (void const* element) const
{ if (Contains (element))
    return 1;
  else
    return 0;
}

INumber
IASortedSetImpl::
RemoveAllOccurrences (void const* element)
{ if (Remove (element))
    return 1;
  else
    return 0;
}

#pragma info (restore)
#pragma pack ()
