/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiexc.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

// ------------------------------------
// IKeySortedBagOnSequenceMngCursorImpl
// ------------------------------------

// public members

template <class Base>
IKeySortedBagOnSequenceMngCursorImpl <Base>::
IKeySortedBagOnSequenceMngCursorImpl
  (IKeySortedBagOnSequenceImpl <Base> const& collection)
: Inherited (IKeySortedBagOnSequenceImpl <Base>::BaseOf (collection))
{ ivCollection = &collection;
}

template <class Base>
IKeySortedBagOnSequenceMngCursorImpl <Base>::
~IKeySortedBagOnSequenceMngCursorImpl ()
{
}

// ---------------------------
// IKeySortedBagOnSequenceImpl
// ---------------------------

// public members

template <class Base>
IKeySortedBagOnSequenceImpl <Base>::
IKeySortedBagOnSequenceImpl (Ops& ops, INumber)
: IAKeySortedBagImpl ((Inherited::Ops&)ops), ivBase (ops)
{ BaseOf (*this).IACollectionImpl::Initialize ();
}

template <class Base>
IKeySortedBagOnSequenceImpl <Base>::
IKeySortedBagOnSequenceImpl
  (Ops& ops, IKeySortedBagOnSequenceImpl <Base> const&)
: IAKeySortedBagImpl ((Inherited::Ops&)ops), ivBase (ops)
{ BaseOf (*this).IACollectionImpl::Initialize ();
}

template <class Base>
IKeySortedBagOnSequenceImpl <Base>::
~IKeySortedBagOnSequenceImpl ()
{
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
Add (void const* element, ICursorImpl& cursor)
{ if (LocateKey (Key (element), cursor))
    Self::SetToNextWithDifferentKey (cursor);
  if (cursor.IsValid ())
    BaseOf (*this).Base::AddAsPrevious (element, cursor);
  else
    BaseOf (*this).Base::AddAsLast (element, cursor);
  return true;
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
AddOrReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (isContained)
    BaseOf (*this).Base::ReplaceAt (cursor, element);
  else
    Self::Add (element, cursor);

  return ! isContained;
}

template <class Base>
IACollectionImpl*
IKeySortedBagOnSequenceImpl <Base>::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
ContainsAllKeysFrom (IACollectionImpl const& collection) const
{ if (Self::ClassName () == collection.ClassName ()) {
    if (this != &collection) {
      Self const& ksb = (Self const&)collection;
      ICursorImpl& thisCursor = CursorOf (*this),
                 & thatCursor = CursorOf (ksb);

      long comp;
      Self::SetToFirst (thisCursor);
      for (ksb.Self::SetToFirst (thatCursor);
           thatCursor.IsValid ();
           ksb.Self::SetToNext (thatCursor)) {
        IBoolean found = false;
        while (! found && thisCursor.IsValid ()) {
          comp = CompareKeys (Self::ElementAt (thisCursor),
                              ksb.Self::ElementAt (thatCursor),
                              kElementElement);
          if (comp <= 0) {
            Self::SetToNext (thisCursor);
            if (comp == 0) {
              found = true;
            }
          }
          else {
            return false;
          }
        }
        if (! found) {
          return false;
        }
      }
    }
    return true;
  }
  else
    return Inherited::ContainsAllKeysFrom (collection);
}

template <class Base>
void
IKeySortedBagOnSequenceImpl <Base>::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

template <class Base>
IKeySortedBagOnSequenceImpl <Base>::Cursor*
IKeySortedBagOnSequenceImpl <Base>::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

template <class Base>
IKeySortedBagOnSequenceImpl <Base>::MngCursor*
IKeySortedBagOnSequenceImpl <Base>::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

template <class Base>
void*
IKeySortedBagOnSequenceImpl <Base>::
ElementWithKey (void const* key) const
{ ICursorImpl& cursor = CursorOf (*this);
  ICHECK_ContainsKey (ClassName (), "ElementWithKey")
  return ElementAt (cursor);
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
LocateElementWithKey (void const* key, ICursorImpl& cursor) const
{ IBoolean isContained = LocateKey (key, cursor);

  if (! isContained)
    cursor.Invalidate ();

  return isContained;
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
LocateNextElementWithKey (void const* key, ICursorImpl& cursor) const
{ Self::SetToNext (cursor);
  IBoolean isContained = LocateNextKey (key, cursor);

  if (! isContained)
    cursor.Invalidate ();

  return isContained;
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
LocateOrAddElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (! isContained)
    Self::Add (element, cursor);

  return isContained;
}

template <class Base>
INumber
IKeySortedBagOnSequenceImpl <Base>::
NumberOfDifferentKeys () const
{ INumber result = 0;
  for (Self::SetToFirst (CursorOf (*this));
       CursorOf (*this).IsValid ();
       Self::SetToNextWithDifferentKey (CursorOf (*this)))
    result++;
  return result;
}

template <class Base>
INumber
IKeySortedBagOnSequenceImpl <Base>::
NumberOfElementsWithKey (void const* key) const
{ INumber result = 0;
  for (Self::LocateElementWithKey (key, CursorOf (*this));
       CursorOf (*this).IsValid ();
       Self::LocateNextElementWithKey (key, CursorOf (*this)))
    result++;
  return result;
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
RemoveElementWithKey (void const* key)
{ IBoolean isContained = LocateKey (key, CursorOf (*this));

  if (isContained)
    Self::RemoveAt (CursorOf (*this));

  return isContained;
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
ReplaceElementWithKey (void const* element, ICursorImpl& cursor)
{ IBoolean isContained = LocateKey (Key (element), cursor);

  if (isContained)
    BaseOf (*this).Base::ReplaceAt (cursor, element);
  else
    cursor.Invalidate ();

  return isContained;
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
SetToNextWithDifferentKey (ICursorImpl& cursor) const
{ void const* key = Key (Self::ElementAt (cursor));
  while (Self::SetToNext (cursor)) {
    if (CompareKeys (Self::ElementAt (cursor), key, kElementKey) != 0)
      return true;
  }
  return false;
}

// private members

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
LocateKey (void const* key, ICursorImpl& cursor) const
{ Self::SetToFirst (cursor);
  return LocateNextKey (key, cursor);
}

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::
LocateNextKey (void const* key, ICursorImpl& cursor) const
{ for (; cursor.IsValid (); Self::SetToNext (cursor)) {
    long comp =
      CompareKeys (Self::ElementAt (cursor), key, kElementKey);
    if (comp >= 0)
      return (comp == 0);
  }
  return false;
}

// ---------------------------------------------
// IKeySortedBagOnSequenceImpl <Base>::Predicate
// ---------------------------------------------

// public members

template <class Base>
IBoolean
IKeySortedBagOnSequenceImpl <Base>::Predicate::
IsEqual (void const* element, void* thisPtr)
{ return ((KSB*)thisPtr)->
    CompareKeys (element , ((KSB*)thisPtr)->ivKey, kElementKey) == 0;
}

#pragma info (restore)
#pragma pack ()
