/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiseqdil.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IASequenceAsDilTableOps)
#endif

// -----------------------------
// ISequenceAsDilTableCursorImpl
// -----------------------------

// public members

ISequenceAsDilTableCursorImpl::
ISequenceAsDilTableCursorImpl (IACollectionImpl const& collection)
: ICursorImpl (collection), ivIndex (0)
{
}

ISequenceAsDilTableCursorImpl::
ISequenceAsDilTableCursorImpl
  (ISequenceAsDilTableCursorImpl const& cursor)
: ICursorImpl (cursor), ivIndex (cursor.ivIndex)
{
}

ISequenceAsDilTableCursorImpl::
~ISequenceAsDilTableCursorImpl ()
{
}

ICursorImpl*
ISequenceAsDilTableCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsDilTableCursorImpl::
Copy (ICursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}


// ------------------------
// ISequenceAsDilTableImpl
// ------------------------

char const*
ISequenceAsDilTableImpl::cvClassName = "SequenceAsDilTable";

// public members

ISequenceAsDilTableImpl::
ISequenceAsDilTableImpl (Ops& ops, INumber numberOfElements)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable ()
{ if (numberOfElements == 0)
    numberOfElements++;
  Self::Initialize (numberOfElements);
}

ISequenceAsDilTableImpl::
ISequenceAsDilTableImpl
  (Ops& ops, ISequenceAsDilTableImpl const& collection)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivTable ()
{ Self::Initialize (collection.ivTable.ivAllocatedElements);
  Self::AddAllFrom (collection);
}

ISequenceAsDilTableImpl::
~ISequenceAsDilTableImpl ()
{
  Self::RemoveAll ();
  DeleteNodes (ivTable.ivNodes);
}

void
ISequenceAsDilTableImpl::
AddAllFrom (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& seq =  (Self const&)collection;
    if (! ivTable.CanAdd (seq.ivTable.ivNumberOfElements))
      GrowBy (seq.ivTable.ivNumberOfElements);
    CopyFrom (seq);
  }
  else
    Inherited::AddAllFrom (collection);
}

void
ISequenceAsDilTableImpl::
AddAsFirst (void const* element, ICursorImpl& cursor)
{ if (Self::IsEmpty ())
    Self::AddAsLast (element, cursor);
  else {
    Self::SetToFirst (cursor);
    Self::AddAsPrevious (element, cursor);
  }
}

void
ISequenceAsDilTableImpl::
AddAsLast (void const* element, ICursorImpl& cursor)
{ if (! ivTable.CanAdd (1))
    GrowBy (1);

  INumber index = ivTable.ivLast;
  ivTable.IncMod (index);
  if (index == ivTable.ivFirst && ! Self::IsEmpty ())
    MakeFree (index);
  ivTable.GetUnused (index);
  ConstructNode (ivTable.PtrAtIndex (index), element);
  IndexOf (cursor) = index;
  ivTable.ivNumberOfElements++;
  ivTable.ivLast = index;
}


void
ISequenceAsDilTableImpl::
AddAsNext (void const* element, ICursorImpl& cursor)
{
  { if (IndexOf (cursor) == ivTable.ivLast)
      Self::AddAsLast (element, cursor);
    else {
      Self::SetToNext (cursor);
      Self::AddAsPrevious (element, cursor);
    }
  }
}

void
ISequenceAsDilTableImpl::
AddAsPrevious (void const* element, ICursorImpl& cursor)
{

  INumber index = IndexOf (cursor); // no reference 'cause of mng cursor
  if (! ivTable.CanAdd (1)) {
    index = ivTable.Pos (index);
    GrowBy (1);
  }

  IBoolean isFirst = (index == ivTable.ivFirst);
  MakeFree (index);
  if (isFirst)
    ivTable.ivFirst = index;
  ivTable.GetUnused (index);
  ConstructNode (ivTable.PtrAtIndex (index), element);
  IndexOf (cursor) = index;
  ivTable.ivNumberOfElements++;
}

void
ISequenceAsDilTableImpl::
AddAtPosition (IPosition position,
               void const* element,
               ICursorImpl& cursor)
{ if (position == Self::NumberOfElements () + 1)
    Self::AddAsLast (element, cursor);
  else {
    Self::SetToPosition (position, cursor);
    Self::AddAsPrevious (element, cursor);
  }
}

IBoolean
ISequenceAsDilTableImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ INumber index = ivTable.ivFirst;
  for (INumber i = 0; i < Self::NumberOfElements (); i++) {
    if (! (*applFunc) (ElementOf (ivTable.ElementAtIndex (index)),
                       addArg))
      return false;
    SetToNextIndex (index);
  }
  return true;
}

IBoolean
ISequenceAsDilTableImpl::
CheckCursor (ICursorImpl const& cursor) const
{
  {
    INumber const& index = IndexOf (cursor);
    if (Self::IsEmpty () || IsRemoved (index))
      return false;
    else if (ivTable.ivFirst <= ivTable.ivLast)
      return ivTable.ivFirst <= index && index <= ivTable.ivLast;
    else
      return (ivTable.ivFirst <= index &&
                index <= ivTable.ivAllocatedElements) ||
              index <= ivTable.ivLast;
  }
}

IACollectionImpl*
ISequenceAsDilTableImpl::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

void
ISequenceAsDilTableImpl::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

ISequenceAsDilTableImpl::Cursor*
ISequenceAsDilTableImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

ISequenceAsDilTableImpl::MngCursor*
ISequenceAsDilTableImpl::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

void*
ISequenceAsDilTableImpl::
ElementAtPosition (IPosition position) const
{ Self::SetToPosition (position, CursorOf (*this));
  return Self::ElementAt (CursorOf (*this));
}

IBoolean
ISequenceAsDilTableImpl::
IsConsistent () const
{ if (Self::IsEmpty ()) {
    INumber last = ivTable.ivLast;
    return (ivTable.IncMod (last) == ivTable.ivFirst);
  } else {
    if (IsRemoved (ivTable.ivFirst) || IsRemoved (ivTable.ivLast))
      return false;
    INumber numberOfElements = 0;
    for (INumber index = ivTable.ivFirst; ; ivTable.IncMod (index)) {
      if (! IsRemoved (index)) {
        numberOfElements++;
      }
      if (index == ivTable.ivLast) {
        break;
      }
    }
    return (numberOfElements == Self::NumberOfElements ());
  }
}

INumber
ISequenceAsDilTableImpl::
RemoveAll ()
{ INumber n = Self::NumberOfElements ();
  INumber index = ivTable.ivFirst;
  for (INumber i = ivTable.ivNumberOfElements; i != 0; i--) {
    DestroyNode (ivTable.PtrAtIndex (index));
    ivTable.PutUnused (index);
    SetToNextIndex (index);
  }


  ivTable.ivFirst = 1;
  ivTable.ivLast = ivTable.ivAllocatedElements;
  ivTable.ivNumberOfElements = 0;
  return n;
}

INumber
ISequenceAsDilTableImpl::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber removed = 0;
  if (! Self::IsEmpty ()) {
    INumber index = ivTable.ivFirst;
    while (1) {
      IBoolean isLast = (index == ivTable.ivLast);
      if ((*predFunc) (ElementOf (ivTable.ElementAtIndex (index)),
                       addArg)) {
        RemoveAtIndex (index);
        removed++;
      }
      if (isLast) {
        break;
      }
      SetToNextIndex (index);
    }
  }
  return removed;
}

void
ISequenceAsDilTableImpl::
RemoveAt (ICursorImpl& cursor)
{ RemoveAtIndex (IndexOf (cursor));
  if (! IsInBetween (cursor))
    cursor.Invalidate ();
}

void
ISequenceAsDilTableImpl::
RemoveAtPosition (IPosition position)
{ Self::SetToPosition (position, CursorOf (*this));
  RemoveAtIndex (IndexOf (CursorOf (*this)));
}

void
ISequenceAsDilTableImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ Self::SetToPosition (position, CursorOf (*this));
  ReplaceAt (CursorOf (*this), element);
}

void
ISequenceAsDilTableImpl::
Reverse ()
{ if (Self::NumberOfElements () > 1) {
    INumber count = Self::NumberOfElements () / 2;
    INumber forward = ivTable.ivFirst,
            backward = ivTable.ivLast;
    if (ivTable.CanAdd (1)) {
      if (forward == backward)
        MakeFree (forward);
      else
        ivTable.DecMod (forward);
      ivTable.ivFirst = forward;
      while (count--) {
        ivTable.PtrAtIndex (forward) = ivTable.PtrAtIndex (backward);
        SetToNextIndex (forward);
        ivTable.PtrAtIndex (backward) = ivTable.PtrAtIndex (forward);
        SetToPreviousIndex (backward);
      }
      ivTable.RemoveAtIndex (forward);
    }
    else {
      void* swap;
      while (count--) {
        swap = ivTable.PtrAtIndex (forward);
        ivTable.PtrAtIndex (forward) = ivTable.PtrAtIndex (backward);
        ivTable.PtrAtIndex (backward) = swap;
        SetToNextIndex (forward);
        SetToPreviousIndex (backward);
      }
    }
  }
}

IBoolean
ISequenceAsDilTableImpl::
SetToFirst (ICursorImpl& cursor) const
{ if (! Self::IsEmpty ())
    IndexOf (cursor) = ivTable.ivFirst;
  else
    IndexOf (cursor) = 0;

  return (IndexOf (cursor) != 0);
}

IBoolean
ISequenceAsDilTableImpl::
SetToLast (ICursorImpl& cursor) const
{ if (! Self::IsEmpty ())
    IndexOf (cursor) = ivTable.ivLast;
  else
    IndexOf (cursor) = 0;

  return (IndexOf (cursor) != 0);
}

void
ISequenceAsDilTableImpl::
SetToPosition (IPosition position, ICursorImpl& cursor) const
{ IndexOf (cursor) = ivTable.ivFirst;
  for (IPosition i = 1; i < position; i++)
    SetToNextIndex (IndexOf (cursor));
}

void
ISequenceAsDilTableImpl::
Sort (ICompFunc compFunc, void* addArg)
{ INumber left, last, newLast;
  INumber index;
  void* swap;

  if (Self::NumberOfElements () > 1) {
    last = ivTable.ivLast;
    while (last != ivTable.ivFirst) {
      index = ivTable.ivFirst;
      left = index;
      while (left != last) {
        SetToNextIndex (index);
        if ((*compFunc) (ElementOf (ivTable.ElementAtIndex (index)),
                         ElementOf (ivTable.ElementAtIndex (left)),
                         addArg) < 0) {
          swap = ivTable.PtrAtIndex (index);
          ivTable.PtrAtIndex (index) = ivTable.PtrAtIndex (left);
          ivTable.PtrAtIndex (left) = swap;
        }
        newLast = left;
        left = index;
      }
      last = newLast;
    }
  }
}

void
ISequenceAsDilTableImpl::
operator= (ISequenceAsDilTableImpl const& collection)
{ if (this != &collection) {
    Self::RemoveAll ();
    if (! ivTable.CanAdd (collection.ivTable.ivNumberOfElements))
      GrowBy (collection.ivTable.ivNumberOfElements);
    CopyFrom (collection);
  }
}

TStream&
ISequenceAsDilTableImpl::
operator<<= (TStream& fromWhere)
{
  return fromWhere;
}

TStream&
ISequenceAsDilTableImpl::
operator>>= (TStream& toWhere) const
{
  return toWhere;
}

// protected members

void
ISequenceAsDilTableImpl::
CopyFrom (ISequenceAsDilTableImpl const& collection)
{ IDilTableImpl const& table = collection.ivTable;
  if (table.ivNumberOfElements != 0) {
    INumber fromIndex, toIndex;
    if (ivTable.ivNumberOfElements != 0 &&
        (ivTable.ivFirst > ivTable.ivLast ?
          (ivTable.ivFirst - ivTable.ivLast - 1) :
          (ivTable.ivFirst + ivTable.ivAllocatedElements
                           - ivTable.ivLast - 1))
         < table.ivNumberOfElements) {
      toIndex = fromIndex = ivTable.ivFirst;
      while (1) {
        if (fromIndex != toIndex) {
          ivTable.PtrAtIndex (toIndex) = ivTable.PtrAtIndex (fromIndex);
        }
        if (! SetToNextIndex (fromIndex))
          break;
        ivTable.IncMod (toIndex);
      }
      ivTable.ivLast = toIndex;
    }
    toIndex = ivTable.ivLast;
    fromIndex = table.ivFirst;
    while (1) {
      ivTable.IncMod (toIndex);
      ivTable.GetUnused (toIndex);
      ConstructNode (ivTable.PtrAtIndex (toIndex),
                     ElementOf (collection.ivTable.ElementAtIndex
                                 (fromIndex)));
      if (fromIndex == table.ivLast)
        break;
      do
        table.IncMod (fromIndex);
      while (collection.ivTable.PtrAtIndex (fromIndex) == 0);
    }
    ivTable.ivNumberOfElements += table.ivNumberOfElements;
    ivTable.ivLast = toIndex;
  }
}

void
ISequenceAsDilTableImpl::
GrowBy (INumber n)
{

  n += ivTable.ivNumberOfElements;

  IDilTableImpl table = ivTable;

  INumber newAllocatedElements = ivTable.ivAllocatedElements;
  while (newAllocatedElements < n)
    newAllocatedElements *= 2;

  Initialize (newAllocatedElements);
  INumber& toIndex   = ivTable.ivLast;
  INumber  fromIndex = table.ivFirst;
  for (INumber i = table.ivNumberOfElements; i != 0; i--) {
    ivTable.IncMod (toIndex);
    ivTable.GetUnused (toIndex);
    ConstructNode (ivTable.PtrAtIndex (toIndex),
                   ElementOf (table.ElementAtIndex (fromIndex)));
    DestroyNode (table.PtrAtIndex (fromIndex));
    do {
      table.IncMod (fromIndex);
    }
    while (table.PtrAtIndex (fromIndex) == 0);
  }
  DeleteNodes (table.ivNodes);
  ivTable.ivNumberOfElements = table.ivNumberOfElements;
}

void
ISequenceAsDilTableImpl::
Initialize (INumber n)
{ ivTable.ivNodes = CreateNodes (n, ivTable.ivNodeSize);

  ivTable.ivFirst = 1;
  ivTable.ivLast = ivTable.ivAllocatedElements = n;
  ivTable.ivNumberOfElements = 0;

  while (--n) {
    *((void**)ivTable.NodeAtIndex (n) + 1) =
              ivTable.NodeAtIndex (n + 1);
  }

  ivTable.ivUnused = ivTable.NodeAtIndex (ivTable.ivFirst);
}

void
ISequenceAsDilTableImpl::
MakeFree (INumber& index)
{ INumber leftFree = index;
  while (leftFree > 1 && ! IsRemoved (leftFree)) {
    if (leftFree == ivTable.ivFirst && leftFree - 1 != ivTable.ivLast)
      ivTable.RemoveAtIndex (leftFree - 1);
    leftFree--;
  }
  if (index - leftFree <= 1 && IsRemoved (leftFree)) {
    index = leftFree;
    return;
  }
  INumber rightFree = index;
  while (rightFree < ivTable.ivAllocatedElements &&
         ! IsRemoved (rightFree)) {
    if (rightFree == ivTable.ivLast && rightFree + 1 != ivTable.ivFirst)
      ivTable.RemoveAtIndex (rightFree + 1);
    rightFree++;
  }
  if (! IsRemoved (leftFree) ||
      (index - leftFree > rightFree - index && IsRemoved (rightFree))) {
    for (INumber i = rightFree,
                 j = rightFree - (index + 1) + 1; j != 0; i--, j--) {
      ivTable.PtrAtIndex (i) = ivTable.PtrAtIndex (i - 1);
    }
    if (index <= ivTable.ivFirst && ivTable.ivFirst < rightFree)
      ivTable.ivFirst++;
    if (index <= ivTable.ivLast && ivTable.ivLast < rightFree)
      ivTable.ivLast++;
  }
  else {
    for (INumber i = leftFree, j = (index - 1) - (leftFree + 1) + 1;
         j != 0; i++, j--) {
      ivTable.PtrAtIndex (i) = ivTable.PtrAtIndex (i + 1);
    }
    if (leftFree < ivTable.ivFirst && ivTable.ivFirst < index)
      ivTable.ivFirst--;
    if (leftFree < ivTable.ivLast && ivTable.ivLast < index)
      ivTable.ivLast--;
    index--;
  }
}

IPosition
ISequenceAsDilTableImpl::
PositionAtIndex (INumber index) const
{ IPosition position = 1;
  for (INumber currentIndex = ivTable.ivFirst;
       currentIndex != index;
       SetToNextIndex (currentIndex), position++);
  return position;
}

void
ISequenceAsDilTableImpl::
RemoveAtIndex (INumber index)
{
  DestroyNode (ivTable.PtrAtIndex (index));
  ivTable.PutUnused (index);
  if (--ivTable.ivNumberOfElements != 0) {
    if (index == ivTable.ivFirst)
      SetToNextIndex (ivTable.ivFirst);
    else if (index == ivTable.ivLast)
      SetToPreviousIndex (ivTable.ivLast);
  }
  else
    ivTable.DecMod (ivTable.ivLast);
}

void
ISequenceAsDilTableImpl::
SetInBetween (MngCursor& cursor) const
{
}

IBoolean
ISequenceAsDilTableImpl::
SetToNextIndex (INumber& index) const
{ if (index != ivTable.ivLast) {
    do {
      ivTable.IncMod (index);
    }
    while (IsRemoved (index));
  }
  else
    index = 0;

  return (index != 0);
}

IBoolean
ISequenceAsDilTableImpl::
SetToPreviousIndex (INumber& index) const
{ if (index != ivTable.ivFirst) {
    do {
      ivTable.DecMod (index);
    }
    while (IsRemoved (index));
  }
  else
    index = 0;

  return (index != 0);
}

#pragma info (nopar, nouse)

void
ISequenceAsDilTableImpl::
UpdateMngCursorsForCopyFrom (INumber fromIndex, INumber toIndex) const
{
}

void
ISequenceAsDilTableImpl::
UpdateMngCursorsForGrowBy () const
{
}

void
ISequenceAsDilTableImpl::
UpdateMngCursorsForRemoveOf (INumber index) const
{
}

void
ISequenceAsDilTableImpl::
UpdateMngCursorsForSwap (INumber index1, INumber index2) const
{
}

#pragma info (restore)

#pragma info (restore)
#pragma pack ()
