/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

// ---------------------------
// ISequenceAsDilTableNodeImpl
// ---------------------------

// public members

template <class Element>
inline
ISequenceAsDilTableNodeImpl <Element>::
ISequenceAsDilTableNodeImpl (Element const& element)
: ivElement (element)
{
}

template <class Element>
inline
ISequenceAsDilTableNodeImpl <Element>::
~ISequenceAsDilTableNodeImpl ()
{
}

#if defined (__LIB_BUILD__)
// -------------
// IDilTableImpl
// -------------

// public members

inline void*
IDilTableImpl::
ElementAtIndex (INumber index) const
{ return &((Node*)((Node*)NodeAtIndex (index))->ivPtr)->ivElement;
}

inline void
IDilTableImpl::
GetUnused (INumber index)
{ IASSERT (index > 0 && index <= ivAllocatedElements)
  IASSERT (ivUnused != 0)

  PtrAtIndex (index) = ivUnused;
  ivUnused = *((void**)ivUnused + 1);
}

inline void*&
IDilTableImpl::
PtrAtIndex (INumber index) const
{ return ((Node*)NodeAtIndex (index))->ivPtr;
}

inline void
IDilTableImpl::
PutUnused (INumber index)
{ IASSERT (index > 0 && index <= ivAllocatedElements)
  IASSERT (PtrAtIndex (index) != 0)

  *((void**)PtrAtIndex (index) + 1) = ivUnused;
  ivUnused = PtrAtIndex (index);
  PtrAtIndex (index) = 0;
}

inline void
IDilTableImpl::
RemoveAtIndex (INumber index)
{ IASSERT (index > 0 && index <= ivAllocatedElements)

  PtrAtIndex (index) = 0;
}

#endif
// -----------------------------
// ISequenceAsDilTableCursorImpl
// -----------------------------

// public members

inline void
ISequenceAsDilTableCursorImpl::
Invalidate ()
{ ivIndex = 0;
}

inline IBoolean
ISequenceAsDilTableCursorImpl::
IsInBetween () const
{ return false;
}

inline IBoolean
ISequenceAsDilTableCursorImpl::
IsValid () const
{ return (ivIndex != 0);
}

inline void
ISequenceAsDilTableCursorImpl::
operator= (ISequenceAsDilTableCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivIndex = cursor.ivIndex;
}

inline IBoolean
ISequenceAsDilTableCursorImpl::
operator== (ICursorImpl const& cursor) const
{ return (Inherited::operator== (cursor) &&
          ivIndex == ((Self const&)cursor).ivIndex);
}

// -----------------------
// ISequenceAsDilTableImpl
// -----------------------

// public members

#if defined (__LIB_BUILD__)

#if ! defined (__2nd_LEVEL__)
#define inline
#endif

inline IBoolean
ISequenceAsDilTableImpl::
Add (void const* element, ICursorImpl& cursor)
{ Self::AddAsLast (element, cursor);
  return true;
}

inline void*
ISequenceAsDilTableImpl::
Any () const
{ return ElementOf (ivTable.ElementAtIndex (ivTable.ivFirst));
}

inline char const*
ISequenceAsDilTableImpl::
ClassName () const
{ return cvClassName;
}

inline void*
ISequenceAsDilTableImpl::
ElementAt (ICursorImpl const& cursor) const
{ return ElementOf (ivTable.ElementAtIndex (IndexOf (cursor)));
}

inline void*
ISequenceAsDilTableImpl::
First () const
{ return ElementOf (ivTable.ElementAtIndex (ivTable.ivFirst));
}

inline IBoolean
ISequenceAsDilTableImpl::
IsBounded () const
{ return false;
}

inline IBoolean
ISequenceAsDilTableImpl::
IsEmpty () const
{ return (ivTable.ivNumberOfElements == 0);
}

inline IBoolean
ISequenceAsDilTableImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ return (IndexOf (cursor) == ivTable.ivFirst);
}

inline IBoolean
ISequenceAsDilTableImpl::
IsFull () const
{ return false;
}

inline IBoolean
ISequenceAsDilTableImpl::
IsLastAt (ICursorImpl const& cursor) const
{ return (IndexOf (cursor) == ivTable.ivLast);
}

inline void*
ISequenceAsDilTableImpl::
Last () const
{ return ElementOf (ivTable.ElementAtIndex (ivTable.ivLast));
}

inline INumber
ISequenceAsDilTableImpl::
NumberOfElements () const
{ return ivTable.ivNumberOfElements;
}

inline IPosition
ISequenceAsDilTableImpl::
PositionAt (ICursorImpl const& cursor) const
{ return PositionAtIndex (IndexOf (cursor));
}

inline void
ISequenceAsDilTableImpl::
RemoveFirst ()
{ RemoveAtIndex (ivTable.ivFirst);
}

inline void
ISequenceAsDilTableImpl::
RemoveLast ()
{ RemoveAtIndex (ivTable.ivLast);
}

inline void
ISequenceAsDilTableImpl::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

inline IBoolean
ISequenceAsDilTableImpl::
SetToNext (ICursorImpl& cursor) const
{
    return SetToNextIndex (IndexOf (cursor));
}

inline IBoolean
ISequenceAsDilTableImpl::
SetToPrevious (ICursorImpl& cursor) const
{
    return SetToPreviousIndex (IndexOf (cursor));
}

#if ! defined (__2nd_LEVEL__)
#undef inline
#endif

// protected members

inline void
ISequenceAsDilTableImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline void*
ISequenceAsDilTableImpl::
CreateNodes (INumber n, size_t& size) const
{ return OpsOf (*this).CreateNodes (n, size);
}

inline void
ISequenceAsDilTableImpl::
ConstructNode (void* node, void const* element) const
{ OpsOf (*this).ConstructNode (node, element);
}

inline void
ISequenceAsDilTableImpl::
DeleteNodes (void* nodes) const
{ OpsOf (*this).DeleteNodes (nodes);
}

inline void
ISequenceAsDilTableImpl::
DestroyNode (void* node) const
{ OpsOf (*this).DestroyNode (node);
}

inline void*
ISequenceAsDilTableImpl::
StreamIn (TStream& fromWhere) const
{ return OpsOf (*this).StreamIn (fromWhere);
}

inline void
ISequenceAsDilTableImpl::
StreamOut (void const* element, TStream& toWhere) const
{ OpsOf (*this).StreamOut (element, toWhere);
}

// private members

inline void*
ISequenceAsDilTableImpl::
ElementOf (void const* node) const
{ return (OpsOf (*this).ivContainsDTSObjects ?
    (void*)*(void const**)node :
    (void*) (void const**)node);

}

inline INumber&
ISequenceAsDilTableImpl::
IndexOf (ICursorImpl const& cursor)
{ return ((Cursor&)cursor).ivIndex;
}

inline IBoolean
ISequenceAsDilTableImpl::
IsInBetween (ICursorImpl const& cursor)
{
  return false;
}

inline IBoolean
ISequenceAsDilTableImpl::
IsRemoved (INumber index) const
{ return (ivTable.PtrAtIndex (index) == 0);
}

inline INumber&
ISequenceAsDilTableImpl::
NextIndexOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivIndex; // just a dummy
}

inline ISequenceAsDilTableImpl::Ops&
ISequenceAsDilTableImpl::
OpsOf (ISequenceAsDilTableImpl const& collection)
{ return (Ops&) IACollectionImpl::OpsOf (collection);
}

inline INumber&
ISequenceAsDilTableImpl::
PreviousIndexOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivIndex;  // just a dummy
}
#endif

// -----------------------
// IASequenceAsDilTableOps
// -----------------------

// public members

inline
IASequenceAsDilTableOps::
~IASequenceAsDilTableOps ()
{
}

// protected members

inline
IASequenceAsDilTableOps::
IASequenceAsDilTableOps (IBoolean containsDTSObjects)
: ivContainsDTSObjects (containsDTSObjects)
{
}
