/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

#include <iiseqlst.h>

#pragma info (nocls, nocnd, nocns, nocnv, noext, nognr, novft)
#pragma pack (4)

#if defined (__SOM_ENABLED__) && defined (__RRBC_LIB__)
#pragma SOMDefine (IASequenceAsListOps)
#endif

// -------------------------
// ISequenceAsListCursorImpl
// -------------------------

// public members

ISequenceAsListCursorImpl::
ISequenceAsListCursorImpl (IACollectionImpl const& collection)
: ICursorImpl (collection), ivNode (0)
{
}

ISequenceAsListCursorImpl::
ISequenceAsListCursorImpl
  (ISequenceAsListCursorImpl const& cursor)
: ICursorImpl (cursor), ivNode (cursor.ivNode)
{
}

ISequenceAsListCursorImpl::
~ISequenceAsListCursorImpl ()
{
}

ICursorImpl*
ISequenceAsListCursorImpl::
Clone () const
{ return (Self*) CheckPointer (new Self (*this));
}

void
ISequenceAsListCursorImpl::
Copy (ICursorImpl const& cursor)
{ operator= ((Self const&)cursor);
}


// --------------------
// ISequenceAsListImpl
// --------------------

char const*
ISequenceAsListImpl::cvClassName = "SequenceAsList";

// public members

ISequenceAsListImpl::
ISequenceAsListImpl (Ops& ops, INumber)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivFirst (0), ivLast (0), ivNumberOfElements (0)
{
}

ISequenceAsListImpl::
ISequenceAsListImpl (Ops& ops, ISequenceAsListImpl const& collection)
: IASequenceImpl ((Inherited::Ops&)ops), ivMngCursors (0),
  ivFirst (0), ivLast (0), ivNumberOfElements (0)
{ Self::AddAllFrom (collection);
}

ISequenceAsListImpl::
~ISequenceAsListImpl ()
{
  Self::RemoveAll ();
}

IBoolean
ISequenceAsListImpl::
Add (void const* element, ICursorImpl& cursor)
{ NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (ivLast, NodeOf (cursor));
  return true;
}

void
ISequenceAsListImpl::
AddAllFrom (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ()) {
    Self const& seq = (Self const&)collection;
    for (Node* node = seq.ivFirst; node != 0; node = node->ivNext)
      AddAsNextNode (ivLast, (Node*) CreateNode (ElementOf (node)));
  }
  else
    Inherited::AddAllFrom (collection);
}

void
ISequenceAsListImpl::
AddAsFirst (void const* element, ICursorImpl& cursor)
{ NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (0, NodeOf (cursor));
}

void
ISequenceAsListImpl::
AddAsLast (void const* element, ICursorImpl& cursor)
{ NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (ivLast, NodeOf (cursor));
}

void
ISequenceAsListImpl::
AddAsNext (void const* element, ICursorImpl& cursor)
{ Node* node = (Node*) CreateNode (element);
  AddAsNextNode (NodeOf (cursor), node);
  NodeOf (cursor) = node;
}

void
ISequenceAsListImpl::
AddAsPrevious (void const* element, ICursorImpl& cursor)
{ Node* node = (Node*) CreateNode (element);
  {
    AddAsNextNode (NodeOf (cursor)->ivPrevious, node);
  }
  NodeOf (cursor) = node;
}

void
ISequenceAsListImpl::
AddAtPosition (IPosition position,
               void const* element,
               ICursorImpl& cursor)
{ Node* node;
  if (position == 1)
    node = 0;
  else {
    node = ivFirst;
    for (int i = 2; i < position; i++)
      node = node->ivNext;
  }
  NodeOf (cursor) = (Node*) CreateNode (element);
  AddAsNextNode (node, NodeOf (cursor));
}

IBoolean
ISequenceAsListImpl::
AllElementsDo (IApplFunc applFunc, void* addArg)
{ for (Node* node = ivFirst; node != 0; node = node->ivNext) {
    if (! (*applFunc) (ElementOf (node), addArg))
      return false;
  }
  return true;
}

IBoolean
ISequenceAsListImpl::
CheckCursor (ICursorImpl const& cursor) const
{ IBoolean result = false;
  {
    for (Node* node = ivFirst; node != 0; node = node->ivNext) {
      if (NodeOf (cursor) == node)
        result = true;
    }
  }
  return result;
}

IACollectionImpl*
ISequenceAsListImpl::
Clone () const
{ return (IACollectionImpl*) CheckPointer (OpsOf (*this).Clone ());
}

void
ISequenceAsListImpl::
Copy (IACollectionImpl const& collection)
{ if (Self::ClassName () == collection.ClassName ())
    operator= ((Self const&)collection);
  else
    Inherited::Copy (collection);
}

ISequenceAsListImpl::Cursor*
ISequenceAsListImpl::
CreateCursor () const
{ return (Cursor*) CheckPointer (new Cursor (*this));
}

ISequenceAsListImpl::MngCursor*
ISequenceAsListImpl::
CreateMngCursor () const
{ return (MngCursor*) CheckPointer (new MngCursor (*this));
}

void*
ISequenceAsListImpl::
ElementAtPosition (IPosition position) const
{ Node* node = 0;
  SetNodeToPosition (position, node);
  return ElementOf (node);
}

IBoolean
ISequenceAsListImpl::
IsConsistent () const
{ INumber numberOfElements = 0;
  if (ivFirst != 0 && ivFirst->ivPrevious != 0)
    return false;
  for (Node* node = ivFirst; node != 0; node = node->ivNext) {
    if (node->ivNext != 0&& node->ivNext->ivPrevious != node)
      return false;
    numberOfElements++;
  }
  return (numberOfElements == ivNumberOfElements);
}

IPosition
ISequenceAsListImpl::
PositionAt (ICursorImpl const& cursor) const
{ IPosition position = 1;
  for (Node* node = ivFirst;
       node != NodeOf (cursor);
       node = node->ivNext, position++);
  return position;
}

INumber
ISequenceAsListImpl::
RemoveAll ()
{ INumber n = Self::NumberOfElements ();
  for (Node *current = ivFirst, *next; current != 0; current = next) {
    next = current->ivNext;
    DeleteNode (current);
  }


  ivFirst = ivLast = 0;
  ivNumberOfElements = 0;
  return n;
}

INumber
ISequenceAsListImpl::
RemoveAll (IPredFunc predFunc, void* addArg)
{ INumber removed = 0;

  for (Node* current = ivFirst, * next = ivFirst;
       current != 0;
       current = next) {
    next = next->ivNext;
    if ((*predFunc) (ElementOf (current), addArg)) {
      RemoveNode (current);
      removed++;
    }
  }

  return removed;
}

void
ISequenceAsListImpl::
RemoveAt (ICursorImpl& cursor)
{ RemoveNode (NodeOf (cursor));
  if (! IsInBetween (cursor))
    cursor.Invalidate ();
}

void
ISequenceAsListImpl::
RemoveAtPosition (IPosition position)
{ Node* node;
  SetNodeToPosition (position, node);
  RemoveNode (node);
}

void
ISequenceAsListImpl::
ReplaceAtPosition (IPosition position, void const* element)
{ SetToPosition (position, CursorOf (*this));
  Self::ReplaceAt (CursorOf (*this), element);
}

void
ISequenceAsListImpl::
Reverse ()
{ Node* next;
  for (Node* current = ivFirst; current != 0; current = next) {
    next = current->ivNext;
    current->ivNext = current->ivPrevious;
    current->ivPrevious = next;
  }


  next = ivLast;
  ivLast = ivFirst;
  ivFirst = next;
}

void
ISequenceAsListImpl::
Sort (ICompFunc compFunc, void* addArg)
{ for (Node* last = ivLast; last != ivFirst; last = last->ivPrevious) {
    for (Node* current = ivFirst; current != last; ) {
      Node* next = current->ivNext;
      if (0 < (*compFunc) (ElementOf (current),
                           ElementOf (current->ivNext), addArg)) {
        current->ivNext = next->ivNext;
        next->ivPrevious = current->ivPrevious;
        if (current->ivNext != 0)
          current->ivNext->ivPrevious = current;
        if (next->ivPrevious != 0)
          next->ivPrevious->ivNext = next;
        current->ivPrevious = next;
        next->ivNext = current;
        if (next == last) {
          last = current;
          if (next == ivLast)
            ivLast = current;
        }
        if (current == ivFirst) {
          ivFirst = next;
        }
      }
      else {
        current = next;
      }
    }
  }
}

void
ISequenceAsListImpl::
operator= (ISequenceAsListImpl const& collection)
{ if (this != &collection) {
    Self::RemoveAll ();
    Self::AddAllFrom (collection);
  }
}

TStream&
ISequenceAsListImpl::
operator<<= (TStream& fromWhere)
{
  return fromWhere;
}

TStream&
ISequenceAsListImpl::
operator>>= (TStream& toWhere) const
{
  return toWhere;
}

// private members

void
ISequenceAsListImpl::
AddAsNextNode (Node* node, Node* newNode)
{ if (node == 0)
    newNode->ivNext = ivFirst;
  else
    newNode->ivNext = node->ivNext;

  newNode->ivPrevious = node;

  if (newNode->ivNext != 0)
    newNode->ivNext->ivPrevious = newNode;

  if (newNode->ivPrevious != 0)
    newNode->ivPrevious->ivNext = newNode;

  if (node == 0)
    ivFirst = newNode;

  if (node == ivLast)
    ivLast = newNode;

  ivNumberOfElements++;
}

void
ISequenceAsListImpl::
RemoveNode (Node* node)
{ if (node == ivFirst)
    ivFirst = node->ivNext;
  if (node == ivLast)
    ivLast = node->ivPrevious;

  if (node->ivNext != 0)
    node->ivNext->ivPrevious = node->ivPrevious;
  if (node->ivPrevious != 0)
    node->ivPrevious->ivNext = node->ivNext;


  DeleteNode (node);

  ivNumberOfElements--;
}

void
ISequenceAsListImpl::
SetInBetween (MngCursor& cursor) const
{
}

void
ISequenceAsListImpl::
SetNodeToPosition (IPosition position, Node*& node) const
{ node = ivFirst;
  for (int i = 1; i < position; i++)
    node = node->ivNext;
}

#pragma info (restore)
#pragma pack ()
