/**********************************************************************
*                                                                     *
*  IBM(R) VisualAge(TM) for C++ for Windows(R), Version 3.5           *
*                                                                     *
*  PID: 5622-880                                                      *
*  - Licensed Material - Program-Property of IBM                      *
*  (C) Copyright IBM Corp. 1991, 1995 - All Right Reserved.           *
*                                                                     *
*  US Government Users Restricted Rights - Use, duplication or        *
*  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  *
*                                                                     *
*  VisualAge, and IBM are trademarks or registered trademarks of      *
*  International Business Machines Corporation.                       *
*  Windows is a registered trademark of Microsoft Corporation.        *
*                                                                     *
**********************************************************************/

// -----------------------
// ISequenceAsListNodeImpl
// -----------------------

// public members

template <class Element>
inline
ISequenceAsListNodeImpl <Element>::
ISequenceAsListNodeImpl (Element const& element)
: ivNext (0), ivPrevious (0), ivElement (element)
{
}

template <class Element>
inline
ISequenceAsListNodeImpl <Element>::
~ISequenceAsListNodeImpl ()
{
}

// -------------------------
// ISequenceAsListCursorImpl
// -------------------------

// public members

inline void
ISequenceAsListCursorImpl::
Invalidate ()
{ ivNode = 0;
}

inline IBoolean
ISequenceAsListCursorImpl::
IsInBetween () const
{ return false;
}

inline IBoolean
ISequenceAsListCursorImpl::
IsValid () const
{ return (ivNode != 0);
}

inline void
ISequenceAsListCursorImpl::
operator= (ISequenceAsListCursorImpl const& cursor)
{ Inherited::operator= (cursor);
  ivNode = cursor.ivNode;
}

inline IBoolean
ISequenceAsListCursorImpl::
operator== (ICursorImpl const& cursor) const
{ return (Inherited::operator== (cursor) &&
          ivNode == ((Self const&)cursor).ivNode);
}

// -------------------
// ISequenceAsListImpl
// -------------------

// public members

#if defined (__LIB_BUILD__)

#if ! defined (__2nd_LEVEL__)
#define inline
#endif

inline void*
ISequenceAsListImpl::
Any () const
{ return ElementOf (ivFirst);
}

inline char const*
ISequenceAsListImpl::
ClassName () const
{ return cvClassName;
}

inline void*
ISequenceAsListImpl::
ElementAt (ICursorImpl const& cursor) const
{ return ElementOf (NodeOf (cursor));
}

inline void*
ISequenceAsListImpl::
First () const
{ return ElementOf (ivFirst);
}

inline IBoolean
ISequenceAsListImpl::
IsBounded () const
{ return false;
}

inline IBoolean
ISequenceAsListImpl::
IsEmpty () const
{ return (ivNumberOfElements == 0);
}

inline IBoolean
ISequenceAsListImpl::
IsFirstAt (ICursorImpl const& cursor) const
{ return (NodeOf (cursor) == ivFirst);
}

inline IBoolean
ISequenceAsListImpl::
IsFull () const
{ return false;
}

inline IBoolean
ISequenceAsListImpl::
IsLastAt (ICursorImpl const& cursor) const
{ return (NodeOf (cursor) == ivLast);
}

inline void*
ISequenceAsListImpl::
Last () const
{ return ElementOf (ivLast);
}

inline INumber
ISequenceAsListImpl::
NumberOfElements () const
{ return ivNumberOfElements;
}

inline void
ISequenceAsListImpl::
RemoveFirst ()
{ RemoveNode (ivFirst);
}

inline void
ISequenceAsListImpl::
RemoveLast ()
{ RemoveNode (ivLast);
}

inline void
ISequenceAsListImpl::
ReplaceAt (ICursorImpl const& cursor, void const* element)
{ Assign (Self::ElementAt (cursor), element);
}

inline IBoolean
ISequenceAsListImpl::
SetToFirst (ICursorImpl& cursor) const
{ NodeOf (cursor) = ivFirst;
  return (NodeOf (cursor) != 0);
}

inline IBoolean
ISequenceAsListImpl::
SetToLast (ICursorImpl& cursor) const
{ NodeOf (cursor) = ivLast;
  return (NodeOf (cursor) != 0);
}

inline IBoolean
ISequenceAsListImpl::
SetToNext (ICursorImpl& cursor) const
{
    NodeOf (cursor) = NodeOf (cursor)->ivNext;
  return (NodeOf (cursor) != 0);
}

inline void
ISequenceAsListImpl::
SetToPosition (IPosition position, ICursorImpl& cursor) const
{ SetNodeToPosition (position, NodeOf (cursor));
}

inline IBoolean
ISequenceAsListImpl::
SetToPrevious (ICursorImpl& cursor) const
{
    NodeOf (cursor) = NodeOf (cursor)->ivPrevious;
  return (NodeOf (cursor) != 0);
}

#if ! defined (__2nd_LEVEL__)
#undef inline
#endif

// protected members

inline void
ISequenceAsListImpl::
Assign (void* element1, void const* element2) const
{ OpsOf (*this).Assign (element1, element2);
}

inline void*
ISequenceAsListImpl::
CreateNode (void const* element) const
{ return OpsOf (*this).CreateNode (element);
}

inline void
ISequenceAsListImpl::
DeleteNode (void* node) const
{ OpsOf (*this).DeleteNode (node);
}

inline void*
ISequenceAsListImpl::
StreamIn (TStream& fromWhere) const
{ return OpsOf (*this).StreamIn (fromWhere);
}

inline void
ISequenceAsListImpl::
StreamOut (void const* element, TStream& toWhere) const
{ OpsOf (*this).StreamOut (element, toWhere);
}

// private members

inline void*
ISequenceAsListImpl::
ElementOf (Node const* node) const
{ return (OpsOf (*this).ivContainsDTSObjects ? (void*) node->ivElement :
                                               (void*)&node->ivElement);
}

inline IBoolean
ISequenceAsListImpl::
IsInBetween (ICursorImpl const& cursor)
{
  return false;
}

inline ISequenceAsListImpl::Node*&
ISequenceAsListImpl::
NextNodeOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivNode; // just a dummy
}

inline ISequenceAsListImpl::Node*&
ISequenceAsListImpl::
NodeOf (ICursorImpl const& cursor)
{ return ((Cursor&)cursor).ivNode;
}

inline ISequenceAsListImpl::Ops&
ISequenceAsListImpl::
OpsOf (ISequenceAsListImpl const& collection)
{ return (Ops&) Inherited::OpsOf (collection);
}

inline ISequenceAsListImpl::Node*&
ISequenceAsListImpl::
PreviousNodeOf (ICursorImpl const& cursor)
{
  return ((Cursor&)cursor).ivNode; // just a dummy
}
#endif

// -------------------
// IASequenceAsListOps
// -------------------

// public members

inline
IASequenceAsListOps::
~IASequenceAsListOps ()
{
}

// protected members

inline
IASequenceAsListOps::
IASequenceAsListOps (IBoolean containsDTSObjects)
: ivContainsDTSObjects (containsDTSObjects)
{
}
