/*******************************************************************************
* FILE NAME: istring2.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the comparison operators for      *
*   the class IString.  Also internationalization (I18N) support routines.                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #include <ctype.h>
  #include <stdlib.h>
  #include <string.h>
  }
#include <istring.hpp>

//#include <ibuffer.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISTRING2_CPP_
  #include <ipagetun.h>
#endif


#ifdef IC_NLS
//          Perform I18N initialization during load time
struct I18N_Init
{
  I18N_Init();
};
#endif


#ifdef IC_PMWIN
#pragma data_seg(ICLNonConst)
#endif

#ifdef IC_NLS
/*  default will be I18N OFF  */
IString::I18NStates IString::fgI18NState = IString::i18nOff;

static I18N_Init I18NInitializer;
#endif

#ifdef IC_PMWIN
#pragma data_seg()
#endif


#ifdef IC_NLS
/*--------------------------------------------------------------------*/
/* I18N support                                                       */
/*--------------------------------------------------------------------*/
#define  ENVAR_I18N    "ICLUI_I18N"

/*-------------------------------------------------------------------------
| I18N_Init::I18N_Init()                                                  |
|                                                                         |
| Call getenv() to see if the environment variable ICLUI_I18N= is         |
| defined.  If it is not found, default fgI18NState to OFF (i.e. SWORD    |
| semantics - just binary compare).  fgI18NState is set to i18nOn         |
| (I18N semantics - locale sensitive) iff ICLUI_I18N=ON.                  |
|                                                                         |
-------------------------------------------------------------------------*/
I18N_Init::I18N_Init()
{
  /*  Get environment variable  ICLUI_I18N=    */
  char cEnvStr[3] = "\0";
  const char * pEnv = getenv(ENVAR_I18N);

  if (pEnv)     /*  if ICLUI_I18N=  exist  */
   {
     if (2 == strlen(pEnv))        /*  2 characters value found  */
      {
        int i;

        strcpy(cEnvStr, pEnv);
        for (i = 0; i < 2 ; i++)
            *(cEnvStr + i) = toupper(*(cEnvStr + i));
        *(cEnvStr + 2) = '\0';
      }
     else  pEnv = NULL;        /*  ignore the invalid values  */
   }

  /*   Default is I18N off, i.e. locale insensitive  */
  if ( pEnv && !strcmp(cEnvStr, "ON"))   /*  ICLUI_I18N=ON ?  */
     IString::enableInternationalization();
  else
    IString::disableInternationalization();

}
#endif


/*------------------------------------------------------------------------------
| operator ==                                                                  |
| operator !=                                                                  |
| operator <                                                                   |
| operator <=                                                                  |
| operator >                                                                   |
| operator >=                                                                  |
|                                                                              |
| Compares two IStrings (or a IString and a char* string) for the specified    |
| condition by calling IBuffer::compare().                                     |
------------------------------------------------------------------------------*/
// See istring.inl

