#ifdef __MVS__
                   ??=ifdef __COMPILER_VER__
                     ??=pragma filetag("IBM-1047")
                   ??=endif
                   #pragma nomargins nosequence
                   #pragma checkout(suspend)
#endif

#ifndef _ITIME_
#define _ITIME_
/*******************************************************************************
* FILE NAME: itime.hpp                                                         *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     ITime                                                                    *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ibase.hpp>
#include <istring.hpp>

/*  The "struct blocks" must be placed before SOM enablement test  */
#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifdef IC_PM

struct _CTIME;

#endif
#endif

struct ICnrTime {
  unsigned char hours;
  unsigned char minutes;
  unsigned char seconds;
  unsigned char ucReserved;
};

#pragma pack(4)

class ostream;


#if defined (__SOM_ENABLED__)
#if defined (__RRBC_LIB__)
#pragma SOMAsDefault (on)
#else
#pragma SOMAsDefault (off)
#endif
#endif



class ITime : public IBase {
public:
/*------------------------ Constructors --------------------------------------*/
  ITime       ( );

  ITime       ( long seconds );

  ITime       ( unsigned hours,
                unsigned minutes,
                unsigned seconds = 0 );

  ITime       ( const ITime &aTime );

  ITime       ( const ICnrTime &cnrTime );

/*-------------------------- Current Time ------------------------------------*/
static ITime
  now         ( );

/*-------------------------- Diagnostics -------------------------------------*/
IString
  asString    ( const char *fmt = "%X" ) const;

friend ostream
 &operator << ( ostream     &aStream,
                const ITime &aTime );


/*-------------------------- Time Queries ------------------------------------*/
long
  asSeconds   ( ) const;

ICnrTime
  asICnrTime  ( ) const;

unsigned
  hours       ( ) const,
  minutes     ( ) const,
  seconds     ( ) const;

/*-------------------------- Comparisons -------------------------------------*/
Boolean
  operator == ( const ITime &aTime ) const,
  operator != ( const ITime &aTime ) const,
  operator <  ( const ITime &aTime ) const,
  operator <= ( const ITime &aTime ) const,
  operator >  ( const ITime &aTime ) const,
  operator >= ( const ITime &aTime ) const;

/*------------------------- Manipulations ------------------------------------*/
ITime
  operator +  ( const ITime &aTime ) const,
  operator -  ( const ITime &aTime ) const;

ITime
 &operator += ( const ITime &aTime ),
 &operator -= ( const ITime &aTime );


protected:
/*------------------------- Initialize ---------------------------------------*/
ITime
 &initialize  ( long seconds );

private:
/*------------------------- PRIVATE ------------------------------------------*/
#if ( IC_OBSOLETE <= IC_OBSOLETE_2 )
#ifdef IC_PM

public:

  ITime   ( const _CTIME &cDate );

_CTIME
  asCTIME ( ) const;

private:

#endif
#endif

long
  ticks;


#if defined (__SOM_ENABLED__ ) && defined (__RRBC_LIB__)

#pragma SOMClassVersion  (*, 3, 0)

#ifdef __MVS__

#pragma SOMReleaseOrder( \
/*   1 */       ITime(long),\
/*   2 */       ITime(unsigned int,unsigned int,unsigned int),\
/*   3 */   /*   ITime(const _CTIME&), OS/2 specific Not Supported on MVS */ \
/*   4 */       now(),\
/*   5 */       asString(const char*) const,\
/*   6 */       asSeconds() const,\
/*   7 */   /*    asCTIME() const,  OS/2 Specific Not Supported on MVS */ \
/*   8 */       hours() const,\
/*   9 */       minutes() const,\
/*  10 */       seconds() const,\
/*  11 */       operator==(const ITime&) const,\
/*  12 */       operator!=(const ITime&) const,\
/*  13 */       operator<(const ITime&) const,\
/*  14 */       operator<=(const ITime&) const,\
/*  15 */       operator>(const ITime&) const,\
/*  16 */       operator>=(const ITime&) const,\
/*  17 */       operator+(const ITime&) const,\
/*  18 */       operator-(const ITime&) const,\
/*  19 */       operator+=(const ITime&),\
/*  20 */       operator-=(const ITime&),\
/*  21 */       initialize(long),\
/*  22 */       ITime(const ICnrTime&),\
/*  23 */       asICnrTime() const)

#else

#pragma SOMReleaseOrder( \
/*   1 */       ITime(long),\
/*   2 */       ITime(unsigned int,unsigned int,unsigned int),\
/*   3 */       ITime(const _CTIME&), \
/*   4 */       now(),\
/*   5 */       asString(const char*) const,\
/*   6 */       asSeconds() const,\
/*   7 */       asCTIME() const,  \
/*   8 */       hours() const,\
/*   9 */       minutes() const,\
/*  10 */       seconds() const,\
/*  11 */       operator==(const ITime&) const,\
/*  12 */       operator!=(const ITime&) const,\
/*  13 */       operator<(const ITime&) const,\
/*  14 */       operator<=(const ITime&) const,\
/*  15 */       operator>(const ITime&) const,\
/*  16 */       operator>=(const ITime&) const,\
/*  17 */       operator+(const ITime&) const,\
/*  18 */       operator-(const ITime&) const,\
/*  19 */       operator+=(const ITime&),\
/*  20 */       operator-=(const ITime&),\
/*  21 */       initialize(long),\
/*  22 */       ITime(const ICnrTime&),\
/*  23 */       asICnrTime() const)

#endif  // for Release order Pragma

#endif    // SOM_ENABLED & RRBC_LIB

}; // ITime


#if defined (__SOM_ENABLED__)
#pragma SOMAsDefault (pop)
#endif

#pragma pack()

#ifndef I_NO_INLINES
  #include <itime.inl>
#endif

#endif /* _ITIME_ */

#ifdef __MVS__
                   #pragma checkout(resume)
#endif

