/* NOSHIP */
/*****************************************************************************
 *
 * ascntobj.h 
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * LIST OF CLASSES IN THIS FILE
 *      CAswContainedObject
 *
 * Description:
 *
 * This class deals with the connection to an embedded object; not its
 * presentation or data renderings, advises, etc...  Each object that is
 * embedded in a container must have an instance of a CAswContainedObject
 * object.  The class implements methods that allow the toolkit user to create
 * new, load persistent, or delete existing objects.  It also allows the
 * toolkit user to manage the state of an object; persistent, loaded and
 * running.
 *
 * CAswContainedObject inherits privately from CAswCOMSite.  Those methods
 * marked above with an "*" indicated that they are derived from CAswCOMSite.
 *
 * Containers must create a CAswContainer object prior to instantiating a
 * CAswContainedObject object. They must also instantiate a storage for the
 * object prior to object creation or loading.  See CAswStorage and
 * CAswRootStorage for more information on how to create a storage for an
 * object.
 *
 * Simple containers will generally instantiate either a CAswPresSite or
 * CAswDataSite in conjunction with the instantiation of CAswContainedObject,
 * depending on what type of object the container wants created.   A single
 * CAswContainedObject can support multiple presentation and/or data sites if
 * desired.  This allows the toolkit user to manage multiple views of the same
 * object.
 *
 *****************************************************************************/

#ifndef ASWAN_ASCNTOBJ_H
#define ASWAN_ASCNTOBJ_H
#include <asstream.h>
#include <asstg.h>
#include <aspack.h>
typedef struct tagASWEXTENTINFO
{
   DWORD    dwAspect;
   SIZEL    sizelExtents;
} ASWEXTENTINFO;
typedef ASWEXTENTINFO *PASWEXTENTINFO;
typedef enum tagASWINITDATASTATE
{
   INITDATASTATE_SAME = 1,
   INITDATASTATE_LEAVERUNNING,
   INITDATASTATE_LEAVELOADED,
   INITDATASTATE_UNLOAD
} ASWINITDATASTATE;
typedef enum tagASWCACHEOPT
{
   ASWCACHEOPT_NONE = 0,         
   ASWCACHEOPT_KEEPALL = 1,      
   ASWCACHEOPT_KEEPPRES = 2,     
   ASWCACHEOPT_KEEPDATA = 3      
} ASWCACHEOPT;
typedef enum tagASWOBJLINKTYPE
{
   LINKTYPE_NONE      = 0,
   LINKTYPE_AUTOMATIC = 1,
   LINKTYPE_MANUAL    = 2
}
   ASWOBJLINKTYPE;
typedef enum tagCREATEFROMDATA
{
   CREATEFROMDATA_LINK   = 1,
   CREATEFROMDATA_OBJECT = 2,
   CREATEFROMDATA_STATIC = 3
}
   ASWCREATEFROMDATA;
class ASWPUBCLASS CAswContainedObject : public CAswCOMSite
{
   ASWAN_DECLARATIONS(5);  
public:
   ASWPUBMETHOD CAswContainedObject( PASWCONTROL pAswControl,
                                     LPUNKNOWN lpUnknownOuter = NULL );
   ASWPUBMETHOD ~CAswContainedObject( void );
   CAswResult ASWPUBMETHOD Setup( PASWCONTAINER pAswContainer,
                                  PASWSTORAGE   pAswParentStorage,
                                  LPASWCOLESTR  pszObjectName );
   CAswResult ASWPUBMETHOD Shutdown( BOOL fEmergency = FALSE );
   CAswResult ASWPUBMETHOD Activate( LPMSG         lpMsg,
                                     PASWPRESSITE  pAswPresSite,
                                     LONG          lVerb = OLEIVERB_PRIMARY );
   CAswResult ASWPUBMETHOD Activate( LPMSG               lpMsg,
                                     PASWINPLACEPRESSITE pAswInPlacePresSite,
                                     LONG                lVerb = OLEIVERB_PRIMARY );
   CAswResult ASWPUBMETHOD Activate( LPMSG         lpMsg,
                                     PASWDATASITE  pAswDataSite,
                                     LONG          lVerb,
                                     HWND          hwnd,
                                     LPCRECT       lprcPosRect );
   CAswResult ASWPUBMETHOD Activate( LPMSG lpMsg,
                                     LONG lVerb = OLEIVERB_PRIMARY );
   CAswResult ASWPUBMETHOD 
   CreateFromInsertObject(DWORD        dwFlags,
                          CLSID        clsid,
                          LPASWCOLESTR lpszFileName = NULL,
                          HGLOBAL      hMetaPict = NULL);
   inline CAswResult
   CreateFromDrop( PASWDROPTARGET   pDropTarget,
                   BOOL             fLink = FALSE,
                   UINT             uCacheOpt = ASWCACHEOPT_KEEPALL,
                   HGLOBAL          hMetaPict = NULL)
   {
      return CreateFromData( pDropTarget->GetDataObjectRH(),
                             (fLink)
                                 ? CREATEFROMDATA_LINK : CREATEFROMDATA_OBJECT,
                             0,
                             uCacheOpt,
                             hMetaPict );
   }
   inline CAswResult
   CreateFromPaste( PASWPASTETARGET pPasteTarget,
                    BOOL            fLink = FALSE,
                    UINT            uCacheOpt = ASWCACHEOPT_KEEPALL,
                    HGLOBAL         hMetaPict = NULL)
   {
      return CreateFromData( pPasteTarget->GetDataObjectRH(),
                             (fLink)
                                 ? CREATEFROMDATA_LINK : CREATEFROMDATA_OBJECT,
                             0,
                             uCacheOpt,
                             hMetaPict );
   }
   CAswResult ASWPUBMETHOD
   CreateFromData( PCASWREFHOLDER      prhDataObject,
                   ASWCREATEFROMDATA   ct,
                   CLIPFORMAT          cfFormat = 0,
                   UINT                uCacheOpt = ASWCACHEOPT_KEEPALL,
                   HGLOBAL             hMetaPict = NULL);
   CAswResult ASWPUBMETHOD CreateLink(LPASWCOLESTR pszFileName,
                                      LPASWCOLESTR pszItemName = NULL);
   CAswResult ASWPUBMETHOD CreateLink(LPMONIKER pmkSrc);
   CAswResult ASWPUBMETHOD CreateFromOLE1Stream(LPOLESTREAM polestm);
   CAswResult ASWPUBMETHOD PersistentlyDelete( void );
   CAswResult ASWPUBMETHOD Duplicate( PASWCONTAINEDOBJECT pSourceContainedObject );
   CAswResult ASWPUBMETHOD CanInitFromData( void );
   CAswResult ASWPUBMETHOD FillWithSelectedData( BOOL             fCreation,
                                                 LPBOOL           lpbInsert,
                                                 ASWINITDATASTATE uLeaveState = INITDATASTATE_SAME );
   CAswResult ASWPUBMETHOD FillFromPasteDrop( BOOL fCreation,
                                              PASWTARGETBASE pAswTargetBase,
                                              LPBOOL lpbInsert,
                                              ASWINITDATASTATE uLeaveState = INITDATASTATE_SAME );
   virtual CAswResult ASWPUBMETHOD Unload( void );
   virtual CAswResult ASWPUBMETHOD Close( DWORD dwSaveOption );
   virtual void ASWPUBMETHOD PrepareForOpeningServer( void );
   CAswResult ASWPUBMETHOD IsStateChangeOK( ASWOBJSTATE uDesiredState );
   CAswResult ASWPUBMETHOD Load( void );
   CAswResult ASWPUBMETHOD GetLinkedState( void );
   virtual ASWOBJDIRTYSTATE ASWPUBMETHOD GetDirtyState( void );
   CAswResult ASWPUBMETHOD BindLinkIfLinkSrcIsRunning( void );
   inline PASWSTORAGE GetStorage( void )
   {
      return &m_AswStorage;
   }
   CAswResult ASWPUBMETHOD GetName( LPASWOLESTR lpszObjName,
                                    UINT        nBuffCount );
   CAswResult ASWPUBMETHOD Rename( LPASWCOLESTR pszObjName );
   CAswResult ASWPUBMETHOD VerbNext( LPLONG        lplVerb,
                                     LPASWOLESTR   lpszVerbName,
                                     LPDWORD       lpdwFlags,
                                     LPDWORD       lpdwAttribs );
   CAswResult ASWPUBMETHOD VerbStartEnumeration( void );
   CAswResult ASWPUBMETHOD VerbEndEnumeration( void );
   CAswResult ASWPUBMETHOD AddVerbMenu( HMENU          hPopupMenu,
                                        UINT           idVerbMin,
                                        UINT           idVerbMax,
                                        BOOL           bAddConvert,
                                        UINT           idConvert,
                                        HMENU ASWPTR * phMenu );
   static CAswResult ASWPUBFUNC AddVerbMenuNoSelectedObject( HMENU hPopupMenu,
                                                             UINT  idVerbMin,
                                                             UINT  idVerbMax,
                                                             UINT  idConvert );
   inline PASWUIOLELINK GetUIOleLink( void )
   {
      return &m_auiolLink;
   }
   CAswResult ASWPUBMETHOD GetUserPresentableName( DWORD dwID,
                                                   LPASWOLESTR lpszString,
                                                   UINT  uSize );
   CAswResult ASWPUBMETHOD GetClipboardData(  );
   CAswResult ASWPUBMETHOD SetColorScheme( LPLOGPALETTE lplogicalPalette );
   CAswResult ASWPUBMETHOD GetEmbeddedObjectData(PASWSTORAGE pAswStorage);
   CAswResult ASWPUBMETHOD GetLinkSourceData(PASWSTREAM pAswStream);
   CAswResult ASWPUBMETHOD CacheNext( LPFORMATETC ASWPTR *  lplpFormatEtc,
                                      LPDWORD               lpdwAdvf,
                                      LPDWORD               lpdwConnection );
   CAswResult ASWPUBMETHOD CacheReset( void );
   CAswResult ASWPUBMETHOD Cache( LPFORMATETC lpFormatEtc,
                                  DWORD       dwAdvf,
                                  LPDWORD     lpdwConnection = NULL );
   CAswResult ASWPUBMETHOD Uncache( LPFORMATETC lpFormatEtc,
                                    DWORD       dwAdvf,
                                    DWORD       lpdwConnection = NULL );
   virtual CAswResult ASWPUBMETHOD Run( void );
   inline CAswResult SaveToCurrent( void )
   {
      return SaveToStorage( NULL, FALSE );
   }
   inline CAswResult SaveToDifferentStorage( PASWSTORAGE pParentStorage )
   {
      return DoSaveFromParent( pParentStorage, FALSE );
   }
   inline CAswResult SaveToNewStorage( PASWSTORAGE pParentStorage )
   {
      return DoSaveFromParent( pParentStorage, TRUE );
   }
   virtual CAswResult ASWPUBMETHOD Save( PASWSTORAGE pNewAswStorage );
   virtual CAswResult ASWPUBMETHOD SaveCompleted( PASWSTORAGE pNewAswStorage );
   CAswResult ASWPUBMETHOD HandsOffStorage( void );
   CAswResult ASWPUBMETHOD SaveToOLE1Stream(LPOLESTREAM polestm);
   virtual void ASWPUBMETHOD OnSave( void );
   virtual void ASWPUBMETHOD OnClose( void );
   virtual CAswResult ASWPUBMETHOD OnServerOpening( BOOL fShow );
   virtual void ASWPUBMETHOD ObjectIsEmpty( void );
public:
   void ASWPUBMETHOD RemoveAllAdvises(void);
   inline PASWCONTAINER GetControllingContainer( void )
   {
      return m_pAswContainer;
   }
   inline PASWDATAMANAGER GetDataManager( void )
   {
      return &m_CODataMgr;
   }
   virtual HRESULT ASWPUBMETHOD STDMETHODCALLTYPE
   QueryInterface( REFIID riid, LPVOID ASWPTR * ppvObj );
   BOOL ASWPUBMETHOD IsInplaceEverAllowed( void );
   inline void SetInsideOutObject( BOOL bInsideOutObject )
   {
      m_bInsideOutObject = bInsideOutObject;
   }
   inline BOOL IsInsideOutObject( void )
   {
      return m_bInsideOutObject;
   }
   inline BOOL  DoesPassCreateGuard( void )
   {
      return (m_fCreatingGuard) ? FALSE : TRUE;
   }
   inline BOOL IsObjectStatic(void)
   {
      return m_bStaticObject;
   }
   inline void SetStaticObject(BOOL bStatic)
   {
      m_bStaticObject = bStatic;
   }
   inline BOOL IsObjectClosing(void)
   {
      return m_fClosingGuard;
   }
   inline BOOL IsObjectCreating(void)
   {
      return m_fCreatingGuard;
   }
   enum ObjectUIState
   {
   ASWOBJ_INACTIVE_UISTATE,     
   ASWOBJ_OPEN_UISTATE,         
   ASWOBJ_ACTIVE_UISTATE,       
   ASWOBJ_ACTIVEUI_UISTATE      
   };
   inline ObjectUIState GetUIState(void) const
   {
      ASWMETHOD("GetUIState", (CH_CLASS_CASWCONTAINEDOBJECT));
      ASWENTRY();
      ASWEXITF(m_ObjectUIState,"UIState=%d", (ASWSTR, m_ObjectUIState));
   }
   inline void SetUIState( ObjectUIState state )
   {
      ASWMETHOD("SetUIState", (CH_CLASS_CASWCONTAINEDOBJECT));
      ASWENTRYF_NC("state=%d", (ASWSTR, state));
      m_ObjectUIState = state;
      ASWEXITX();
   }
   virtual ASWOBJSTATE ASWPUBMETHOD GetState( void );
   inline LPOLECLIENTSITE GetOleClientSite(void)
   {
      return &m_implOleClientSite;
   }
#ifdef ASWANASSERT
   inline BOOL DebugVerifyObjState( void )
   {
      ASWMETHOD( "DebugVerifyObjState", (CH_CLASS_CASWCONTAINEDOBJECT) );
      BOOL bRunning = FALSE;
      BOOL bRetVal = FALSE;
      LPOLEOBJECT pOleObject = (LPOLEOBJECT)(GetOleObjectRefHolder()->Get());
      if( pOleObject )
      {
         bRunning = OleIsRunning( pOleObject );
         bRetVal = FALSE;
         if( bRunning )
         {
            ASWASSERT ( OBJSTATE_RUNNING_CANUNLOAD==CAswCOMSite::GetState() ||
                 OBJSTATE_RUNNING_CANNOTUNLOAD==CAswCOMSite::GetState() ||
                 OBJSTATE_RUNNING_UNLOADUNSURE==CAswCOMSite::GetState() ||
                 OBJSTATE_RUNNING_BUTCLOSED==CAswCOMSite::GetState() )
            if( OBJSTATE_RUNNING_CANUNLOAD==CAswCOMSite::GetState() ||
                 OBJSTATE_RUNNING_CANNOTUNLOAD==CAswCOMSite::GetState() ||
                 OBJSTATE_RUNNING_UNLOADUNSURE==CAswCOMSite::GetState() ||
                 OBJSTATE_RUNNING_BUTCLOSED==CAswCOMSite::GetState() )
               bRetVal = TRUE;
         }
         else
         {
            ASWASSERT ( OBJSTATE_LOADED==CAswCOMSite::GetState() );
            if( OBJSTATE_LOADED==CAswCOMSite::GetState() )
               bRetVal =  TRUE;
         }
      }
      else
      {
         ASWASSERT( OBJSTATE_PASSIVE == CAswCOMSite::GetState() );
         if( OBJSTATE_PASSIVE == CAswCOMSite::GetState() )
            bRetVal = TRUE;
      }
      ASWEXITX();
      return bRetVal;
   }
#endif
   inline CAswAspectListHead * GetAspectListHead(void)
   {
      return &m_lhListOfAspects;
   }
   inline void AddAspectToList(PASWASPECT pAspect)
   {
      m_lhListOfAspects.Insert( pAspect );
   }
   inline void RemoveAspectFromList(PASWASPECT pAspect)
   {
      m_lhListOfAspects.Remove( pAspect );
   }
   inline PASWASPECT GetFirstAspect( void )
   {
      return m_lhListOfAspects.GetFirst();
   }
   inline PASWASPECT GetNextAspect( void )
   {
      return m_lhListOfAspects.GetNext();
   }
   inline PASWREFHOLDER GetOleObjectRefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIOleObject;
   }
   inline PASWREFHOLDER GetOleInPlaceObjectRefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIOleInPlaceObject;
   }
   inline PASWREFHOLDER GetPersistStorageRefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIPersistStorage;
   }
   inline PASWREFHOLDER GetViewRefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIViewObject;
   }
   inline PASWREFHOLDER GetView2RefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIViewObject2;
   }
   inline PASWREFHOLDER GetOleCacheRefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIOleCache;
   }
   inline PASWREFHOLDER GetOleCache2RefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIOleCache2;
   }
   inline PASWREFHOLDER GetOleLinkRefHolder( void ) const
   {
      return (PASWREFHOLDER)&m_refhIOleLink;
   }
   inline LPADVISESINK GetAdviseSinkInterface( void )
   {
      return &m_implAdviseSink;
   }
   inline PASWINPLACEPRESSITE GetActivatedIPPresSite( void ) const
   {
      return m_pActivatedIPPresSite;
   }
   inline void SetActivatedIPPresSite(PASWINPLACEPRESSITE pIPSite)
   {
      m_pActivatedIPPresSite = pIPSite;
   }
   inline void ClearActiveSites(void)
   {
      m_pActivatedPresSite = NULL;
      m_pActivatedDataSite = NULL;
      m_pActivatedIPPresSite = NULL;
   }
   void ASWPUBMETHOD SetLinkType( ASWOBJLINKTYPE LinkType );
   void ASWPUBMETHOD SetLinkType( DWORD dwLinkType );
   ASWOBJLINKTYPE ASWPUBMETHOD GetLinkType( void );
   inline BOOL IsLinked( void )
   {
      return ( LINKTYPE_NONE != GetLinkType() );
   }
   inline BOOL IsLinkAvailable( void ) const
   {
      return ( ! m_fLinkUnavailable );
   }
   inline void SetLinkAvailable( BOOL fLinkAvailable )
   {
      m_fLinkUnavailable = ! fLinkAvailable;
   }
   CAswResult ASWPUBMETHOD GetMoniker(DWORD              dwAssign,
                                      DWORD              dwWhichMoniker,
                                      LPMONIKER ASWPTR*  ppmk);
   CAswResult ASWPUBMETHOD SetMoniker(DWORD dwWhichMoniker, LPMONIKER pmk);
   CAswResult ASWPUBMETHOD GetRelativeMoniker(DWORD               dwAssign,
                                              LPMONIKER ASWPTR*   ppmk);
   CAswResult ASWPUBMETHOD GetFullMoniker(DWORD             dwAssign,
                                          LPMONIKER ASWPTR* ppmk);
   PASWPRESSITE        ASWPUBMETHOD LinkGetPresSite( void );
   PASWINPLACEPRESSITE ASWPUBMETHOD LinkGetIPPresSite( void );
   PASWDATASITE        ASWPUBMETHOD LinkGetDataPresSite( void );
   CAswResult ASWPUBMETHOD BreakLink(void);
   BOOL ASWPUBMETHOD IsUndoSupported( void );
   CAswResult WriteExtentInfoStream( PASWSTORAGE pAswStorage );
   CAswResult ReadExtentInfoStream( PASWSTORAGE pAswStorage );
   CAswResult WriteLinkInfoStream(PASWSTORAGE pAswStorage);
   CAswResult GetLinkInfo(LPASWOLESTR ASWPTR*  ppszDisplayName,
                          ULONG       ASWPTR*  plenFileName,
                          LPASWOLESTR ASWPTR*  ppszFullLinkType,
                          LPASWOLESTR ASWPTR*  ppszShortLinkType,
                          BOOL        ASWPTR*  pfSourceAvailable,
                          BOOL        ASWPTR*  pfIsSelected = NULL);
   virtual CAswResult ASWPUBMETHOD IsUpToDate( void );
   virtual CAswResult ASWPUBMETHOD Update( void );
   virtual void ASWPUBMETHOD NotifyOfRename( LPMONIKER pmk );
   virtual void ASWPUBMETHOD NotifyOfDocRename( LPASWCOLESTR pszDocumentName );
   inline BOOL IsObjectSavingItself(void)
   {
      return m_fObjectSavingItself;
   }
   inline void StartObjectSave(void)
   {
      m_fObjectSavingItself = TRUE;
   }
   inline void EndObjectSave(void)
   {
      m_fObjectSavingItself = FALSE;
   }
   inline void SetPersistDirty(BOOL fDirty = TRUE)
   {
      m_fPersistChange = fDirty;
   }
   inline IsPersistDirty(void)
   {
      return m_fPersistChange;
   }
   inline void SetOpenObjectSaveNeeded( BOOL fOpenObjectSaveNeeded )
   {
      m_fOpenObjectSaveNeeded = fOpenObjectSaveNeeded;
   }
   inline BOOL GetOpenObjectSaveNeeded( void ) const
   {
      return m_fOpenObjectSaveNeeded;
   }
private:
   CAswResult CreateObjectChildStorage( PASWSTORAGE  pAswParentStg,
                                        LPASWCOLESTR lpszObjectName );
   CAswResult DoCreate( DWORD      dwFlags,
                        CLSID      clsid,
                        LPASWCOLESTR  lpszFileName,
                        LPUNKNOWN * lplpUnknownObj );
   CAswResult DoCreateData( ASWCREATEFROMDATA  ct,
                            PCASWREFHOLDER     prhDataObject,
                            CLIPFORMAT         cfFormat,
                            UINT               uCacheOpt,
                            LPUNKNOWN * lplpUnknownObj );
   CAswResult SetupCreatedObject( LPUNKNOWN lpUnknownObj );
   CAswResult SetupOleObject( LPUNKNOWN lpUnknownObj );
   CAswResult UseIconAspect( HGLOBAL hMetaPict );
   CAswResult ASWPUBMETHOD 
   SaveToStorage( PASWSTORAGE pNewAswStorage, BOOL fRemember );
   CAswResult ASWPUBMETHOD
   DoSaveFromParent( PASWSTORAGE pParentStorage, BOOL fRemember );
   inline void SetHandsOff( BOOL fHandsOff )
   {
      m_fHandsOff = fHandsOff;
   }
   inline BOOL GetHandsOff( void ) const
   {
      return m_fHandsOff;
   }
   inline void SetSaveDone( BOOL fSaveDone )
   {
      m_fSaveDone = fSaveDone;
   }
   inline BOOL GetSaveDone( void ) const
   {
      return m_fSaveDone;
   }
   inline void SetCreateType( DWORD dwFlags )
   {
      ASWMETHOD( "SetCreateType", (CH_CLASS_CASWCONTAINEDOBJECT) );
      if ( (dwFlags & IOF_CREATELINKOBJECT) )
         m_dwOleCreateType = IOF_CREATELINKOBJECT;
      else if ( (dwFlags & IOF_CREATEFILEOBJECT) )
         m_dwOleCreateType = IOF_CREATEFILEOBJECT;
      else
         m_dwOleCreateType = IOF_CREATENEWOBJECT;
      ASWEXIT();
   }
   static CAswResult FindMenuItemPosition( HMENU  hPopup,
                                           UINT   idItemMin,
                                           UINT   idItemMax,
                                           UINT   idItem,
                                           UINT * pPosition);
   CAswResult DoActivate( LPMSG lpMsg,
                          PASWPRESSITE pAswPresSite,
                          LONG         lVerb );
   BOOL ResetStorage(BOOL fDelete = FALSE);
   inline BOOL IsInOleRun(void)
   {
      return m_fRunGuard;
   }
public:
   CAswListEl     m_leContainedObjectListElement;
private:
   CAswStorage             m_AswStorage;
   PASWCONTAINER           m_pAswContainer;
   CAswOleClientSiteImpl   m_implOleClientSite;
   CAswOleInPlaceSiteImpl  m_implOleInPlaceSite;
   CAswAdviseSink2Impl     m_implAdviseSink;
   CAswDataManager         m_CODataMgr;
   CAswAspectListHead      m_lhListOfAspects;
   CAswRefHolder           m_refhIOleObject;          
   CAswRefHolder           m_refhIOleInPlaceObject;   
   CAswRefHolder           m_refhIPersistStorage;     
   CAswRefHolder           m_refhIViewObject;         
   CAswRefHolder           m_refhIViewObject2;        
   CAswRefHolder           m_refhIOleCache;           
   CAswRefHolder           m_refhIOleCache2;          
   CAswRefHolder           m_refhIOleLink;            
   CAswRefHolder           m_refhIEnumOLEVERB;        
   DWORD                   m_dwAdviseConnection;
   BOOL                    m_fCreatingGuard;
   BOOL                    m_fClosingGuard;
   BOOL                    m_fRunGuard;
   DWORD                   m_dwOleCreateType;
   ASWOBJLINKTYPE          m_LinkType;
   BOOL                    m_fLinkTypeSet;
   BOOL                    m_fLinkUnavailable;
   BOOL                    m_fMonikerAssigned;
   BOOL                    m_fHandsOff;
   BOOL                    m_fSaveDone;
   BOOL                    m_fOpenObjectSaveNeeded;
   BOOL                    m_fPersistChange;
   BOOL                    m_fObjectSavingItself;
   ObjectUIState           m_ObjectUIState;
   BOOL                    m_bInsideOutObject;
   BOOL                    m_bStaticObject;
   PASWPRESSITE            m_pActivatedPresSite;
   PASWINPLACEPRESSITE     m_pActivatedIPPresSite;
   PASWDATASITE            m_pActivatedDataSite;
   CAswUIOleLink           m_auiolLink;
};
MAKELISTHEADCLASS(CAswContainedObject);
#include <aspop.h>
#endif      
