/* NOSHIP */
/*****************************************************************************
 *
 * Class: CAswFrameWindow
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Description:
 *    An Aswan Core Class, CAswFrameWindow need only be instantiated by top
 *    level applications that support editing of objects in-place.  If the
 *    container does not support in-place objects there is no need to
 *    instantiate, or derive a class from this class.  This class should be
 *    instantiated for both MDI and SDI Applications.   For SDI applications,
 *    CAswDocWindow should not be instantiate - CAswFrameWindow should be
 *    associate with the document window.
 *
 *    CAswFrameWindow inherits from CAswWindowBase, those methods listed above
 *    that  are appended with an "*" are derived from CAswWindowBase.
 *
 *    Servers that are also containers only need to instantiate this class when
 *    OPEN or LINKED, and they are showing there own top level frame.
 *
 *****************************************************************************/

#ifndef ASFRMWND_H
#define ASFRMWND_H
#include <aspack.h>
class ASWPUBCLASS TKOleInPlaceFrame : public CAswIOleInPlaceFrame
{
public:
   TKOleInPlaceFrame(PASWCONTROL pControl, PASWFRAMEWINDOW pFrameWindow);
   STDMETHOD(GetWindow) (THIS_ HWND FAR* lphwnd);
   STDMETHOD(ContextSensitiveHelp) (THIS_ BOOL fEnterMode);
   STDMETHOD(GetBorder) (THIS_ LPRECT lprectBorder);
   STDMETHOD(RequestBorderSpace) (THIS_ LPCBORDERWIDTHS lpborderwidths);
   STDMETHOD(SetBorderSpace) (THIS_ LPCBORDERWIDTHS lpborderwidths);
   STDMETHOD(SetActiveObject) (THIS_ LPOLEINPLACEACTIVEOBJECT lpActiveObject,
                   LPCOLESTR lpszObjName);   
   STDMETHOD(InsertMenus) (THIS_ HMENU hmenuShared,
                                       LPOLEMENUGROUPWIDTHS lpMenuWidths);
   STDMETHOD(SetMenu) (THIS_ HMENU hmenuShared, HOLEMENU holemenu,
                                       HWND hwndActiveObject);
   STDMETHOD(RemoveMenus) (THIS_ HMENU hmenuShared);
   STDMETHOD(SetStatusText) (THIS_ LPCOLESTR lpszStatusText);     
   STDMETHOD(EnableModeless) (THIS_ BOOL fEnable);
   STDMETHOD(TranslateAccelerator) (THIS_ LPMSG lpmsg, WORD wID);
   inline PASWFRAMEWINDOW GetFrameWindow(void)
   {
      return m_pFrameWindow;
   }
private:
   PASWFRAMEWINDOW m_pFrameWindow;
};
class ASWPUBCLASS CAswFrameWindow : public CAswWindowBase
{
   ASWAN_DECLARATIONS(3);    
public:
   ASWPUBMETHOD CAswFrameWindow( PASWCONTROL pAswControl, LPUNKNOWN pUnk = NULL );
   virtual ASWPUBMETHOD ~CAswFrameWindow( void );
   CAswResult ASWPUBMETHOD Setup( HWND           hFrameWnd,
                                  HWND           hMDIClientWnd,
                                  BOOL           bTrueMDI,
                                  PASWMENUBASE   pAswMenuBase,
                                  PASWSPACEBASE  pAswSpaceBase = NULL,
                                  PASWCONTACCEL  pAswAccel = NULL );
   CAswResult ASWPUBMETHOD Shutdown( BOOL fEmergency = FALSE );
   virtual HRESULT ASWPUBMETHOD SetStatusText( LPASWCOLESTR lpszStatusText );
   virtual void ASWPUBMETHOD EnableModelessWindows( BOOL fEnable );
   void ASWPUBMETHOD EnableObjectsModelessWindows( BOOL fEnable );
   virtual BOOL ASWPUBMETHOD IsFrameWindow( void );
   virtual PASWFRAMEWINDOW ASWPUBMETHOD GetFrameWindow(void);
   virtual LPOLEINPLACEUIWINDOW ASWPUBMETHOD GetOleInPlaceUIWindow( void );
   inline PASWMENUBASE GetMenuHandler(void) const
   {
      return m_pAswMenuBase;
   }
   inline PASWCONTACCEL GetAccelHandler(void) const
   {
      return m_pAswAccel;
   }
   inline BOOL IsMDI()
   {
      return m_bTrueMDI;
   }
   inline LPOLEINPLACEFRAME GetOleInPlaceFrame( void )
   {
      return (LPOLEINPLACEFRAME)&m_implOleInPlaceFrame;
   }
   inline PASWCONTACCEL GetAccelObj( void )
   {
      return m_pAswAccel;
   }
   inline HWND GetMDIClientWnd( void )
   {
      ASWMETHOD("GetMDIClientWnd", (CH_CLASS_CASWFRAMEWINDOW));
      ASWENTRY();
      ASWASSERT( IsMDI() );
      ASWASSERT( m_hMDIClientWnd );
      ASWEXITX();
      return m_hMDIClientWnd;
   }
   inline HWND GetFrameWnd(void)
   {
      return m_hFrameWnd;
   }
private:
   TKOleInPlaceFrame    m_implOleInPlaceFrame;
   BOOL                 m_bTrueMDI;
   PASWCONTACCEL        m_pAswAccel;
   PASWMENUBASE         m_pAswMenuBase;
   HWND                 m_hMDIClientWnd;
   HWND                 m_hFrameWnd;
};
#include <aspop.h>
#endif   
