/* NOSHIP */
/*****************************************************************************
 *
 * aslist.h
 *
 * (C) Copyright Lotus Development Coporation 1994, 1996. All Rights Reserved.
 *
 * Classes in this file:
 *      CAswListEl
 *      CAswListHead
 *      CAswEmbeddedListHead
 *      XXXListHead (Generated by MAKELISTHEAD macro)
 *  
 * Description:
 *    Header describing classes related to aslist, a strongly typed linked list 
 *    for use in Aswan.  
 *    
 *    The   CAswListEl class defines a data structure which can be embedded
 *    in a class to be put on lists.   Multiple use is   possible, and
 *    the   list elements need not be at the beginning of the class:
 *
 *       //
 *       // A class that can be in two lists at once
 *       //
 *       class ClassInTwoLists 
 *       {
 *            int some_member_here_if_you_like;
 *            ListEl m_List1,   m_List2;
 *       };
 *
 *    XXXListHead is a strongly typed linked list (BTW:  XXXListHead
 *    would be a template if we were willing to rely on that compiler feature):
 *
 *       class SomeClass   
 *       {
 *            ListEl m_le;
 *       };
 *
 *       MAKELISTHEADCLASS(SomeClass);
 *
 *       class ClassWithHeadOfList:public CAswCtlBase 
 *       {
 *           int some_other_members_here_if_you_like;
 *           SomeClassListHead m_ListHead;
 *       };
 *
 *    To use the list:
 *
 *        ClassWithHeadOfList::SomeMethod()
 *        {
 *            SomeClass *pSc;    // Note pointer is strongly typed
 *
 *            for(pSc=ListHead.GetFirst(); pSc !=NULL; pSc=GetNext())
 *            {
 *               ...process element pSc.
 *            }  
 *        }
 *
 *****************************************************************************/

#ifndef  ASLIST_H
#define  ASLIST_H
#include <aspack.h>
typedef short AswPointerOffset_t;   
class ASWPUBCLASS CAswListEl : public CAswMemoryBase
{
public:
   inline CAswListEl( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      InitializeElement();
   }
   inline void InitializeElement( void ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      InitializePtrToNextEl();
      InitializePtrToPrevEl();
   }
   inline void PointtoSelf( void ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      SetPtrToNextEl( this );
      SetPtrToPrevEl( this );
   }
   inline PASWLISTEL GetPtrToNextEl( void ) const 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
#ifdef ASWANASSERT
      if ( m_pNextEl )
      {
         ASWGLOBALASSERT_RPTROKSIZE( m_pNextEl ); 
      }
#endif
      return m_pNextEl;
   }
   inline PASWLISTEL GetPtrToPrevEl( void ) const 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
#ifdef ASWANASSERT
      if ( m_pPrevEl )
      {
         ASWGLOBALASSERT_RPTROKSIZE( m_pPrevEl ); 
      }
#endif
      return m_pPrevEl;
   }
   inline void SetPtrToNextEl( PASWLISTEL pEl ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
#ifdef ASWANASSERT
      if ( pEl )
      {
         ASWGLOBALASSERT_RPTROKSIZE( pEl ); 
      }
#endif
      m_pNextEl = pEl;
   }
   inline void InitializePtrToNextEl( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      m_pNextEl = NULL;
   }
   inline void SetPtrToPrevEl( PASWLISTEL pEl ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
#ifdef ASWANASSERT
      if ( pEl )
      {
         ASWGLOBALASSERT_RPTROKSIZE( pEl ); 
      }
#endif
      m_pPrevEl = pEl;
   }
   inline void InitializePtrToPrevEl( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      m_pPrevEl = NULL;
   }
   inline BOOL IsInList( void ) const  
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      return  ( NULL != GetPtrToNextEl() );
   }
#ifdef ASWANASSERT
   BOOL CrossCheckElement( void ) const;
#endif
private:
   PASWLISTEL m_pNextEl;   
   PASWLISTEL m_pPrevEl;   
};
class ASWPUBCLASS CAswListHead : public CAswListEl 
{
public:
   inline CAswListHead( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      InitCurrentElement();               
      PointtoSelf();                      
      ASWAN_CROSSCHECKP( this );          
   }
#if defined(ASWANASSERT)
   inline ~CAswListHead( void )
   {
      ASWAN_CROSSCHECKP( this );             
   }
#endif
   inline PASWLISTEL GetFirst( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      ASWGLOBALASSERT( IsInList() );
      InitCurrentElement();
      return ( GetNext() );
   }
   PASWLISTEL ASWPUBMETHOD GetNext( void );
   inline BOOL IsEmpty( void ) const
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      ASWGLOBALASSERT( IsInList() );
      return ( CAswListEl::GetPtrToNextEl() == (PASWLISTEL) this );
   }
   void ASWPUBMETHOD InsertAt( PASWLISTEL pLe, PASWLISTEL prev );
   inline void Insert( PASWLISTEL pLe ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      ASWGLOBALASSERT_RPTROKSIZE( pLe );  
      InsertAt( pLe, this );              
   }                                    
   inline void Remove( PASWLISTEL pLe ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      ASWGLOBALASSERT( IsInList() );
      ASWAN_CROSSCHECKP( this );              
      ASWGLOBALASSERT_RPTROKSIZE( pLe );      
      ASWGLOBALASSERT( pLe->IsInList() );     
      if ( pLe->IsInList() )
      {
         PASWLISTEL next = pLe->GetPtrToNextEl();
         PASWLISTEL prev = pLe->GetPtrToPrevEl();
         ASWGLOBALASSERT( next );                
         ASWGLOBALASSERT( prev );                
         prev->SetPtrToNextEl( next );           
         next->SetPtrToPrevEl( prev );
      }
      pLe->InitializeElement();               
   }
   void ASWPUBMETHOD RemoveAll( void ); 
   inline void Reset( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      PointtoSelf();                      
   }
   inline void SetCurrentElement( PASWLISTEL pLe )
   {
      ASWGLOBALASSERT_RPTROKSIZE( pLe );
      ASWGLOBALASSERT( pLe->IsInList() );
      m_pleCurrentElement = pLe;
   }
#ifdef ASWANASSERT
   BOOL ASWPUBMETHOD CrossCheck( void );
#endif
private:
   PASWLISTEL m_pleCurrentElement;        
   inline void InitCurrentElement( void ) 
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      m_pleCurrentElement = this;         
   }
};
#ifndef __TURBOC__
const AswPointerOffset_t CASWMANAGEDLIST_UNINIT (-1);
#else
const AswPointerOffset_t CASWMANAGEDLIST_UNINIT = -1;
#endif
class ASWPUBCLASS CAswEmbeddedListHead : public CAswListHead 
{
public:
   inline CAswEmbeddedListHead( void )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      m_offset = CASWMANAGEDLIST_UNINIT;
   }
   inline void SetOffset( LPVOID ClassOrigin, 
                          LPVOID List )
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      ASWGLOBALASSERT( NULL != List );      
      m_offset = (AswPointerOffset_t) ((LPSTR)List - (LPSTR)ClassOrigin);
   }
   inline AswPointerOffset_t GetClassOffset( void ) const 
   { 
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      return m_offset; 
   }
   inline LPVOID GetClassPointer( LPVOID List ) const
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
#ifdef ASWANASSERT
      if ( List )
      {
         ASWGLOBALASSERT_RPTROK( List );
         ASWGLOBALASSERT_RPTROK( (LPVOID) (((LPSTR) List) - GetClassOffset()) );
      }
#endif
      LPVOID petpeeve;
      if ( NULL != List )
         petpeeve = (((LPSTR) List) - GetClassOffset() );
      else
         petpeeve = NULL;
      return petpeeve;
   }
   inline BOOL IsInitialized( void ) const
   {
      ASWGLOBALASSERT_RPTROKSIZE( this ); 
      return GetClassOffset() != CASWMANAGEDLIST_UNINIT;
   }
private:
   AswPointerOffset_t m_offset;     
};
#define  MAKELISTHEADCLASS(c) class ASWPUBCLASS c##ListHead:public CAswEmbeddedListHead{public:\
inline c##ListHead(CAswListEl c::*O){ASWGLOBALASSERT_RPTROKSIZE(this);m_ListOffset=O;}\
inline PASWLISTEL GetLE(c ASWPTR *p)const{ASWGLOBALASSERT_RPTROKSIZE(this);ASWGLOBALASSERT_RPTROKSIZE(p);return&(p->*m_ListOffset);}\
inline c ASWPTR *GetFirst(void){return(c ASWPTR *)GetClassPointer(CAswListHead::GetFirst());}\
inline c ASWPTR *GetNext(void){return(c ASWPTR *)GetClassPointer(CAswListHead::GetNext());}\
inline void InsertAt(c ASWPTR *p,c ASWPTR *prv){ASWGLOBALASSERT_RPTROKSIZE(this);CAswListHead::InsertAt(GetLE(p),GetLE(prv));}\
inline void Insert(c ASWPTR *p){ASWGLOBALASSERT_RPTROKSIZE(this);ASWGLOBALASSERT_RPTROKSIZE(p);if (!IsInitialized()){SetOffset(p,GetLE(p));}CAswListHead::Insert(GetLE(p));}\
inline void Remove(c ASWPTR *p){ASWGLOBALASSERT_RPTROKSIZE(this);CAswListHead::Remove(GetLE(p));}\
inline void SetCurrentElement(c ASWPTR *p){ASWGLOBALASSERT_RPTROKSIZE(this);CAswListHead::SetCurrentElement(GetLE(p));}\
void ASWPUBCLASS Iterate(void(c::*f)());\
BOOL ASWPUBCLASS ANDIterate(BOOL(c::*f)());\
BOOL ASWPUBCLASS ORIterate(BOOL(c::*f)());\
private:CAswListEl c::*m_ListOffset;}
#define  MAKELISTHEADCLASSIMPL(cls)\
void cls##ListHead::Iterate(void (cls::*f)())\
{ASWGLOBALASSERT_RPTROKSIZE(this);cls ASWPTR *p;\
for(p=GetFirst();NULL!=p;p=GetNext()){ASWGLOBALASSERT_RPTROKSIZE(p);(p->*f)();}\
}\
BOOL cls##ListHead::ANDIterate(BOOL (cls::*f)())\
{ASWGLOBALASSERT_RPTROKSIZE(this);int r=TRUE;cls ASWPTR *p;\
for(p=GetFirst();NULL!=p;p=GetNext()){ASWGLOBALASSERT_RPTROKSIZE(p);r&=(p->*f)();}return r;\
}\
BOOL cls##ListHead::ORIterate(BOOL (cls::*f)())\
{ASWGLOBALASSERT_RPTROKSIZE(this);int r=FALSE;cls ASWPTR *p;\
for(p=GetFirst();NULL!=p;p=GetNext()){ASWGLOBALASSERT_RPTROKSIZE(p);r|=(p->*f)();}return r;\
}
#include <aspop.h>
#endif 
