/*******************************************************************************
* FILE NAME: idataex.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the classes:                                             *
*     IAswClipboardSource                                                      *
*     IAswPasteTarget                                                          *
*     IAswDragSource                                                           *
*     IAswDropTarget                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*   Portions or this component are based on work originating from Lotus.       *
*   (C) Copyright Lotus Development Corporation. 1996 - All Rights Reserved.   *
*                                                                              *
*******************************************************************************/
#include <precomp.hpp>
#ifdef IC_SCREEN_DEBUG            
#include <iostream.h>
#endif

#include "idataex.hpp"

#include <iview.hpp>
#include <istatnry.hpp>
#include <iguibndl.hpp>
#include <icompnen.hpp>
#include <icompimp.hpp>
#include <iembmod.hpp>
#include <iecmpimp.hpp>


/*******************************************************************************
*                                                                              *
*                             IAswClipboardSource                              *
*                                                                              *
*******************************************************************************/

// -----------------------------------------------------------------------------
// IAswClipboardSource
// -----------------------------------------------------------------------------
// constructor
IAswClipboardSource::IAswClipboardSource( PASWCONTROL pControl ) :
    CAswClipboardSource(pControl),
    fpComponent( NULL ),
    fClipRootStg(pControl),
    fClipStg(pControl),
    fClipDocFile(pControl),
    fAswObjDescriptor(pControl)
{
}

// -----------------------------------------------------------------------------
// ~IAswClipboardSource
// -----------------------------------------------------------------------------
// Destructor
IAswClipboardSource::~IAswClipboardSource(void)
{
   Shutdown(TRUE);
}

// -----------------------------------------------------------------------------
// Setup
// -----------------------------------------------------------------------------
// Creates a temp file in which to place snapshots of the copied objects
CAswResult ASWCLIENTMETHOD
IAswClipboardSource::Setup( PASWSOURCEFORMATSET )
{
    CAswResult  ar;

    // setup the base class
    ar = CAswClipboardSource::Setup();
    IASWASSERTSTATE( ar );

    ar = fClipRootStg.Setup();
    IASWASSERTSTATE( ar );

    ar = fAswObjDescriptor.Setup();
    IASWASSERTSTATE( ar );

    // fstgmetafile is a general-purpose memory handle
    fstgMetaFile.tymed = TYMED_NULL;
    fstgMetaFile.hGlobal = NULL;
    fstgMetaFile.pUnkForRelease = NULL;

    ar = fClipDocFile.Setup();
    IASWASSERTSTATE( ar );

    // create the temp file
    DOCFILESTGCREATE dfsCreate;
    dfsCreate.lpstrName = NULL;
    dfsCreate.dwgrfMode = STGM_CREATE | STGM_READWRITE | STGM_DIRECT | STGM_DELETEONRELEASE;
    dfsCreate.dwReserved = 0;
    ar = fClipDocFile.Create( &fClipRootStg, &dfsCreate );
    IASWASSERTSTATE( ar );

    return ar;

}

// -----------------------------------------------------------------------------
// Shutdown
// -----------------------------------------------------------------------------
CAswResult ASWCLIENTMETHOD
IAswClipboardSource::Shutdown( BOOL fEmergency )
{
    CAswResult  ar;

    // check to see if a shutdown is OK
    if( fEmergency || IsShutdownOK() )
    {
        ReleaseStgMedium(&fstgMetaFile);
        fClipStg.Shutdown(fEmergency);
        fClipRootStg.Shutdown(fEmergency);
        fClipDocFile.Shutdown(fEmergency);
        fAswObjDescriptor.Shutdown(fEmergency);
        CAswClipboardSource::Shutdown(fEmergency);
        ar.Set(S_OK, ASD_OK);
    }
    else
    {
        ar.Set(ASW_E_SHUTDOWNNOTOK,ASD_UNCHANGED);
    }

    return ar;
}

// -----------------------------------------------------------------------------
// RenderMetafilePict
// -----------------------------------------------------------------------------
// This method extracts the metafile presentation from the cache
HGLOBAL
/*IAswClipboardSource::*/RenderMetafilePict( IEmbeddedComponentImp* pCompImp )
{

    // Create an empty memory metafile DC
    HDC hDC = (HDC)CreateMetaFile( NULL ); // TODO: must use CreateEnhMetaFile for 32 bit GDI functions
    if( hDC == NULL )
        return NULL;

    // Set the mapping mode so that the metafile is "scalable"
    // then set the size (f into the metafile
    SetMapMode( hDC, MM_ANISOTROPIC );
    // get the current bounds of our window
    RECTL rc, rcBounds;
    IRectangle rect = pCompImp->area();

    rc.left   = 0; //rect.minX();
    rc.top    = 0; //rect.minY();
    rc.right  = rect.width();  //rect.maxX();
    rc.bottom = rect.height(); //rect.maxY();

    rcBounds.left   = 0;
    rcBounds.top    = 0;
    rcBounds.right  = rect.width();
    rcBounds.bottom = rect.height();

    // set origin of bitmap to 0,0
    SetWindowOrgEx( hDC, 0, 0, NULL );
    SetWindowExtEx( hDC, rect.width(), rect.height(), NULL );
    SetViewportOrgEx( hDC, 0, 0, NULL );
    SetViewportExtEx( hDC, rect.width(), rect.height(), NULL );

    //Erase the background for bitmaps and metafiles.
    HPEN hPen = (HPEN)SelectObject( hDC, GetStockObject(NULL_PEN) );
    Rectangle( hDC, rc.left, rc.top, rc.right+1, rc.bottom+1 );
    SelectObject( hDC, hPen );

    // call draw on the object's pres site
    pCompImp->presSite().Draw( &rc, NULL, hDC, FALSE, NULL, &rcBounds );

    HMETAFILE hMF = CloseMetaFile( hDC ); // TODO: use CloseEnhMetaFile
    if( hMF == NULL )
        return NULL;

    // now that we have a handle to the metafile we wrap it into a METAFILEPICT structure
    // in global memory
    HGLOBAL hMem = GlobalAlloc( GMEM_DDESHARE | GMEM_MOVEABLE, sizeof(METAFILEPICT) );
    if( hMem == NULL )
    {
        DeleteMetaFile(hMF);
        return NULL;
    }

    LPMETAFILEPICT pMF = (LPMETAFILEPICT)GlobalLock( hMem );
    pMF->hMF = hMF;
    pMF->mm = MM_ANISOTROPIC;

    //Insert the extents in MM_HIMETRIC units.
    // RectConvertMappings(&rc, FALSE);
    pMF->xExt = XformWidthInPixelsToHimetric( NULL, rc.right );
    pMF->yExt = XformHeightInPixelsToHimetric( NULL, rc.bottom );

    GlobalUnlock(hMem);
    return hMem;

}

// -----------------------------------------------------------------------------
// PutItemOnClipboard
// -----------------------------------------------------------------------------
// snaps a copy of the object to be put on the clipboard into the temporary
// storage, then calls PutObjectOnClipboard to expose the available clipboard
// formats.
BOOL
IAswClipboardSource::PutItemOnClipboard( IEmbeddedComponent* pComponent )
{

    BOOL fRet = FALSE;

    // pComponent will be NULL for the entire model
    fpComponent = pComponent;

    // our native model
    IComponentImp* pInstance = IComponentStationery::stationery().bundle().component().objectInstance();
    
    do
    {
        // Reuse the clip storage (shutdown and setup)
        CAswResult ar = fClipStg.Shutdown();
        if( ASWFAILED(ar) )
        {
            IASWSHOWERROR( ar );
            break;
        }

        ar = fClipStg.Setup();
        if( ASWFAILED(ar) )
        {
            IASWSHOWERROR( ar );
            break;
        }

        // Create the storage in the clipboard file
        ar = fClipRootStg.CreateChildStorage( &fClipStg,ASWOLESTR("CLIPBOARD"),
                STGM_DIRECT | STGM_CREATE | STGM_READWRITE);
        if( ASWFAILED(ar) )
        {
            IASWSHOWERROR( ar );
            break;
        }

        // ------------------------------
        // object descriptor
        // ------------------------------
        if( fpComponent )
        {
            SIZEL sizeHi, sizePix;
            sizePix = fpComponent->area().size().asSIZEL();
            XformSizeInPixelsToHimetric( NULL, &sizePix, &sizeHi );
            fRet = fAswObjDescriptor.FillFromContainedObject(
                    fpComponent->containedObject(), 
                    &fpComponent->containedObject()->aspect(),
                    NULL, NULL, &sizeHi );
        }
        else
        {
            fRet = fAswObjDescriptor.FillFromObjectInstance( pInstance, L"" );
        }
        if( !fRet )
            break;


        // ------------------------------
        // Metafile
        // ------------------------------
        // snap a copy of the current presentation into fstgMetaFile (instance var)
        ReleaseStgMedium( &fstgMetaFile );
        fstgMetaFile.tymed = TYMED_NULL;
        HGLOBAL hMetaFile = NULL;

        if( fpComponent )
        {
            // call on embedded component to render
            hMetaFile = RenderMetafilePict( fpComponent->containedObject() );
        }
        else
        {
            // call on model to render
            hMetaFile = pInstance->RenderMetafilePict( false );
        }

        if( !hMetaFile )
        {
            #ifdef IC_SCREEN_DEBUG            
                cout << "IAswClipboardSource: failed to create metafile" << endl;
            #endif            
            break;
        }

        fstgMetaFile.tymed = TYMED_MFPICT;
        fstgMetaFile.hGlobal = hMetaFile;


        // ------------------------------
        // take a snapshot of the data
        // ------------------------------

        if( fpComponent )
        {
            // Copy the contents of the item's storage into the clipboard

            // First, get the persist storage from the contained object
            PASWREFHOLDER prhPStg = fpComponent->containedObject()->GetPersistStorageRefHolder();
            LPPERSISTSTORAGE pPStg = (LPPERSISTSTORAGE) prhPStg->Get();

            // If the item's storage is dirty then write to new storage. Otherwise,
            // do a copy.
            if( pPStg->IsDirty() == NOERROR )
            {
                // Dirty, tell persist to save to new storage
                HRESULT hr = OleSave( pPStg, fClipStg.GetRealStorage(), FALSE );
                pPStg->SaveCompleted( fClipStg.GetRealStorage() );
                ar.Set( hr, ASD_UNAVAILABLE );
                IASWSHOWERROR( ar );
            }
            else
            {
                // Not dirty, do a copy to
                PASWSTORAGE pStg = fpComponent->containedObject()->GetStorage();
                ar = pStg->CopyTo( &fClipStg );
                IASWSHOWERROR( ar );
            }
        }

        else
        {
            // snap a copy of the model and all embedded objects into the temporary storage
            ar = pInstance->GetEmbedSourceData( &fClipStg );
            IASWSHOWERROR( ar );
        }

        // Check for error
        if( ASWFAILED(ar) )
            break;

        // -------------------------------------------------
        // Put the available format sets on the clipboard
        // -------------------------------------------------
        SetFormatSet( &IComponentStationery::stationery().sourceFormatSet() );
        ar = PutObjectOnClipboard();
        if( ASWFAILED(ar) )
        {
            IASWSHOWERROR( ar );
            break;
        }

        fRet = TRUE;


    } while (FALSE);


    return fRet;

}



// -----------------------------------------------------------------------------
// GetData
// -----------------------------------------------------------------------------
// gets the data out of the temporary storage
CAswResult
IAswClipboardSource::GetData( LPFORMATETC pFormatEtc,
                              PASWSTGMEDUNION pAswStgMedium )
{

#ifdef IC_SCREEN_DEBUG            
cout << "Get Data: " <<
    "CLIPFORMAT=" << pFormatEtc->cfFormat <<
    ", Aspect="  << pFormatEtc->dwAspect <<
    ", tymed=" << pFormatEtc->tymed <<
    ", tymed=" << pAswStgMedium->tymed << endl;
#endif

    CAswResult  ar(DATA_E_FORMATETC,ASD_UNCHANGED);

    // ------------------
    // Object Descriptor 
    // ------------------
    // this is common to both native model and embedded objects, since the appropriate
    // info would have been saved into fAswObjDescriptor in PutItemOnClipboard
    // Note: a link source descriptor is the same as an object descriptor
    if(  ( pFormatEtc->cfFormat == IComponentStationery::stationery().cfObjectDescriptor() ||
           pFormatEtc->cfFormat == IComponentStationery::stationery().cfLinkSrcDescriptor() ) &&
         ( pFormatEtc->tymed & TYMED_HGLOBAL ) &&
           fAswObjDescriptor.IsValid() )
    {

        // Make a copy of the object descriptor that we saved when the
        // object was put on the clipboard.
        pAswStgMedium->tymed = TYMED_HGLOBAL;
        pAswStgMedium->pUnkForRelease = NULL;
        ar = fAswObjDescriptor.CopyTo( &pAswStgMedium->hGlobal );
        IASWSHOWERROR( ar );
        return ar;
    }



    // see if the calling program allocated the storage or if we have to
    switch( pAswStgMedium->tymed )
    {
        // The storage medium is not allocated (GetData)
        case TYMED_NULL:

            // ------------------
            // Metafile?
            // ------------------
            if( pFormatEtc->cfFormat == CF_METAFILEPICT &&
                ( pFormatEtc->tymed & TYMED_MFPICT ) &&
                ( fstgMetaFile.tymed == TYMED_MFPICT ))
            {

                // Make a copy of the meta file that we have stored
                HGLOBAL hNewPict = OleDuplicateData( fstgMetaFile.hGlobal, CF_METAFILEPICT,GHND|GMEM_SHARE );
                if( hNewPict )
                {
                    pAswStgMedium->tymed = TYMED_MFPICT;
                    pAswStgMedium->pUnkForRelease = NULL;
                    pAswStgMedium->hGlobal = hNewPict;
                    ar.Set(S_OK, ASD_OK);
                }

            }
            #ifdef IC_SCREEN_DEBUG            
            else
            // the only cases we handle for TYMED_NULL are object descriptor and metafile
            // assert( false );
            cout << "    *** error - did not handle GetData ***" << endl;
            #endif                

            break;

        // Handle the cases where we are to fill in a storage
        // the only case we support is an Embedded object
        case TYMED_ISTORAGE:
            // ------------------
            // Embedded Object
            // ------------------
            if( /*pFormatEtc->cfFormat == IComponentStationery::stationery().cfEmbeddedObject() &&*/
                (pFormatEtc->tymed & TYMED_ISTORAGE) )
            {
                // copy the saved info from the temp storage to the supplied storage
                ar = fClipStg.CopyTo( pAswStgMedium->pAswStorage );
                IASWSHOWERROR( ar );

            }
            #ifdef IC_SCREEN_DEBUG            
            else
                cout << "*** error - did not handle GetData ***" << endl;
            #endif                

            break;

        // Handle the cases where we are to fill in a stream
        // The only case we support is a link source
        case TYMED_ISTREAM:
            // ------------------
            // Link Source
            // ------------------
            if( pFormatEtc->cfFormat == IComponentStationery::stationery().cfLinkSource() )
            {
                if( fpComponent )
                    // ask the embedded object for a link
                    ar = fpComponent->containedObject()->GetLinkSourceData( pAswStgMedium->pAswStream );
                else
                    // ask the model for link info
                    ar = IComponentStationery::stationery().bundle().component().objectInstance()
                                ->GetLinkSourceData( pAswStgMedium->pAswStream );

                IASWSHOWERROR( ar );

            }
            #ifdef IC_SCREEN_DEBUG            
            else
                cout << "*** error - did not handle GetData ***" << endl;
            #endif                
            break;

    } // switch

    return ar;
}


/*******************************************************************************
*                                                                              *
*                                 IAswPasteTarget                              *
*                                                                              *
*******************************************************************************/

// -----------------------------------------------------------------------------
// Constructor
// -----------------------------------------------------------------------------
IAswPasteTarget::IAswPasteTarget( PASWCONTROL  pControl ) :
    CAswPasteTarget( pControl )
{ 
};

// -----------------------------------------------------------------------------
// QueryFormatSet
// -----------------------------------------------------------------------------
PASWTARGETFORMATSET
IAswPasteTarget::QueryFormatSet()
{
      return &IComponentStationery::stationery().targetFormatSet();
}


// -----------------------------------------------------------------------------
// ObjectPasted
// -----------------------------------------------------------------------------
// called when the user selects "Edit-Paste"
BOOL 
IAswPasteTarget::ObjectPasted(LPFORMATETC  pBestFormatetc,
                              BOOL         fLink,
                              HGLOBAL      hMetaPict)
{

   
    // get pointer to the embedder model
    IEmbedderModel* pModel = NULL;
    ::dynamicCastTo( pModel, IComponentStationery::stationery().bundle().component().model() );
    if( pModel )
    {
        IEmbeddedComponent* pComponent = pModel->createEmbeddedComponent();
        pModel->adopt( pComponent );
        CAswResult ar = pComponent->containedObject()->
            CreateFromPaste( this, fLink, ASWCACHEOPT_KEEPALL, hMetaPict );
        if( ASWFAILED( ar ) )
        {
            const char* msg = IBase::messageText(IC_CDF_INSERT_FAILED);
            IComponentStationery::stationery().messageBox( msg,
                                                  IMessageBox::moveable  |
                                                  IMessageBox::applicationModal|
                                                  IMessageBox::errorIcon |
                                                  IMessageBox::okButton);
//>>>>>            ShowAswResult( ar, msg );
            delete pModel->orphan( *pComponent );
            return FALSE;
        }


        // see if there is a source object descriptor
        // if so extract the original extent (if any) and the 
        // original aspect
        SIZEL sizeHi;
        PASWOBJECTDESCRIPTOR pDescriptor = GetSourceObjectDescriptor();
        if( pDescriptor )
        {

            // get extent from descriptor (must change AFTER call to initialize)
            sizeHi = pDescriptor->GetExtent();

            // get aspect - must change BEFORE call to initialize
            DWORD dwAspect = pDescriptor->GetDrawAspect();
            if( dwAspect == DVASPECT_ICON )
                pComponent->setAspect( IComponent::kIcon );

        } // there was a source object descriptor


        // if the aspect was OTHER than content then switch aspect
        // initialize the component
        pComponent->initialize( true /* new */, false /* activate */ );

        // if we got a size from the obj descriptor resize the component
        // (this must be done after initialize)
        if( sizeHi.cx != 0 )
        {

            SIZEL sizePix;
            // size is in himetric units - convert to pixels
            XformSizeInHimetricToPixels( NULL, &sizeHi, &sizePix );
            ISize newSize( sizePix );
            // cascade the item (to make room for adornments in in-place activation)
            IPoint newPos( 50,50 );
            // move and size the component
            pComponent->resize( IRectangle( newPos, newSize ) );

        } // a size was specified in the source obj desc.
    } // we are an embedder model
    return TRUE;
}
      

/*******************************************************************************
*                                                                              *
*                                 IAswDragSource                               *
*                                                                              *
*******************************************************************************/

// -----------------------------------------------------------------------------
// Constructor
// -----------------------------------------------------------------------------
IAswDragSource::IAswDragSource( PASWCONTROL  pControl ) :
    CAswDragSource( pControl )
{ 
};

// -----------------------------------------------------------------------------
// QueryFormatSet
// -----------------------------------------------------------------------------
PASWSOURCEFORMATSET
IAswDragSource::QueryFormatSet()
{
    return &IComponentStationery::stationery().sourceFormatSet();

}

// -----------------------------------------------------------------------------
// GetData
// -----------------------------------------------------------------------------
CAswResult 
IAswDragSource::GetData(LPFORMATETC     pFormatEtc,
                        PASWSTGMEDUNION pAswStgMedium )
{

    CAswResult ar( DATA_E_FORMATETC,ASD_UNCHANGED );

    // get a pointer to the currently selected component (if any)
    IEmbedderModel* pModel = NULL;
    ::dynamicCastTo( pModel, IComponentStationery::stationery().bundle().component().model() );
    IEmbeddedComponent* pComponent  = pModel->selectedComponent();

    // The currently selected item is always the source in a drag drop.
    // The Framework does not support drag/drop on the native model or "select all"
    if( !pComponent )
        return CAswResult( E_NOTIMPL, ASD_UNCHANGED );
    
    IEmbeddedComponentImp* pCompImp = pComponent->containedObject();    

    switch( pAswStgMedium->tymed )
    {
        // The storage medium is not allocated
        // This will be for an Object Descriptor or a Metafile
        case TYMED_NULL:
            // -------------------
            // Object Descriptor
            // -------------------
            if( pFormatEtc->cfFormat == IComponentStationery::stationery().cfObjectDescriptor()
                && (TYMED_HGLOBAL & pFormatEtc->tymed))
            {
                // Provide an object descriptor
                CAswObjectDescriptor aswDescriptor( IComponentStationery::stationery() );
                ar = aswDescriptor.Setup();
                IASWSHOWERROR( ar );
                // get current size
                SIZEL sizeHi, sizePix;
                sizePix = pComponent->area().size().asSIZEL();
                // size is in himetric units - convert to pixels
                XformSizeInPixelsToHimetric( NULL, &sizePix, &sizeHi );                

                if( ASWSUCCEEDED(ar) &&
                        aswDescriptor.FillFromContainedObject( pCompImp,
                            &pCompImp->aspect(), NULL, NULL, &sizeHi ) )
                {
                    // Set the offset points: top,left of rect and mouse position
                    POINTL ptlTL,ptlMouse;
                    POINT    pt;

                    // top,left of rect
                    ptlTL.x = 0;
                    ptlTL.y = 0;

                    // mouse down position
                    IPoint offset = pComponent->dragMouseOffset();
                    pt.x = offset.x();
                    pt.y = offset.y();

                    ptlMouse.x = (long)XformWidthInPixelsToHimetric( NULL, pt.x );
                    ptlMouse.y = (long)(-XformHeightInPixelsToHimetric( NULL, pt.y ));

                    aswDescriptor.SetOffsetPoint( ptlTL,ptlMouse );

                    // Copy the object descriptor to the return storage medium
                    ar = aswDescriptor.CopyTo(&pAswStgMedium->hGlobal);
                    if( ASWSUCCEEDED(ar) )
                    {
                        pAswStgMedium->tymed = TYMED_HGLOBAL;
                        pAswStgMedium->pUnkForRelease = NULL;
                    }

                    else
                    {
                        IASWSHOWERROR( ar );
                    }

                } // decriptor setup OK

            } // want an object descriptor

/*
            else
            // -------------------
            // metafile
            // -------------------
            if (CF_METAFILEPICT == pFormatEtc->cfFormat
            &&
            (TYMED_MFPICT & pFormatEtc->tymed))
            {
            // Provide a metafilepict. Ask the object for one.
            do
            {
            // Get the IDataObject from the contained object
            PASWREFHOLDER  prhData;
            pItem->GetContainedObject()->GetDataManager()->GetRefHolder(&prhData);
            if (!prhData)
            break;

            // Ask for a metafile from the data object
            LPDATAOBJECT   pData = (LPDATAOBJECT) prhData->Get();
            if (!pData)
            break;

            FORMATETC   fmtetc;
            fmtetc.cfFormat = CF_METAFILEPICT;
            fmtetc.dwAspect = pFormatEtc->dwAspect;
            fmtetc.ptd      = NULL;
            fmtetc.tymed    = TYMED_MFPICT;
            fmtetc.lindex   = -1;

            STGMEDIUM stg;
            stg.tymed = TYMED_NULL;
            HRESULT  hr = pData->GetData(&fmtetc,&stg);
            if (FAILED(hr))
            break;

            // Setup the return
            SetAswStgMediumFromStgMedium(pAswStgMedium,&stg);
            ar.Set(hr,ASD_OK);
            }
            while (FALSE);
            }
*/
            break;

        // Handle the cases where we are to fill in a storage
        case TYMED_ISTORAGE:

            if (pFormatEtc->cfFormat == IComponentStationery::stationery().cfEmbeddedObject()
                    && (TYMED_ISTORAGE & pFormatEtc->tymed))
            {

                // Clone our current storage into the storage medium
                //
                // First, get the persist storage from the contained object
                PASWREFHOLDER prhPStg = pCompImp->GetPersistStorageRefHolder();
                LPPERSISTSTORAGE pPStg = (LPPERSISTSTORAGE) prhPStg->Get();

                // Next, build an OLE storage medium from our medium
                STGMEDIUM medium;
                SetStgMediumFromAswStgMedium(&medium,pAswStgMedium);

                // Use the OLE utility method to do the copy
                HRESULT hr = OleStdGetOleObjectData(pPStg,pFormatEtc,&medium,FALSE);

                // Make sure to release the storage from our temp medium.
                // I know I have a storage, so this will only do a release.
                ReleaseStgMedium(&medium);

                // Set the aswan error return
                ar.Set(hr, ASD_UNAVAILABLE);

            }
            #ifdef IC_SCREEN_DEBUG            
            else
                cout << "*** error - did not handle embedded object drag GetData ***" << endl;
            #endif            


            break;

        // Handle the cases where we are to fill in a stream
        // The only case we support is a link source
        case TYMED_ISTREAM:
            // ------------------
            // Link Source
            // ------------------
            if( pFormatEtc->cfFormat == IComponentStationery::stationery().cfLinkSource() )
            {
                // ask the embedded object for a link
                ar = pCompImp->GetLinkSourceData( pAswStgMedium->pAswStream );
                IASWSHOWERROR( ar );

            }
            #ifdef IC_SCREEN_DEBUG            
            else
                cout << "*** error - did not handle Link source drag GetData ***" << endl;
            #endif            
            break;
    }

    return ar;
}


/*******************************************************************************
*                                                                              *
*                                 IAswDropTarget                               *
*                                                                              *
*******************************************************************************/

// -----------------------------------------------------------------------------
// Constructor
// -----------------------------------------------------------------------------
IAswDropTarget::IAswDropTarget( PASWCONTROL pControl ) :
    CAswDropTarget( pControl, NULL ),
    fpFeedback( NULL )
{
}


// -----------------------------------------------------------------------------
// ObjectDropped
// -----------------------------------------------------------------------------
BOOL ASWCLIENTMETHOD
IAswDropTarget::ObjectDropped( POINTL      pt,
                               LPFORMATETC pBestFormatetc,
                               DWORD       dwEffect,
                               BOOL        /*fRightMouseDrop*/ )

{
    CAswResult ar;

    // get pointer to the embedder model
    IEmbedderModel* pModel = NULL;
    ::dynamicCastTo( pModel, IComponentStationery::stationery().bundle().component().model() );
    if( pModel )
    {
        IEmbeddedComponent* pComponent = NULL;
        PASWOBJECTDESCRIPTOR pDescriptor = GetSourceObjectDescriptor();

        // if this is an internal action and they want to "move"
        // we just want to move the object on the screen
        if( dwEffect == DROPEFFECT_MOVE && IsInternalAction() )
            pComponent = pModel->selectedComponent();

        else
        {
            // not an internal action - create a new component
            pComponent = pModel->createEmbeddedComponent();
            pModel->adopt( pComponent );
            BOOL bLink = dwEffect & DROPEFFECT_LINK;
            ar = pComponent->containedObject()->CreateFromDrop( this, bLink );
            if( ASWFAILED( ar ) )
            {
                const char* msg = IBase::messageText(IC_CDF_INSERT_FAILED);
                IComponentStationery::stationery().messageBox( msg,
                                                  IMessageBox::moveable  |
                                                  IMessageBox::applicationModal|
                                                  IMessageBox::errorIcon |
                                                  IMessageBox::okButton);
//>>>>>>                ShowAswResult( ar, msg );
                delete pModel->orphan( *pComponent );
                return FALSE;
            }

            // get aspect from descriptor
            // (aspect from pBestFormatetc always seems to be CONTENT no matter what)
            // if it's an ICON we must change BEFORE the call to initialize
            if( pDescriptor )
                if( pDescriptor->GetDrawAspect() == DVASPECT_ICON )
                    pComponent->setAspect( IComponent::kIcon );

            // set up a presentation area for the object
            pComponent->initialize( true /* new */, false /* no launch */ );

        } // not an internal action

        // move the component to the insertion point
        // convert screen coordinates to client coordinates
        IView* pView = &IComponentStationery::stationery().bundle().objectView();
        POINT newPt;
        newPt.x = pt.x;
        newPt.y = pt.y;
        MapWindowPoints( NULL, (HWND)(void*)pView->handle(), &newPt, 1 );

        // new pos is now the top-left corner in View coordinates
        IPoint newPos( newPt );
        ISize newSize;

        // if this is an internal action then offset the point by the amount
        // representing where the original click was inside the view
        if( dwEffect == DROPEFFECT_MOVE && IsInternalAction() )
        {
            newPos = newPos - pComponent->dragMouseOffset();
            newSize = pComponent->area().size();

        } // was an internal action

        else // not an internal action
        {
            
            SIZEL sizeHi, sizePix;
            sizeHi.cx = 0;

            // see if there is a source object descriptor
            // if so, extract the original offset and extent
            if( pDescriptor )
            {
                POINTL pt = pDescriptor->GetOffsetPoint();
                POINTL ptlMouse;
                ptlMouse.x = (long)XformWidthInHimetricToPixels( NULL, pt.x );
                ptlMouse.y = (long)XformHeightInHimetricToPixels( NULL, pt.y );
                newPos = newPos - IPoint( ptlMouse );

                sizeHi = pDescriptor->GetExtent();

            } // there was a descriptor

            // did we get a size?
            if( sizeHi.cx == 0 )
            {
                // since there is no object descriptor, must get extent from the server
                pComponent->containedObject()->aspect().GetExtent( &sizeHi );

            } // no object descriptor or no size in descriptor

            // size is in himetric units - convert to pixels
            XformSizeInHimetricToPixels( NULL, &sizeHi, &sizePix );
            newSize = ISize( sizePix );

        } // not an internal action

        // move and size the component
        pComponent->resize( IRectangle( newPos, newSize ) );

    } // we are an embedder model

    

   return true;
}
                               

// -----------------------------------------------------------------------------
// Scroll
// -----------------------------------------------------------------------------
// We do not handle scrolling in this release
void ASWCLIENTMETHOD
IAswDropTarget::Scroll(WORD wScrollDirection)
{
/*
    // Start with no scroll
   WORD  wCode = MAKEWORD(-1,-1);

   // Horizontal scrolls
   if (wScrollDirection & SCROLL_LEFT)
      wCode = MAKEWORD(SB_LINEUP,HIBYTE(wCode));
   else
   if (wScrollDirection & SCROLL_RIGHT)
      wCode = MAKEWORD(SB_LINEDOWN,HIBYTE(wCode));

   // Vertical Scrolls
   if (wScrollDirection & SCROLL_UP)
      wCode = MAKEWORD(LOBYTE(wCode),SB_LINEUP);
   else
   if (wScrollDirection & SCROLL_DOWN)
      wCode = MAKEWORD(LOBYTE(wCode),SB_LINEDOWN);

   // The above will handle scrolling horizontal and vertical at once


   // Do the scroll
   GetView()->OnScroll(wCode,0,TRUE);
*/
}

// -----------------------------------------------------------------------------
// RenderTargetFeedback
// -----------------------------------------------------------------------------
void ASWCLIENTMETHOD
IAswDropTarget::RenderTargetFeedback(POINTL pt, DWORD dwEffect, WORD wState)
{
    
    // If starting feedback, allocate a new feedback object
    if( wState == FEEDBACK_START )
    {
        fpFeedback = new CAswTargetFeedback(IComponentStationery::stationery());
        if( fpFeedback )
        {
            if (ASWFAILED(fpFeedback->Setup(this)))
            {
                delete fpFeedback;
                fpFeedback = NULL;
            }
        }
    }

    if (fpFeedback)
    {
        // Render the feedback
        fpFeedback->Render(pt,dwEffect,wState);

        // If feedback end then delete the feedback object
        if (FEEDBACK_END == wState)
        {
            delete fpFeedback;
            fpFeedback = NULL;
        }
    }


}

// -----------------------------------------------------------------------------
// QueryFormatSet
// -----------------------------------------------------------------------------
PASWTARGETFORMATSET
IAswDropTarget::QueryFormatSet()
{
      return &IComponentStationery::stationery().targetFormatSet();
}

