/* NOSHIP */
/*******************************************************************************
* FILE NAME: idataex.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IAswClipboardSource                                                      *
*     IAswPasteTarget                                                          *
*     IAswDragSource                                                           *
*     IAswDropTarget                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*   Portions or this component are based on work originating from Lotus.       *
*   (C) Copyright Lotus Development Corporation. 1996 - All Rights Reserved.   *
*                                                                              *
*******************************************************************************/

#ifndef _ICLIPBRD_
#define _ICLIPBRD_

// forward references
class IEmbeddedComponent;

// classes defined in this module
class IAswClipboardSource;
class IAswPasteTarget;

/*******************************************************************************
*                                                                              *
*                                 IAswClipboardSource                          *
*                                                                              *
*******************************************************************************/

class IAswClipboardSource : public CAswClipboardSource
{
//--------------------------------------------------
// Public
//--------------------------------------------------
public:

   // Constructor/Destructor
   IAswClipboardSource( PASWCONTROL pControl );
   ~IAswClipboardSource( void );

   //--------------------------------------------------
   // Base Class overrides
   //--------------------------------------------------
   // Setup and shutdown
   CAswResult ASWCLIENTMETHOD Setup(PASWSOURCEFORMATSET pAswSourceFormatSet = NULL);
   virtual CAswResult ASWCLIENTMETHOD Shutdown(BOOL fEmergency = FALSE);

   // Get the source data in the specified medium
   virtual CAswResult ASWCLIENTMETHOD GetData(LPFORMATETC     pFormatEtc,
                                              PASWSTGMEDUNION pAswStgMedium);


   //--------------------------------------------------
   // helper functions
   //--------------------------------------------------
   
   // cache state of objects into a temp file then put format sets on clipboard
   BOOL PutItemOnClipboard( IEmbeddedComponent* pComponent = NULL );


//--------------------------------------------------
// Private
//--------------------------------------------------
private:


    CAswDocFileStorage      fClipDocFile;          // Doc File
    CAswStorage             fClipRootStg;          // Root storage for clipboard
    CAswStorage             fClipStg;              // Clipboard contents
    CAswObjectDescriptor    fAswObjDescriptor;     // Object descriptor

    STGMEDIUM               fstgMetaFile;          // Meta file pict of object

    IEmbeddedComponent*     fpComponent;            // pointer to embedded component

};


/*******************************************************************************
*                                                                              *
*                                 IAswPasteTarget                              *
*                                                                              *
*******************************************************************************/

class IAswPasteTarget : public CAswPasteTarget
{

// --------------------------------------
// Public
// --------------------------------------
public:

   // Constructors/Destructors
   // ------------------------

                        IAswPasteTarget( PASWCONTROL  pControl );


   // Public Methods
   // ------------------------

   // Paste was specified
   virtual BOOL ASWCLIENTMETHOD ObjectPasted( LPFORMATETC pBestFormatetc,
                                              BOOL        fLink,
                                              HGLOBAL     hMetaPict );


   // provides a context-sensitive format set
   virtual PASWTARGETFORMATSET ASWPUBMETHOD QueryFormatSet();


};


/*******************************************************************************
*                                                                              *
*                                 IAswDragSource                               *
*                                                                              *
*******************************************************************************/

class IAswDragSource : public CAswDragSource
{
//--------------------------------------------------
// Public
//--------------------------------------------------
public:
   
                                IAswDragSource( PASWCONTROL  pControl );



   // Get the source data in the specified medium
   virtual CAswResult ASWCLIENTMETHOD GetData( LPFORMATETC     pFormatEtc, 
                                               PASWSTGMEDUNION pAswStgMedium );

   // provides a context-sensitive format set
   virtual PASWSOURCEFORMATSET ASWPUBMETHOD QueryFormatSet();

};


/*******************************************************************************
*                                                                              *
*                                 IAswDropTarget                               *
*                                                                              *
*******************************************************************************/

class IAswDropTarget : public CAswDropTarget
{

// --------------------------------------
// Public
// --------------------------------------
public:

    // Constructors/Destructors
    // ------------------------

    IAswDropTarget( PASWCONTROL pControl );

    // Setup and shutdown are inherited from CAswDropTarget

    // Public Methods
    // ------------------------

    // Object was dropped on the target
    virtual BOOL ASWCLIENTMETHOD ObjectDropped(POINTL        pt,
                                               LPFORMATETC   pBestFormatetc,
                                               DWORD         dwEffect,
                                               BOOL          fRightMouseDrop);

    // Render target feedback during a drag operation
    virtual void ASWCLIENTMETHOD RenderTargetFeedback(POINTL  pt,
                                                      DWORD   dwEffect,
                                                      WORD    wState);

    // Notification that scrolling of the Drop Target window should occur. 
    virtual void ASWCLIENTMETHOD Scroll(WORD wScrollDirection);

    // provides a context-sensitive format set
    virtual PASWTARGETFORMATSET ASWPUBMETHOD QueryFormatSet();

    // record the start position of (potentially internal) drag
    void dragStarted( const IPoint& p ) { fDragStartPosition = p; }

private:
    IPoint               fDragStartPosition;

    // Use a tracker rect to provide feedback
    RECT                 frcLast;
    RECT                 frcStart;
    POINT                fptStart;

    // Point to a render target feedback object
    PASWTARGETFEEDBACK   fpFeedback;


};


#endif // _ICLIPBRD_
