/*******************************************************************************
* FILE NAME: imodel.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the classes:                                             *
*     IModel                                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*   Portions or this component are based on work originating from Taligent.    *
*   (C) Copyright Taligent, Inc. 1996 - All rights reserved.                   *
*                                                                              *
*******************************************************************************/
#include <precomp.hpp>

#include "imodel.hpp"

#include <istatnry.hpp>
#include <icompnen.hpp>
#include <icompimp.hpp>
#include <ibasstrm.hpp>
#include <inotifev.hpp>
#include <ipoint.hpp>

IModel::~IModel()
{
}

void
IModel::notifyOfChange( const INotificationEvent& theNotification )
{
    notifier().notifyObservers( theNotification );
    if ( theNotification.hasNotifierAttrChanged() )
    {
        setChanged(true);
        // Note: SetIsDirty (called by setChanged) also calls AdviseOnDataChange
    }
    else
    {
        component().objectInstance()->AdviseOnDataChange();
    }
}
        
Boolean
IModel::hasChanged() const
{
    return component().objectInstance()->GetIsDirty();
}

void
IModel::setChanged( Boolean val )
{
    component().objectInstance()->SetIsDirty( val );
}
    
INotifier&
IModel::notifier() const
{
    return component().notifier();
}

void
IModel::attachTo( IComponent& c )
{
    fpComponent = &c;
}

void
IModel::fileRenamed( const IString& name )
{
    CAswOLEString pszOLEFile(name);
    fpComponent->objectInstance()->FileRenamed( pszOLEFile );
}

IBaseStream&
IModel::operator>>=( IBaseStream& towhere ) const
{
    writeVersion( towhere );

    fpComponent->fExtent.width() >>= towhere;
    fpComponent->fExtent.height() >>= towhere;
}

IBaseStream&
IModel::operator<<=( IBaseStream& fromwhere )
{
    readVersion( fromwhere );

    IPair::Coord width, height;
    width  <<= fromwhere;
    height <<= fromwhere;

    IComponent* pComponent = 
        &IComponentStationery::stationery().bundle().component();

    pComponent->fExtent.setWidth( width );
    pComponent->fExtent.setHeight( height );
}

TypeExtensionMacro( IModel )

IModel::IModel() : fpComponent(NULL)
{
}

IModel::IModel( const IModel& other ) : fpComponent(other.fpComponent)
{
    assert( false );
}
