/*******************************************************************************
* FILE NAME: immttime.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immttime.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
   {
   #include <string.h>
   #define INCL_MCIOS2
   #define IC_MULTIMEDIA
   #include <immdefs.h>
   }

#include <itrace.hpp>
#include <immcdda.hpp>
#include <immttime.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMMTTIME_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(const IMMTrackMinSecFrameTime &tmsf)
                       : IMMTime(tmsf),
                         fStart(tmsf.fStart),
                         fEnd(tmsf.fEnd),
                         fTrackMinSecFrame(tmsf.fTrackMinSecFrame)
{
   IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMTrackMinSecFrameTime&)") ;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMAudioCDContents &toc,
                                                 unsigned long value)
                       : IMMTime(),
                         fTrackMinSecFrame(value)
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMAudioCDContents&, unsigned long)") ;
  unsigned long track = value & 0xFF;
  if (track  &&  track <= toc.numberOfTracks())
  {
     fEnd = toc.timeAt(track,false);
     fStart = toc.timeAt(track,true);
     setMMTime(IMMTime(REDBOOKTOMM((value >> 8))) + fStart);
  } else {
    fEnd = IMMTime();
    fStart = IMMTime();
    setMMTime(IMMTime(REDBOOKTOMM((value >> 8))));
  } /* endif */
  if (asMMTime() <= fEnd)
     setValid(true);
  else
     setValid(false);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMAudioCDContents &toc,
                         unsigned long track,
                         unsigned long minutes,
                         unsigned long seconds,
                         unsigned long frames)
                       : IMMTime(),
                         fTrackMinSecFrame(track | (minutes << 8) |
                                           (seconds << 16) | (frames << 24))
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::IMMTrackMinSecFrameTime(IMMAudioCDContents&, track, min, sec, frame)") ;
  if (track && track <= toc.numberOfTracks())
  {
     fEnd = toc.timeAt(track,false);
     fStart = toc.timeAt(track,true);
     setMMTime(IMMTime(REDBOOKTOMM(minutes | (seconds << 8) | (frames << 16))) + fStart);
  } else {
     fEnd = IMMTime();
     fStart = IMMTime();
     setMMTime(IMMTime(REDBOOKTOMM(minutes | (seconds << 8) | (frames << 16))));
  } /* endif */
  if (asMMTime() <= fEnd)
     setValid(true);
  else
     setValid(false);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::~IMMTrackMinSecFrameTime                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::~IMMTrackMinSecFrameTime()
{
   IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::~IMMTrackMinSecFrameTime") ;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::frames                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::frames() const
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::frames") ;
  return ((fTrackMinSecFrame >> 24) & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::minutes                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::minutes() const
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::minutes") ;
  return ((fTrackMinSecFrame >> 8) & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::seconds                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::seconds() const
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::seconds") ;
  return ((fTrackMinSecFrame >> 16) & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::track                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IMMTrackMinSecFrameTime::track() const
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::tracks") ;
  return (fTrackMinSecFrame & 0xFF);
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::asString                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IString IMMTrackMinSecFrameTime::asString() const
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::asString") ;
  return IString(track()) + " " + IString(minutes()) + ":" +
         IString(seconds()).rightJustify(2, '0') + "." +
         IString(frames()).rightJustify(3, '0');
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator unsigned long                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime::operator unsigned long () const
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::operator(unsigned long)") ;
  return fTrackMinSecFrame;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator =                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime &IMMTrackMinSecFrameTime::operator = (const IMMTrackMinSecFrameTime &time)
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::operator=") ;
  setMMTime(time.asMMTime());
  setValid(time.isValid());
  fStart = time.fStart;
  fEnd   = time.fEnd;
  fTrackMinSecFrame  = time.fTrackMinSecFrame;
  return *this;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator +=                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime &IMMTrackMinSecFrameTime::operator += (const IMMTime &time)
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::operator+=") ;
  setMMTime(asMMTime() + time.asMMTime());
  if (asMMTime() > fEnd.asMMTime() ||
      asMMTime() < fStart.asMMTime())
  {
     setValid(false);
  }
  else
  {
     fTrackMinSecFrame = (REDBOOKFROMMM(asMMTime() - fStart.asMMTime()) << 8) | (fTrackMinSecFrame & 0xFF);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator -=                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime &IMMTrackMinSecFrameTime::operator -= (const IMMTime &time)
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::operator-=") ;
  setMMTime(asMMTime() - time.asMMTime());

  if (asMMTime() > fEnd.asMMTime() ||
      asMMTime() < fStart.asMMTime())
  {
     setValid(false);
  }
  else
  {
     fTrackMinSecFrame = (REDBOOKFROMMM(asMMTime() - fStart.asMMTime()) << 8) | (fTrackMinSecFrame & 0xFF);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator +                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime IMMTrackMinSecFrameTime::operator + (const IMMTime &time)
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::operator+") ;
  IMMTrackMinSecFrameTime temp(*this);
  temp += time;
  return temp;
}

/*------------------------------------------------------------------------------
| IMMTrackMinSecFrameTime::operator -                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMMTrackMinSecFrameTime IMMTrackMinSecFrameTime::operator - (const IMMTime &time)
{
  IMODTRACE_DEVELOP("IMMTrackMinSecFrameTime::operator-") ;
  IMMTrackMinSecFrameTime temp(*this);
  temp -= time;
  return temp;
}

