/*******************************************************************************
* FILE NAME: ibutton1.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ibtnnhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#define INCL_WININPUT
#define INCL_WINBUTTONS

extern "C" {
   #include <iwindefs.h>
}

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IBUTTON_
  #include <ibutton.hpp>
#endif

#ifndef _IBTNNHDR_
  #include <ibtnnhdr.hpp>
#endif

#ifndef _IEVENT_
  #include <ievent.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#ifndef _ICOLOR_
  #include <icolor.hpp>
#endif

#ifndef _IRECT_
  #include <irect.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _IBUTTON1_CPP_
  #include <ipagetun.h>
#endif

// Class to clean up static objects.
class IButtonNotifyStatics {
public:
 ~IButtonNotifyStatics ( )
  {
    delete (buttonNotifyHandler);
    buttonNotifyHandler = 0;
  }
static IButtonNotifyHandler
 &notificationHandler ( );

static IButtonNotifyHandler
 *buttonNotifyHandler;
}; // IButtonNotifyStatics

// Initialize static obects and notification strings.
#pragma data_seg(ICLStaticConst)
static IButtonNotifyStatics buttonStatics;
INotificationId const IButton::buttonClickId = "IButton::buttonClick";
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IButtonNotifyHandler* IButtonNotifyStatics::buttonNotifyHandler = 0;
#pragma data_seg()

/*------------------------------------------------------------------------------
| IButtonNotifyStatics::notificationHandler                                    |
|                                                                              |
| Static function to create and return a notification handler for IButton.     |
------------------------------------------------------------------------------*/
IButtonNotifyHandler& IButtonNotifyStatics::notificationHandler ()
{
  if (!(IButtonNotifyStatics::buttonNotifyHandler))
  {
    IButtonNotifyStatics::buttonNotifyHandler =
                                   new IButtonNotifyHandler();
  }
  return (*IButtonNotifyStatics::buttonNotifyHandler);
}

/*------------------------------------------------------------------------------
| IButton::enableNotification                                                  |
|                                                                              |
| Flags this part as being ready to accept notifications.  A Notifier, like    |
| IButton, does not send notifications to observer objects until it is         |
| enabled.                                                                     |
|                                                                              |
| In the window hierarchy we override this function to allocate the correct    |
| notification handler.                                                        |
------------------------------------------------------------------------------*/
IButton& IButton::enableNotification (Boolean enabled)
{
  if (!notificationHandler())
  {
    setNotificationHandler(
               &(IButtonNotifyStatics::notificationHandler()));
  }
  IWindow::enableNotification(enabled);
  return *this;
}

/*------------------------------------------------------------------------------
| IButtonNotifyHandler::IButtonNotifyHandler                                   |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IButtonNotifyHandler::IButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IButtonNotifyHandler::~IButtonNotifyHandler                                  |
|                                                                              |
| Destructor.                                                                  |
------------------------------------------------------------------------------*/
IButtonNotifyHandler::~IButtonNotifyHandler()
{}

/*------------------------------------------------------------------------------
| IButtonNotifyHandler::dispatchHandlerEvent                                   |
|                                                                              |
| This notifies IButton Windows of the following events:                       |
|                                                                              |
| buttonClicked(Id)                                                            |
------------------------------------------------------------------------------*/
#ifdef IC_PMWIN
Boolean IButtonNotifyHandler::dispatchHandlerEvent (IEvent& anEvent)
{
#ifdef IC_PM
  IWindow* thisWindow = anEvent.window();
  static Boolean button1down = false;

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      case WM_BUTTON1DBLCLK:
      case WM_BUTTON1DOWN:
      {
        if (thisWindow->isEnabled())
        {
          button1down = true;
        }
      break;
      }

      case WM_BUTTON1UP:
      {
        if (button1down)
        {
          /**************************************************************/
          /* Post private message to delay notification until after     */
          /* the selection state has been modified                      */
          /**************************************************************/
          thisWindow->postEvent( IC_UM_DELAYNOTIFY,
                                 IEventParameter1( anEvent.parameter1() ),
                                 IEventParameter2( IC_NOTDEL_PUSHBT_SELECT ));
          button1down = false;
        }
      break;
      } /* end case */


      /****************************************************************/
      /* This is the delayed notification message.  Check to see which*/
      /* notification should be sent to the observers based on info   */
      /* stored in parameter 2 of the private message.  Parameter 1   */
      /* contains the information passed in the original event that   */
      /* needs to be sent with the notification                       */
      /****************************************************************/
      case IC_UM_DELAYNOTIFY:

        if (anEvent.parameter2() == IC_NOTDEL_PUSHBT_SELECT)
        {
          IRectangle buttonRect = thisWindow->rect();
          IEventData mX = anEvent.parameter1().lowNumber();
          IEventData mY = anEvent.parameter1().highNumber();
          unsigned long mouseX = mX.asUnsignedLong();
          unsigned long mouseY = mY.asUnsignedLong();
          mouseX += buttonRect.left();
          mouseY += buttonRect.bottom();

          if (buttonRect.contains(IPoint(mouseX, mouseY)))
          {
            thisWindow->notifyObservers(INotificationEvent(
                                        IButton::buttonClickId,
                                        *thisWindow, false, 0));
          }
        }
      break;

      default:
        break;
    }  /* end switch */
  }
#else //IC_WIN
  IWindow* thisWindow = anEvent.window();

  if (thisWindow)
  {
    switch (anEvent.eventId())
    {
      /***********************************************************************/
      /* In Windows, all button click events (mouse, space bar, and mnemonic)*/
      /* can be obtained through WM_CONTROL since we are mapping all         */
      /* WM_COMMAND to WM_CONTROL.  Not so in PM.                            */
      /***********************************************************************/
      case WM_CONTROL:
      {
        IControlEvent ctlevt( anEvent );
        IWindowHandle hwndControl = ctlevt.controlHandle();
        IWindow* controlWindow = thisWindow->windowWithHandle(ctlevt.controlHandle());
        if (hwndControl)
        {
          unsigned long subMsg = ctlevt.parameter1().number2();
          IWindowClassName className( hwndControl );
          if ((className == WC_BUTTON) && (subMsg == BN_CLICKED))
          {
          /**************************************************************/
          /* Post private message to delay notification until after     */
          /* the selection state has been modified                      */
          /**************************************************************/
            controlWindow->postEvent( IC_UM_DELAYNOTIFY,
                                   IEventParameter1( anEvent.parameter2() ),
                                   IEventParameter2( IC_NOTDEL_PUSHBT_SELECT ));
          }
        } /* endif */
      }
      break;
      case IC_UM_DELAYNOTIFY:
      {
        if (anEvent.parameter2() == IC_NOTDEL_PUSHBT_SELECT)
        {
            thisWindow->notifyObservers(INotificationEvent(
                                        IButton::buttonClickId,
                                        *thisWindow, false, 0));
        }
      }
      break;
      default:
        break;
    }  /* end switch */
  }
#endif
  return (Inherited::dispatchHandlerEvent(anEvent));
}
#endif // IC_PMWIN

