/*******************************************************************************
* FILE NAME: icheckbx.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icheckbx.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

#ifndef _IBASE_
  #include <ibase.hpp>
#endif
#ifdef IC_PM
#define INCL_WINBUTTONS
#define INCL_WINERRORS
#define INCL_WINPOINTERS
#define INCL_GPIBITMAPS
#define INCL_WINSYS
#define INCL_GPIPRIMITIVES
#endif
#include <iwindefs.h>

#ifndef _ICHECKBX_
  #include <icheckbx.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _IFONT_
  #include <ifont.hpp>
#endif
#ifndef _ICOLOR_
  #include <icolor.hpp>
#endif
#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif
#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif
#ifndef _IPOINT_
  #include <ipoint.hpp>
#endif
#ifndef _IRECT_
  #include <irect.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICHECKBX_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public check box styles                                                      |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const ICheckBox::Style
  ICheckBox::autoSelect        ( 0, IBS_AUTOCHECKBOX ),
  ICheckBox::classDefaultStyle ( WS_VISIBLE, IBS_AUTOCHECKBOX );
#pragma data_seg()


/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
  ICheckBox::Style
        ICheckBox::currentDefaultStyle ( WS_VISIBLE, IBS_AUTOCHECKBOX );
#pragma data_seg()

#ifdef IC_WIN
class ICheckBoxData : public IBase {
/*------------------------------------------------------------------------------
| The ICheckBoxData class encapsulates private data and functions              |
| used by the ICheckBox class.  An object of this class is created in          |
| the ICheckBox constructors.  This is the os2/windows version.                |
------------------------------------------------------------------------------*/
public:
  ICheckBoxData ( );

virtual
 ~ICheckBoxData ( );

HBRUSH
  bkgrndbrush;

}; // ICheckBoxData

/*------------------------------------------------------------------------------
| ICheckBoxData::ICheckBoxData                                                 |
|                                                                              |
------------------------------------------------------------------------------*/
ICheckBoxData::ICheckBoxData ( )
  : bkgrndbrush( 0 )
{ }

/*------------------------------------------------------------------------------
| ICheckBoxData::~ICheckBoxData                                                |
|                                                                              |
------------------------------------------------------------------------------*/
ICheckBoxData::~ICheckBoxData ( )
{
  if ( bkgrndbrush != 0 )
    DeleteObject(bkgrndbrush);
}

class ICheckHandler : public IHandler {
typedef IHandler
  Inherited;
public:

/*------------------------------- Constructors -------------------------------*/
  ICheckHandler ( );

virtual
  ~ICheckHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

}; // class ICheckHandler

/*------------------------------------------------------------------------------
| ICheckHandler::ICheckHandler                                                 |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
ICheckHandler::ICheckHandler()
{ }

/*------------------------------------------------------------------------------
| ICheckHandler::~ICheckHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ICheckHandler::~ICheckHandler()
{ }

/*------------------------------------------------------------------------------
| ICheckHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Intercepts control color event                                               |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
IBase::Boolean ICheckHandler::dispatchHandlerEvent( IEvent& event )
{
  Boolean
    result = false;

  switch( event.eventId() )
  {

    case WM_CTLCOLORBTN:
    case WM_CTLCOLORSTATIC:
         {
         IWindow* ctlwin =
           IWindow::windowWithHandle(IWindowHandle(event.parameter2()) );
         IColor clrFgnd =
           ((ICheckBox *) ctlwin)->foregroundColor();
         IColor clrBgnd =
           ((ICheckBox *) ctlwin)->backgroundColor();

         // Set foreground and background text colors in device context
         SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
         SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

         // Return HBRUSH for painting controls background
         if ( ((ICheckBox *) ctlwin)->fCheckBoxData->bkgrndbrush != 0 )
           DeleteObject( ((ICheckBox *) ctlwin)->fCheckBoxData->bkgrndbrush);
         ((ICheckBox *) ctlwin)->fCheckBoxData->bkgrndbrush =
           CreateSolidBrush (NEARESTPALETTECOLOR(clrBgnd.asRGBLong()));
         event.setResult( ((ICheckBox *) ctlwin)->fCheckBoxData->bkgrndbrush );

         result = true;
         }
      break;

    default:
      break;
  }
  return result;
}

/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointer to the object |
| when the wrapper is destructed.                                              |
------------------------------------------------------------------------------*/
static struct DefaultHandler {
  operator ICheckHandler* ();
ICheckHandler
 *operator -> ()
    {
    return *this;
    }
 ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
ICheckHandler
 *ptr;
} defaultHandler;

DefaultHandler::operator ICheckHandler* ( )
  {
  if ( !ptr )
    ptr = new ICheckHandler;
  return ptr;
  }
#endif //IC_WIN

/*------------------------------------------------------------------------------
| ICheckBox::ICheckBox                                                         |
|                                                                              |
| Constructor to create a check box on a standard window.                      |
------------------------------------------------------------------------------*/
ICheckBox :: ICheckBox(unsigned long ulId,
                       IWindow* pwndParent,
                       IWindow* pwndOwner,
                       const IRectangle& rectInit,
                       const Style& chbsStyle)
#ifdef IC_WIN
                     : fCheckBoxData( new ICheckBoxData )
#else
                     : fCheckBoxData( 0 )
#endif // IC_WIN
{

  IASSERTPARM(pwndParent!=0);          //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | chbsStyle.asExtendedUnsignedLong() );

  //Create the check box control
  IWindowHandle ownerHandle = IWindowHandle(0);
  if (pwndOwner)
     ownerHandle = pwndOwner->handle();
  IWindowHandle whCheckBox =
      this -> create( ulId,
                      0,
                      convertToGUIStyle( chbsStyle ),
                      WC_BUTTON,
                      pwndParent->handle(),
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( chbsStyle, true ) );
#ifdef IC_WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(whCheckBox);
}


/*------------------------------------------------------------------------------
| ICheckBox::ICheckBox                                                         |
|                                                                              |
| Constructor to instantiate an object for a dialog template.                  |
------------------------------------------------------------------------------*/
ICheckBox :: ICheckBox(unsigned long ulId,
                       IWindow* pdlgwndParent)
#ifdef IC_WIN
                     : fCheckBoxData( new ICheckBoxData )
#else
                     : fCheckBoxData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow(false);
#ifdef IC_WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(ulId, pdlgwndParent);
}


/*------------------------------------------------------------------------------
| ICheckBox::ICheckBox                                                         |
|                                                                              |
| Constructor to instantiate an object from an existing check box control.     |
------------------------------------------------------------------------------*/
ICheckBox :: ICheckBox(const IWindowHandle& wh)
#ifdef IC_WIN
                     : fCheckBoxData( new ICheckBoxData )
#else
                     : fCheckBoxData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow(false);
#ifdef IC_WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(wh);
}


/*------------------------------------------------------------------------------
| ICheckBox::~ICheckBox                                                        |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ICheckBox :: ~ICheckBox()
{
#ifdef IC_WIN
  removeHandler(defaultHandler);
  if ( fCheckBoxData != 0 )
    delete fCheckBoxData;
#endif //IC_WIN
}


/*------------------------------------------------------------------------------
| ICheckBox::defaultStyle                                                      |
|                                                                              |
| Return the default style for new check box objects.                          |
------------------------------------------------------------------------------*/
ICheckBox::Style  ICheckBox :: defaultStyle()
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| ICheckBox::setDefaultStyle                                                   |
------------------------------------------------------------------------------*/
void  ICheckBox :: setDefaultStyle(const Style& chbsStyle)
{
  currentDefaultStyle = chbsStyle;
}


/*------------------------------------------------------------------------------
| ICheckBox::convertToGUIStyle                                                 |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long ICheckBox::convertToGUIStyle(const IBitFlag& guiStyle,
                                           Boolean bExtOnly) const
{
  // Obtain the style from the class (ISettingButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXTCHECKBX;

    // Handle overlapping styles
    if (ulTmp & IBS_AUTOCHECKBOX)
      ulStyle |= BS_AUTOCHECKBOX;
    else
      ulStyle |= BS_CHECKBOX;

#ifdef IC_WIN
    ulStyle |= WS_CHILD;
#endif
  }


  return( ulStyle );
}


/*------------------------------------------------------------------------------
| ICheckBox::enableAutoSelect                                                  |
|                                                                              |
| Sets or removes the autoSelect style for the check box control.              |
------------------------------------------------------------------------------*/
ICheckBox&  ICheckBox :: enableAutoSelect(Boolean bTurnOn)
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= ~BS_PRIMARYSTYLES;  //Use masking flag from pmwin.h
    ulStyle |= BS_AUTOCHECKBOX;
  }
  else
  {
    ulStyle &= ~BS_PRIMARYSTYLES;  //Use masking flag from pmwin.h
    ulStyle |= BS_CHECKBOX;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                        //Force repaint of control
  }
  return *this;
}


/*------------------------------------------------------------------------------
| ICheckBox::disableAutoSelect                                                 |
------------------------------------------------------------------------------*/
ICheckBox&  ICheckBox :: disableAutoSelect()
{
  enableAutoSelect(false);
  return *this;
}


/*------------------------------------------------------------------------------
| ICheckBox::isAutoSelect                                                      |
------------------------------------------------------------------------------*/
Boolean  ICheckBox :: isAutoSelect() const
{
  return( (style() & BS_AUTOCHECKBOX & ~(BS_CHECKBOX)) ? true : false );
}


/*------------------------------------------------------------------------------
| ICheckBox::calcMinimumSize                                                   |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The minimum size   |
| is based on the maximum height of the font and the font width of the         |
| check box text (plus the size of the check box itself).                      |
------------------------------------------------------------------------------*/
ISize  ICheckBox :: calcMinimumSize() const
{
#ifdef IC_PM
   BITMAPINFOHEADER pbmpData;
   pbmpData.cbFix=sizeof(BITMAPINFOHEADER);
#endif
#ifdef IC_WIN
   BITMAP           pbmpData;
#endif
   HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
   IQUERYBITMAPPARAMETERS(hbit, &pbmpData);    //Retrieve size of check
   IDELETEBITMAP(hbit);                        //box bitmap

   ISize size = displaySize(text());
   unsigned long ulWidth = size.width();
   unsigned long i=textLength();
#ifdef IC_PM
   long bmpWidth = pbmpData.cx;
#endif
#ifdef IC_WIN
   long bmpWidth = pbmpData.bmWidth;
#endif

   if (i==0)
     ulWidth += ((bmpWidth/4) + 2 + (2 * IQUERYSYSVALUE(SV_CXBORDER)));
   else
     ulWidth += (ulWidth/i + (bmpWidth/4) + 2 +
                                    (2 * IQUERYSYSVALUE(SV_CXBORDER)));
   unsigned long ulHeight = size.height() +
                                     (2 * IQUERYSYSVALUE(SV_CYBORDER));
   return ISize(ulWidth, ulHeight);
}
