/*******************************************************************************
* FILE NAME: icnrctl3.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the view switching and            *
*   quering functions for the class IContainerControl declared                 *
*   in icnrctl.hpp.                                                            *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDCNR

#include <icnrrec.hpp>  // Must be first for OS flags

#include <itrace.hpp>
#include <iexcept.hpp>
#include <icnrctl.hpp>


// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTL3_CPP_
  #include <ipagetun.h>
#endif

/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: arrangeIconView

 Implementation: Flow the icons in the icon view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: arrangeIconView()
{
   Boolean fSuccess = (Boolean)sendEvent( CM_ARRANGE, 0, 0);
   if(!fSuccess)
   {
      ITHROWGUIERROR("CM_ARRANGE");
   }
   return *this;
}


#ifndef IC_MOTIF_FLAGNOP
/*-----------------------------------------------------------------------------
 Function Name: IContainerControl :: showMiniIcons

 Implementation: Show mini-icons in any non-text view.
------------------------------------------------------------------------------*/
IContainerControl& IContainerControl :: showMiniIcons( Boolean mini)
{
   if (mini)
     setContainerAttributes( 0, miniIcons.asUnsignedLong() );
   else
     setContainerAttributes( miniIcons.asUnsignedLong(), 0 );
   return *this;
}
#endif


void IContainerControl :: showView(unsigned long ulViewFlags)
{
   setContainerAttributes((textView | treeView | nameView |
                          iconView | detailsView |
                          flowedView).asUnsignedLong(),
                          ulViewFlags);
}

IContainerControl& IContainerControl :: showTextView()
{
    showView(textView.asUnsignedLong());
    return *this;
}

IContainerControl& IContainerControl :: showFlowedTextView()
{
   showView((textView | flowedView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showNameView()
{
   showView(nameView.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showFlowedNameView()
{
   showView((nameView | flowedView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showDetailsView()
{
   showView(detailsView.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showIconView()
{
   showView(iconView.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showTreeIconView()
{
   showView((iconView | treeView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showTreeTextView()
{
   showView((textView | treeView).asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: showTreeNameView()
{
   showView((nameView | treeView).asUnsignedLong());
   return *this;
}


IContainerControl& IContainerControl :: setNormalTargetEmphasis()
{
   setContainerAttributes((mixedTargetEmphasis | orderedTargetEmphasis).asUnsignedLong()
                             , 0);
   return *this;
}

IContainerControl& IContainerControl :: setOrderedTargetEmphasis()
{
   setContainerAttributes(mixedTargetEmphasis.asUnsignedLong(),
                          orderedTargetEmphasis.asUnsignedLong());
   return *this;
}

IContainerControl& IContainerControl :: setMixedTargetEmphasis()
{
   setContainerAttributes(orderedTargetEmphasis.asUnsignedLong(),
                          mixedTargetEmphasis.asUnsignedLong());
   return *this;
}

/*-------------------------------------------------------------------
  Since there is no normal target emphasis bit(it is the default),
  we need to check that the other 2 (ordered, mixed) are not set.
-------------------------------------------------------------------*/
IBase::Boolean IContainerControl :: hasNormalTargetEmphasis() const
{
   return ( (containerAttributes() & mixedTargetEmphasis.asUnsignedLong()) &&
            (containerAttributes() & orderedTargetEmphasis.asUnsignedLong())
             ? false : true );
}

IBase::Boolean IContainerControl :: hasOrderedTargetEmphasis() const
{
   return ( (containerAttributes() & orderedTargetEmphasis.asUnsignedLong())
             ? true : false );
}

IBase::Boolean IContainerControl :: hasMixedTargetEmphasis() const
{
   return ( (containerAttributes() & mixedTargetEmphasis.asUnsignedLong())
             ? true : false );
}

IBase::Boolean IContainerControl :: isTextView(Boolean textOnly) const
{
  if (textOnly)  //text view and not tree-text view
    return( ( (containerAttributes()  & textView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) )
            ? true : false);
  else
    return((containerAttributes() & textView.asUnsignedLong()) ? true : false);
}

IBase::Boolean IContainerControl :: isNameView(Boolean nameOnly) const
{
  if (nameOnly)  //name view and not tree-name view
    return( ( (containerAttributes()  & nameView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) )
            ? true : false);
  else
    return((containerAttributes() & nameView.asUnsignedLong()) ? true : false);
}

IBase::Boolean IContainerControl :: isIconView(Boolean iconOnly) const
{
  if (iconOnly)  //icon view and not tree-icon view
    return( ( (containerAttributes()  & iconView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) )
            ? true : false);
  else
    return((containerAttributes() & iconView.asUnsignedLong()) ? true : false);
}

IBase::Boolean IContainerControl :: isDetailsView() const
{
   return ((containerAttributes() & detailsView.asUnsignedLong()) ? true : false);
}

IBase::Boolean IContainerControl :: isTreeView() const
{
   return ((containerAttributes() & treeView.asUnsignedLong()) ? true : false);
}

IBase::Boolean IContainerControl :: isFlowed() const
{
   return ((containerAttributes() & flowedView.asUnsignedLong()) ? true : false);
}

IBase::Boolean IContainerControl :: isFlowedTextView() const
{
   return ( ( (containerAttributes()  & textView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) &&
              (containerAttributes()  & flowedView.asUnsignedLong())  )
            ? true : false);
}

IBase::Boolean IContainerControl :: isFlowedNameView() const
{
   return ( ( (containerAttributes()  & nameView.asUnsignedLong()) &&
              !(containerAttributes() & treeView.asUnsignedLong()) &&
              (containerAttributes()  & flowedView.asUnsignedLong())  )
            ? true : false);
}

IBase::Boolean IContainerControl :: isTreeIconView() const
{
   return ( ( (containerAttributes()  & treeView.asUnsignedLong()) &&
              (containerAttributes()  & iconView.asUnsignedLong())  )
            ? true : false);
}

IBase::Boolean IContainerControl :: isTreeNameView() const
{
   return ( ( (containerAttributes()  & treeView.asUnsignedLong()) &&
              (containerAttributes()  & nameView.asUnsignedLong())  )
            ? true : false);
}

IBase::Boolean IContainerControl :: isTreeTextView() const
{
   return ( ( (containerAttributes()  & treeView.asUnsignedLong()) &&
              (containerAttributes()  & textView.asUnsignedLong())  )
            ? true : false);
}

#ifndef IC_MOTIF_FLAGNOP
IBase::Boolean IContainerControl :: isShowingMiniIcons() const
{
   return ( (containerAttributes()  & miniIcons.asUnsignedLong())
            ? true : false);
}
#endif

