/*******************************************************************************
* FILE NAME: icnrctlw.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of the native window container       *
*   functions declared in icnrctlw.hpp.                                        *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <icnrctlw.hpp>

#ifdef IC_WIN

#include <iexcept.hpp>
#include <iapp.hpp>
#include <ireslib.hpp>
#include <icnrobjw.hpp>
#include <istattxt.hpp>
#include <ifont.hpp>
#include <imle.hpp>
#include <itimer.hpp>
#include <icconst.h>
#include <icoordsy.hpp>
#include <idmhndlr.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICNRCTLW_CPP_
  #include <ipagetun.h>
#endif

VOID FAR CreateNLSDateString  ( LPCDATE pDate,  LPSTR pszDate  );
VOID FAR CreateNLSTimeString  ( LPCTIME pTime,  LPSTR pszTime  );
Boolean  CreateNLSNumberString( DWORD dwNumber, LPSTR lpNumber );

/*------------------------------------------------------------------------------
| ICnrControlData::ICnrControlData                                             |
|                                                                              |
| Control data default constructor.                                            |
------------------------------------------------------------------------------*/
ICnrControlData::ICnrControlData( )
  : pCnr( 0 ),
    cclStyle( 0 ),
    pmCompatible( true ),
    showTitle( false ),
    showTitleSeparator( false ),
    doubleClickOccurred( false ),
    bCheckingDragStart( false ),
    needArrange( false ),
    collapseFirstColumn( false ),
    useScaledImages( false ),
    expandNotificationDone( false ),
    pListview( 0 ),
    pTreeview( 0 ),
    pActiveview( 0 ),
    pTitle( 0 ),
    pEditColumn( 0 ),
    pEditObject( 0 ),
    pDetailsEdit( 0 ),
    fTimer( 0 ),
    largeImages( 0 ),
    scaledImages( 0 ),
    smallImages( 0 ),
    noImages( 0 ),
    inUseImage( 0 ),
    scaledInUseImage( 0 ),
    smallInUseImage( 0 ),
    av( ICnrControlData::none ),
    pointerSet(0),
    columnSet(0),
    pointLButtonDown()
{}

/*------------------------------------------------------------------------------
| ICnrControlData::ICnrControlData                                             |
|                                                                              |
| Control data informed constructor.                                           |
------------------------------------------------------------------------------*/
ICnrControlData::ICnrControlData( IContainerControl* cnr,
                                  unsigned long      ulStyle,
                                  IContainerControl::Attribute attribute )
  : pCnr( cnr ),
    cclStyle( ulStyle ),
    pmCompatible( false ),
    showTitle( false ),
    showTitleSeparator( false ),
    doubleClickOccurred( false ),
    bCheckingDragStart( false ),
    needArrange( false ),
    collapseFirstColumn( false ),
    useScaledImages( false ),
    expandNotificationDone( false ),
    pListview( 0 ),
    pTreeview( 0 ),
    pActiveview( 0 ),
    pTitle(new IStaticText( CID_CNRTITLEWND,
                            cnr,
                            cnr,
                            IRectangle(),
                            IStaticText::defaultStyle() | IStaticText::border3D )),
    pEditColumn( 0 ),
    pEditObject( 0 ),
    pDetailsEdit( 0 ),
    fTimer( 0 ),
    largeImages(ImageList_Create    ( GetSystemMetrics (SM_CXICON),
                                      GetSystemMetrics (SM_CYICON),
                                      ILC_MASK, 40, 5 )),
    scaledImages( 0 ),
    smallImages(ImageList_Create    ( GetSystemMetrics (SM_CXSMICON),
                                      GetSystemMetrics (SM_CYSMICON),
                                      ILC_MASK, 40, 5 )),
    noImages   (ImageList_Create    ( 1,   1, ILC_MASK, 1,  0 )),
    inUseImage (ImageList_Create    ( GetSystemMetrics (SM_CXICON),
                                      GetSystemMetrics (SM_CYICON),
                                      ILC_MASK, 1,  0 )),
    scaledInUseImage( 0 ),
    smallInUseImage(ImageList_Create( GetSystemMetrics (SM_CXSMICON),
                                      GetSystemMetrics (SM_CYSMICON),
                                      ILC_MASK, 1,  0 )),
    av( ICnrControlData::none ),
    pointerSet(new ICnrPointerSet()),
    columnSet(new IColumnSet()),
    pointLButtonDown()
{
  pTitle->setAlignment( IStaticText::centerCenter );
  pTitle->setBackgroundColor( IGUIColor( IGUIColor::menuBgnd ));

  // Initialize the container info structure.

  cnrInfo.flWindowAttr = attribute.asUnsignedLong();
  cnrInfo.strCnrTitle  = 0;

  // Use the largeImages ImageList to set the size of the icons
  int width, height;
  if (ImageList_GetIconSize( largeImages, &width, &height) )
  {
    cnrInfo.slBitmapOrIcon.cx = width;
    cnrInfo.slBitmapOrIcon.cy = height;
  }
  // Initialize tree values to zero - change later when view is established in
  // setWindowAttr
  cnrInfo.slTreeBitmapOrIcon.cx = 0;
  cnrInfo.slTreeBitmapOrIcon.cy = 0;

  ImageList_AddIcon( inUseImage,      IApplication::current().resourceLibrary().loadIcon(IC_INUSEICON) );
  ImageList_AddIcon( smallInUseImage, IApplication::current().resourceLibrary().loadIcon(IC_SMALLINUSEICON) );
}

/*------------------------------------------------------------------------------
| ICnrControlData::~ICnrControlData                                            |
|                                                                              |
| Control data destructor.                                                     |
------------------------------------------------------------------------------*/
ICnrControlData::~ICnrControlData( )
{
  if (pListview)
    delete pListview;

  if (pTreeview)
    delete pTreeview;

  if (pTitle)
  {
    delete pTitle;
  }
  if (pDetailsEdit)
  {
    delete pDetailsEdit;
  }
  if (fTimer)
  {
    delete fTimer;
  }
  if (pointerSet)
  {
    delete pointerSet;
  }
  if (columnSet)
  {
    delete columnSet;
  }
  // Check for image list before destroying.
  if (largeImages)
  {
    ImageList_Destroy( largeImages );
  }
  if (scaledImages)
  {
    ImageList_Destroy( scaledImages );
  }
  if (smallImages)
  {
    ImageList_Destroy( smallImages );
  }
  if (noImages)
  {
    ImageList_Destroy( noImages );
  }
  if (inUseImage)
  {
    ImageList_Destroy( inUseImage );
  }
  if (scaledInUseImage)
  {
    ImageList_Destroy( scaledInUseImage );
  }
  if (smallInUseImage)
  {
    ImageList_Destroy( smallInUseImage );
  }
  if (cnrInfo.strCnrTitle)
  {
    delete cnrInfo.strCnrTitle;
  }
}


/*------------------------------------------------------------------------------
| ICnrControlData::setCnrInfo                                                  |
|                                                                              |
| Change cnrInfo structure according to passed flag.                           |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::setCnrInfo(ICnrInfo* pcnrInfo, unsigned long setFlag)
{
  if (setFlag & CMA_PSORTRECORD)
  {
    cnrInfo.pSortRecord = pcnrInfo->pSortRecord;
    // Do something with the sort record function
  }

  if (setFlag & CMA_PFIELDINFOLAST)
  {
    cnrInfo.pFieldInfoLast = pcnrInfo->pFieldInfoLast;
    // Do something with the last field info struct
  }

  if (setFlag & CMA_CNRTITLE)
  {
    if (cnrInfo.strCnrTitle)
      delete cnrInfo.strCnrTitle;
    cnrInfo.strCnrTitle = (char*)malloc( strlen( pcnrInfo->strCnrTitle ) + 1 );
    strcpy( cnrInfo.strCnrTitle, pcnrInfo->strCnrTitle );
    // Now force an update of the title window/
    if (pTitle)
      pTitle->setText( cnrInfo.strCnrTitle );
  }

  if (setFlag & CMA_FLWINDOWATTR)
  {
    cnrInfo.flWindowAttr = pcnrInfo->flWindowAttr;
    setWindowAttr(cnrInfo.flWindowAttr);
  }

  if (setFlag & CMA_PTLORIGIN)
  {
    cnrInfo.ptlOrigin = pcnrInfo->ptlOrigin;
    // Do something with the origin
  }

  if (setFlag & CMA_DELTA)
  {
    cnrInfo.cDelta = pcnrInfo->cDelta;
    // Do something with the delta.
  }

  if (setFlag & CMA_SLBITMAPORICON)
  {
    // Only change the size if it is different from the current size.
    if ((cnrInfo.slBitmapOrIcon.cx != pcnrInfo->slBitmapOrIcon.cx) &&
        (cnrInfo.slBitmapOrIcon.cy != pcnrInfo->slBitmapOrIcon.cy))
    {
       cnrInfo.slBitmapOrIcon = pcnrInfo->slBitmapOrIcon;
       setImageSize();
    }
  }

  if (setFlag & CMA_SLTREEBITMAPORICON)
  {
    cnrInfo.slTreeBitmapOrIcon = pcnrInfo->slTreeBitmapOrIcon;
  }

  if (setFlag & CMA_TREEBITMAP)
  {
    cnrInfo.hbmExpanded = pcnrInfo->hbmExpanded;
    cnrInfo.hbmCollapsed = pcnrInfo->hbmCollapsed;
  }

  if (setFlag & CMA_TREEICON)
  {
    cnrInfo.hptrExpanded = pcnrInfo->hptrExpanded;
    cnrInfo.hptrCollapsed = pcnrInfo->hptrCollapsed;
  }

  if (setFlag & CMA_LINESPACING)
  {
    cnrInfo.cyLineSpacing = pcnrInfo->cyLineSpacing;
  }

  if (setFlag & CMA_CXTREEINDENT)
  {
    cnrInfo.cxTreeIndent = pcnrInfo->cxTreeIndent;
    if ( pTreeview )
      TreeView_SetIndent( pTreeview->handle(), cnrInfo.cxTreeIndent );
  }

  if (setFlag & CMA_CXTREELINE)
  {
    cnrInfo.cxTreeLine = pcnrInfo->cxTreeLine;
  }

  if (setFlag & CMA_XVERTSPLITBAR)
  {
    cnrInfo.xVertSplitbar = pcnrInfo->xVertSplitbar;
  }

  // There are other CMA_ flags we may support in the future
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::setImageSize                                                |
|                                                                              |
| Set the size of the images.                                                  |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::setImageSize()
{
  Boolean done = false;
  int cx, cy;
  ImageList_GetIconSize (largeImages, &cx, &cy);
  // If the size requested matches the large image list, use that.
  if ((cnrInfo.slBitmapOrIcon.cx == cx) &&
      (cnrInfo.slBitmapOrIcon.cy == cy))
  {
     useScaledImages = false;
     cnrInfo.flWindowAttr &= ~CV_MINI;
     setImageListWithView (cnrInfo.flWindowAttr);
     done = true;
  }
  else
  {
     ImageList_GetIconSize (smallImages, &cx, &cy);
     // If the size requested matches the small image list, use that.
     if ((cnrInfo.slBitmapOrIcon.cx == cx) &&
         (cnrInfo.slBitmapOrIcon.cy == cy))
     {
        useScaledImages = false;
        cnrInfo.flWindowAttr |= CV_MINI;
        setImageListWithView (cnrInfo.flWindowAttr);
        done = true;
     }
     else
     // If the size requested matches the current scaled list, use that.
     {
        if (scaledImages)
        {
           ImageList_GetIconSize (scaledImages, &cx, &cy);
           if ((cnrInfo.slBitmapOrIcon.cx == cx) &&
               (cnrInfo.slBitmapOrIcon.cy == cy))
           {
              useScaledImages = true;
              cnrInfo.flWindowAttr &= ~CV_MINI;
              setImageListWithView (cnrInfo.flWindowAttr);
              done = true;
           }
        }
     }
  }
  // Otherwise, we have to scale to get the right size.  Scale from the large
  // images.
  if (!done)
  {
     // If we had a previous scaled list, destroy it.
     if (scaledImages)
        ImageList_Destroy (scaledImages);
     if (scaledInUseImage)
        ImageList_Destroy (scaledInUseImage);

     // Create and add the scaled in use image.
     scaledInUseImage = ImageList_Create (cnrInfo.slBitmapOrIcon.cx,
                                      cnrInfo.slBitmapOrIcon.cy,
                                      ILC_MASK, 1, 0);
     ImageList_AddIcon( scaledInUseImage,
            IApplication::current().resourceLibrary().loadIcon(IC_INUSEICON) );

     // Create the scaled image list and add the icons from the large image list.
     scaledImages = ImageList_Create (cnrInfo.slBitmapOrIcon.cx,
                                      cnrInfo.slBitmapOrIcon.cy,
                                      ILC_MASK, 40, 5);
     int numberOfIcons = ImageList_GetImageCount (largeImages);
     for (int i=0; i < numberOfIcons; i++)
     {
        // Get icon, add to scaled list and free it (Image API creates it)
        HICON image = ImageList_GetIcon (largeImages, i, ILD_NORMAL);
        ImageList_AddIcon (scaledImages, image);
        DestroyIcon( image );
     }
     useScaledImages = true;
     cnrInfo.flWindowAttr &= ~CV_MINI;
     setImageListWithView (cnrInfo.flWindowAttr);
  }

  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::queryCnrInfo                                                |
|                                                                              |
| Return cnrInfo structure.                                                    |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::queryCnrInfo(ICnrInfo* pInfo)
{
  *pInfo = cnrInfo;
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::setSelection                                                |
|                                                                              |
| Change selection model of the container.                                     |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::setSelection(unsigned long selection)
{
  // If a listview exists, then change its selection type.
  if (pListview)
  {
    if (selection == CCS_SINGLESEL)
    {
      DWORD dwStyle = GetWindowLong(pListview->handle(), GWL_STYLE);
      if (!(dwStyle & LVS_SINGLESEL))
      {
        SetWindowLong(pListview->handle(),
                      GWL_STYLE,
                      (dwStyle | LVS_SINGLESEL | LVS_SHOWSELALWAYS));
      }
    }
    else if (selection == CCS_MULTIPLESEL)
    {
      DWORD dwStyle = GetWindowLong(pListview->handle(), GWL_STYLE);
      if (dwStyle & LVS_SINGLESEL)
      {
        SetWindowLong(pListview->handle(),
                      GWL_STYLE,
                      (dwStyle & ~LVS_SINGLESEL | LVS_SHOWSELALWAYS));
      }
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::migrateContainerObjects                                     |
|                                                                              |
| Build tree view from list view OR vice versa.                                |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::migrateContainerObjects( Boolean fromListToTree )
{
  if (fromListToTree)
  {
    int             numListItems(ListView_GetItemCount(pListview->handle()));
    LV_ITEM         lvItem;
    TV_ITEM         treeItem;
    TV_INSERTSTRUCT tvis;

    lvItem.mask       = LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
    lvItem.iSubItem   = 0;
    tvis.hParent      = TVI_ROOT;
    tvis.hInsertAfter = TVI_LAST;

    for (int index=0; index < numListItems; index++)
    {
      lvItem.iItem = index;
      if (ListView_GetItem(pListview->handle(), &lvItem))
      {
        IContainerObject* pCnrObj((IContainerObject*)lvItem.lParam);
        IMiniCnrRecord* pRec(IRecFromObj(pCnrObj) );

        // Add to list but ONLY if an icon is specified.
        int imageIndex = -1;
        if (pRec->hptrIcon)
          imageIndex = addIconToList(pRec->hptrIcon);

        // Look at this state business later...
        treeItem.state          = 0;
        treeItem.mask           = TVIF_IMAGE | TVIF_TEXT | TVIF_PARAM | TVIF_SELECTEDIMAGE;
        treeItem.stateMask      = 0x0000;
        treeItem.pszText        = pRec->strIcon;
        treeItem.cchTextMax     = 64;
        treeItem.iImage         = imageIndex;
        treeItem.iSelectedImage = imageIndex;
        treeItem.lParam         = lvItem.lParam;
        tvis.item               = treeItem;

        // Insert the item into the tree view.
        HTREEITEM ti = TreeView_InsertItem(pTreeview->handle(), &tvis);

        pCnrObj->ppd->pCnrSet->updateTreeItem( pCnr, ti );
      }
    }
  }
  else
  {
    LV_ITEM          lvItem;
    TV_ITEM          treeItem;
    HTREEITEM        rootNode;

    treeItem.mask = TVIF_IMAGE | TVIF_PARAM;
    lvItem.mask   = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;

    int index;
    for (rootNode = TreeView_GetRoot(pTreeview->handle()), index=0;
         rootNode != 0;
         rootNode = TreeView_GetNextSibling(pTreeview->handle(), rootNode), index++)
    {
      treeItem.hItem = rootNode;

      if (TreeView_GetItem(pTreeview->handle(), &treeItem))
      {
        IMiniCnrRecord* pRec(IRecFromObj( (IContainerObject*)treeItem.lParam) );


        // Look at this state business later...
        lvItem.state      = 0;
        lvItem.stateMask  = 0x0000;
        lvItem.iItem      = index;

        lvItem.iSubItem   = 0;  //Note:  Need to add loop for column subitems?
                                //       Loop thru
        lvItem.pszText    = pRec->strIcon;
        lvItem.cchTextMax = 64;
        lvItem.iImage     = treeItem.iImage;
        lvItem.lParam     = treeItem.lParam;

        ListView_InsertItem(pListview->handle(), &lvItem);
      }
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::layoutActiveView                                            |
|                                                                              |
| Position container window and title window if any.                           |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::layoutActiveView( const ISize& parentSize )
{
  int titleHeight(24);
  if (showTitle && pTitle)
  {
    SetWindowPos( pTitle->handle(),
                  HWND_TOP,
                  0,0,
                  parentSize.width(),
                  titleHeight,
                  SWP_SHOWWINDOW | SWP_FRAMECHANGED );

    if (showTitleSeparator)
      titleHeight++;

    SetWindowPos( pActiveview->handle(),
                  HWND_TOP,
                  0, titleHeight,
                  parentSize.width(),
                  parentSize.height()- titleHeight,
                  SWP_SHOWWINDOW );

  }
  else
  {
    if (pTitle)
      pTitle->hide();
    SetWindowPos( pActiveview->handle(),
                  HWND_TOP,
                  0,0,
                  parentSize.width(),
                  parentSize.height(),
                  SWP_SHOWWINDOW );
  }
  ITRACE_DEVELOP(IString("======> LAYOUT ACTIVE VIEW <====== Width: ") +
             IString(parentSize.width()) + IString("  Height: ") +
             IString(parentSize.height()));
  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::addIconToList                                               |
|                                                                              |
| Put icon handle into pointer set and add it to the image list.               |
------------------------------------------------------------------------------*/
int ICnrControlData::addIconToList ( const IPointerHandle& iconHandle )
{
  int index;

  // Check if the pointer had already been inserted into the
  // image list.  If it has, return its index.
  if (pointerSet->containsElementWithKey( iconHandle.asUnsigned() ))
  {
    IPointerElement* pointerElement(pointerSet->elementWithKey( iconHandle.asUnsigned() ));
    index = pointerElement->imageListIndex;
  }
  // Otherwise, insert the icon into the image list.
  else
  {
    index = ImageList_AddIcon( smallImages, iconHandle );
            ImageList_AddIcon( largeImages, iconHandle );
    if (useScaledImages)
       ImageList_AddIcon( scaledImages, iconHandle );

    // Add the icon into the pointer set.
    pointerSet->addPointerElement( iconHandle, index );
  }
  return index;
}

/*------------------------------------------------------------------------------
| ICnrControlData::addInUseToList                                              |
|                                                                              |
| Put in use icon into pointer set and add it to the image list.               |
------------------------------------------------------------------------------*/
int ICnrControlData::addInUseToList ( const IPointerHandle& iconHandle )
{
  int index;

  // Check if the pointer had already been inserted into the
  // image list.  If not, return an error (-1).
  if (pointerSet->containsElementWithKey( iconHandle.asUnsigned() ))
  {
    IPointerElement* pointerElement(pointerSet->elementWithKey( iconHandle.asUnsigned() ));

    // If the image has not been merged, merge it with the in use image and
    // add it to the list.  Otherwise, return the merged icon's index.
    if (pointerElement->inUseIndex == -1)
    {
      HICON      hIcon;
      HIMAGELIST hTemp = ImageList_Merge( inUseImage,
                                          0,
                                          largeImages,
                                          pointerElement->imageListIndex,
                                          0, 0 );

      hIcon = ImageList_GetIcon( hTemp, 0, ILD_NORMAL );
      index = ImageList_AddIcon( largeImages, hIcon );
      ImageList_Destroy( hTemp );
      DestroyIcon( hIcon );

      pointerElement->inUseIndex = index;

      hTemp = ImageList_Merge( smallInUseImage, 0,
                               smallImages,
                               pointerElement->imageListIndex,
                               0, 0 );

      hIcon = ImageList_GetIcon( hTemp, 0, ILD_NORMAL );
      index = ImageList_AddIcon( smallImages, hIcon );
      ImageList_Destroy( hTemp );
      DestroyIcon( hIcon );
      if (useScaledImages)
      {
         hTemp = ImageList_Merge( scaledInUseImage, 0,
                                  scaledImages,
                                  pointerElement->imageListIndex,
                                  0, 0 );

         hIcon = ImageList_GetIcon( hTemp, 0, ILD_NORMAL );
         index = ImageList_AddIcon( scaledImages, hIcon );
         ImageList_Destroy( hTemp );
         DestroyIcon( hIcon );
      }
    }
    else
    {
      index = pointerElement->inUseIndex;
    }
  }
  else
  {
    index = -1;
  }
  return index;
}


/*------------------------------------------------------------------------------
| ICnrControlData::setWindowAttr                                               |
|                                                                              |
| Set window attributes structure according to CV_* flags.                     |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::setWindowAttr(unsigned long flags)
{
  // Don't need to check for CV_FLOW since we've
  // hard-coded the GRID place style in createContainer.
  // Basically, the native container is always in flowed view, so
  // showTextView == showFlowedTextView, etc.
  // Illegal combinations, e.g., tree & detail will do one or the other
  // or nothing.
  // Only the combinations used by IContainerControl are considered.

  // showTextView             ( ), // CV_TEXT & !CV_TREE
  // showFlowedTextView       ( ), // CV_FLOW & CV_TEXT & !CV_TREE
  // showNameView             ( ), // CV_NAME & !CV_TREE
  // showFlowedNameView       ( ), // CV_NAME & CV_FLOW & !CV_TREE
  // showDetailsView          ( ), // CV_DETAILS
  // showTreeIconView         ( ), // CV_TREE & CV_ICON
  // showTreeTextView         ( ), // CV_TREE & CV_TEXT
  // showTreeNameView         ( ), // CV_TREE & CV_NAME
  // showIconView             ( ); // CV_ICON & !CV_TREE

  // For all view flags, set the active view, set the image list, and
  // change window word if necessary (SetWindowLong).
  if (((flags & CV_TREE) && (flags & CV_ICON)) ||
      ((flags & CV_TREE) && (flags & CV_TEXT)))
  {
    setActiveView( ICnrControlData::tree );
    setImageListWithView (flags);
    DWORD dwStyle = GetWindowLong(pTreeview->handle(), GWL_STYLE);
    SetWindowLong(pTreeview->handle(),GWL_STYLE, (dwStyle | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_HASLINES));
  }

  if ((flags & CV_TREE) && (flags & CV_NAME))
  {
    setActiveView( ICnrControlData::tree );
    setImageListWithView (flags);
    DWORD dwStyle = GetWindowLong(pTreeview->handle(), GWL_STYLE);
    SetWindowLong(pTreeview->handle(),GWL_STYLE, ((dwStyle | TVS_HASLINES) & ~(TVS_HASBUTTONS | TVS_LINESATROOT)));
  }

  if (flags & CV_DETAIL)
  {
    setActiveView( ICnrControlData::list );
    setImageListWithView (flags);
    DWORD dwStyle = GetWindowLong(pListview->handle(), GWL_STYLE);
    if ((dwStyle & LVS_TYPEMASK) != LVS_REPORT)
    {
      SetWindowLong(pListview->handle(),
                    GWL_STYLE,
                    (dwStyle & ~LVS_TYPEMASK) | LVS_REPORT | LVS_SHOWSELALWAYS);
    }
  }

  if ( (flags & CV_ICON) && !(flags & CV_TREE) )
  {
    setActiveView( ICnrControlData::list );
    setImageListWithView (flags);
    DWORD dwStyle = GetWindowLong(pListview->handle(), GWL_STYLE);
    if ((dwStyle & LVS_TYPEMASK) != LVS_ICON)
    {
      SetWindowLong(pListview->handle(),
                    GWL_STYLE,
                    (dwStyle & ~LVS_TYPEMASK) | LVS_ICON | LVS_SHOWSELALWAYS);
    }
  }

  if (((flags & CV_NAME) && !(flags & CV_TREE)) ||
     ((flags & CV_TEXT) && !(flags & CV_TREE)))
  {
    setActiveView( ICnrControlData::list );
    setImageListWithView (flags);
    DWORD dwStyle = GetWindowLong(pListview->handle(), GWL_STYLE);
    if ((dwStyle & LVS_TYPEMASK) != LVS_LIST)
    {
      SetWindowLong(pListview->handle(),
                    GWL_STYLE,
                    (dwStyle & ~LVS_TYPEMASK) | LVS_LIST | LVS_SHOWSELALWAYS );
    }
  }

  if (flags & CA_TITLELEFT)
  {
    pTitle->setAlignment( IStaticText::centerLeft );
  }

  if (flags & CA_TITLECENTER)
  {
    pTitle->setAlignment( IStaticText::centerCenter );
  }

  if (flags & CA_TITLERIGHT)
  {
    pTitle->setAlignment( IStaticText::centerRight );
  }

  if (flags & CA_TITLESEPARATOR)
  {
    showTitleSeparator = true;
    layoutActiveView(pCnr->size());
  }
  else
  {
    showTitleSeparator = false;
    layoutActiveView(pCnr->size());
  }

  if (flags & CA_CONTAINERTITLE)
  {
      // Set the font of the title window.
      pTitle->setFont(pActiveview->font());

    if (cnrInfo.strCnrTitle)
      pTitle->setText( cnrInfo.strCnrTitle );

    showTitle = true;
    layoutActiveView(pCnr->size());
  }
  else
  {
    showTitle = false;
    layoutActiveView(pCnr->size());
  }

  if (flags & CA_TREELINE)
  {
     // Native container always has them or not.  Can't turn them on and off.
  }

  if (flags & CA_DETAILSVIEWTITLES)
  {
     // Title is always required in native container.  Can't turn it on or off.
  }

  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::setImageListWithView                                        |
|                                                                              |
| Set the image list according to the view.                                    |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::setImageListWithView ( unsigned long flags )
{
   int width, height;
   // For all views, set the image list by checking the following: if mini,
   // if scaled, and otherwise.  Store the image width and height in
   // case it is queried.
   // treeIcon and treeName have images.
   if (((flags & CV_TREE) && (flags & CV_ICON)) ||
       ((flags & CV_TREE) && (flags & CV_NAME)))
   {
      if (flags & CV_MINI)
      {
        useScaledImages = false;
        TreeView_SetImageList(pTreeview->handle(), smallImages, TVSIL_NORMAL);
        if (ImageList_GetIconSize( smallImages, &width, &height) )
        {
          cnrInfo.slTreeBitmapOrIcon.cx = width;
          cnrInfo.slTreeBitmapOrIcon.cy = height;
          cnrInfo.slBitmapOrIcon.cx = width;
          cnrInfo.slBitmapOrIcon.cy = height;
        }
      }
     else
     if (useScaledImages)
     {
        TreeView_SetImageList(pTreeview->handle(), scaledImages, TVSIL_NORMAL);
        if (ImageList_GetIconSize( scaledImages, &width, &height) )
        {
          cnrInfo.slTreeBitmapOrIcon.cx = width;
          cnrInfo.slTreeBitmapOrIcon.cy = height;
          cnrInfo.slBitmapOrIcon.cx = width;
          cnrInfo.slBitmapOrIcon.cy = height;
        }
     }
     else
     {
       TreeView_SetImageList(pTreeview->handle(), largeImages, TVSIL_NORMAL);
       if (ImageList_GetIconSize( largeImages, &width, &height) )
       {
         cnrInfo.slTreeBitmapOrIcon.cx = width;
         cnrInfo.slTreeBitmapOrIcon.cy = height;
         cnrInfo.slBitmapOrIcon.cx = width;
         cnrInfo.slBitmapOrIcon.cy = height;
       }
     }
   }
   else
   // treeText has no images.
   if ((flags & CV_TREE) && (flags & CV_TEXT))
   {
     // Using a NULL image list causes the TreeView to draw the collapse/expand
     // icons itself.
     TreeView_SetImageList(pTreeview->handle(), NULL, TVSIL_NORMAL);
     cnrInfo.slTreeBitmapOrIcon.cx = 0;
     cnrInfo.slTreeBitmapOrIcon.cy = 0;
     cnrInfo.slBitmapOrIcon.cx = width;
     cnrInfo.slBitmapOrIcon.cy = height;
   }
   else
   // Details, icon, and name all have images.
   if ((flags & CV_DETAIL) ||
       ((flags & CV_ICON) && !(flags & CV_TREE)) ||
       ((flags & CV_NAME) && !(flags & CV_TREE)))
   {
     if (flags & CV_MINI)
     {
       useScaledImages = false;
       ListView_SetImageList(pListview->handle(), smallImages, LVSIL_SMALL );
       ListView_SetImageList(pListview->handle(), smallImages, LVSIL_NORMAL );
       if (ImageList_GetIconSize( smallImages, &width, &height) )
       {
         cnrInfo.slTreeBitmapOrIcon.cx = width;
         cnrInfo.slTreeBitmapOrIcon.cy = height;
         cnrInfo.slBitmapOrIcon.cx = width;
         cnrInfo.slBitmapOrIcon.cy = height;
       }
     }
     else
     if (useScaledImages)
     {
        ListView_SetImageList(pListview->handle(), scaledImages, LVSIL_SMALL);
        ListView_SetImageList(pListview->handle(), scaledImages, LVSIL_NORMAL);
        if (ImageList_GetIconSize( scaledImages, &width, &height) )
        {
          cnrInfo.slTreeBitmapOrIcon.cx = width;
          cnrInfo.slTreeBitmapOrIcon.cy = height;
          cnrInfo.slBitmapOrIcon.cx = width;
          cnrInfo.slBitmapOrIcon.cy = height;
        }
     }
     else
     {
       ListView_SetImageList(pListview->handle(), largeImages, LVSIL_SMALL );
       ListView_SetImageList(pListview->handle(), largeImages, LVSIL_NORMAL );
       if (ImageList_GetIconSize( largeImages, &width, &height) )
       {
         cnrInfo.slTreeBitmapOrIcon.cx = width;
         cnrInfo.slTreeBitmapOrIcon.cy = height;
         cnrInfo.slBitmapOrIcon.cx = width;
         cnrInfo.slBitmapOrIcon.cy = height;
       }
     }
   }
   else
   // Plain text view has no images.
   if ((flags & CV_TEXT) && !(flags & CV_TREE))
     // Set a dummy image list that contains zero-sized images.
     ListView_SetImageList(pListview->handle(), noImages, LVSIL_SMALL );

   return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::setActiveView                                               |
|                                                                              |
| Show the tree view or list view.  Create them if they don't exist.           |
------------------------------------------------------------------------------*/
ICnrControlData& ICnrControlData::setActiveView ( ActiveView activeView )
{
  // If this is the first time, we have to do some special processing.
  Boolean fFirstTime(false);
  if (activeView == ICnrControlData::none)
    fFirstTime = true;

  // If the view being requested is the one we're in, do nothing.
  if (activeView != av)
  {
    // Create a place holder for the prior view.
    INativeContainerControl *pOldview(0);
    // Set the active view.
    av = activeView;

    // View requested is tree view.
    if (activeView == ICnrControlData::tree)
    {
      // A TreeView control doesn't yet exist -> Create It.
      if (!pTreeview)
      {
        // A TreeView control doesn't yet exist -> Create It.
        unsigned long currentStyle(pCnr->style());
        pTreeview = new INativeContainerControl( IC_NATIVE_TREEVIEW,
                                                 pCnr->handle(),
                                                 pCnr->handle(),
                                                 IRectangle(IPoint(0,0),
                                                            pCnr->size()),
                                                 convertToNativeStyle( currentStyle, false ),
                                                 false );

        // If a list view exists, populate the tree view from the list view
        if (pListview)
        {
          migrateContainerObjects( true );
          pTreeview->setFont(pListview->font());
        }

        // If the Child Indention has been changed from the default, set it.
        if ( cnrInfo.cxTreeIndent )
          TreeView_SetIndent( pTreeview->handle(), cnrInfo.cxTreeIndent );

      }

      pActiveview = pTreeview;

      if (pListview)
      {
        pOldview = pListview;
        pTreeview->moveSizeTo( pListview->rect() );

        // Get rid of the edit field if it exists.
        ListView_EditLabel( pListview->handle(), (-1) );

        if (pCnr->ppd->pDetailsEdit)                                 /* D8097 */
          // Send a fake enter message to the mle so that it will close.
          pCnr->ppd->pDetailsEdit->sendEvent( WM_CHAR,
                                              IEventParameter1( VK_RETURN ));

        // Fake out the container to let it think it is still in a non tree
        // view so that the CM_QUERYRECORDEMPHASIS logic below will work.
        cnrInfo.flWindowAttr &= ~CV_TREE;

        // Query the cursored object in the list view.  This
        // will be the selected object in the tree view.
        IContainerObject* cursored(pCnr->cursoredObject());
        cnrInfo.flWindowAttr |= CV_TREE;

        if (cursored)
        {
          // Set the selected object in the tree view to
          // the cursored object in the tree view.
          pCnr->setSelected(cursored);
        }
      }
    }
    else
    {
      if (!pListview)
      {
        // A ListView control doesn't yet exist -> Create It.
        unsigned long currentStyle(pCnr->style());
        pListview = new INativeContainerControl( IC_NATIVE_LISTVIEW,
                                                 pCnr->handle(),
                                                 pCnr->handle(),
                                                 IRectangle(IPoint(0,0),
                                                            pCnr->size()),
                                                 convertToNativeStyle( currentStyle, true ),
                                                 true );

        // Associate the image list with the list view
        ListView_SetImageList(pListview->handle(), smallImages, LVSIL_SMALL);
        ListView_SetImageList(pListview->handle(), largeImages, LVSIL_NORMAL);
        // If a tree view exists, populate the list view from the tree view
        if (pTreeview)
        {
          migrateContainerObjects( false );
          pListview->setFont(pTreeview->font());
        }

        // Add the required first column for details view.
        LV_COLUMN    lvColumn;

        lvColumn.mask     = LVCF_TEXT | LVCF_WIDTH;
        lvColumn.pszText  = "";
        lvColumn.cx = 0;

        ListView_InsertColumn( pListview->handle(), 0, &lvColumn );

      }

      pActiveview = pListview;

      if (pTreeview)
      {
        pOldview = pTreeview;
        pListview->moveSizeTo( pTreeview->rect() );

        // Get rid of the edit field if it exists.
        TreeView_EndEditLabelNow( pTreeview->handle(), false );

        // Fake out the container to let it think it is still in a tree
        // view so that the CM_QUERYRECORDEMPHASIS logic below will work.
        cnrInfo.flWindowAttr |= CV_TREE;

        // Query the cursored object in the tree view.  This
        // will be the selected object in the list view.
        IContainerObject* cursored(pCnr->cursoredObject());
        cnrInfo.flWindowAttr &= ~CV_TREE;

        if (cursored)
        {
          //We should remove selection from all objects other than the
          //cursor because of mult. selection....Or that a different object
          //may have been selected when last in the listview.
          IContainerControl::ObjectCursor  listCursor =
                              IContainerControl::ObjectCursor(*pCnr, 0, false);
          forCursor ( listCursor )
            pCnr->removeSelected( pCnr->objectAt( listCursor ) );

          // Set the selected & cursored object in the list view
          // to the root of the cursored object in the tree view.
          IContainerObject*  rootObj= pCnr->ppd->rootObject( cursored );
          pCnr->setSelected( rootObj );
          pCnr->setCursor  ( rootObj );
        }
      }
    }

    pActiveview->show();
    if (pOldview)
      pOldview->hide();

    // Do something that causes the view to be redrawn.
  }

  if (pCnr->ppd->pDetailsEdit)                                     /* D8097 */
    // Send a fake enter message to the mle so that it will close.
    pCnr->ppd->pDetailsEdit->sendEvent( WM_CHAR,
                                        IEventParameter1( VK_RETURN ));

  if (fFirstTime)
  {
    pCnr->setFont(pActiveview->font());
    // Set the font of the title window.
    pTitle->setFont(pActiveview->font());
  }

  return *this;
}

/*------------------------------------------------------------------------------
| ICnrControlData::rootObject                                                  |
|                                                                              |
| Return the root object of the tree view.                                     |
------------------------------------------------------------------------------*/
IContainerObject* ICnrControlData::rootObject(IContainerObject* treeObj)
{
  IContainerObject*   rootObj = 0;

  while ( treeObj )
  {
    rootObj = treeObj;
    treeObj = pCnr->parentObject( treeObj );
  }

  return ( rootObj );
}

/*------------------------------------------------------------------------------
| ICnrControlData::activeView                                                  |
|                                                                              |
| Return the active view.                                                      |
------------------------------------------------------------------------------*/
ICnrControlData::ActiveView ICnrControlData::activeView( ) const
{
  return av;
}

/*------------------------------------------------------------------------------
| ICnrControlData::registerContainer                                           |
|                                                                              |
| Register the wrapper window class.                                           |
------------------------------------------------------------------------------*/
void ICnrControlData::registerContainer( )
{
  static Boolean fRegistered(false);

  // Register the wrapper window class and query the size of the large
  // and small image lists.
  if (!fRegistered)
  {
    InitCommonControls();

    fRegistered = true;
    WNDCLASS wndClass;

    wndClass.style         = CS_HREDRAW | CS_VREDRAW;
    wndClass.cbClsExtra    = 0;
    wndClass.cbWndExtra    = sizeof(LONG);
    wndClass.hCursor       = LoadCursor(0,IDC_ARROW);
    wndClass.hInstance     = GetModuleHandle(0);
    wndClass.lpfnWndProc   = (WNDPROC)IDEFWINDOWPROC;
    wndClass.hIcon         = 0;
    wndClass.hbrBackground = GetStockObject(BLACK_BRUSH);
    wndClass.lpszMenuName  = 0;
    wndClass.lpszClassName = WC_NATIVECONTAINER;

    if (!RegisterClass(&wndClass))
      ITHROWGUIERROR( "RegisterClass" );

#ifdef ASK_ABOUT_SHELL_STUFF
    // Get the width and heigth of the system image lists.
    SHFILEINFO sfi;

    HIMAGELIST
    sysImageList = (HIMAGELIST) SHGetFileInfo( "C:\\",
                                              0,
                                              &sfi,
                                              sizeof(SHFILEINFO),
                                              SHGFI_SYSICONINDEX | SHGFI_SMALLICON );

    ImageList_GetIconSize( sysImageList, &smallImageWidth, &smallImageHeight );

    sysImageList = (HIMAGELIST) SHGetFileInfo( "C:\\",
                                               0,
                                               &sfi,
                                               sizeof(SHFILEINFO),
                                               SHGFI_SYSICONINDEX | SHGFI_LARGEICON );

    ImageList_GetIconSize( sysImageList, &largeImageWidth, &largeImageHeight );
#endif
  }
}

/*------------------------------------------------------------------------------
| ICnrControlData::convertToNativeStyle                                        |
|                                                                              |
| Convert CCL container style to native container style.                       |
------------------------------------------------------------------------------*/
unsigned long ICnrControlData::convertToNativeStyle( unsigned long cclStyle,
                                                     Boolean       fListView )
{
  unsigned long windowStyle(0);

  if (fListView)
  {
    windowStyle = LVS_ICON | LVS_SHOWSELALWAYS;
    if (~(cclStyle & CCS_READONLY))
      windowStyle |= LVS_EDITLABELS;
    if (cclStyle & CCS_AUTOPOSITION)
      windowStyle |= LVS_AUTOARRANGE;
    if (cclStyle & CCS_SINGLESEL)
      windowStyle |= LVS_SINGLESEL;
  }
  else
  {
    windowStyle = TVS_SHOWSELALWAYS | TVS_HASLINES | TVS_HASBUTTONS | TVS_LINESATROOT;
    if (~(cclStyle & CCS_READONLY))
      windowStyle |= TVS_EDITLABELS;
  }

  return windowStyle;
}

/*------------------------------------------------------------------------------
| INativeContainerControl::INativeContainerControl                             |
|                                                                              |
| Native container control constructor.                                        |
------------------------------------------------------------------------------*/
INativeContainerControl::INativeContainerControl( unsigned long id,
                                                  const IWindowHandle& parent,
                                                  const IWindowHandle& owner,
                                                  const IRectangle& initial,
                                                  unsigned long style,
                                                  Boolean fCreateListView )
  : INativeContainerControl::Inherited( )
{
  IWindowHandle whContainer;

  if (fCreateListView)
  {
    whContainer = this->create( IC_NATIVE_LISTVIEW,
                                0,
                                style | WS_CHILD,
                                WC_LISTVIEW,
                                parent,
                                owner,
                                initial,
                                0,
                                0,
                                defaultOrdering(),
                                WS_EX_CLIENTEDGE );
  }
  else
  {
    whContainer = this->create( IC_NATIVE_TREEVIEW,
                                0,
                                style | WS_CHILD,
                                WC_TREEVIEW,
                                parent,
                                owner,
                                initial,
                                0,
                                0,
                                defaultOrdering(),
                                WS_EX_CLIENTEDGE );
  }
  this->startHandlingEventsFor( whContainer );

  IMessageHandler::defaultHandler()->handleEventsFor( this );
}

/*------------------------------------------------------------------------------
| INativeContainerControl::~INativeContainerControl                            |
|                                                                              |
| Native container control destructor                                          |
------------------------------------------------------------------------------*/
INativeContainerControl::~INativeContainerControl( )
{
  IMessageHandler::defaultHandler()->stopHandlingEventsFor(this);
}

/**********************************************************************/
/* Class to handle destruction of the static INativeContainerHandler  */
/* object.                                                            */
/**********************************************************************/
class INativeContainerHandlerStatics
{
public:
~INativeContainerHandlerStatics ()
 {
   if(pNativeContainerHandler)
      delete pNativeContainerHandler;
   if(pMessageHandler)
      delete pMessageHandler;
   if(pEditHandler)
      delete pEditHandler;
 }

INativeContainerHandler
 *pNativeContainerHandler;
IMessageHandler
 *pMessageHandler;
IDetailsEditHandler
 *pEditHandler;
};   // INativeContainHandlerStatics

#pragma data_seg(ICLNonConst)
static INativeContainerHandlerStatics IDefaultNCStatics;
#pragma data_seg()


/*------------------------------------------------------------------------------
| INativeContainerHandler::INativeContainerHandler                             |
| Native container handler constructor.                                        |
------------------------------------------------------------------------------*/
INativeContainerHandler::INativeContainerHandler( )
  : INativeContainerHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| INativeContainerHandler::~INativeContainerHandler                            |
| Native container handler destructor.                                         |
------------------------------------------------------------------------------*/
INativeContainerHandler::~INativeContainerHandler( )
{ }

/*------------------------------------------------------------------------------
| INativeContainerHandler::defaultHandler                                      |
|                                                                              |
| Returns the default handler (creates one if necessary).                      |
------------------------------------------------------------------------------*/
INativeContainerHandler* INativeContainerHandler::defaultHandler ( )
{
   if(!IDefaultNCStatics.pNativeContainerHandler)
      IDefaultNCStatics.pNativeContainerHandler = new INativeContainerHandler();
   return IDefaultNCStatics.pNativeContainerHandler;
}

/*------------------------------------------------------------------------------
| INativeContainerHandler::containerHandleFromEvent                            |
|                                                                              |
| Extract the container handle from an event.                                  |
------------------------------------------------------------------------------*/
IWindowHandle INativeContainerHandler::containerHandleFromEvent ( const IEvent& event )
{
  IContainerControl *container((IContainerControl*)event.window());
  IWindowHandle view;
  view = container->ppd->pActiveview->handle();
  return view;
}

/*------------------------------------------------------------------------------
| INativeContainerHandler::dispatchHandlerEvent                                |
|                                                                              |
| Handle events to the native container wrapper.                               |
------------------------------------------------------------------------------*/
Boolean INativeContainerHandler::dispatchHandlerEvent( IEvent& event )
{
#define ptrTV_DISPINFO ((TV_DISPINFO *)event.parameter2().asUnsignedLong())
#define ptrLV_DISPINFO ((LV_DISPINFO *)event.parameter2().asUnsignedLong())

  Boolean fStopHandling( false );
  IContainerControl *pCnrCtl((IContainerControl*)event.window());

  switch (event.eventId())
  {
    // Eat the keyboard messages so they are not passed up the owner chain.
    case  WM_KEYDOWN:
    case  WM_KEYUP:
    case  WM_CHAR:
      // Pass tab key to owner for tab processing.
      if ((event.eventId() == WM_CHAR) && (event.parameter1() == VK_TAB))
      {
        IWindow* owner(event.window()->owner());
        owner->sendEvent( event );
      }
      event.setResult(true);
    break;

    case WM_GETDLGCODE:
      event.setResult( (long)DLGC_WANTARROWS | DLGC_WANTCHARS );
      fStopHandling = true;
    break;

    // If the wrapper window is receiving the input focus,
    // pass it onto the active control window.
    case WM_SETFOCUS:
      if (pCnrCtl->ppd->pActiveview &&
          pCnrCtl->ppd->pActiveview->isVisible())
        pCnrCtl->ppd->pActiveview->setFocus();
    break;

    case CM_ALLOCDETAILFIELDINFO:
    case CM_ALLOCRECORD:
      /*******************************************************************/
      /* These messages are not implemented because we do not call them  */
      /* for the native windows container.  The memory allocation is done*/
      /* only via the CCL allocation container.                          */
      /*******************************************************************/
      event.setResult(0);
      fStopHandling = true;
    break;

    case CM_ARRANGE:
    {
      /******************************************************************/
      /* Arrange the Native ListView Control's Icon View.               */
      /******************************************************************/
      //If we have size and the list view exists, do the arrange now.
      //Otherwise, set the needArrange flag for later when we are given
      //size (WM_SIZE), or the view is changed (CM_SETCNRINFO).

      if (pCnrCtl->ppd->pListview)
      {
         ISize cnrSize = pCnrCtl->ppd->pListview->size();
         Boolean hasSize = (!(cnrSize.width() == 0)) &&
                           (!(cnrSize.height() == 0));
         if (hasSize)
         {
            pCnrCtl->ppd->needArrange = false;
            ListView_Arrange( pCnrCtl->ppd->pListview->handle(), LVA_ALIGNTOP );
         }
         else
            pCnrCtl->ppd->needArrange = true;
      }
      else
         pCnrCtl->ppd->needArrange = true;

      event.setResult( true );
      fStopHandling = true;
    }
    break;

    case CM_CLOSEEDIT:
      /*********************************************************************/
      /* Close the current edit field.                                     */
      /*********************************************************************/

      //If in a treeView, close its edit field, otherwise close the
      //listview's edit field.  We assume that switching views closes
      //the edit fields, thus both views can't have an open field
      //simultaneously.
      if ( pCnrCtl->isTreeView() )
        event.setResult( TreeView_EndEditLabelNow(
                                 pCnrCtl->ppd->pTreeview->handle(), false ) );
      // If we're not in details view, let the control close the open edit field.
      else if (!pCnrCtl->isDetailsView())
           event.setResult( ListView_EditLabel( pCnrCtl->ppd->pListview->handle(),
                                                (-1) ) );
      // If we're in details view, we opened the edit field so we have to close it.
      else if (pCnrCtl->ppd->pDetailsEdit)                          /* D8017 */
      {
        // Send a fake enter message to the mle so that it will close.
        pCnrCtl->ppd->pDetailsEdit->sendEvent( WM_CHAR,
                                               IEventParameter1( VK_RETURN ));
        event.setResult(true);
      }

      fStopHandling = true;
      break;

    case CM_ERASERECORD:
      /********************************************************************/
      /* There is no Native Windows equivalent.                           */
      /* This function is not needed for moving icons in Windows Listview.*/
      /********************************************************************/
      event.setResult( true );
      fStopHandling = true;
    break;


    case CM_EXPANDTREE:
    case CM_COLLAPSETREE:
    {
      /*********************************************************************/
      /* Expand/Collapse the given object.                                 */
      /* If object=0, then expand/collapse all objects.                    */
      /*********************************************************************/
      //Set changeFlag appropriately, for the given message.
      UINT  changeFlag=0;
      if (event.eventId() == CM_EXPANDTREE)
        changeFlag = TVE_EXPAND;
      else
        changeFlag = TVE_COLLAPSE;
      IMiniCnrRecord* pRec((IMiniCnrRecord*)event.parameter1().asUnsignedLong());

      // If we have a tree view, do the expand/collapse whether it is currently
      // showing or not.
      if ( pCnrCtl->ppd->pTreeview )
      {
        IContainerObject*  pCnrObj=0;
        HTREEITEM          hItem;

        if ( pRec )    //Just expand/collapse the given object.
        {
          pCnrObj  = IObjFromRec( pRec );
          hItem = pCnrObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl);
          // If an expand or collapse actually occurs, TreeView_Expand
          // returns true, and we need to notify our owner.  TreeView_Expand
          // DOES NOT send a TVN_ITEMEXPANDED, as documented in the MSDN.
          // Check our own flag to see if expand or collapse notification has
          // been done.  Windows does notify the first time,
          // but not thereafter.
          pCnrCtl->ppd->expandNotificationDone = false;
          if (TreeView_Expand(pCnrCtl->ppd->pTreeview->handle(),
              hItem, changeFlag))
          {
             if (!pCnrCtl->ppd->expandNotificationDone)
             {
                pCnrCtl->ppd->expandNotificationDone = true;
                IWindow* owner(event.window()->owner());

                if (owner)
                {
                  if (changeFlag == TVE_EXPAND)
                  {
                    owner->sendEvent( WM_CONTROL,
                                      IEventParameter1( event.window()->id(),
                                                        CN_EXPANDTREE ),
                                      IEventParameter2( (RECORDCORE*)pRec ));
                  }
                  else
                  {
                    owner->sendEvent( WM_CONTROL,
                                      IEventParameter1( event.window()->id(),
                                                        CN_COLLAPSETREE ),
                                      IEventParameter2( (RECORDCORE*)pRec ));
                  }
                }
             }
          }
          // Always return true.  TreeView_Expand returns false if no change
          // occurred, i.e., if expand is requested on an already expanded item.
          event.setResult( true );                                   /* D7942 */
        }
        else           //Expand/Collapse the whole tree.
        {
          IContainerControl::ObjectCursor cursor(*pCnrCtl);

          //Cursor thru and expand/collapse all objects in the tree view.
          forCursor (cursor)
          {
            pCnrObj = pCnrCtl->objectAt(cursor);
            hItem = pCnrObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl);
            TreeView_Expand( pCnrCtl->ppd->pTreeview->handle(),
                             hItem, changeFlag );
          }
          //Always return true
          event.setResult( true );
        }
      }
      fStopHandling = true;
    }
    break;

    case CM_FILTER:
      //Not implemented for Native Windows.
    break;

    case CM_FREEDETAILFIELDINFO:
    case CM_FREERECORD:
      /*******************************************************************/
      /* These messages are not implemented because we do not call them  */
      /* for the native windows container.  The freeing is only done via */
      /* the CCL allocationContainer.  We do not need to explicity free  */
      /* windows objects or columns, just removing them is enough.       */
      /*******************************************************************/
      event.setResult(0);
      fStopHandling = true;
    break;

    case CM_HORZSCROLLSPLITWINDOW:
    {
      /*******************************************************************/
      /* Scroll the Details view.                                        */
      /* Only scroll the left window.  If right is requested, do nothing.*/
      /*******************************************************************/
      Boolean         retVal    ( false );
      unsigned short  usWindow  ( event.parameter1().lowNumber() );
      long            scrollInc ( (long)event.parameter2() );

      if ( pCnrCtl->isDetailsView() )
      {
        if ( usWindow == CMA_LEFT )
          event.setResult( ListView_Scroll( pCnrCtl->ppd->pListview->handle(),
                                            (int)scrollInc, 0 ) );
        retVal = true;
      }

      event.setResult( retVal );
      fStopHandling = true;
    }
    break;

    case CM_INSERTDETAILFIELDINFO:
    {
      /*******************************************************************/
      /* Insert a new column into the list view.                         */
      /*******************************************************************/
      LPFIELDINFO
            pFI((LPFIELDINFO)event.parameter1().asUnsignedLong());
      LPFIELDINFOINSERT
           pFIInsert((LPFIELDINFOINSERT)event.parameter2().asUnsignedLong());
      LV_COLUMN    lvColumn;
      int   columnId( 0 ),
            retVal( 1 ),
            insertPosition( 0 );

      // If a ListView control doesn't yet exist, create it.
      if ( !pCnrCtl->ppd->pListview )
      {
        unsigned long currentStyle( pCnrCtl->style() );
        pCnrCtl->ppd->pListview = new INativeContainerControl(
                                          IC_NATIVE_LISTVIEW,
                                          pCnrCtl->handle(),
                                          pCnrCtl->handle(),
                                          IRectangle(IPoint(0,0),
                                                     pCnrCtl->size()),
                                          pCnrCtl->ppd->convertToNativeStyle(
                                                           currentStyle, true ),
                                          true );

        // Set the image list for the new view.
        pCnrCtl->ppd->setImageListWithView (pCnrCtl->ppd->cnrInfo.flWindowAttr);
        // If a tree view exists, populate the list view from the tree view
        if ( pCnrCtl->ppd->pTreeview )
          pCnrCtl->ppd->migrateContainerObjects( false );

        // Add the required first column for details view.
        lvColumn.mask     = LVCF_TEXT | LVCF_WIDTH;
        lvColumn.pszText  = "";
        lvColumn.cx = 0;

        ListView_InsertColumn( pCnrCtl->ppd->pListview->handle(),
                               0, &lvColumn );

      }

      // If the column being inserted is (DataSource=isIcon or isIconViewText)
      // then don't insert it as it is already inserted as required.
      if ( pFI->offStruct < pCnrCtl->baseRecordSize() )
      {
        pCnrCtl->ppd->collapseFirstColumn = false;
        if ( pFI->offStruct == (unsigned long)&(((ICnrRecord *)0)->strIcon) )
        {
          // DataSource=isIconViewText, then update first column title
          // and column size.
          lvColumn.mask = LVCF_TEXT | LVCF_WIDTH;
          if ( pFI->cxWidth == 0 )
            lvColumn.cx   = 100;
          else
            lvColumn.cx   = pFI->cxWidth;

          if ( (pFI->flTitle & CFA_STRING) && pFI->pTitleData )
          {
            // In native windows, only single line headings supported, so we
            // will strip out any newline characters and replace with blanks.
            IString temp( (char*)pFI->pTitleData );
            temp.change( "\n", " " );
            strcpy((char*)(pFI->pTitleData), temp );
            lvColumn.pszText = (char*)pFI->pTitleData;
          }
          else
            lvColumn.pszText = "";

          // Check if the isIconViewText column has been added previously.
          IContainerColumn* pcnrcol =
                   pCnrCtl->ppd->columnSet->containerColumnFromColumnId(0);

          // If column 0 has already been added, then remove from set
          // before adding the new one (because we are only replacing the
          // current one in the native control, don't want to count twice)
          if ( pcnrcol != 0 )
          {
            pCnrCtl->ppd->columnSet->remove( pcnrcol, false );
            delete pcnrcol;
          }

          ListView_SetColumn( pCnrCtl->ppd->pListview->handle(),
                              0, &lvColumn );

          // Add this to the columnSet for this container.
          pCnrCtl->ppd->columnSet->add( (IContainerColumn*)pFI->pUserData,
                                        0, 0, true );
        }
        else
        if ( pFI->offStruct == (unsigned long)&(((ICnrRecord *)0)->hptrIcon) )
        {
          // DataSource=isIcon, then update first column size.
          // Do this only if the isIconViewText column has not been added.
          // The application can only control the size and heading of this
          // column via the isIconViewText column.
          if ( pCnrCtl->ppd->columnSet->containerColumnFromColumnId(0) == 0 )
          {
            lvColumn.mask = LVCF_WIDTH;
            lvColumn.cx   = 100;
            ListView_SetColumn( pCnrCtl->ppd->pListview->handle(),
                                0, &lvColumn );
          }

          // Add this to the columnSet for this container.
          pCnrCtl->ppd->columnSet->add( (IContainerColumn*)pFI->pUserData,
                                        (-1), (-1), true );
        }
      }
      else
      {
        int  prevSubItemId(0);
        Boolean  colVisible( true );
        if ( pFI->flData & CFA_BITMAPORICON )
          colVisible = false;

        lvColumn.mask = LVCF_FMT | LVCF_SUBITEM | LVCF_TEXT | LVCF_WIDTH;

        // CCL allows both the title and the column data to have separate
        // alignment styles.  ListView only has one so we will use the
        // data alignment for both.
        if ( pFI->flData & CFA_CENTER )
          lvColumn.fmt      = LVCFMT_CENTER;
        else if ( pFI->flData & CFA_RIGHT )
          lvColumn.fmt      = LVCFMT_RIGHT;
        else    //default is LEFT
          lvColumn.fmt      = LVCFMT_LEFT;

        // ListView does not support icons for column titles.
        // ListView only supports strings for column titles.
        // Use an empty string if style is not string, or if
        // a string is not yet provided at this point.
        if ( (pFI->flTitle & CFA_STRING) && pFI->pTitleData )
        {
          // In native windows, only single line headings supported, so we
          // will strip out any newline characters and replace with blanks.
          IString temp( (char*)pFI->pTitleData );
          temp.change( "\n", " " );
          strcpy((char*)(pFI->pTitleData), temp );
          lvColumn.pszText = (char*)pFI->pTitleData;
        }
        else
          lvColumn.pszText = "";

        // Query for the next available subItemId.
        lvColumn.iSubItem = pCnrCtl->ppd->columnSet->newSubItemId();

        // Specify the column width, otherwise use the default.
        if ( pFI->cxWidth == 0 )
          lvColumn.cx = 100;   //what to use for default?
        else
          lvColumn.cx = pFI->cxWidth;

        // Determine the position to place the new column.
        if ( pFIInsert->pFieldInfoOrder ==(PFIELDINFO)CMA_FIRST )
          insertPosition = 1;
        else if ( pFIInsert->pFieldInfoOrder == (PFIELDINFO)CMA_END )
        {
          insertPosition = pCnrCtl->ppd->columnSet->columnCount();

          //Get The current last items SubItemId
          prevSubItemId = pCnrCtl->ppd->columnSet->lastSubItemId();
        }
        else    //Assume valid column.
        {
          insertPosition = pCnrCtl->ppd->columnSet->columnIdFromFieldInfo(
                                          pFIInsert->pFieldInfoOrder);
          if (insertPosition < 1)
            insertPosition = 1;
          else
            insertPosition++;

          // Update subItem so that it represents the previous subItem.
          prevSubItemId = pCnrCtl->ppd->columnSet->subItemIdFromFieldInfo(
                                         pFIInsert->pFieldInfoOrder);
        }

        lvColumn.iSubItem = insertPosition;

        if ( colVisible )
          columnId = ListView_InsertColumn( pCnrCtl->ppd->pListview->handle(),
                                            insertPosition,
                                            &lvColumn );

        // Add this to the columnSet for this container.
        if ( columnId != (-1) )
          pCnrCtl->ppd->columnSet->add( (IContainerColumn*)pFI->pUserData,
                                        prevSubItemId,
                                        insertPosition,
                                        colVisible);
        else
          retVal = 0;
      }

      pCnrCtl->ppd->cnrInfo.cFields++;
      event.setResult( retVal );     //Total # of cols inserted?
      fStopHandling = true;
    }
    break;

    case CM_INSERTRECORD:
    {
      IMiniCnrRecord*
            pRec((IMiniCnrRecord*)event.parameter1().asUnsignedLong());
      LPRECORDINSERT
           pRecordInsert((LPRECORDINSERT)event.parameter2().asUnsignedLong());
      IContainerObject* pCnrObj(IObjFromRec(pRec));

      // If no icon, then don't add to list and pass index of -1
      int imageIndex = -1;
      if (pRec->hptrIcon)
        imageIndex = pCnrCtl->ppd->addIconToList(pRec->hptrIcon);
      ICnrObjSetInfo  *pCOSInfo;

      //Create the IContainerObject's private data if this
      //object has not been inserted before.
      if ( pCnrObj->ppd == 0 )
        pCnrObj->ppd = new ICnrObjPrivateData( );

      //Check if any objects are already inserted.
      //If none, then give object cursored emphasis.
      //See if there is a better way of accomplishing this.
      Boolean   noObjectsInserted = true;
      if ( ((pCnrCtl->ppd->pListview) &&
            (ListView_GetItemCount(pCnrCtl->ppd->pListview->handle()) == 0)) ||
           ((pCnrCtl->ppd->pTreeview ) &&
            (TreeView_GetCount(pCnrCtl->ppd->pTreeview->handle()) == 0)) )
        pRec->flRecordAttr |= CRA_CURSORED;

      //Check and see if a child is being inserted.
      //If so, then make sure a tree view has been created before
      //we attempt to insert the child object below.
      if ( pRecordInsert->pRecordParent && !pCnrCtl->ppd->pTreeview )
      {
        //Create TreeView and migrate objects.
        unsigned long currentStyle(pCnrCtl->style());
        pCnrCtl->ppd->pTreeview = new INativeContainerControl
                                         ( IC_NATIVE_TREEVIEW,
                                           pCnrCtl->handle(),
                                           pCnrCtl->handle(),
                                           IRectangle(IPoint(0,0),
                                                      pCnrCtl->size()),
                                           pCnrCtl->ppd->convertToNativeStyle(
                                                                currentStyle,
                                                                false ),
                                           false );

        // If a list view exists, populate the tree view from the list view
        if (pCnrCtl->ppd->pListview)
          pCnrCtl->ppd->migrateContainerObjects( true );

        // If the Child Indention has been changed from the default, set it.
        if ( pCnrCtl->ppd->cnrInfo.cxTreeIndent )
          TreeView_SetIndent( pCnrCtl->ppd->pTreeview->handle(),
                              pCnrCtl->ppd->cnrInfo.cxTreeIndent );

      }

      if ( pCnrCtl->ppd->pTreeview )
      {
        TV_ITEM          treeItem;
        TV_INSERTSTRUCT  tvis;
        HTREEITEM        pTreeObj;

        treeItem.mask = TVIF_TEXT | TVIF_IMAGE | TVIF_STATE | TVIF_PARAM | TVIF_SELECTEDIMAGE;
        treeItem.state = 0;
        if ( pRec->flRecordAttr & CRA_SELECTED )
          treeItem.state |= TVIS_SELECTED;
        if ( pRec->flRecordAttr & CRA_CURSORED )       //may not need
          treeItem.state |= TVIS_FOCUSED;
        if ( pRec->flRecordAttr & CRA_TARGET )         //may not need
          treeItem.state |= TVIS_DROPHILITED;
        if ( pRec->flRecordAttr & CRA_EXPANDED )
          treeItem.state |= TVIS_EXPANDED;

        treeItem.stateMask      = 0x00FF;
        treeItem.pszText        = pRec->strIcon;
        treeItem.cchTextMax     = 64;
        treeItem.iImage         = imageIndex;
        treeItem.iSelectedImage = imageIndex;
        treeItem.lParam         = (unsigned long)IObjFromRec(pRec);

        if ( pRecordInsert->pRecordParent )
          //I just realized that the parent & after must be specified
          //for native windows.(unlike CCL) So when afterObject is called
          //the parent might need to be queried.
        {
          IMiniCnrRecord*   pParentRec((IMiniCnrRecord*)
                                             pRecordInsert->pRecordParent);
          IContainerObject* pParentObj(IObjFromRec(pParentRec));

          tvis.hParent = pParentObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl);

          // If parent object not found, put it as a root object.
          if (tvis.hParent == 0)
             tvis.hParent = TVI_ROOT;
        }
        else
          tvis.hParent = TVI_ROOT;


        if ( pRecordInsert->pRecordOrder == (PRECORDCORE)CMA_FIRST )
          tvis.hInsertAfter = TVI_FIRST;
        else if ( pRecordInsert->pRecordOrder == (PRECORDCORE)CMA_END )
        {
          tvis.hInsertAfter = TVI_LAST;
        }
        else if ( pRecordInsert->pRecordOrder != 0 )
        {
          // Insert after the given object.
          IMiniCnrRecord*   pAfterRec((IMiniCnrRecord*)
                                             pRecordInsert->pRecordOrder);
          IContainerObject* pAfterObj(IObjFromRec(pAfterRec));

          tvis.hInsertAfter =
                       pAfterObj->ppd->pCnrSet->treeItemFromObject( pCnrCtl);

          // If insertion after object not found, put it at the end.
          if (tvis.hInsertAfter == 0)
             tvis.hInsertAfter = TVI_LAST;
        }

        // Set the insertion item to the tree item.
        tvis.item = treeItem;

        // Insert the item.
        if ( pTreeObj=TreeView_InsertItem(pCnrCtl->ppd->pTreeview->handle(),
                                          &tvis) )
        {
          pCOSInfo = new ICnrObjSetInfo(pCnrCtl, pTreeObj, imageIndex);
          pCnrObj->ppd->pCnrSet->add( pCOSInfo );

          event.setResult( true );
        }
        else
          event.setResult( false );
      }

      //If a ListView exists, then insert the object there, providing
      //the object is not a child object.
      if ( pCnrCtl->ppd->pListview && !pRecordInsert->pRecordParent )
      {
        if ( !pCnrCtl->ppd->pTreeview )
        {
          pCOSInfo = new ICnrObjSetInfo(pCnrCtl, 0, imageIndex);
          pCnrObj->ppd->pCnrSet->add( pCOSInfo );
        }

        LV_ITEM   item;

        item.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
        item.state = 0;
        if ( pRec->flRecordAttr & CRA_SELECTED )
          item.state |= LVIS_SELECTED;
        if ( pRec->flRecordAttr & CRA_CURSORED )       //may not need
          item.state |= LVIS_FOCUSED;
        if ( pRec->flRecordAttr & CRA_TARGET )         //may not need
          item.state |= LVIS_DROPHILITED;

        item.stateMask = 0x000F;

        if ( pRecordInsert->pRecordOrder == (PRECORDCORE)CMA_FIRST )
          item.iItem = 0;
        else if ( pRecordInsert->pRecordOrder == (PRECORDCORE)CMA_END )
        {
          int  count=ListView_GetItemCount(pCnrCtl->ppd->pListview->handle());
          item.iItem = count;
        }
        else       //maybe check if valid objec?
        {  //Need to insert after pRecordInsert->pRecordOrder
          IMiniCnrRecord*   pAfterRec((IMiniCnrRecord*)
                                             pRecordInsert->pRecordOrder);
          IContainerObject* pAfterObj(IObjFromRec(pAfterRec));
          int           itemIndex;
          LV_FINDINFO   lvFindInfo;
          lvFindInfo.flags  = LVFI_PARAM;
          lvFindInfo.lParam = (LPARAM)pAfterObj;

          // Get the objects list view index.
          itemIndex = IContainerControl::ListViewFindItem(
                                         pCnrCtl->ppd->pListview->handle(),
                                         (-1), &lvFindInfo );
          item.iItem = itemIndex + 1;
        }

        item.iSubItem = 0;  //Note:  Need to add loop for column subitems?
                            //       Loop thru
        item.pszText = pRec->strIcon;
        item.cchTextMax = 64;
        item.iImage = imageIndex;
        item.lParam = (unsigned long)IObjFromRec(pRec);

        if (ListView_InsertItem(pCnrCtl->ppd->pListview->handle(), &item) == -1)
          event.setResult( false );
        else
          event.setResult( true );
      }

      //Make sure we have a valid insert first.
      if ( pRec->flRecordAttr & CRA_RECORDREADONLY )
        pCOSInfo->setObjectState(0, CRA_RECORDREADONLY);
      if ( pRec->flRecordAttr & CRA_DROPONABLE )
        pCOSInfo->setObjectState(0, CRA_DROPONABLE);

      fStopHandling = true;
      break;
    }

    case CM_INVALIDATEDETAILFIELDINFO:
      if ( !pCnrCtl->isTreeView() )
      {
        ListView_RedrawItems( pCnrCtl->ppd->pListview->handle(),
                              0,
                              ListView_GetItemCount(
                                        pCnrCtl->ppd->pListview->handle()) );

        InvalidateRect( pCnrCtl->ppd->pListview->handle(), 0, true );
      }
      event.setResult( true );
      fStopHandling = true;
    break;

    case CM_INVALIDATERECORD:
    {
      /*******************************************************************/
      /* Need to carefully examine this one.  Need to refresh the        */
      /* emphasis states, and position, and generate a repaint.          */
      /*                                                                 */
      /* Only accept invalidation of one object.                         */
      /*******************************************************************/
      IContainerObject* pCnrObj;
      unsigned short    cNumRecord       (event.parameter2().lowNumber());
      unsigned short    fInvalidateRecord(event.parameter2().highNumber());
      IASSERTSTATE( cNumRecord <= 1 );

      if ( cNumRecord )
      {
        //parameter1 is a pointer to a IMiniCnrRecord*
        IMiniCnrRecord*
             pRec(*((IMiniCnrRecord**)event.parameter1().asUnsignedLong()));
        pCnrObj = IObjFromRec(pRec);

        // Add/replace image in list but ONLY if an icon is specified.
        int imageIndex = -1;
        if (pRec->hptrIcon)
          imageIndex = pCnrCtl->ppd->addIconToList(pRec->hptrIcon);
        int  itemIndex=(-1);

        // I do not think the emphasis states need to be updated.
        // We update them immediately, via CM_SETRECORDEMPHASIS.

        if ( pCnrCtl->ppd->pListview )
        {
          //If a ListView exists, then update the icon view position.
          LV_FINDINFO   lvFindInfo;
          lvFindInfo.flags  = LVFI_PARAM;
          lvFindInfo.lParam = (LPARAM)pCnrObj;

          // Get the objects list view index.
          itemIndex = IContainerControl::ListViewFindItem(
                                         pCnrCtl->ppd->pListview->handle(),
                                         (-1), &lvFindInfo );

          if ( itemIndex != (-1) )
          {
            ListView_SetItemPosition32( pCnrCtl->ppd->pListview->handle(),
                                        itemIndex, pRec->ptlIcon.x,
                                        pRec->ptlIcon.y );

            if ( fInvalidateRecord & CMA_TEXTCHANGED )
            {
               // The icon changed and/or the icon text may have changed.
               LV_ITEM lvItem;

               // Prime the lv item
               lvItem.iItem    = itemIndex;
               lvItem.mask     = LVIF_IMAGE | LVIF_TEXT;
               lvItem.iSubItem = 0;
               lvItem.pszText  = pCnrObj->iconText();
               lvItem.iImage   = imageIndex;

               ListView_SetItem( pCnrCtl->ppd->pListview->handle(), &lvItem );
            }

            if ( !pCnrCtl->isTreeView() )
            {
              ListView_RedrawItems( pCnrCtl->ppd->pListview->handle(),
                                    itemIndex, itemIndex);

              UpdateWindow( pCnrCtl->ppd->pListview->handle() );
            }
          }
        }
        if (pCnrCtl->ppd->pTreeview)
        {
          ICnrObjSetInfo* pObjInfo = pCnrObj->ppd->pCnrSet->objectFromCnr( pCnrCtl );
          HTREEITEM treeItem = pObjInfo->pTVItem;

          if (treeItem && (fInvalidateRecord & CMA_TEXTCHANGED))
          {
            // The icon changed and/or the icon text may have changed.
            TV_ITEM tvItem;

            tvItem.hItem = treeItem;
            tvItem.mask  = TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_TEXT;
            tvItem.iImage =
            tvItem.iSelectedImage = imageIndex;
            tvItem.pszText = pCnrObj->iconText();
            tvItem.cchTextMax = 64;

            TreeView_SetItem( pCnrCtl->ppd->pTreeview->handle(), &tvItem );
          }
        }
      }
      else   //All listview objects
      {
        if ( !pCnrCtl->isTreeView() )
        {
          ListView_RedrawItems( pCnrCtl->ppd->pListview->handle(),
                                0,
                                ListView_GetItemCount(
                                          pCnrCtl->ppd->pListview->handle()) );

          InvalidateRect( pCnrCtl->ppd->pListview->handle(), 0, true );
        }
      }

      if ( pCnrCtl->isTreeView() )
        InvalidateRect( pCnrCtl->ppd->pTreeview->handle(), 0, true );

      fStopHandling = true;
      event.setResult( true );
    }
    break;


    case CM_OPENEDIT:
    {
      event.setResult( true );

      /*********************************************************************/
      /* Open an edit field (on a container object only).                  */
      /* Currently limited function.  We assume that a valid object        */
      /* is coming in the CNREDITDATA structure.  No editing columns,      */
      /* titles, col. titles, etc.  right now.                             */
      /*********************************************************************/
      PCNREDITDATA
           pCnrEditData((PCNREDITDATA)event.parameter1().asUnsignedLong());
      IMiniCnrRecord*   pRec((IMiniCnrRecord*)pCnrEditData->pRecord);
      IContainerObject* pCnrObj(IObjFromRec(pRec));

      if (!(pCnrCtl->style() & IContainerControl::readOnly.asUnsignedLong()) &&
          pCnrObj && pCnrObj->isWriteable())
      {
        if ( pCnrCtl->isTreeView() )     //currently in a tree view.
        {
          //Convert the ICnrObj* to an HTREEITEM and start the treeview edit.
          event.setResult( TreeView_EditLabel(
                             pCnrCtl->ppd->pTreeview->handle(),
                             pCnrObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl)));
        }
        else       //currently not in a tree view.
        {
          //Convert the ICnrObj* to an index and start the listview edit.
          int          index;
          LV_FINDINFO  lvFindInfo;

          lvFindInfo.flags  = LVFI_PARAM;
          lvFindInfo.lParam = (unsigned long)pCnrObj;

          index = IContainerControl::ListViewFindItem(
                                    pCnrCtl->ppd->pListview->handle(),
                                    (-1), &lvFindInfo);

          // If we are in details view then ensure that we are not
          // attempting to edit the required column.
          if ( pCnrCtl->isDetailsView() )
          {
            //Start Direct Edit in a Details View Column
            IContainerColumn* pColumn((IContainerColumn*)pCnrEditData->pFieldInfo->pUserData);

            if ((pColumn->pfieldinfoCl->flData & CFA_STRING) &&
                pColumn->isWriteable())
            {
              // If there is already a field open, we have to close it.
              if (pCnrCtl->ppd->pDetailsEdit)                      /* D8067 */
                // Send a fake enter message to the mle so that it will close.
                pCnrCtl->ppd->pDetailsEdit->sendEvent( WM_CHAR,
                                                       IEventParameter1( VK_RETURN ));

              // If we are in details view then ensure that we are not
              // attempting to edit the required column.
              if ( pCnrEditData->pFieldInfo &&
                   (pCnrCtl->ppd->columnSet->columnIdFromFieldInfo(pCnrEditData->pFieldInfo)
                    != 0) )
              {
                // Save the column and container objects so that we can
                // retrieve the edit info when sending the CN_REALLOCPSZ
                // and CN_EDITEDIT notifications.
                pCnrCtl->ppd->pEditColumn = pColumn;
                pCnrCtl->ppd->pEditObject = pCnrObj;

                LPTSTR *pColumnText(((LPTSTR*)(((LPBYTE)pRec) + pColumn->pfieldinfoCl->offStruct)));

                IRectangle detailsRect( pCnrCtl->detailsObjectRectangle( pCnrObj,
                                                                         pColumn ));

                IFont listviewFont( pCnrCtl->ppd->pListview->font() );

                detailsRect.sizeTo(ISize((listviewFont.textWidth( *pColumnText )*1.5),
                                         detailsRect.height()));

                detailsRect.shrinkBy( IPair(0, (detailsRect.height()-
                                 (listviewFont.maxCharHeight()+8))/2 ));

                // Shift edit control right by approx margin if left-justified.
                if (pColumn->pfieldinfoCl->flData & CFA_LEFT)
                  detailsRect.moveBy( IPair( listviewFont.textWidth( "xi" ), 0 ));

                pCnrCtl->ppd->pDetailsEdit = new IMultiLineEdit( CID_MLE,
                                                   pCnrCtl,
                                                   pCnrCtl,
                                                   detailsRect,
                                                   IWindow::visible | IMultiLineEdit::border );

                pCnrCtl->ppd->pDetailsEdit->setAutoDeleteObject( true );

                pCnrCtl->ppd->pDetailsEdit->setText( *pColumnText ).
                // select all the text in the mle
                              selectRange().
                // Set the font of the edit window to use the font of the list view control.
                              setFont( listviewFont );

                IDetailsEditHandler::defaultHandler()->handleEventsFor(
                                                        pCnrCtl->ppd->pDetailsEdit);

                CNREDITDATA cnrEditData;
                cnrEditData.cb         = sizeof( CNREDITDATA );
                cnrEditData.hwndCnr    = pCnrCtl->handle();
                cnrEditData.pRecord    = (RECORDCORE*)pRec;
                cnrEditData.pFieldInfo = pColumn->pfieldinfoCl;
                cnrEditData.ppszText   = pColumnText;
                cnrEditData.cbText     = strlen( *pColumnText );
                cnrEditData.id         = pCnrCtl->id();

                pCnrCtl->owner()->sendEvent( WM_CONTROL,
                                        IEventParameter1( cnrEditData.id, CN_BEGINEDIT ),
                                        IEventParameter2( &cnrEditData ) );

                pCnrCtl->ppd->pDetailsEdit->setFocus();
              }
              else
              {
                event.setResult( ListView_EditLabel(pCnrCtl->ppd->pListview->handle(),
                                                   index) );
              }
            }
          }
          else
          {
            event.setResult( ListView_EditLabel(pCnrCtl->ppd->pListview->handle(),
                                               index) );
          }
        }
      }
      fStopHandling = true;
    }
    break;

    case CM_PAINTBACKGROUND:
      /********************************************************************/
      /* This message is generated by the CCL control.  The CCL container */
      /* sends this message to itself, thereby allowing the owner to      */
      /* subclass and process this.  There is no equivalent sent by either*/
      /* the native ListView or TreeView, thus handleDrawBackground will  */
      /* never be called.                                                 */
      /********************************************************************/
    break;

    case CM_QUERYCNRINFO:
      /********************************************************************/
      /* Query all the attributes of the container and insert them into   */
      /* the ICnrInfo structure.                                          */
      /********************************************************************/
      ((IContainerControl*)event.window())->ppd->queryCnrInfo(
                            (ICnrInfo*)event.parameter1().asUnsignedLong() );
      event.setResult( true );
      fStopHandling = true;
    break;

    case CM_QUERYDETAILFIELDINFO:
    {
      /********************************************************************/
      /* Query details attributes of the container.  Not all CMA_xx flags */
      /* are supported.                                                   */
      /********************************************************************/
      LPFIELDINFO
            pFI((LPFIELDINFO)event.parameter1().asUnsignedLong());
      unsigned short ulCmd(event.parameter2().lowNumber());

      // Only respond to this message if a list view exists.
      if ( pCnrCtl->ppd->pListview )
      {
        switch(ulCmd)
        {
          case CMA_FIRST:
            event.setResult( pCnrCtl->ppd->columnSet->firstFieldInfo() );
          break;

          case CMA_LAST:
            event.setResult( pCnrCtl->ppd->columnSet->lastFieldInfo() );
          break;

          case CMA_NEXT:
            event.setResult( pCnrCtl->ppd->columnSet->nextFieldInfo(pFI) );
          break;

          case CMA_PREV:
            event.setResult( pCnrCtl->ppd->columnSet->prevFieldInfo(pFI) );
          break;

          case CMA_DATAWIDTH:
          {
            int subItem( pCnrCtl->ppd->columnSet->subItemIdFromFieldInfo(pFI) );
            if ( subItem > (-10) )
            {
              int colIndex =
                       pCnrCtl->ppd->columnSet->columnIdFromFieldInfo(pFI);

              if ( (colIndex==(-1)) &&
                   (pCnrCtl->ppd->columnSet->containerColumnFromSubId(0)) )
                event.setResult( 0 );
              else
              {
                if ( colIndex==(-1) )
                  colIndex = 0;
                event.setResult( ListView_GetColumnWidth(
                                 pCnrCtl->ppd->pListview->handle(), colIndex) );
              }
            }
            else
              event.setResult( 0 );
          }
          break;

          default:
            event.setResult( (LPFIELDINFO)(-1) );
          break;
        }
      }
      else
        event.setResult( 0 );

      fStopHandling = true;
    }
    break;

#ifdef NOT_SUPPORTED
    case CM_QUERYDRAGIMAGE:
      // Talk with Jim about this one...
    break;
#endif

    case CM_QUERYRECORD:
    {
      /********************************************************************/
      /* Return the requested record according to the passed in CMA_xx    */
      /* flag.                                                            */
      /* For the purposes of the IContainerControl class, the only        */
      /* implementation necessary is for the ObjectCursor case.  Other    */
      /* uses may fail.                                                   */
      /********************************************************************/
      unsigned short ulCmd(event.parameter2().lowNumber());

      if ( pCnrCtl->ppd->pTreeview )
      {
        TV_ITEM    item;
        HTREEITEM  pTreeItem, pNextItem;
        IContainerObject*  pObj=0;

        IMiniCnrRecord* pRec((IMiniCnrRecord*)event.parameter1().asUnsignedLong());
        if ( pRec )
          pObj = IObjFromRec( pRec );
        //Need to double check if pObj=0 what it means below

        switch(ulCmd)
        {
          case CMA_FIRST:
            pTreeItem =  TreeView_GetRoot(pCnrCtl->ppd->pTreeview->handle());
          break;

          case CMA_FIRSTCHILD:
            pTreeItem = TreeView_GetChild(pCnrCtl->ppd->pTreeview->handle(),
                            pObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) );
          break;

          case CMA_LAST:
          {
            pNextItem =
            pTreeItem =  TreeView_GetRoot(pCnrCtl->ppd->pTreeview->handle());

            while (pNextItem)
            {
              pTreeItem = pNextItem;
              pNextItem = TreeView_GetNextSibling(pCnrCtl->ppd->pTreeview->handle(),
                                                  pNextItem );
            }
          }
          break;

          case CMA_LASTCHILD:
          {
            //Check for pObj=0, then same as CMA_LAST
            if ( pObj )
              pNextItem =
              pTreeItem = TreeView_GetChild(pCnrCtl->ppd->pTreeview->handle(),
                              pObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) );
            else
              pNextItem =
              pTreeItem =  TreeView_GetRoot(pCnrCtl->ppd->pTreeview->handle());

            while (pNextItem)
            {
              pTreeItem = pNextItem;
              pNextItem = TreeView_GetNextSibling(pCnrCtl->ppd->pTreeview->handle(),
                                                  pNextItem );
            }
          }
          break;

          case CMA_NEXT:
            pTreeItem = TreeView_GetNextSibling(pCnrCtl->ppd->pTreeview->handle(),
                            pObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) );
          break;

          case CMA_PARENT:
            pTreeItem = TreeView_GetParent(pCnrCtl->ppd->pTreeview->handle(),
                            pObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) );
          break;

          case CMA_PREV:
            pTreeItem = TreeView_GetPrevSibling(pCnrCtl->ppd->pTreeview->handle(),
                            pObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) );
          break;

          default:
            event.setResult( -1 );
            fStopHandling=true;
          break;
        }

        if (!fStopHandling)
        {
          if ( pTreeItem )
          {
            item.mask =  TVIF_PARAM;
            item.hItem = pTreeItem;

            TreeView_GetItem(pCnrCtl->ppd->pTreeview->handle(), &item);
            fStopHandling = true;
            event.setResult( (IMiniCnrRecord*)IRecFromObj(
                                              (IContainerObject*)item.lParam) );
          }
          else
            event.setResult( 0 );
        }
      }
      else  //consider ListView only
            //currently only considers being called by a cursor
      {
        LV_ITEM   item;
        item.iSubItem = 0;
        IContainerControl::ObjectCursor* pObjCur((IContainerControl::ObjectCursor*)
                                           event.parameter1().asUnsignedLong());

        switch(ulCmd)
        {
          case CMA_FIRST:
            item.iItem=0;
          break;

          case CMA_LAST:
            item.iItem = ListView_GetItemCount(
                                       pCnrCtl->ppd->pListview->handle());
          break;

          case CMA_NEXT:
            item.iItem= pObjCur->currentIndex + 1;
          break;

          case CMA_PREV:
            item.iItem = pObjCur->currentIndex -1;     //check for 0
          break;

          case CMA_FIRSTCHILD:
          case CMA_LASTCHILD:
          case CMA_PARENT:
            event.setResult( 0 );
            fStopHandling=true;
          break;

          default:
            event.setResult( -1 );
            fStopHandling=true;
          break;
        }
                /*   check this logic for all cases above */
        if (!fStopHandling)
        {
          item.mask = LVIF_PARAM;
          if ( ListView_GetItem(pCnrCtl->ppd->pListview->handle(), &item) )
          {
            pObjCur->currentIndex = item.iItem;
            event.setResult( (IMiniCnrRecord*)IRecFromObj(
                                              (IContainerObject*)item.lParam) );
          }
          else
            event.setResult( 0 );
        }
      }
      fStopHandling = true;
    }
    break;

    case CM_QUERYRECORDEMPHASIS:
    {
      /*******************************************************************/
      /* Query the object's current emphasis states for this container.  */
      /* This is used by the cursor classes.                             */
      /*******************************************************************/
      unsigned long      ulEmphasis(event.parameter2().asUnsignedLong());
      IMiniCnrRecord*    pStartRec((IMiniCnrRecord*)event.parameter1().
                                                    asUnsignedLong());
      IContainerObject*  pStartObj=0;

      //Whoa.....it could be CMA_FIRST

      if ( pStartRec  && ( pStartRec != (IMiniCnrRecord*)CMA_FIRST ) )
        pStartObj = IObjFromRec( pStartRec );

      //flag something for cma_first -> just let pStartObj=0?
      //Need to double check if pStartObj=0 what it means below

      if ( pCnrCtl->isTreeView() )
      {
        TV_ITEM    item;
        HTREEITEM  pTreeItem;

        if ( ulEmphasis & CRA_INUSE )
        {
          if ( ulEmphasis & ( CRA_SELECTED | CRA_CURSORED ) )
          {
            pTreeItem =TreeView_GetSelection(pCnrCtl->ppd->pTreeview->handle());
            if ( pStartObj &&
                 (pStartObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) ==
                  pTreeItem) )
              event.setResult( 0 );
            else
            {
              item.mask =  TVIF_PARAM;
              item.hItem = pTreeItem;

              if (TreeView_GetItem(pCnrCtl->ppd->pTreeview->handle(), &item))
                        //what about error=-1  verses  possible good value=0
              {
                IContainerObject*  pCnrObj;
                pCnrObj = (IContainerObject*)(item.lParam);
                if ( pCnrObj->ppd->pCnrSet->isInUse(pCnrCtl) )
                  event.setResult( pCnrObj );
                //Marty......should change InUse to inuse?
              }
            }
          }
          else   //Inuse only....Could also be garbage though.
          {
            IContainerObject*                pCnrObj;
            IContainerControl::ObjectCursor  cursor(*pCnrCtl);

            if ( pStartObj )
              cursor.setCurrent( pStartObj );

            //else...do Nothing. Thus when setToNext() is called in
            //the cursor below, it will actually go to first(CMA_FIRST)

            event.setResult( false );

            //Cursor thru and query inuse for all objects in the tree view.
            //Need to stop cursor.
            for( cursor.setToNext(); cursor.isValid(); cursor.setToNext() )
            {
              pCnrObj = pCnrCtl->objectAt(cursor);
              if ( pCnrObj->ppd->pCnrSet->isInUse(pCnrCtl) )
              {
                event.setResult( pCnrObj );
                cursor.invalidate();   //must now stop cursor
              }
            }
          }
        }
        else    //No Inuse emphasis to worry about.
        {
          if ( ulEmphasis & ( CRA_SELECTED | CRA_CURSORED ) )
          {
            pTreeItem =TreeView_GetSelection(pCnrCtl->ppd->pTreeview->handle());
            if (pStartObj && ( pStartObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) ==
                          pTreeItem ) )
              event.setResult( 0 );
            else
            {
              item.mask =  TVIF_PARAM;
              item.hItem = pTreeItem;

              if (TreeView_GetItem(pCnrCtl->ppd->pTreeview->handle(), &item))
                        //what about error=-1  verses  possible good value=0
                event.setResult((IMiniCnrRecord*)IRecFromObj((IContainerObject*)item.lParam));
            }
          }
          //ulEmphasis must be invalid.
          else
            event.setResult( -1 );
        }
      }
      else  //consider ListView only
            //what about pStartObj being CMA_FIRST?
      {
        LV_ITEM item;
        int     index=(-1);
        UINT    flags=0;

        //get current index
        if (pStartObj)
        {
          LV_FINDINFO  lvFindInfo;

          lvFindInfo.flags  = LVFI_PARAM;
          lvFindInfo.lParam = (unsigned long)pStartObj;

          index = IContainerControl::ListViewFindItem(
                                    pCnrCtl->ppd->pListview->handle(),
                                    (-1), &lvFindInfo);
        }
        if ( ulEmphasis & CRA_INUSE )
        {
          if ( ulEmphasis & ( CRA_SELECTED | CRA_CURSORED ) )
          {
            IContainerObject*  pCnrObj;

            if (ulEmphasis & CRA_SELECTED)
              flags |= LVNI_SELECTED;

            if (ulEmphasis & CRA_CURSORED)
              flags |= LVNI_FOCUSED;

            while ( index != (-1) )
            {
              index = ListView_GetNextItem(pCnrCtl->ppd->pListview->handle(),
                                           index, flags );

              if ( index != (-1) )
              {
                item.mask =  LVIF_PARAM;
                item.iItem = index;
                item.iSubItem = 0;
                ListView_GetItem(pCnrCtl->ppd->pListview->handle(), &item);

                pCnrObj = (IContainerObject*)(item.lParam);
                if ( pCnrObj->ppd->pCnrSet->isInUse(pCnrCtl) )
                {
                  index = (-1);
                  event.setResult( IRecFromObj(pCnrObj) );
                }
              }
              else
                event.setResult( 0 );
            }
          }
          else   //Inuse only.
          {
            IContainerObject*   pCnrObj;
            Boolean  notFound = true;
            int  numListItems( ListView_GetItemCount(
                                         pCnrCtl->ppd->pListview->handle()) );

            for (int iter=index+1; iter < numListItems; iter++)
            {
              item.mask =  LVIF_PARAM;
              item.iItem = iter;
              item.iSubItem = 0;
              ListView_GetItem(pCnrCtl->ppd->pListview->handle(), &item);
              pCnrObj = (IContainerObject*)(item.lParam);

              if ( pCnrObj->ppd->pCnrSet->isInUse(pCnrCtl) )
              {
                notFound = false;
                iter     = numListItems;
                event.setResult( IRecFromObj(pCnrObj) );
              }
            }

            if ( notFound )
              event.setResult( 0 );
          }
        }
        else    //No inuse emphasis to worry about.
        {
          if (ulEmphasis & CRA_SELECTED)
            flags |= LVNI_SELECTED;

          if (ulEmphasis & CRA_CURSORED)
            flags |= LVNI_FOCUSED;

          item.iItem = ListView_GetNextItem(pCnrCtl->ppd->pListview->handle(),
                                            index, flags );

          if ( item.iItem != (-1) )
          {
            item.mask =  LVIF_PARAM;
            item.iSubItem = 0;
            ListView_GetItem(pCnrCtl->ppd->pListview->handle(), &item);

            event.setResult( IRecFromObj((IContainerObject*)item.lParam) );
          }
          else
            event.setResult( 0 );
        }
      }
      fStopHandling = true;
    }
    break;

    case CM_QUERYRECORDFROMRECT:
    {
      /*******************************************************************/
      /* Get the object intersecting the given rectangle.                */
      /*                                                                 */
      /* Only what IContainerControl requires has been implemented.      */
      /* We only allow a rectangle of a point to determine the           */
      /* object under that point.                                        */
      /* If the starting point is anything other than CMA_FIRST, we      */
      /* will return 0.                                                  */
      /*******************************************************************/
      IMiniCnrRecord*
          pSearchAfter( (IMiniCnrRecord*)event.parameter1().asUnsignedLong() );
      LPQUERYRECFROMRECT
       pQryRecFromRect((LPQUERYRECFROMRECT)event.parameter2().asUnsignedLong());

      if ( pSearchAfter == (IMiniCnrRecord*)CMA_FIRST )
      {
        if ( pCnrCtl->isTreeView() )
        {
          HTREEITEM        treeHandle;
          TV_HITTESTINFO   treeHitTestInfo;

          treeHitTestInfo.pt.x  = pQryRecFromRect->rect.right;
          treeHitTestInfo.pt.y  = pQryRecFromRect->rect.bottom;
          treeHitTestInfo.flags = TVHT_ONITEM | TVHT_ONITEMICON |
                                  TVHT_ONITEMLABEL;
          treeHandle = TreeView_HitTest( pCnrCtl->ppd->pTreeview->handle(),
                                         &treeHitTestInfo );
          if ( treeHandle )
          {
            TV_ITEM    treeItem;

            treeItem.hItem = treeHandle;
            treeItem.mask  = TVIF_PARAM;

            TreeView_GetItem( pCnrCtl->ppd->pTreeview->handle(), &treeItem );
            event.setResult( IRecFromObj( (IContainerObject*)treeItem.lParam) );
          }
          else
            event.setResult( 0 );
        }
        else    //ListView
        {
          int              itemIndex;
          LV_HITTESTINFO   listHitTestInfo;

          listHitTestInfo.pt.x  = pQryRecFromRect->rect.right;
          listHitTestInfo.pt.y  = pQryRecFromRect->rect.bottom;
          listHitTestInfo.flags = LVHT_ONITEMICON | LVHT_ONITEMLABEL |
                                  LVHT_ONITEMSTATEICON;
          itemIndex = ListView_HitTest( pCnrCtl->ppd->pListview->handle(),
                                        &listHitTestInfo );
          if ( itemIndex != (-1) )
          {
            LV_ITEM    listItem;

            listItem.iItem    = itemIndex;
            listItem.iSubItem = 0;
            listItem.mask     = LVIF_PARAM;

            ListView_GetItem( pCnrCtl->ppd->pListview->handle(), &listItem );
            event.setResult( IRecFromObj( (IContainerObject*)listItem.lParam) );
          }
          else
          {
            if ( pCnrCtl->isDetailsView() &&
                 ListView_GetItemCount(pCnrCtl->ppd->pListview->handle()) > 0  )
            {
              //Check to see if point is over anothercolumn.
              //HitTest only checks the first column.

              POINT viewPortOffset;
              int      index( 0 );
              Boolean  notDone( true );
              long   leftSide( 0 ),
                     rightSide( 0 );

              ListView_GetItemPosition( pCnrCtl->ppd->pListview->handle(),
                                        0, &viewPortOffset );
              leftSide += viewPortOffset.x;


              //Check and see if this point fits vertically in any of
              //the details view rows.
              itemIndex = (-1);
              int numListItems(ListView_GetItemCount(
                                          pCnrCtl->ppd->pListview->handle()));
              RECTL  rect;
              for (index=0; index < numListItems; index++)
              {
                ListView_GetItemRect( pCnrCtl->ppd->pListview->handle(),
                                      index, &rect, LVIR_BOUNDS );
                if ( (rect.top <= listHitTestInfo.pt.y) &&
                     (listHitTestInfo.pt.y <= rect.bottom) )
                {
                  itemIndex = index;
                  index = numListItems;
                }
              }

              event.setResult( 0 );

              if ( itemIndex != (-1) )
              {
                // We have a potential candidate lets see if the given
                // x value falls within the other columns.

                int  numColumns( pCnrCtl->ppd->columnSet->columnCount() );
                for (index=0; index < numColumns; index++)
                {
                  rightSide = leftSide + ListView_GetColumnWidth(
                                    pCnrCtl->ppd->pListview->handle(), index );
                  if ( rightSide > leftSide )
                  {
                    if ( (leftSide <= pQryRecFromRect->rect.right) &&
                         (pQryRecFromRect->rect.right <= rightSide ) )
                    {
                      LV_ITEM    listItem;

                      listItem.iItem    = itemIndex;
                      listItem.iSubItem = 0;
                      listItem.mask     = LVIF_PARAM;

                      ListView_GetItem( pCnrCtl->ppd->pListview->handle(),
                                        &listItem );
                      event.setResult( IRecFromObj(
                                        (IContainerObject*)listItem.lParam) );
                      index = numColumns;
                    }
                    leftSide = rightSide;
                  }
                }
              }
            }
            else
              event.setResult( 0 );
          }
        }
      }
      else
        event.setResult( 0 );

      fStopHandling = true;
    }
    break;

    case CM_QUERYRECORDINFO:
    {
      /*******************************************************************/
      /* Update the objects flRecordAttr and ptlIcon fields.             */
      /*                                                                 */
      /* Only what IContainerControl requires has been implemented.      */
      /* We only allow cNumRecord = 1.  An exception is thrown for       */
      /* all other values.                                               */
      /*                                                                 */
      /* What about filtered, source, and droponable?                    */
      /*******************************************************************/
      //parameter1 is a pointer to a IMiniCnrRecord*
      IMiniCnrRecord*
           pRec(*((IMiniCnrRecord**)event.parameter1().asUnsignedLong()));
      unsigned short    cNumRecord   (event.parameter2().lowNumber());
      IContainerObject* pObj;
      TV_ITEM       treeItem;
      LV_FINDINFO   lvFindInfo;
      int           itemIndex;
      Boolean       existsInTree = false;

      IASSERTSTATE( cNumRecord = 1 );
      IASSERTSTATE( pRec != 0 );
      pRec->flRecordAttr = 0;
      pRec->ptlIcon.x = pRec->ptlIcon.y = 0;
      pObj = IObjFromRec( pRec );

      // If a ListView control exists, then get the items icon view
      // position there even if currently in a non-tree view.
      // Otherwise, let it default to (0,0)
      if ( pCnrCtl->ppd->pListview )
      {
        lvFindInfo.flags  = LVFI_PARAM;
        lvFindInfo.lParam = (LPARAM)pObj;

        // Get the objects list view index.
        itemIndex = IContainerControl::ListViewFindItem(
                                       pCnrCtl->ppd->pListview->handle(),
                                       (-1), &lvFindInfo );

        if ( itemIndex != (-1) )
          ListView_GetItemPosition( pCnrCtl->ppd->pListview->handle(),
                                    itemIndex, &(pRec->ptlIcon) );
      }

      // If a TreeView control exists, then check for collapsed/expanded
      // there even if currently in a non-tree view.
      if ( pCnrCtl->ppd->pTreeview )
      {
        treeItem.mask      = TVIF_STATE;
        treeItem.stateMask = 0x00FF;
        treeItem.hItem     = pObj->ppd->pCnrSet->treeItemFromObject( pCnrCtl );

        existsInTree = TreeView_GetItem( pCnrCtl->ppd->pTreeview->handle(),
                                         &treeItem );
        if ( existsInTree )
        {
          if ( treeItem.state & TVIS_EXPANDED )
            pRec->flRecordAttr |= CRA_EXPANDED;
          else
            pRec->flRecordAttr |= CRA_COLLAPSED;
        }
      }
      else  //Default to collapsed, as CCL does.
        pRec->flRecordAttr |= CRA_COLLAPSED;

      // Check the object's private data for inUse emphasis.
      if ( pObj->ppd->pCnrSet->isInUse(pCnrCtl) )
        pRec->flRecordAttr |= CRA_INUSE;

      // Check the object's private data for readonly attribute.
      if ( pObj->ppd->pCnrSet->isReadOnly(pCnrCtl) )
        pRec->flRecordAttr |= CRA_RECORDREADONLY;

      // Check the object's private data for droponable attribute.
      if ( pObj->ppd->pCnrSet->isDropOnable(pCnrCtl) )
        pRec->flRecordAttr |= CRA_DROPONABLE;

      // Check the current view for the emphasis attributes.
      if ( pCnrCtl->isTreeView() )
      {
        if ( existsInTree )
        {
          if ( treeItem.state & TVIS_FOCUSED )
            pRec->flRecordAttr |= CRA_CURSORED;
          if ( treeItem.state & TVIS_SELECTED )
            pRec->flRecordAttr |= CRA_SELECTED;
          if ( treeItem.state & TVIS_DROPHILITED )
            pRec->flRecordAttr |= CRA_TARGET;
        }
        else
        {
          //Couldn't find the object in the TreeView.
          //Thus it must not be in the ListView either b/c all
          //objects in the ListView must be in the TreeView.
          fStopHandling = true;
        }
      }
      else       //listview
      {
        unsigned int  itemState(0);

        if ( itemIndex != (-1) )
        {
          itemState = ListView_GetItemState( pCnrCtl->ppd->pListview->handle(),
                                             itemIndex, 0x000F );

          if ( itemState & LVIS_FOCUSED )
            pRec->flRecordAttr |= CRA_CURSORED;
          if ( itemState & LVIS_SELECTED )
            pRec->flRecordAttr |= CRA_SELECTED;
          if ( itemState & LVIS_DROPHILITED )
            pRec->flRecordAttr |= CRA_TARGET;

          event.setResult(true );
        }
        else
          //Couldn't find the object in the ListView.
          //Its possible it is a child object in the TreeView.
          if ( !existsInTree )
            fStopHandling = true;
      }

      // fStopHandling = true -> Object doesn't exist in this container.
      // Set the event result appropriately.
      event.setResult( !fStopHandling );
      fStopHandling = true;
    }
    break;

    case CM_QUERYRECORDRECT:
    {
      /*******************************************************************/
      /* Get the rectangle for the given object.                         */
      /*                                                                 */
      /*******************************************************************/
      IContainerObject*   pObj;
      LPQUERYRECORDRECT
       pQryRecordRect((LPQUERYRECORDRECT)event.parameter2().asUnsignedLong());
      PRECTL prclItem( (PRECTL)event.parameter1().asUnsignedLong() );
      Boolean    fSuccess;

      IASSERTSTATE( pQryRecordRect->pRecord != 0 );
      pObj = IObjFromRec( (IMiniCnrRecord*)pQryRecordRect->pRecord );


      if ( pCnrCtl->isTreeView() )
      {
        HTREEITEM  treeHandle = pObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl);
        fSuccess = TreeView_GetItemRect( pCnrCtl->ppd->pTreeview->handle(),
                                         treeHandle, prclItem, true );

        if ( fSuccess && (pQryRecordRect->fsExtent & CMA_ICON) )
        {
          Boolean  notDone( true );
          int      index;
          TV_HITTESTINFO  hitTestInfo;

          hitTestInfo.pt.x = prclItem->left;
          hitTestInfo.pt.y = (prclItem->top + prclItem->bottom) / 2;
          for ( index=1; notDone; index++ )
          {
            hitTestInfo.pt.x--;
            TreeView_HitTest(pCnrCtl->ppd->pTreeview->handle(), &hitTestInfo);
            if (hitTestInfo.flags == TVHT_ONITEMICON)
            {
              while( hitTestInfo.flags == TVHT_ONITEMICON )
              {
                hitTestInfo.pt.x--;
                TreeView_HitTest(pCnrCtl->ppd->pTreeview->handle(),
                                 &hitTestInfo);
              }
              notDone=false;
            }
            else if ( index = 500 )
            {
              notDone=false;
              fSuccess=false;
            }
          }

          if ( fSuccess )
          {
            if (!(pQryRecordRect->fsExtent & CMA_TEXT))
            {
              prclItem->right = prclItem->left - index;
            }
            prclItem->left = hitTestInfo.pt.x +1;
          }
        }
      }
      else    //listView
      {
        int          index=0;
        int          code=0;

        LV_FINDINFO  lvFindInfo;

        lvFindInfo.flags  = LVFI_PARAM;
        lvFindInfo.lParam = (unsigned long)pObj;

        index = IContainerControl::ListViewFindItem(
                                   pCnrCtl->ppd->pListview->handle(),
                                   (-1), &lvFindInfo);

        if ( (pQryRecordRect->fsExtent & CMA_TEXT)  &&
             (pQryRecordRect->fsExtent & CMA_ICON)  )
          code = LVIR_BOUNDS;
        if ( (pQryRecordRect->fsExtent & CMA_TEXT)  &&
             !(pQryRecordRect->fsExtent & CMA_ICON)  )
          code = LVIR_LABEL;
        if ( !(pQryRecordRect->fsExtent & CMA_TEXT)  &&
             (pQryRecordRect->fsExtent & CMA_ICON)  )
          code = LVIR_ICON;

        fSuccess = ListView_GetItemRect( pCnrCtl->ppd->pListview->handle(),
                                         index, prclItem, code );
      }

      // Adjust for possible title window.
      if ( fSuccess && pCnrCtl->isTitleVisible() )
      {
        IWindow *nCnr;
        if (pCnrCtl->isTreeView() )
          nCnr = pCnrCtl->ppd->pTreeview;
        else
          nCnr = pCnrCtl->ppd->pListview;

        POINTL newOrigin;
        newOrigin.x = 0;
        newOrigin.y = 0;

        IMAPWINDOWPOINTS( nCnr->handle(), pCnrCtl->handle(), &newOrigin, 1 );

        prclItem->top    += newOrigin.y;
        prclItem->bottom += newOrigin.y;
      }

      event.setResult( fSuccess );
      fStopHandling = true;
    }
    break;

    case CM_QUERYVIEWPORTRECT:
    {
      /*******************************************************************/
      /* Query the current viewport rectangle.                           */
      /*                                                                 */
      /* This is only implemented for the viewport in window coordinates */
      /* I know of no way to calculate the viewport in workspace coords. */
      /* The fRightSplitWindow flag is ignored.                          */
      /*******************************************************************/
      BOOL      retVal;
      PRECTL    pCnrRect( (PRECTL)event.parameter1().asUnsignedLong() );
      unsigned short  usIndicator(event.parameter2().lowNumber());
      LPRECT    pClientRect;


      pCnrRect->left   = 0;
      pCnrRect->right  = 0;
      pCnrRect->top    = 0;
      pCnrRect->bottom = 0;

      ISize  viewSize;
      if ( pCnrCtl->isTreeView() )
        viewSize = pCnrCtl->ppd->pTreeview->size();
      else
        viewSize = pCnrCtl->ppd->pListview->size();

      pCnrRect->right  = viewSize.width();
      pCnrRect->bottom = viewSize.height();

      // Adjust for Possible title window.
      if ( pCnrCtl->isTitleVisible() )
      {
        IWindow *nCnr;
        if (pCnrCtl->isTreeView() )
          nCnr = pCnrCtl->ppd->pTreeview;
        else
          nCnr = pCnrCtl->ppd->pListview;

        POINTL newOrigin;
        newOrigin.x = 0;
        newOrigin.y = 0;

        IMAPWINDOWPOINTS( nCnr->handle(), pCnrCtl->handle(), &newOrigin, 1 );

        pCnrRect->top    += newOrigin.y;
        pCnrRect->bottom += newOrigin.y;
      }

#if 0
      //ppb fix this below
      //there needs to be an object for this to work.
      POINT pt;
      RECT  rect;
  //      ListView_GetItemPosition( pCnrCtl->ppd->pListview->handle(),
  //                                1, &pt );
  //      ListView_GetItemRect( pCnrCtl->ppd->pListview->handle(),
  //                            1, &rect, LVIR_BOUNDS );

      IASSERTSTATE(  (usIndicator == CMA_WINDOW) ||
                     (usIndicator == CMA_WORKSPACE) );

      retVal = GetClientRect( pCnrCtl->ppd->pTreeview->handle(), pClientRect);

      // Need to adjust for possible title?, details view titles?
      // What about possible scrollBars?

      pCnrRect->left   = pClientRect->left;
      pCnrRect->right  = pClientRect->right;
      pCnrRect->top    = pClientRect->top;
      pCnrRect->bottom = pClientRect->bottom;

      if ( (usIndicator == CMA_WORKSPACE) &&    //ONLY WORKS FOR ICON VIEW
           pCnrCtl->isIconView() )
      {
        LPRECT    pWorkspaceRect;

        ListView_GetViewRect( pCnrCtl->ppd->pListview->handle(),
                              pWorkspaceRect );

        pCnrRect->left   -= pWorkspaceRect->left;
        pCnrRect->right  -= pWorkspaceRect->left;
        pCnrRect->top    -= pWorkspaceRect->top;
        pCnrRect->bottom -= pWorkspaceRect->top;
      }
#endif

      event.setResult( true );
      fStopHandling = true;
    }
    break;

    case CM_REMOVEDETAILFIELDINFO:
    {
      /*******************************************************************/
      /* Remove the column from the ListView.                            */
      /*                                                                 */
      /* Only what IContainerControl requires has been implemented.      */
      /* We are only considering 2 possibilities:                        */
      /*   1) cNumFieldInfo = 0 -> All columns being removed.            */
      /*   2) cNumFieldInfo = 1 -> Just one being removed.               */
      /*******************************************************************/
      Boolean            retVal( true );
      IContainerColumn*  pCnrColumn;
      unsigned short     cNumFieldInfo   (event.parameter2().lowNumber());

      IASSERTSTATE( cNumFieldInfo <= 1 );

      // ListView must exist if we are to remove columns.
      if (pCnrCtl->ppd->pListview)
      {
        if ( cNumFieldInfo )   //Assume just one.
        {
          //parameter1 is a pointer to a PFIELDINFO
          PFIELDINFO pFI(*((PFIELDINFO*)event.parameter1().asUnsignedLong()));
          pCnrColumn = (IContainerColumn*)pFI->pUserData;
          int  colId( pCnrCtl->ppd->columnSet->columnIdFromContainerColumn(
                                                                  pCnrColumn) );

          if ( colId >= (-1) )    // (-2) is invalid.
          {
            // Remove it from the list, which also does ListView_RemoveItem.
            pCnrCtl->ppd->columnSet->remove( pCnrColumn );
            ListView_RedrawItems( pCnrCtl->ppd->pListview->handle(), 0,
                                  ListView_GetItemCount(
                                      pCnrCtl->ppd->pListview->handle() )-1 );
            InvalidateRect( pCnrCtl->ppd->pListview->handle(), 0, true );
            pCnrCtl->ppd->cnrInfo.cFields--;
          }
          else
            retVal = false;
        }
        else    //Remove and Free All Columns
        {
          int colCount( pCnrCtl->ppd->columnSet->columnCount() );

          //take care of icon column here before loop.

          for ( int colId = 0; colId < colCount; colId++ )
          {

            pCnrColumn = pCnrCtl->ppd->columnSet->containerColumnFromColumnId(
                                                                         colId);

            if ( pCnrColumn )
              pCnrCtl->ppd->columnSet->remove( pCnrColumn );
          }
          pCnrCtl->ppd->cnrInfo.cFields = 0;
        }
      }

      event.setResult( retVal );
      fStopHandling = true;
    }
    break;

    case CM_REMOVERECORD:
    {
      /*******************************************************************/
      /* Remove the items from the TreeView and/or the ListView.         */
      /*                                                                 */
      /* Only what IContainerControl requires has been implemented.      */
      /* We are only considering 2 possibilities:                        */
      /*   1) cNumRecord = 0 -> All records being removed.               */
      /*   2) cNumRecord = 1 -> Just one being removed.                  */
      /* We do not accept CMA_FREE, or cNumRecord > 1, as this           */
      /* is all the class library requires.                              */
      /*                                                                 */
      /* NOTE: Actually, deleteAllObjects uses CMA_FREE, but this        */
      /*       has been bypassed, because of the problems posed by       */
      /*       trying to implement noSharedObjects optimization.         */
      /*       See deleteAllObjects for more info.                       */
      /*                                                                 */
      /* We are currently ignoring CMA_INVALIDATE.                       */
      /*******************************************************************/

      IContainerObject* pCnrObj;
      unsigned short    cNumRecord   (event.parameter2().lowNumber());
      unsigned short    fRemoveRecord(event.parameter2().highNumber());
      IASSERTSTATE( cNumRecord <= 1 );
      IASSERTSTATE( !(Boolean)(fRemoveRecord & CMA_FREE) );

      if ( cNumRecord )   //Assume just one.
      {
        //parameter1 is a pointer to a IMiniCnrRecord*
        IMiniCnrRecord*
             pRec(*((IMiniCnrRecord**)event.parameter1().asUnsignedLong()));
        pCnrObj = IObjFromRec(pRec);

        // This container must contain the object before we can continue.  If
        // it doesn't, ignore this call like CCL does.
        if (pCnrCtl->containsObject (pCnrObj))                       /* D8111 */
        {
          //If TreeView exists, convert ICnrObj* to PTREEITEM and remove it.
          if ( pCnrCtl->ppd->pTreeview )
          {
            HTREEITEM pTreeItem;
            pTreeItem = pCnrObj->ppd->pCnrSet->treeItemFromObject( pCnrCtl );

            if ( pTreeItem )
              TreeView_DeleteItem(pCnrCtl->ppd->pTreeview->handle(), pTreeItem);
          }

          //If ListView exists, convert ICnrObj* to an index and remove it.
          if ( pCnrCtl->ppd->pListview )
          {
            int           index=0;
            LV_FINDINFO   lvFindInfo;
            lvFindInfo.flags  = LVFI_PARAM;
            lvFindInfo.lParam = (LPARAM)pCnrObj;

            // Get the objects list view index.
            index = IContainerControl::ListViewFindItem(
                                       pCnrCtl->ppd->pListview->handle(),
                                       (-1), &lvFindInfo );

            if ( index != (-1) )
              ListView_DeleteItem(pCnrCtl->ppd->pListview->handle(), index);
          }

          //Remove the corresponding element from the objects CnrSet.
          pCnrObj->ppd->pCnrSet->remove( pCnrCtl );
        }
      }
      else   //Remove All
      {
        if ( pCnrCtl->ppd->pListview && !pCnrCtl->ppd->pTreeview )
        {
          //Cursor through and remove from each objects CnrSet
          //for this ListView only container.
          //This is done because the LVN_DELETEITEM notification does
          //not return the LV_ITEM.lParam = ICnrObj field but only the
          //index.

          //What does an ListView_GetItem return when we get the notification?

          int     numListItems(ListView_GetItemCount(
                                     pCnrCtl->ppd->pListview->handle()));
          LV_ITEM         lvItem;

          lvItem.mask       = LVIF_PARAM;
          lvItem.iSubItem   = 0;

          for (int index=0; index < numListItems; index++)
          {
            lvItem.iItem = index;
            if (ListView_GetItem(pCnrCtl->ppd->pListview->handle(), &lvItem))
            {
              IContainerObject* pCnrObj((IContainerObject*)lvItem.lParam);
              pCnrObj->ppd->pCnrSet->remove( pCnrCtl );
            }
          }
        }

        //If the views exist, remove all from the native control.
        if ( pCnrCtl->ppd->pTreeview )
        {
          //Set some container ppd value here to check when
          //we get the notification?

          TreeView_DeleteAllItems(pCnrCtl->ppd->pTreeview->handle() );
        }

        if ( pCnrCtl->ppd->pListview )
          ListView_DeleteAllItems(pCnrCtl->ppd->pListview->handle() );
      }

      fStopHandling = true;
      break;
    }

    case CM_SCROLLWINDOW:
    {
      /*******************************************************************/
      /* Scroll the TreeView or the ListView.                            */
      /*                                                                 */
      /*******************************************************************/
      unsigned short  scrollDirection( event.parameter1().lowNumber() );
      long            scrollInc      ( (long)event.parameter2() );

      // Must be either CMA_VERTICAL or CMA_HORIZONTAL
      IASSERTSTATE( ( scrollDirection == CMA_VERTICAL)  ||
                    ( scrollDirection == CMA_HORIZONTAL)  );

      if ( pCnrCtl->isTreeView() )
      {
        /***************************************************************/
        /*  This doesn't seem to work..and may not with this approach  */
        /*  If we can get ahold of the scroll bar handle, then maybe   */
        /*  we send tell it to scroll?                                 */
        /***************************************************************/
        int      retVal;
        RECT     cnrRect;
        POINTL   pt;

        GetWindowRect( pCnrCtl->ppd->pTreeview->handle(), &cnrRect);

        pt.x = cnrRect.left;
        pt.y = cnrRect.top;
        IMAPWINDOWPOINTS( 0, pCnrCtl->ppd->pTreeview->handle(), &pt, 1);
        cnrRect.left = pt.x;
        cnrRect.top  = pt.y;

        pt.x = cnrRect.right;
        pt.y = cnrRect.bottom;
        IMAPWINDOWPOINTS( 0, pCnrCtl->ppd->pTreeview->handle(), &pt, 1);
        cnrRect.right = pt.x;
        cnrRect.bottom  = pt.y;

        if ( scrollDirection == CMA_VERTICAL )
          retVal = ScrollWindowEx( pCnrCtl->ppd->pTreeview->handle(),
                                   0, (int)scrollInc, 0, &cnrRect, 0, 0,
                                   SW_ERASE | SW_INVALIDATE );
        else
          retVal = ScrollWindowEx( pCnrCtl->ppd->pTreeview->handle(),
                                   (int)scrollInc, 0, 0, &cnrRect, 0, 0,
                                   SW_ERASE | SW_INVALIDATE );

        if ( retVal = ERROR )
          event.setResult( false );
        else
          event.setResult( true );
      }
      else          //ListView
      {
        if ( scrollDirection == CMA_VERTICAL )
          event.setResult( ListView_Scroll( pCnrCtl->ppd->pListview->handle(),
                                            0, (int)scrollInc ) );
        else
          event.setResult( ListView_Scroll( pCnrCtl->ppd->pListview->handle(),
                                            (int)scrollInc, 0 ) );
      }

      fStopHandling = true;
    }
    break;

    case CM_SEARCHSTRING:
    {
      /*******************************************************************/
      /* Search the TreeView or the ListView for the given string.       */
      /*                                                                 */
      /* This is only implemented for listview currently and doesn't     */
      /* support details view or the treeView.                           */
      /* It currently doesn't support case sensitivity                   */
      /*                                                                 */
      /* The tree view could possibly create a tree cursor and then      */
      /* call TreeView_GetItem for each one untill a string matches.     */
      /*******************************************************************/
      IMiniCnrRecord*
          pSearchAfter( (IMiniCnrRecord*)event.parameter2().asUnsignedLong() );
      LPSEARCHSTRING
          pSearchString( (LPSEARCHSTRING)event.parameter1().asUnsignedLong() );
      int   itemIndex=0;

      IASSERTSTATE( pSearchAfter != 0 );

      if ( pSearchAfter == (IMiniCnrRecord*)CMA_FIRST )
      {
        itemIndex = (-1);
      }

      if ( pCnrCtl->isTreeView() )
      {
         //We dont yet have this implemented.  We may need to create a
         //cursor and iterate the container as mentioned above.  If so,
         //we may as well implement the list view the same way, especially
         //since it does not seem to support case sensitivity.
      }
      else    //ListView
      {
        LV_FINDINFO   lvFindInfo;

        if ( itemIndex != (-1) )
        {
          //Convert Given object to item number.

          lvFindInfo.flags  = LVFI_PARAM;
          lvFindInfo.lParam = (LPARAM)IObjFromRec( pSearchAfter );

          itemIndex = IContainerControl::ListViewFindItem(
                                         pCnrCtl->ppd->pListview->handle(),
                                         (-1), &lvFindInfo );
        }

        lvFindInfo.psz   = pSearchString->pszSearch;
        lvFindInfo.flags = LVFI_STRING;

        if ( pSearchString->fsPrefix )
          lvFindInfo.flags |= LVFI_PARTIAL;

        itemIndex = IContainerControl::ListViewFindItem(
                                        pCnrCtl->ppd->pListview->handle(),
                                       (-1), &lvFindInfo );

        if ( itemIndex != (-1) )
        {
          LV_ITEM    listItem;

          listItem.iItem    = itemIndex;
          listItem.iSubItem = 0;
          listItem.mask     = LVIF_PARAM;

          ListView_GetItem( pCnrCtl->ppd->pListview->handle(), &listItem );
          event.setResult( IRecFromObj( (IContainerObject*)listItem.lParam) );
        }
        else
          event.setResult( 0 );
      }

      fStopHandling = true;
    }
    break;

    case CM_SETCNRINFO:
      ((IContainerControl*)event.window())->ppd->setCnrInfo( (ICnrInfo*)event.parameter1().asUnsignedLong(),
                                                             event.parameter2().asUnsignedLong() );
      // Do an arrange if we are in icon view, have size, and the
      // arrange flag is set
      if (pCnrCtl->ppd->pListview && pCnrCtl->ppd->needArrange)
      {
         ISize cnrSize = pCnrCtl->ppd->pListview->size();
         Boolean hasSize = (!(cnrSize.width() == 0)) &&
                           (!(cnrSize.height() == 0));
         if (hasSize)
         {
            pCnrCtl->ppd->needArrange = false;
            ListView_Arrange( pCnrCtl->ppd->pListview->handle(), LVA_ALIGNTOP );
         }
      }
      event.setResult( true );
      fStopHandling = true;
      break;

    case CM_SETRECORDEMPHASIS:
    {
      /********************************************************************/
      /* Sets Selection/Cursored/InUse states for current View            */
      /* This only sets these styles for the current view.                */
      /* Sets the InUse state for both views if they exist.               */
      /*                                                                  */
      /* Added support for setAttributes (CRA_RECORDREADONLY, DROPONABLE  */
      /* This way CM_INVALIDATERECORD doesn't need to worry about the     */
      /* flRecordAttr field                                               */
      /********************************************************************/
      IContainerObject*  pObj=0;
      IMiniCnrRecord* pRec((IMiniCnrRecord*)event.parameter1().asUnsignedLong());
      Boolean         fChangeEmphasis( (Boolean)event.parameter2().number1() );
      unsigned short  fEmphAttribute (  event.parameter2().number2() );
      ICnrObjSetInfo* pObjInfo;
      Boolean         fSuccess;
      int             itemIndex(-1);
      HTREEITEM       treeItem(0);
      HWND            hwndList(0);
      HWND            hwndTree(0);

      event.setResult(true);

      // Template is placed, but we can determine details.
      // Maybe we do nothing for whole container source emphasis?
      if ( fEmphAttribute == CRA_SOURCE )
      {
        if ( pRec )
        {
          // Source Emphasis on an object.
          pObj = IObjFromRec( pRec );
          if ( fChangeEmphasis )
          {
            // Aquiring it.
          }
          else
          {
            // Losing it.
          }
        }
        else
        {
          // Source Emphasis on entire container.
          if ( fChangeEmphasis )
          {
            // Aquiring it.
          }
          else
          {
            // Losing it.
          }
        }
        event.setResult(true);
        fStopHandling = true;
        break;
      }

      IASSERTSTATE( pRec != 0 );
      pObj = IObjFromRec( pRec );

      IASSERTSTATE( pObj->ppd != 0 );
      pObjInfo = pObj->ppd->pCnrSet->objectFromCnr( pCnrCtl );

      //Turn readOnly on/off for the object.
      if ( fEmphAttribute & CRA_RECORDREADONLY )
      {
        if ( fChangeEmphasis )
          pObjInfo->setObjectState(0, CRA_RECORDREADONLY);
        else
          pObjInfo->setObjectState(CRA_RECORDREADONLY, 0);
      }

      //Turn droponable on/off for the object.
      if ( fEmphAttribute & CRA_DROPONABLE )
      {
        if ( fChangeEmphasis )
          pObjInfo->setObjectState(0, CRA_DROPONABLE);
        else
          pObjInfo->setObjectState(CRA_DROPONABLE, 0);
      }

      // Turn inUse emphasis on/off in both tree & list if they exist.
      if ( fEmphAttribute & CRA_INUSE)
      {
        if (pCnrCtl->ppd->pListview)
        {
          hwndList = pCnrCtl->ppd->pListview->handle();

          LV_FINDINFO lvFindInfo;

          // Get the index of the lv_item
          lvFindInfo.flags  = LVFI_PARAM;
          lvFindInfo.lParam = (LPARAM)pObj;

          // Prime the lv item
          itemIndex = IContainerControl::ListViewFindItem(hwndList, -1,
                                                          &lvFindInfo);

          // Make sure we get a valid index back.  It could be
          // setting inUse on a child object that doesn't exist
          // in the list view.
          if (itemIndex != -1)
          {
            LV_ITEM lvItem;

            // Prime the lv item
            lvItem.iItem    = itemIndex;
            lvItem.mask     = LVIF_IMAGE;
            lvItem.iSubItem = 0;

            if ( fChangeEmphasis )
            {
              lvItem.iImage = pCnrCtl->ppd->addInUseToList( pRec->hptrIcon );
              pObjInfo->setObjectState(0, CRA_INUSE, pObj );
            }
            else
            {
              lvItem.iImage = pCnrCtl->ppd->addIconToList( pRec->hptrIcon );
              pObjInfo->setObjectState(CRA_INUSE, 0, pObj );
            }
            fSuccess = ListView_SetItem( hwndList, &lvItem );
          }
        }

        if (pCnrCtl->ppd->pTreeview)
        {
          hwndTree = pCnrCtl->ppd->pTreeview->handle();
          treeItem = pObjInfo->pTVItem;

          if (treeItem)
          {
            TV_ITEM tvItem;
            tvItem.hItem = treeItem;
            tvItem.mask  = TVIF_IMAGE | TVIF_SELECTEDIMAGE;
            if ( fChangeEmphasis )
            {
              tvItem.iImage =
              tvItem.iSelectedImage = pCnrCtl->ppd->addInUseToList( pRec->hptrIcon );
              pObjInfo->setObjectState(0, CRA_INUSE, pObj);
            }
            else
            {
              tvItem.iImage =
              tvItem.iSelectedImage = pCnrCtl->ppd->addIconToList( pRec->hptrIcon );
              pObjInfo->setObjectState(CRA_INUSE, 0, pObj);
            }
            TreeView_SetItem( hwndTree, &tvItem );
          }
        }
        // Get out now....CRA_INUSE should not be accompanied with
        // other states.
        event.setResult(true);
        fStopHandling = true;
        break;
      }


      //Turn selection/cursor  on/off only in the current view.
      if ( !pCnrCtl->isTreeView() )         //Consider only ListView
      {
        hwndList = pCnrCtl->ppd->pListview->handle();

        LV_FINDINFO lvFindInfo;
        lvFindInfo.flags  = LVFI_PARAM;
        lvFindInfo.lParam = (LPARAM)pObj;

        // Get the objects list view index.
        itemIndex = IContainerControl::ListViewFindItem(hwndList, -1,
                                                        &lvFindInfo);

        if( (fEmphAttribute & CRA_CURSORED) || (fEmphAttribute & CRA_SELECTED) )
        {
          unsigned int itemState(0);
          //Must first query before changing the state.
          itemState = ListView_GetItemState( hwndList, itemIndex, 0x000F );

          if ( (fEmphAttribute & CRA_CURSORED) &&  fChangeEmphasis )
            itemState |= LVIS_FOCUSED;

          if ( fEmphAttribute & CRA_SELECTED )
            if ( fChangeEmphasis )
              itemState |= LVIS_SELECTED;
            else
              itemState &= ~(LVIS_SELECTED);

          ListView_SetItemState( hwndList, itemIndex, itemState, 0x000F );
        }
        else if (!(fEmphAttribute & CRA_INUSE))                     /* D7981 */
          event.setResult( false );
      }

      else                           //Consider only TreeView.
      {
        hwndTree = pCnrCtl->ppd->pTreeview->handle();
        treeItem = pObjInfo->pTVItem;

        if( ((fEmphAttribute & CRA_CURSORED) || (fEmphAttribute & CRA_SELECTED))
            && fChangeEmphasis )
        {    //MUST BE TURNING IT ON... cant turn off in tree view.
          TreeView_SelectItem( hwndTree, treeItem );
        }
        else if (!(fEmphAttribute & CRA_INUSE))
        {
          event.setResult( false );
        }
      }

      fStopHandling = true;
    }
    break;

    case CM_SORTRECORD:
    {
      /********************************************************************/
      /* Sorts the objects for both the Listview and the Treeview.        */
      /********************************************************************/

      // Process the TreeView Sort only if the TreeView is already created.
      if (pCnrCtl->ppd->pTreeview)
      {
        TV_SORTCB   treeSort;
        treeSort.lpfnCompare = (PFNTVCOMPARE)((void*)event.parameter1());
        treeSort.lParam      = (LPARAM)event.parameter2();
        treeSort.hParent     = 0;
#ifdef  _WIN32S
        TreeView_SortChildrenCB( pCnrCtl->ppd->pTreeview->handle(), &treeSort, 0 );
        event.setResult( true ); //indicate success
#else
        event.setResult( TreeView_SortChildrenCB(
                                            pCnrCtl->ppd->pTreeview->handle(),
                                            &treeSort, 0 ) );
#endif
        //Create Container object cursor and iterate all objects and
        //call TreeView sort for any object that has children.
        //This is necessary because the native TreeView does not
        //sort the entire tree hierarchy with one sort request.
        IContainerControl::ObjectCursor  treeCursor =
                         IContainerControl::ObjectCursor(*pCnrCtl, 0, true);
        forCursor ( treeCursor )
        {
          IContainerObject* pCnrObj(pCnrCtl->objectAt(treeCursor));
          HTREEITEM pTreeItem;
          pTreeItem = TreeView_GetChild(pCnrCtl->ppd->pTreeview->handle(),
                          pCnrObj->ppd->pCnrSet->treeItemFromObject(pCnrCtl) );
          if ( pTreeItem )
          {
            treeSort.hParent =
                        pCnrObj->ppd->pCnrSet->treeItemFromObject( pCnrCtl );
            TreeView_SortChildrenCB( pCnrCtl->ppd->pTreeview->handle(),
                                     &treeSort, 0 );
          }
        }
      }

      // Process the ListView Sort only if the ListView is already created.
      if (pCnrCtl->ppd->pListview)
      {  //mkb, the following is expected to return false under _WIN32S
        event.setResult( ListView_SortItems( pCnrCtl->ppd->pListview->handle(),
                                             (WPARAM)event.parameter1(),
                                             (LPARAM)event.parameter2() ) );
      }

      fStopHandling = true;
    }
    break;

    case WM_SHOWWINDOW:
    {
      if ((Boolean)event.parameter1())
      {
        ShowWindow( pCnrCtl->ppd->pActiveview->handle(), SW_SHOW );
      }
    }
    break;

    // Check to see if we need to do an arrange in icon view after we have
    // been sized.
    case WM_SIZE:
    {
       if (pCnrCtl->ppd->pListview && pCnrCtl->ppd->needArrange)
       {
          ISize cnrSize = pCnrCtl->ppd->pListview->size();
          Boolean hasSize = (!(cnrSize.width() == 0)) &&
                            (!(cnrSize.height() == 0));
          if (hasSize)
          {
             pCnrCtl->ppd->needArrange = false;
             ListView_Arrange( pCnrCtl->ppd->pListview->handle(), LVA_ALIGNTOP );
          }
       }
    }
    break;

    case WM_WINDOWPOSCHANGED:
    {                              // size/move event found
//    defaultProcedure( event );
      PWINDOWPOS pswp = (PWINDOWPOS)(char*)event.parameter2();
      if (!(pswp->flags & SWP_NOSIZE))
      {                           // size event found
        pCnrCtl->ppd->layoutActiveView( ISize(pswp->cx, pswp->cy) );
      }
      break;
    }

    case WM_CONTEXTMENU:
    {
      IMiniCnrRecord* pRec(0);
      POINT cursorPos;
      cursorPos.x = event.parameter2().number1();
      cursorPos.y = event.parameter2().number2();

      // Need to get the container object if we are in details
      // view... Talk to pete.

      GetCursorPos( &cursorPos );

      MapWindowPoints( HWND_DESKTOP, pCnrCtl->handle(), &cursorPos, 1 );

      IPoint cursorPoint( cursorPos.x, cursorPos.y );

      cursorPoint = ICoordinateSystem::convertToApplication( cursorPoint,
                                                             pCnrCtl->size());

      IContainerObject *pObject(pCnrCtl->objectUnderPoint( cursorPoint ));
      if ( pObject )
        pRec = IRecFromObj( pObject );

      CNRMENUINIT cnrMenuInit;
      cnrMenuInit.pRecord = (RECORDCORE*)pRec;
      // Wrapper the record in a cnrmenuinit structure so that the wm_control
      // message is converted by the wrapper.
      event.window()->owner()->sendEvent( WM_CONTROL,
                                          IEventParameter1( event.window()->id(),
                                                            CN_CONTEXTMENU ),
                                          IEventParameter2( &cnrMenuInit ));
      fStopHandling = true;
    }
    break;

    case WM_HELP:
    {
      HELPINFO helpInfo=*((LPHELPINFO)(void *)(event.parameter2()));
      if (helpInfo.iContextType == HELPINFO_MENUITEM)
      {
        fStopHandling = false;
      }
      else
      {
        // If no object is cursored, send -1 as parm2
        IContainerObject* cursored(pCnrCtl->cursoredObject());
        IMiniCnrRecord* pRec=(IMiniCnrRecord*)(void*)(-1);

        if (cursored)
        {
          pRec = IRecFromObj(cursored);
        }
        event.window()->owner()->sendEvent( WM_CONTROL,
                                          IEventParameter1( event.window()->id(),
                                                            CN_HELP ),
                                          IEventParameter2( (RECORDCORE*)pRec ));
        fStopHandling = true;
      }
    }
    break;

    case WM_SETFONT:
    {
      // Pass font change messages onto the listview and treeview controls.
      if (pCnrCtl->ppd->pListview)
        pCnrCtl->ppd->pListview->sendEvent( event );

      if (pCnrCtl->ppd->pTreeview)
        pCnrCtl->ppd->pTreeview->sendEvent( event );

      // Also pass to title if one was specified.
      if ( pCnrCtl->ppd->pTitle )
        pCnrCtl->ppd->pTitle->setFont(pCnrCtl->ppd->pActiveview->font());
    }
    break;

    case WM_DESTROY:
    {
      // Free any associated lists BEFORE the control is destroyed.
      if (pCnrCtl->ppd->pListview)
      {
        ListView_SetImageList( pCnrCtl->ppd->pListview->handle(),
                               0, LVSIL_SMALL );
        ListView_SetImageList( pCnrCtl->ppd->pListview->handle(),
                               0, LVSIL_NORMAL );
      }
      if (pCnrCtl->ppd->pTreeview)
        TreeView_SetImageList( pCnrCtl->ppd->pTreeview->handle(),
                               0, TVSIL_NORMAL);
    }
    break;


/*------------------------------------------------------------------------------
| The following events are notification messages coming FROM the tree view     |
| or list view control.                                                        |
|                                                                              |
------------------------------------------------------------------------------*/

    case WM_NOTIFY:
    {
      fStopHandling = true;
      event.setResult(false);
      switch( ((LPNMHDR)event.parameter2().asUnsignedLong())->code)
      {
        case NM_DBLCLK:
        {
          pCnrCtl->ppd->doubleClickOccurred = true;
          NMHDR *pnmhdr((LPNMHDR)event.parameter2().asUnsignedLong());
          IWindow *owner(event.window()->owner());

          if (pCnrCtl->ppd->activeView() == ICnrControlData::tree)
          {
            // find the focused item and send a CN_ENTER notification.
            NOTIFYRECORDENTER notifyEnter;
            IContainerObject* cursored(pCnrCtl->cursoredObject());

            if (cursored)
            {
              notifyEnter.hwndCnr = event.window()->handle();
              notifyEnter.fKey    = false;
              notifyEnter.pRecord = (RECORDCORE*)IRecFromObj(cursored);

              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(), CN_ENTER ),
                                IEventParameter2( &notifyEnter ));
            }
          }
          else
          {
            POINT          cursorPos;
            LV_HITTESTINFO hitTestInfo;

            GetCursorPos( &cursorPos );

            MapWindowPoints( HWND_DESKTOP, pCnrCtl->ppd->pListview->handle(),
                             &cursorPos, 1 );

            hitTestInfo.pt = cursorPos;
            ListView_HitTest( pCnrCtl->ppd->pListview->handle(), &hitTestInfo );
            if (hitTestInfo.iItem != -1)
            {
              NOTIFYRECORDENTER notifyEnter;
              LV_ITEM           lvItem;
              lvItem.iItem    = hitTestInfo.iItem;
              lvItem.mask     = LVIF_PARAM;
              lvItem.iSubItem = 0;
              ListView_GetItem( pCnrCtl->ppd->pListview->handle(), &lvItem );

              IMiniCnrRecord* pRec(IRecFromObj((IContainerObject*)lvItem.lParam));

              notifyEnter.hwndCnr = event.window()->handle();
              notifyEnter.fKey    = false;
              notifyEnter.pRecord = (RECORDCORE*)pRec;

              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(), CN_ENTER ),
                                IEventParameter2( &notifyEnter ));

            }
          }
          ITRACE_DEVELOP(IString("---> Double Click"));
        }
        break;

        case NM_RETURN:
          ITRACE_DEVELOP(IString("---> Enter Key"));
        break;

        case NM_SETFOCUS:
        {
          IWindow *owner(event.window()->owner());
          if (owner)
            owner->sendEvent( WM_CONTROL,
                              IEventParameter1( event.window()->id(),
                                                CN_SETFOCUS ),
                              IEventParameter2( event.window()->handle() ));
        }
        break;

        case NM_KILLFOCUS:
        {
          IWindow *owner(event.window()->owner());
          if (owner)
            owner->sendEvent( WM_CONTROL,
                              IEventParameter1( event.window()->id(),
                                                CN_KILLFOCUS ),
                              IEventParameter2( event.window()->handle() ));
        }
        break;

        case LVN_BEGINDRAG:
          ITRACE_DEVELOP(IString("---> LVN_BEGINDRAG"));
        break;

        case LVN_BEGINRDRAG:
          ITRACE_DEVELOP(IString("---> LVN_BEGINRDRAG"));
        break;

        case LVN_BEGINLABELEDIT:
        case TVN_BEGINLABELEDIT:
        {
          IContainerObject* pCnrObj;
          unsigned long     controlId;

          if (((LPNMHDR)event.parameter2().asUnsignedLong())->code == LVN_BEGINLABELEDIT)
          {
            pCnrObj = (IContainerObject*)ptrLV_DISPINFO->item.lParam;
            controlId = IC_NATIVE_LISTVIEW;
          }
          else
          {
            pCnrObj = (IContainerObject*)ptrTV_DISPINFO->item.lParam;
            controlId = IC_NATIVE_TREEVIEW;
          }

          if ( pCnrObj->ppd->pCnrSet->isReadOnly( pCnrCtl ) )
          {
            //Cancel Edit Now, Object is ReadOnly
            if ( pCnrCtl->isTreeView() )
              TreeView_EndEditLabelNow( pCnrCtl->ppd->pTreeview->handle(),
                                        false );
            else
            {
              ListView_EditLabel( pCnrCtl->ppd->pListview->handle(),
                                  (-1) );
              if (pCnrCtl->ppd->pDetailsEdit)                      /* D8067 */
                // Send a fake enter message to the mle so that it will close.
                pCnrCtl->ppd->pDetailsEdit->sendEvent( WM_CHAR,
                                                       IEventParameter1( VK_RETURN ));
            }
          }
          else   //Object is not ReadOnly -> process editing.
          {
            IMiniCnrRecord*   pRec(IRecFromObj(pCnrObj) );

            CNREDITDATA cnrEditData;
            cnrEditData.cb         = sizeof( CNREDITDATA );
            cnrEditData.hwndCnr    = event.window()->handle();
            cnrEditData.pRecord    = (RECORDCORE*)pRec;
            // the field info struct is 0 if we are not editing a details view.
            cnrEditData.pFieldInfo = 0;
            cnrEditData.ppszText   = (LPSTR*)&(pRec->strIcon);
            cnrEditData.cbText     = pRec->strIcon.length() + 1;
            cnrEditData.id         = event.window()->id();

            event.window()->owner()->sendEvent( WM_CONTROL,
                                       IEventParameter1( cnrEditData.id, CN_BEGINEDIT ),
                                       IEventParameter2( &cnrEditData ) );
          }
        }
        break;

        case LVN_COLUMNCLICK:
          ITRACE_DEVELOP(IString("---> LVN_COLUMNCLICK"));
        break;

        case LVN_DELETEALLITEMS:
          ITRACE_DEVELOP(IString("---> LVN_DELETEALLITEMS"));
        break;

        case LVN_DELETEITEM:
          ITRACE_DEVELOP(IString("---> LVN_DELETEITEM"));
        break;

        case LVN_ENDLABELEDIT:
        case TVN_ENDLABELEDIT:
        {
          IContainerObject* pCnrObj;
          unsigned long     controlId;
          Boolean           fListview;
          Boolean           fEditCancelled;
          IWindow*          owner(event.window()->owner());

          if (((LPNMHDR)event.parameter2().asUnsignedLong())->code == LVN_ENDLABELEDIT)
          {
            pCnrObj   = (IContainerObject*)ptrLV_DISPINFO->item.lParam;
            controlId = IC_NATIVE_LISTVIEW;
            fListview = true;
            fEditCancelled = ((ptrLV_DISPINFO->item.iItem == -1) ||
                              (ptrLV_DISPINFO->item.pszText == 0));
          }
          else
          {
            pCnrObj   = (IContainerObject*)ptrTV_DISPINFO->item.lParam;
            controlId = IC_NATIVE_TREEVIEW;
            fListview = false;
            fEditCancelled = (ptrTV_DISPINFO->item.pszText == 0);
          }

          IMiniCnrRecord* pRec(IRecFromObj(pCnrObj));

          CNREDITDATA cnrEditData;
          cnrEditData.cb         = sizeof( CNREDITDATA );
          cnrEditData.hwndCnr    = event.window()->handle();
          cnrEditData.pRecord    = (RECORDCORE*)pRec;
          // the field info struct is 0 if we are not editing a details view.
          cnrEditData.pFieldInfo = 0;
          cnrEditData.ppszText   = (LPSTR*)&(pRec->strIcon);
          cnrEditData.cbText     = pRec->strIcon.length() + 1;
          cnrEditData.id         = event.window()->id();

          // If the user did not cancel editing, send a message to
          // reallocate the string.
          if (!fEditCancelled)
          {
            if ((Boolean)owner->sendEvent( WM_CONTROL,
                                           IEventParameter1( cnrEditData.id,
                                                             CN_REALLOCPSZ ),
                                           IEventParameter2( &cnrEditData ) ))
            {
              if (fListview)
              {
                pRec->strIcon = ptrLV_DISPINFO->item.pszText;
                ListView_SetItemText( pCnrCtl->ppd->pListview->handle(),
                                      ptrLV_DISPINFO->item.iItem,
                                      0,
                                      ptrLV_DISPINFO->item.pszText );

                // If a tree view exists, update the tree item text.
                if ( pCnrCtl->ppd->pTreeview )
                {
                  TV_ITEM tvItem;
                  tvItem.hItem = pCnrObj->ppd->pCnrSet->treeItemFromObject( pCnrCtl );
                  tvItem.mask = TVIF_TEXT;
                  tvItem.pszText = pRec->strIcon;
                  TreeView_SetItem( pCnrCtl->ppd->pTreeview->handle(), &tvItem);
                }
              }
              else
              {
                pRec->strIcon = ptrTV_DISPINFO->item.pszText;
                ptrTV_DISPINFO->item.mask = TVIF_TEXT;

                TreeView_SetItem( pCnrCtl->ppd->pTreeview->handle(),
                                  &(ptrTV_DISPINFO->item));

                // If a tree view exists, update the tree item text.
                if ( pCnrCtl->ppd->pListview )
                {
                  // If the tree item has no parent, then it exists
                  // in the listview at well.
                  if (!TreeView_GetParent(pCnrCtl->ppd->pTreeview->handle(),
                                          ptrTV_DISPINFO->item.hItem))
                  {
                    int index(ICnrObjPrivateData::listItemFromCnr( pCnrCtl,
                                                                   pCnrObj ));
                    if (index != -1)
                    {
                      ListView_SetItemText( pCnrCtl->ppd->pListview->handle(),
                                            index,
                                            0,
                                            ptrTV_DISPINFO->item.pszText );
                    }
                  }
                }
              }
            }
          }
          owner->sendEvent( WM_CONTROL,
                            IEventParameter1( event.window()->id(), CN_ENDEDIT ),
                            IEventParameter2( &cnrEditData ));
        }
        break;

        case LVN_GETDISPINFO:
        {
          if ((ptrLV_DISPINFO->item.iSubItem > 0) &&
              (ptrLV_DISPINFO->item.mask & LVIF_TEXT))
          {
            IContainerObject*
               pCnrObj((IContainerObject*)ptrLV_DISPINFO->item.lParam);
            IContainerColumn*
               pColumn( pCnrCtl->ppd->columnSet->containerColumnFromSubId(
                                               ptrLV_DISPINFO->item.iSubItem) );
            IMiniCnrRecord*
               pRec(IRecFromObj(pCnrObj) );

            if ( pColumn->pfieldinfoCl->flData & CFA_STRING )
              ptrLV_DISPINFO->item.pszText = (*((LPTSTR*)(((LPBYTE)pRec) +
                                           pColumn->pfieldinfoCl->offStruct)));
            else
            {
              // Two items to think about.
              // 1. Is the pszText field zero'd out for the first getdispinfo request.
              // 2. We need to delete this buffer when we are deleting this subitem
              //    from the container when removing subitems.
              if (!ptrLV_DISPINFO->item.pszText)
              {
                ptrLV_DISPINFO->item.pszText = new char[15];
              }

              if ( pColumn->pfieldinfoCl->flData & CFA_ULONG )
              {
                 CreateNLSNumberString (((unsigned long)*(LPDWORD)(((LPBYTE)pRec) +
                                                 pColumn->pfieldinfoCl->offStruct)),
                                        ptrLV_DISPINFO->item.pszText);
              }
              else if ( pColumn->pfieldinfoCl->flData & CFA_DATE )
              {
                 CreateNLSDateString (((LPCDATE)(((LPBYTE)pRec) +
                                                 pColumn->pfieldinfoCl->offStruct)),
                                      ptrLV_DISPINFO->item.pszText);
              }
              else if ( pColumn->pfieldinfoCl->flData & CFA_TIME )
              {
                 CreateNLSTimeString (((LPCTIME)(((LPBYTE)pRec) +
                                                 pColumn->pfieldinfoCl->offStruct)),
                                      ptrLV_DISPINFO->item.pszText);
              }
            }
          }
        }
        break;

        case LVN_INSERTITEM:
          ITRACE_DEVELOP(IString("---> LVN_INSERTITEM"));
        break;

        case LVN_ITEMCHANGING:
          ITRACE_DEVELOP(IString("---> LVN_ITEMCHANGING"));
        break;

        case LVN_ITEMCHANGED:
        {
          NM_LISTVIEW* pNMListView((NM_LISTVIEW*)event.parameter2().asUnsignedLong());
          NOTIFYRECORDEMPHASIS notifyRecord;
          IContainerObject* pCnrObj((IContainerObject*)pNMListView->lParam);
          IWindow* owner(event.window()->owner());

          if (pCnrObj && owner)
          {
            ICnrObjSetInfo* pCnrObjInfo(pCnrObj->ppd->pCnrSet->objectFromCnr( pCnrCtl ));

            notifyRecord.hwndCnr       = event.window()->handle();
            notifyRecord.pRecord       = (RECORDCORE*)IRecFromObj((IContainerObject*)pNMListView->lParam);
            notifyRecord.fEmphasisMask = 0;

            if (pNMListView->uChanged & LVIF_STATE)
            {
              if ((pNMListView->uNewState & LVIS_SELECTED) ||
                  (pNMListView->uOldState & LVIS_SELECTED))
                notifyRecord.fEmphasisMask |= CRA_SELECTED;
              if ((pNMListView->uNewState & LVIS_FOCUSED) ||
                  (pNMListView->uOldState & LVIS_FOCUSED))
                notifyRecord.fEmphasisMask |= CRA_CURSORED;
            }

            owner->sendEvent( WM_CONTROL,
                              IEventParameter1( event.window()->id(), CN_EMPHASIS ),
                              IEventParameter2( &notifyRecord ));

          }
        }
        break;

        case LVN_KEYDOWN:
        case TVN_KEYDOWN:
        {
          LV_KEYDOWN* pLVKeyDown((LV_KEYDOWN*)event.parameter2().asUnsignedLong());
          IWindow* owner(event.window()->owner());

          if (pLVKeyDown->wVKey == VK_RETURN && owner)
          {
            // find the focused item and send a CN_ENTER notification.
            NOTIFYRECORDENTER notifyEnter;
            IContainerObject* cursored(pCnrCtl->cursoredObject());

            if (cursored)
            {
              notifyEnter.hwndCnr = event.window()->handle();
              notifyEnter.fKey    = true;
              notifyEnter.pRecord = (RECORDCORE*)IRecFromObj(cursored);

              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(), CN_ENTER ),
                                IEventParameter2( &notifyEnter ));
            }
          }
          ITRACE_DEVELOP(IString("---> LVN_KEYDOWN | TVN_KEYDOWN"));
        }
        break;

        case LVN_SETDISPINFO:
          ITRACE_DEVELOP(IString("---> LVN_SETDISPINFO"));
        break;

        case TVN_BEGINDRAG:
          ITRACE_DEVELOP(IString("---> TVN_BEGINDRAG"));
        break;

        case TVN_BEGINRDRAG:
          ITRACE_DEVELOP(IString("---> TVN_BEGINRDRAG"));
        break;

        case TVN_DELETEITEM:
        {
          ITRACE_DEVELOP(IString("---> TVN_DELETEITEM"));
          NM_TREEVIEW* pNMTreeView((NM_TREEVIEW*)event.parameter2().asUnsignedLong());
          IContainerObject* pCnrObj((IContainerObject*)pNMTreeView->itemOld.lParam);

          //Possibly check the ppd value that we set above in CM_REMOVEREC?

          if (pCnrObj)
          {
            pCnrObj->ppd->pCnrSet->remove( pCnrCtl );
          }
        }
        break;

        case TVN_ITEMEXPANDED:
        {
          NM_TREEVIEW* pNMTreeView((NM_TREEVIEW*)event.parameter2().asUnsignedLong());
          IContainerObject* pCnrObj((IContainerObject*)pNMTreeView->itemNew.lParam);
          IMiniCnrRecord*   pRec(IRecFromObj(pCnrObj) );
          IWindow* owner(event.window()->owner());

          // Set the expand or collapse notification done flag.
          pCnrCtl->ppd->expandNotificationDone = true;

          if (owner)
          {
            if (pNMTreeView->itemNew.state & TVIS_EXPANDED)
            {
              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(),
                                                  CN_EXPANDTREE ),
                                IEventParameter2( (RECORDCORE*)pRec ));
            }
            else
            {
              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(),
                                                  CN_COLLAPSETREE ),
                                IEventParameter2( (RECORDCORE*)pRec ));
            }
          }
        }
        break;

        case TVN_ITEMEXPANDING:
          ITRACE_DEVELOP(IString("---> TVN_ITEMEXPANDING"));
        break;

        case TVN_SELCHANGING:
          ITRACE_DEVELOP(IString("---> TVN_SELCHANGING"));
        break;

        case TVN_SELCHANGED:
        {
          NM_TREEVIEW* pNMTreeView((NM_TREEVIEW*)event.parameter2().asUnsignedLong());
          NOTIFYRECORDEMPHASIS notifyRecord;
          IContainerObject* pCnrObj(0);
          IWindow* owner(event.window()->owner());

          if (owner)
          {
            notifyRecord.hwndCnr       = event.window()->handle();
            notifyRecord.fEmphasisMask = CRA_SELECTED | CRA_CURSORED;

            if (pNMTreeView->itemOld.hItem)
            {
              pCnrObj = (IContainerObject*)pNMTreeView->itemOld.lParam;
              notifyRecord.pRecord = (RECORDCORE*)IRecFromObj(pCnrObj);
              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(), CN_EMPHASIS ),
                                IEventParameter2( &notifyRecord ));
            }

            if (pNMTreeView->itemNew.hItem)
            {
              pCnrObj = (IContainerObject*)pNMTreeView->itemNew.lParam;
              notifyRecord.pRecord = (RECORDCORE*)IRecFromObj(pCnrObj);
              owner->sendEvent( WM_CONTROL,
                                IEventParameter1( event.window()->id(), CN_EMPHASIS ),
                                IEventParameter2( &notifyRecord ));
            }
          }
        }
        break;

      }
    }
  }
  return fStopHandling;
}

/*------------------------------------------------------------------------------
| ICnrControlData::timerPopped                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
void ICnrControlData::timerPopped ( unsigned long timerId )
{
  // Kill the timer.
  if (fTimer)
  {
    fTimer->stop();
  }

  // If a double click has not occured, then check if the column and
  // object can be edited.
  if (!doubleClickOccurred)
  {
    MapWindowPoints( pListview->handle(), pCnr->handle(), &editPoint, 1 );

    IPoint cursorPoint( editPoint.x, editPoint.y );

    IContainerColumn *pColumn( pCnr->columnUnderPoint( cursorPoint ));

    cursorPoint = ICoordinateSystem::convertToApplication( cursorPoint,
                                                           pCnr->size());

    IContainerObject *pObject(pCnr->objectUnderPoint( cursorPoint ));

    if (pColumn &&                                      // column exists
        pObject &&                                      // container object exists
        (pColumn->pfieldinfoCl->flData & CFA_STRING) && // column is a string
        pColumn->isWriteable() &&                       // column is editable
        pCnr->isCursored( pObject ) &&                  // object is cursored
        pCnr->isWriteable( pObject ))
    {
      pCnr->editObject( pObject, pColumn );
    }
  }

  if (fTimer)
  {
    delete fTimer;
    fTimer = 0;
  }
}

/*------------------------------------------------------------------------------
| IMessageHandler::IMessageHandler                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IMessageHandler::IMessageHandler( )
  : IMessageHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IMessageHandler::~IMessageHandler                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IMessageHandler::~IMessageHandler( )
{ }

/*------------------------------------------------------------------------------
| IMessageHandler::defaultHandler                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IMessageHandler* IMessageHandler::defaultHandler ( )
{
   if(!IDefaultNCStatics.pMessageHandler)
      IDefaultNCStatics.pMessageHandler = new IMessageHandler();
   return IDefaultNCStatics.pMessageHandler;
}

/*------------------------------------------------------------------------------
| IMessageHandler::dispatchHandlerEvent                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMessageHandler::dispatchHandlerEvent( IEvent& event )
{
  Boolean fStopHandling( false );

  switch (event.eventId())
  {
    // Push mouse move messages upto the wrapper window so that
    // drag drop works.
    case WM_MOUSEMOVE:
    {
      IEventData p2( event.parameter2() );
      IContainerControl* pCnrCtl((IContainerControl*)event.window()->parent());
      if ( pCnrCtl->ppd->showTitle )
      {
        // If the title is showing then we need to remap the event to
        // the container wrapper window.
        // The point must be in native coordinates regardless of which
        // orientation has been selected. IMouseEvent member function
        // mousePosition will take care of any re-coordination issues,
        // but asssumes that the point is in native coordinates.
        unsigned long mousePos = event.parameter2().asUnsignedLong();

        POINTL newOrigin;
        newOrigin.x = 0;
        newOrigin.y = 0;
        IMAPWINDOWPOINTS( event.window()->handle(),
                          event.window()->parent()->handle(),
                          &newOrigin, 1 );

        p2 = IEventData( (short)( LOWORD(mousePos) ),
                         (short)( HIWORD(mousePos) + newOrigin.y ) );
      }
      // A new event with the wrapper container window as the dispatching
      // handle must be created for dispatching.
      IEvent passUpEvent( event.window()->handle(),
                          event.eventId(),
                          event.parameter1(),
                          p2 );

      event.window()->owner()->sendEvent( passUpEvent );
      fStopHandling = true;
    }
    break;

    case WM_LBUTTONDOWN:
    {
      IContainerControl* pCnrCtl((IContainerControl*)event.window()->parent());
      pCnrCtl->ppd->doubleClickOccurred = false;

      // If in details view...
      if (pCnrCtl->isDetailsView() && !(pCnrCtl->style() & IContainerControl::readOnly.asUnsignedLong()))
      {
        // If we are not editing a column...
        if (!pCnrCtl->ppd->pDetailsEdit && !ListView_GetEditControl( pCnrCtl->ppd->pListview->handle() ))
        {
          // If the timer has already been started...
          if (!pCnrCtl->ppd->fTimer)
          {
            LV_HITTESTINFO hitTestInfo;

            hitTestInfo.pt.x = event.parameter2().number1();
            hitTestInfo.pt.y = event.parameter2().number2();

            ListView_HitTest( pCnrCtl->ppd->pListview->handle(), &hitTestInfo );

            // If the listview detected the mouse is not over a list view item
            // then we can assume it is over an object not in the first column.
            // We let the list view handle editing for the first column.
            if (hitTestInfo.iItem == -1)
            {
              pCnrCtl->ppd->editPoint = hitTestInfo.pt;
              pCnrCtl->ppd->fTimer = new ITimer( new ITimerMemberFn<ICnrControlData>(*pCnrCtl->ppd, ICnrControlData::timerPopped),
                                                 GetDoubleClickTime());
            }
          }
        }
        else if (pCnrCtl->ppd->pDetailsEdit)
        {

          // Send a fake enter message to the mle so that it will
          // commit the edit changes.
          pCnrCtl->ppd->pDetailsEdit->sendEvent( WM_CHAR,
                                                 IEventParameter1( VK_RETURN ));

          fStopHandling = true;
        }
      }
      if (!fStopHandling)
      {
        // Save parameter2, it may need to be changed if the title is
        // showing.
        IEventData p2( event.parameter2() );
        if ( pCnrCtl->ppd->showTitle )
        {
          // If the title is showing then we need to remap the event to
          // the container wrapper window.
          // The point must be in native coordinates regardless of which
          // orientation has been selected. IMouseEvent member function
          // mousePosition will take care of any re-coordination issues,
          // but asssumes that the point is in native coordinates.
          unsigned long mousePos = event.parameter2().asUnsignedLong();

          POINTL newOrigin;
          newOrigin.x = 0;
          newOrigin.y = 0;
          IMAPWINDOWPOINTS( event.window()->handle(),
                            event.window()->parent()->handle(),
                            &newOrigin, 1 );

          p2 = IEventData( (short)( LOWORD(mousePos) ),
                           (short)( HIWORD(mousePos) + newOrigin.y ) );
        }
        // A new event with the wrapper container window as the dispatching
        // handle must be created for dispatching.
        IEvent passUpEvent( event.window()->owner()->handle(),
                            event.eventId(),
                            event.parameter1(),
                            p2 );

        if (!event.window()->owner()->dispatch( passUpEvent ))
          this->defaultProcedure( event );
        fStopHandling = true;
      }
    }
    break;

    case WM_LBUTTONUP:
    case WM_LBUTTONDBLCLK:
    case WM_RBUTTONDOWN:
    case WM_RBUTTONUP:
    case WM_RBUTTONDBLCLK:
    case WM_MBUTTONDOWN:
    case WM_MBUTTONUP:
    case WM_MBUTTONDBLCLK:
    {
      // Save parameter2, it may need to be changed if the title is
      // showing.
      IEventData p2( event.parameter2() );
      IContainerControl* pCnrCtl((IContainerControl*)event.window()->parent());
      if ( pCnrCtl->ppd->showTitle )
      {
        // If the title is showing then we need to remap the event to
        // the container wrapper window.
        // The point must be in native coordinates regardless of which
        // orientation has been selected. IMouseEvent member function
        // mousePosition will take care of any re-coordination issues,
        // but asssumes that the point is in native coordinates.
        unsigned long mousePos = event.parameter2().asUnsignedLong();

        POINTL newOrigin;
        newOrigin.x = 0;
        newOrigin.y = 0;
        IMAPWINDOWPOINTS( event.window()->handle(),
                          event.window()->parent()->handle(),
                          &newOrigin, 1 );

        p2 = IEventData( (short)( LOWORD(mousePos) ),
                         (short)( HIWORD(mousePos) + newOrigin.y ) );
      }
      // A new event with the wrapper container window as the dispatching
      // handle must be created for dispatching.
      IEvent passUpEvent( event.window()->owner()->handle(),
                          event.eventId(),
                          event.parameter1(),
                          p2 );

      if (!event.window()->owner()->dispatch( passUpEvent ))
        this->defaultProcedure( event );
      fStopHandling = true;
    }
    break;

//  case WM_PAINT:
//  {
//    IContainerControl* pCnrCtl((IContainerControl*)event.window()->parent());
//    if (pCnrCtl->ppd->pListview && pCnrCtl->ppd->pListview->handle() == event.handle())
//    {
//      ListView_SetBkColor( event.handle(), pCnrCtl->backgroundColor().asRGBLong() );
//    }
//  }
    break;
  }
  return fStopHandling;
}

/*------------------------------------------------------------------------------
| IDetailsEditHandler::IDetailsEditHandler                                     |
|                                                                              |
------------------------------------------------------------------------------*/
IDetailsEditHandler::IDetailsEditHandler ( )
  : IDetailsEditHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| IDetailsEditHandler::~IDetailsEditHandler                                    |
|                                                                              |
------------------------------------------------------------------------------*/
IDetailsEditHandler::~IDetailsEditHandler ( )
{ }

/*------------------------------------------------------------------------------
| IDetailsEditHandler::defaultHandler                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDetailsEditHandler* IDetailsEditHandler::defaultHandler ( )
{
   if(!IDefaultNCStatics.pEditHandler)
      IDefaultNCStatics.pEditHandler = new IDetailsEditHandler();
   return IDefaultNCStatics.pEditHandler;
}

/*------------------------------------------------------------------------------
| IDetailsEditHandler::virtualKeyPress                                         |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IDetailsEditHandler::virtualKeyPress( IKeyboardEvent& event )
{
  if (event.virtualKey() == IKeyboardEvent::esc ||
      event.virtualKey() == IKeyboardEvent::enter )
  {
    IContainerControl *pCnr((IContainerControl*)event.window()->parent());

    IMiniCnrRecord* pRec(IRecFromObj(pCnr->ppd->pEditObject) );
    LPTSTR *pColumnText(((LPTSTR*)(((LPBYTE)pRec) +
                       pCnr->ppd->pEditColumn->pfieldinfoCl->offStruct)));

    CNREDITDATA cnrEditData;
    cnrEditData.cb         = sizeof( CNREDITDATA );
    cnrEditData.hwndCnr    = pCnr->handle();
    cnrEditData.pRecord    = (RECORDCORE*)pRec;
    cnrEditData.pFieldInfo = pCnr->ppd->pEditColumn->pfieldinfoCl;
    cnrEditData.ppszText   = pColumnText;
    cnrEditData.cbText     = strlen( *pColumnText );
    cnrEditData.id         = pCnr->id();

    if (event.virtualKey() == IKeyboardEvent::enter)
    {
      if ((Boolean)pCnr->owner()->sendEvent( WM_CONTROL,
                                     IEventParameter1( cnrEditData.id,
                                                       CN_REALLOCPSZ ),
                                     IEventParameter2( &cnrEditData ) ))
      {
        IString newText(((IMultiLineEdit*)event.window())->text());

        IString *pObjectText((IString*)pColumnText);
        *pObjectText = newText;
      }
    }
    IDetailsEditHandler::defaultHandler()->stopHandlingEventsFor( event.window() );

    pCnr->owner()->sendEvent( WM_CONTROL,
                              IEventParameter1( cnrEditData.id,
                                                CN_ENDEDIT ),
                              IEventParameter2( &cnrEditData ));

    IDESTROYWINDOW( event.window()->handle() );

    // Get the item index of the edit object so we can redraw.
    LV_FINDINFO lvFindInfo;
    lvFindInfo.lParam = (long)pCnr->ppd->pEditObject;
    lvFindInfo.flags  = LVFI_PARAM;
    int itemIndex(IContainerControl::ListViewFindItem(
                                     pCnr->ppd->pListview->handle(),
                                    (-1), &lvFindInfo ));            /* D8067 */

    ListView_RedrawItems( pCnr->ppd->pListview->handle(),
                          itemIndex,
                          itemIndex );                               /* D8067 */

    pCnr->ppd->pEditObject  = 0;
    pCnr->ppd->pEditColumn  = 0;
    pCnr->ppd->pDetailsEdit = 0;

    return true;
  }
  return false;
}

/*------------------------------------------------------------------------------
| IColumnElement::IColumnElement                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IColumnElement::IColumnElement ( IContainerColumn* pCol,
                                 int               subItemIdentifier,
                                 int               columnIdentifier,
                                 int               visibilityIdentifier)
  : pColumn(pCol), subItemId(subItemIdentifier),
    columnId( columnIdentifier ), visibilityId(visibilityIdentifier)
{
}

/*------------------------------------------------------------------------------
| IColumnElement::~IColumnElement                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IColumnElement::~IColumnElement ( )
{
}

/*------------------------------------------------------------------------------
| IColumnElement::operator=                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IColumnElement& IColumnElement::operator= ( const IColumnElement& element )
{
  if (this == &element)
    return *this;
  this->pColumn   = element.pColumn;
  this->subItemId = element.subItemId;
  this->columnId  = element.columnId;
  this->visibilityId = element.visibilityId;
  return *this;
}

/*------------------------------------------------------------------------------
| IColumnElement::operator==                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IColumnElement::operator == ( const IColumnElement& element ) const
{
  return (Boolean)( this->pColumn   == element.pColumn &&
                    this->subItemId == element.subItemId &&
                    this->columnId  == element.columnId &&
                    this->visibilityId == element.visibilityId);
}

/*------------------------------------------------------------------------------
| IColumnElement::operator<                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IColumnElement::operator < ( const IColumnElement& element ) const
{
  return (Boolean)(this->subItemId < element.subItemId);
}

/*------------------------------------------------------------------------------
| IColumnSet::IColumnSet                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IColumnSet::IColumnSet( )
 : IKeySortedSetOnSortedTabularSequence< IColumnElement, int > ( 8 ),
   nextInvalidSubId(-10)
{}

/*------------------------------------------------------------------------------
| IColumnSet::~IColumnSet                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IColumnSet::~IColumnSet( )
{ }

/*------------------------------------------------------------------------------
| IColumnSet::add                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IColumnSet::add( IContainerColumn* pCol,
                         int               subItemIdentifier,
                         int               columnIdentifier,
                         Boolean           visible)
{
  int  visibilityId( 0 );

  if ( visible )            //valid & visible column being added.
  {
    //In this case subItemIdentifier = prevCol's subItemIdentifier
    //It could be representing either a visible or hidden column.
    if ( subItemIdentifier > (-10) )    //previous column is visible
    {
      if ( columnIdentifier > 0 )
      {
        IColumnSet::Cursor cursor(*this);
        forCursor (cursor)
        {
          if ( elementAt(cursor).columnId >= columnIdentifier )
          {
            elementAt(cursor).columnId++;

            // Only increment the subItem Id's on the visible columns
            if (elementAt(cursor).subItemId > 0 )
              elementAt(cursor).subItemId++;
          }

          // if the prev col had hidden cols after it, update their colId
          if ( (elementAt(cursor).columnId  == (columnIdentifier-1) ) &&
               elementAt(cursor).visibilityId > 0 )
          {
            elementAt(cursor).columnId++;
          }
        }
      }
    }
    else         //previous column is hidden
    {
      if (containsElementWithKey( subItemIdentifier ))
      {
        IColumnElement prevElement(elementWithKey( subItemIdentifier ));

        if ( columnIdentifier > 0 )
        {
          IColumnSet::Cursor cursor(*this);
          forCursor (cursor)
          {
            if ( elementAt(cursor).columnId >= columnIdentifier )
            {
              elementAt(cursor).columnId++;

              // Only increment the subItem Id's on the visible columns
              if (elementAt(cursor).subItemId > 0 )
                elementAt(cursor).subItemId++;
            }

            //Update possible hidden columns following the new column
            if ( (elementAt(cursor).columnId  == (columnIdentifier-1) ) &&
                 elementAt(cursor).visibilityId > prevElement.visibilityId )
            {
              elementAt(cursor).columnId++;
            }
          }
        }
      }
      else
        return false;
    }

    subItemIdentifier = columnIdentifier;
  }
  else   //Invalid column, not visible.
  {
    //Given columnId is one too many.
    columnIdentifier--;

    if (containsElementWithKey( subItemIdentifier ))
    {
      IColumnElement prevElement(elementWithKey( subItemIdentifier ));
      visibilityId = prevElement.visibilityId + 1;
    }
    else
      return false;

    IColumnSet::Cursor cursor(*this);
    forCursor (cursor)
    {
      if ( (elementAt(cursor).columnId == columnIdentifier) &&
           (elementAt(cursor).visibilityId >= visibilityId) )
      {
        elementAt(cursor).visibilityId++;
      }
    }

    nextInvalidSubId--;
    subItemIdentifier = nextInvalidSubId;
  }

  IColumnElement newElement( pCol, subItemIdentifier,
                             columnIdentifier, visibilityId );

  Inherited::add( newElement );
  return false;
}

/*------------------------------------------------------------------------------
| IColumnSet::remove                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IColumnSet::remove( IContainerColumn* pCol,
                            Boolean           collapseColumns )
{
  int colId(0);
  int visibilityId(0);

  IColumnSet::Cursor cursor(*this);

  // Find given column in our columnSet and save information.
  forCursor (cursor)
  {
    IColumnElement columnElement(elementAt( cursor ));
    if (columnElement.pColumn == pCol)
    {
      colId        = columnElement.columnId;
      visibilityId = columnElement.visibilityId;
      removeAt( cursor );
      break;
    }
  }

  // Update our columnSet, based on removing above column.
  if ( visibilityId > 0 )     //Removed column was hidden
  {
    //Only need to update succeeding hidden columns with same columnId
    forCursor (cursor)
    {
      if ( (elementAt(cursor).columnId == colId) &&
           (elementAt(cursor).visibilityId > visibilityId) )
      {
        elementAt(cursor).visibilityId--;
      }
    }
  }
  else                        //Removed column was visible
  {
    if ( colId > 0 )          //Only need to update if colId>0
    {
      ListView_DeleteColumn( pCol->container()->ppd->pListview->handle(),
                             colId );

      // Determine the number of hidden columns after previous visible col.
      int prevHiddenCount(0);
      forCursor (cursor)
      {
        if ( elementAt(cursor).columnId == (colId - 1) )
          if ( elementAt(cursor).visibilityId > prevHiddenCount )
            prevHiddenCount = elementAt(cursor).visibilityId;
      }

      // Update our columnSet, based on removing above column.
      forCursor (cursor)
      {
        if ( elementAt(cursor).columnId >= colId )
        {
          if ( elementAt(cursor).visibilityId == 0 )
            elementAt(cursor).subItemId--;
          else if ( elementAt(cursor).columnId == colId )
            elementAt(cursor).visibilityId += prevHiddenCount;

          elementAt(cursor).columnId--;
        }
      }
    }
    else
    {
      if (collapseColumns)
      {
        // When both the isIcon and isIconViewText columns have been
        // removed, then we should size the first column to 0.
        if ( !containerColumnFromSubId(0) && !containerColumnFromSubId(-1) )
        {
          if ( pCol->container()->isDetailsView() )
            ListView_SetColumnWidth( pCol->container()->ppd->pListview->handle(),
                                     0, 0 );
          else
            pCol->container()->ppd->collapseFirstColumn = true;
        }
      }
    }
  }

  return true;
}

/*------------------------------------------------------------------------------
| IColumnSet::containerColumnFromSubId                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerColumn* IColumnSet::containerColumnFromSubId( int subItemIdentifier ) const
{
  if (containsElementWithKey( subItemIdentifier ))
  {
    IColumnElement columnElement(elementWithKey( subItemIdentifier ));
    return columnElement.pColumn;
  }
  else
  {
    return 0;
  }
}

/*------------------------------------------------------------------------------
| IColumnSet::newSubItemId                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int IColumnSet::newSubItemId( ) const
{
  int newId(1);
  if (!isEmpty())
  {
    IColumnElement columnElement(lastElement());
    // We need to check to see if the only columns inserted are invisible
    // (subItem<-1), or the isIcon column (subItem=-1), or the isIconViewText
    // column (subItem=0), or any combination of the above.  If this is the
    // case then we need to return newId=1.
    if (columnElement.subItemId > 0)
      newId = columnElement.subItemId + 1;
  }
  return newId;
}

/*------------------------------------------------------------------------------
| IColumnSet::containerColumnFromColumnId                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IContainerColumn* IColumnSet::containerColumnFromColumnId( int columnId ) const
{
  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    IColumnElement columnElement(elementAt( cursor ));
    if ( columnElement.columnId == columnId )
    {
      return( columnElement.pColumn );
      break;
    }
  }

  return 0;
}

/*------------------------------------------------------------------------------
| IColumnSet::columnCount                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int IColumnSet::columnCount( ) const
{
  int count( 0 );

  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    if ( elementAt(cursor).columnId > count )
      count = elementAt(cursor).columnId;
  }
  return (count + 1);  //Account for first column whose Id=0
}

/*------------------------------------------------------------------------------
| IColumnSet::lastSubItemId                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int IColumnSet::lastSubItemId( ) const
{
  int subItemId( 0 );
  int columnId( columnCount() - 1 );
  int visibilityId( -1 );

  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    if ( elementAt(cursor).columnId == columnId )
    {
      if ( elementAt(cursor).visibilityId > visibilityId)
      {
        visibilityId = elementAt(cursor).visibilityId;
        subItemId = elementAt(cursor).subItemId;
      }
    }
  }
  return (subItemId);
}

/*------------------------------------------------------------------------------
| IColumnSet::lastFieldInfo                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
LPFIELDINFO IColumnSet::lastFieldInfo( ) const
{
  PFIELDINFO  pFI( 0 );
  int columnId( columnCount() - 1 );
  int visibilityId( -1 );

  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    if ( elementAt(cursor).columnId == columnId )
    {
      if ( elementAt(cursor).visibilityId > visibilityId)
      {
        visibilityId = elementAt(cursor).visibilityId;
        pFI = elementAt(cursor).pColumn->pfieldinfoCl;
      }
    }
  }
  return ( (LPFIELDINFO)pFI );
}

/*------------------------------------------------------------------------------
| IColumnSet::firstFieldInfo                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
LPFIELDINFO IColumnSet::firstFieldInfo( ) const
{
  PFIELDINFO  pFI( 0 );
  int columnId(2);
  int visibilityId(1);

  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    if ( elementAt(cursor).columnId < columnId )
    {
      columnId     = elementAt(cursor).columnId;
      visibilityId = elementAt(cursor).visibilityId;
      pFI = elementAt(cursor).pColumn->pfieldinfoCl;
    }
    else if ( elementAt(cursor).columnId == columnId )
    {
      if ( elementAt(cursor).visibilityId < visibilityId )
      {
        visibilityId = elementAt(cursor).visibilityId;
        pFI = elementAt(cursor).pColumn->pfieldinfoCl;
      }
    }
  }

  return ( (LPFIELDINFO)pFI );
}

/*------------------------------------------------------------------------------
| IColumnSet::nextFieldInfo                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
LPFIELDINFO IColumnSet::nextFieldInfo(LPFIELDINFO pFI ) const
{
  PFIELDINFO  nextPFI( 0 );
  int  columnId;
  int  visibilityId;

  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    if ( elementAt(cursor).pColumn->pfieldinfoCl == pFI )
    {
      columnId = elementAt(cursor).columnId;
      visibilityId = elementAt(cursor).visibilityId;
      break;
    }
  }

  // If column 0 is not inserted, then consider as special case.
  if ( (columnId == (-1)) && !containerColumnFromSubId(0) )
  {
    forCursor (cursor)
    {
      if ( (elementAt(cursor).columnId == 0) &&
           (elementAt(cursor).visibilityId == 1) )
        nextPFI =elementAt(cursor).pColumn->pfieldinfoCl;
    }
    if ( !nextPFI )
    {
      if ( containerColumnFromSubId(1) )
        nextPFI = containerColumnFromSubId(1)->pfieldinfoCl;
    }
  }
  else
  {
    forCursor (cursor)
    {
      if ( (elementAt(cursor).columnId == (columnId + 1)) &&
           (elementAt(cursor).visibilityId == 0) )
      {
        nextPFI = elementAt(cursor).pColumn->pfieldinfoCl;
      }
      if ( (elementAt(cursor).columnId == columnId) &&
           (elementAt(cursor).visibilityId == (visibilityId + 1)) )
      {
        nextPFI = elementAt(cursor).pColumn->pfieldinfoCl;
        break;
      }
    }
  }

  return ( (LPFIELDINFO)nextPFI );
}

/*------------------------------------------------------------------------------
| IColumnSet::prevFieldInfo                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
LPFIELDINFO IColumnSet::prevFieldInfo(LPFIELDINFO pFI ) const
{
  PFIELDINFO  prevPFI( 0 );
  int  columnId;
  int  visibilityId;

  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    if ( elementAt(cursor).pColumn->pfieldinfoCl == pFI )
    {
      columnId = elementAt(cursor).columnId;
      visibilityId = elementAt(cursor).visibilityId;
      break;
    }
  }

  if (visibilityId > 0)    //Column is hidden
  {
    forCursor (cursor)
    {
      if ( (elementAt(cursor).columnId == columnId) &&
           (elementAt(cursor).visibilityId == (visibilityId - 1)) )
      {
        prevPFI = elementAt(cursor).pColumn->pfieldinfoCl;
        break;
      }
    }
  }
  else                    //Column is visible
  {
    int tempColId( -2 );
    int tempVisId;

    forCursor (cursor)
    {
      if (elementAt(cursor).columnId < columnId)
      {
        if (elementAt(cursor).columnId > tempColId)
        {
          tempColId = elementAt(cursor).columnId;
          tempVisId = elementAt(cursor).visibilityId;
          prevPFI = elementAt(cursor).pColumn->pfieldinfoCl;
        }
        else if ( (elementAt(cursor).columnId == tempColId) &&
                  (elementAt(cursor).visibilityId > tempVisId) )
        {
          tempVisId = elementAt(cursor).visibilityId;
          prevPFI = elementAt(cursor).pColumn->pfieldinfoCl;
        }
      }
    }
  }

  return ( (LPFIELDINFO)prevPFI );
}

/*------------------------------------------------------------------------------
| IColumnSet::subItemIdFromFieldInfo                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int IColumnSet::subItemIdFromFieldInfo ( PFIELDINFO pFI ) const
{
  Boolean found(false);
  int     subItemId(0);
  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    IColumnElement columnElement(elementAt( cursor ));
    if ( columnElement.pColumn->pfieldinfoCl == pFI )
    {
      subItemId = columnElement.subItemId;
      found = true;
      break;
    }
  }
  IASSERTSTATE( found );
  return subItemId;
}

/*------------------------------------------------------------------------------
| IColumnSet::columnIdFromFieldInfo                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int IColumnSet::columnIdFromFieldInfo ( PFIELDINFO pFI ) const
{
  Boolean found(false);
  int     colId(-2);
  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    IColumnElement columnElement(elementAt( cursor ));
    if ( columnElement.pColumn->pfieldinfoCl == pFI )
    {
      colId = columnElement.columnId;
      found = true;
      break;
    }
  }
  IASSERTSTATE( found );
  return colId;
}

/*------------------------------------------------------------------------------
| IColumnSet::columnIdFromContainerColumn                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
int IColumnSet::columnIdFromContainerColumn ( IContainerColumn* pCol ) const
{
  int colId(-2);
  IColumnSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    IColumnElement columnElement(elementAt( cursor ));
    if (columnElement.pColumn == pCol)
    {
      colId = columnElement.columnId;
      break;
    }
  }
  return colId;
}

/*------------------------------------------------------------------------------
| IPointerElement::IPointerElement                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPointerElement::IPointerElement( const IPointerHandle& pointer,
                                  unsigned long index )
: pointerHandle(pointer), imageListIndex(index), inUseIndex(-1), lph(pointer.asUnsigned())
{}

/*------------------------------------------------------------------------------
| IPointerElement::~IPointerElement                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPointerElement::~IPointerElement( )
{
}

/*------------------------------------------------------------------------------
| ICnrPointerSet::ICnrPointerSet                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICnrPointerSet::ICnrPointerSet()
  // Initially allocate 20 IPointerElements
: IKeySet < IPointerElement*, unsigned long > ( 20 )
{}

/*------------------------------------------------------------------------------
| ICnrPointerSet::~ICnrPointerSet                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICnrPointerSet::~ICnrPointerSet()
{
  ICnrPointerSet::Cursor cursor(*this);
  forCursor (cursor)
  {
    delete this->elementAt( cursor );
  }
  this->removeAll();
}

/*------------------------------------------------------------------------------
| ICnrPointerSet::addPointerElement                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICnrPointerSet& ICnrPointerSet::addPointerElement( const IPointerHandle& pointer,
                                                   int                   imageIndex )
{
  IPointerElement* newPointerElement(new IPointerElement(pointer, imageIndex));
  this->add( newPointerElement );
  return *this;
}

/*------------------------------------------------------------------------------
| reverse                                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
VOID FAR reverse(LPSTR psz)
{
  int     i, j;
  char    c;

  for (i = 0, j = lstrlen(psz) - 1; i<j; i++, j-- )
  {
    c = psz[i];
    psz[i] = psz[j];
    psz[j] = c;
  }
}

/*------------------------------------------------------------------------------
| BuildDateTimeString                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
LPSTR FAR BuildDateTimeString( int Field, LPSTR pszDateTime,
                               LPSTR DateTimeSep, Boolean LeadingZero )
{
  if ((Field < 10) && (LeadingZero))
    *pszDateTime++ = '0';

  itoa (Field, pszDateTime, 10);

  /*********************************************************************/
  /* If there is a separator, add it to the end of the string.         */
  /*********************************************************************/
  if (DateTimeSep)
    lstrcat (pszDateTime, DateTimeSep);

  /*********************************************************************/
  /* Return a pointer to the end of the string, ie. to the terminating */
  /* NULL.                                                             */
  /*********************************************************************/
  while (*pszDateTime)
    pszDateTime++;

  return (pszDateTime);
}

/*------------------------------------------------------------------------------
| CreateNLSDateString                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
VOID FAR CreateNLSDateString( LPCDATE pDate, LPSTR pszDate )
{
 int            aDate[3];
 int            Field1;
 int            Field2;
 int            Field3;
 int            DAY   = 0;
 int            MONTH = 1;
 int            YEAR  = 2;
 static DWORD   DateFormat (GetProfileInt((LPSTR)"intl", (LPSTR)"iDate", 0));
 static Boolean LeadingZero(GetProfileInt((LPSTR)"intl", (LPSTR)"iLzero", 0));
 static Boolean fNeedStateInfo(true);
 static char    DateSep[2];
 Boolean        useLeadingZero(LeadingZero);

 if (fNeedStateInfo)
 {
   fNeedStateInfo = false;
   /******************************************************************/
   /* Get the Date Separator.                                        */
   /******************************************************************/
   GetProfileString ( (LPSTR)"intl", (LPSTR)"sDate", (LPSTR)"/",
                      DateSep, 2);
 }

 /*********************************************************************/
 /* Fill up my array with the values in the CDATE structure.          */
 /*********************************************************************/
 aDate[DAY]   = pDate->day;
 aDate[MONTH] = pDate->month;
 aDate[YEAR]  = pDate->year;

 /*********************************************************************/
 /* Determine the Date Format that is to be displayed.  There are 3   */
 /* possibilities. Month, Day, Year is the default.  The values of 2, */
 /* 1, and 0 for pccinfon->DateFormat are the values returned by the  */
 /* system during the query.  See CnrInitNLSInfo.                     */
 /*********************************************************************/
 switch ((int)DateFormat)
 {
    case 2:
       Field1 = YEAR;
       Field2 = MONTH;
       Field3 = DAY;
    break;

    case 1:
       Field1 = DAY;
       Field2 = MONTH;
       Field3 = YEAR;
    break;

    case 0:
    default:
       Field1 = MONTH;
       Field2 = DAY;
       Field3 = YEAR;
    break;
 }

  if (Field1 == YEAR)
  {
    useLeadingZero = true;
  }

  pszDate = BuildDateTimeString (aDate[Field1],
                                 pszDate,
                                 DateSep,
                                 useLeadingZero);

  useLeadingZero = LeadingZero;

  if (Field2 == YEAR)
  {
    useLeadingZero = true;
  }

  pszDate = BuildDateTimeString (aDate[Field2],
                                 pszDate,
                                 DateSep,
                                 useLeadingZero);

  useLeadingZero = LeadingZero;

  if (Field3 == YEAR)
  {
    useLeadingZero = true;
  }

  /*********************************************************************/
  /* Don't pass in the Date Separator for the last field.              */
  /*********************************************************************/
  pszDate = BuildDateTimeString (aDate[Field3],pszDate,
                                 0, useLeadingZero);

  return;
}

/*------------------------------------------------------------------------------
| CreateNLSTimeString                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
VOID FAR CreateNLSTimeString( LPCTIME pTime, LPSTR pszTime )
{
  int            Hours;
  Boolean        PM(false);
  static char    am[9];
  static char    pm[9];
  static char    TimeSep[2];
  static Boolean MilitaryFormat(GetProfileInt ((LPSTR)"intl", (LPSTR)"iTime", 0));
  static Boolean fNeedStateInfo(true);
  static Boolean LeadingZero(GetProfileInt((LPSTR)"intl", (LPSTR)"iLzero", 0));

  if (fNeedStateInfo)
  {
    fNeedStateInfo = false;
    /******************************************************************/
    /* Get the AM string.                                             */
    /******************************************************************/
    GetProfileString ( (LPSTR)"intl", (LPSTR)"s1159", (LPSTR)"am", am, 9);

    /******************************************************************/
    /* Get the PM string.                                             */
    /******************************************************************/
    GetProfileString ( (LPSTR)"intl", (LPSTR)"s2359", (LPSTR)"pm", pm, 9);

    /******************************************************************/
    /* Get the Time Separator.                                        */
    /******************************************************************/
    GetProfileString ( (LPSTR)"intl", (LPSTR)"sTime", (LPSTR)":",TimeSep, 2);
  }

  Hours = (int)pTime->hours;

  /*********************************************************************/
  /* If we are in the 12 hour format, we need to convert the hours to  */
  /* this format.                                                      */
  /*********************************************************************/
  if (!MilitaryFormat)
  {
    if (Hours >= 12)
    {
      Hours -= 12;
      PM = true;
    }

    if (Hours == 0)
    {
      Hours = 12;
    }
  }

  pszTime = BuildDateTimeString (Hours, pszTime, TimeSep, LeadingZero);

  pszTime = BuildDateTimeString ((int)pTime->minutes, pszTime, TimeSep, true);

  /*********************************************************************/
  /* Don't pass in the Time Separator for the last field.              */
  /*********************************************************************/
  pszTime = BuildDateTimeString ((int)pTime->seconds, pszTime, 0, true);

  /*********************************************************************/
  /* If we are in the 12 hour format, add the am/pm indicator.         */
  /*********************************************************************/
  if (!MilitaryFormat)
  {
    lstrcat (pszTime, " ");
    lstrcat (pszTime, (PM) ? pm : am);
  }

  return;
}

/*------------------------------------------------------------------------------
| CreateNLSTimeString                                                          |
|                                                                              |
| Description:    Builds a string from a DWORD number.                         |
|                                                                              |
| Input:          dwNumber  - DWORD number to be converted to a                |
|                             string.                                          |
|                 lpNumber - pointer to a buffer to hold string.               |
|                                                                              |
| Output:         TRUE  - Success                                              |
|                 FALSE - Error                                                |
|                                                                              |
| NOTES:  This function only works on unsigned long numbers.                   |
|         Negative numbers are not supported.                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean CreateNLSNumberString( DWORD dwNumber, LPSTR lpNumber )
{
 LPSTR      pszLead;           /* Temp Pointer used to create string. */
 LPSTR      pszBack;           /* Temp Pointer used to create string. */
 LPSTR      pszTemp;           /* Temp Pointer used to  free the      */
 WORD       i;                 /* Loop counter.                       */
 static char Thousands[2];
 static Boolean fNeedStateInfo(true);

 if (fNeedStateInfo)
 {
   fNeedStateInfo = false;
   /******************************************************************/
   /* Get the Number Separator.                                      */
   /******************************************************************/
   GetProfileString ( (LPSTR)"intl", (LPSTR)"sThousand", (LPSTR)",",
                      Thousands, 2);
 }

 /* If the number is such that we don't have to worry about adding
  * the thousands separater, convert it and leave.
  */
 if (dwNumber < 1000)
 {
    ultoa (dwNumber, lpNumber, 10);
    return (TRUE);
 }

 /*********************************************************************/
 /* Allocate a temporary pointer to be used to create the number      */
 /* string.                                                           */
 /*********************************************************************/
 if (!(pszLead = new char[30]))
 {
   return (FALSE);
 }

 /*********************************************************************/
 /* Save this pointer so we can free the memory at the bottom of this */
 /* function.                                                         */
 /*********************************************************************/
 pszTemp = pszLead;

 /*********************************************************************/
 /* Convert the number to a string.                                   */
 /*********************************************************************/
 ultoa (dwNumber, pszLead, 10);

 /*********************************************************************/
 /* Reverse the string.                                               */
 /*********************************************************************/
 reverse (pszLead);

 /* Loop through the string and add in the thousands separator.
  */
 while (*pszLead)
 {
   i = 0;
   pszBack = pszLead;

   while ((*pszLead) && (i < 3))
   {
     pszLead = AnsiNext(pszLead);
     i++;
   }

   if (*pszLead)
   {
     strncat (lpNumber, pszBack, 3);
     lstrcat (lpNumber, Thousands);
   }
   else
   {
     lstrcat (lpNumber, pszBack);
   }
 }

 /*********************************************************************/
 /* Reverse the string again before we return.                        */
 /*********************************************************************/
 reverse (lpNumber);

 /*********************************************************************/
 /* Free the temporary storage that was allocated previously.         */
 /*********************************************************************/
 delete [] pszTemp;

 return (TRUE);
}

/*------------------------------------------------------------------------------
| IContainerControl::ListViewFindItem                                          |
|                                                                              |
| Static function that for Win32, calls the ListView_FindItem function, but    |
| for Win32s, implements the function here.  The function is broken on Win32s  |
| and we needed to provide our own version.                                    |
------------------------------------------------------------------------------*/
int IContainerControl::ListViewFindItem(const IWindowHandle& handle,
                                        int start, void* lvFindInfo)
{
  int index;

#ifdef _WIN32S
  LV_ITEM  lvItem;

  lvItem.mask     = LVIF_PARAM;
  lvItem.iSubItem = 0;
  int numItems = ListView_GetItemCount(handle);

  for (index=0; index < numItems; index++)
  {
    lvItem.iItem = index;
    if (ListView_GetItem(handle, &lvItem))
    {
      if (lvItem.lParam == ((LV_FINDINFO*)lvFindInfo)->lParam)
      {
        break;
      }
    }
  }
  if (index == numItems)
    index = -1;
#else
  index = ListView_FindItem(handle, start, (LV_FINDINFO*)lvFindInfo);
#endif
  return index;
}
#else

/*------------------------------------------------------------------------------
| ICnrControlData::ICnrControlData                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICnrControlData::ICnrControlData( )
  : pmCompatible( true )
{}

/*------------------------------------------------------------------------------
| ICnrControlData::~ICnrControlData                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ICnrControlData::~ICnrControlData( )
{}

#endif
