/*******************************************************************************
* FILE NAME: icolor.cpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icolor.hpp.                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_GPILOGCOLORTABLE
#define INCL_WINSYS
#define INCL_WINDESKTOP
#include <iwindefs.h>

#ifndef _ICCONST_
  #include <icconst.h>
#endif
#ifndef _ICOLOR_
  #include <icolor.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif
#include <iplatfrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ICOLOR_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN
#define IRGBLONG(r, g, b) \
      ((long) ((r & 0xFFul) | ((g & 0xFFul)<<8) | ((b & 0xFFul) << 16)) )
#define IRGB_RED(rgb)    ((unsigned char)(rgb & 0x000000FF) )
#define IRGB_GREEN(rgb)  ((unsigned char)((rgb & 0x0000FF00) >> 8) )
#define IRGB_BLUE(rgb)   ((unsigned char)((rgb & 0x00FF0000) >> 16) )
#endif

#ifdef IC_PM
#define IRGBLONG(r, g, b) \
      ((long) ((b & 0xFFul) | ((g & 0xFFul)<<8) | ((r & 0xFFul) << 16)) )
#define IRGB_BLUE(rgb)   ((unsigned char)(rgb & 0x000000FF) )
#define IRGB_GREEN(rgb)  ((unsigned char)((rgb & 0x0000FF00) >> 8) )
#define IRGB_RED(rgb)    ((unsigned char)((rgb & 0x00FF0000) >> 16) )

#endif
#define ISETRGB(rgb) \
   {ucClRed =   IRGB_RED(rgb); \
    ucClGreen = IRGB_GREEN(rgb); \
    ucClBlue =  IRGB_BLUE(rgb);}

#define IRGB_COLOR_WHITE       IRGBLONG( 255, 255, 255)
#define IRGB_COLOR_BLACK       IRGBLONG( 0, 0, 0)
#define IRGB_COLOR_BLUE        IRGBLONG( 0, 0, 255)
#define IRGB_COLOR_RED         IRGBLONG( 255, 0, 0)
#define IRGB_COLOR_PINK        IRGBLONG( 255, 0, 255)
#define IRGB_COLOR_GREEN       IRGBLONG( 0, 255, 0)
#define IRGB_COLOR_CYAN        IRGBLONG( 0, 255, 255)
#define IRGB_COLOR_YELLOW      IRGBLONG( 255, 255, 0)
#ifdef IC_WIN
  #define IRGB_COLOR_DARKGRAY  IRGBLONG( 128, 128, 128)
#else
  #define IRGB_COLOR_DARKGRAY  IRGBLONG( 131, 131, 131)
#endif
#ifdef IC_WIN
  #define IRGB_COLOR_DARKBLUE  IRGBLONG( 0, 0, 128)
#else
  #define IRGB_COLOR_DARKBLUE  IRGBLONG( 0, 0, 170)
#endif
#define IRGB_COLOR_DARKRED     IRGBLONG( 128, 0, 0)
#ifdef IC_WIN
  #define IRGB_COLOR_DARKPINK  IRGBLONG( 128, 0, 128 )
#else
#define IRGB_COLOR_DARKPINK    IRGBLONG( 170, 0, 170)
#endif
#ifdef IC_WIN
  #define IRGB_COLOR_DARKGREEN IRGBLONG( 0, 128, 0)
#else
  #define IRGB_COLOR_DARKGREEN IRGBLONG( 0, 146, 0)
#endif
#ifdef IC_WIN
  #define IRGB_COLOR_DARKCYAN  IRGBLONG( 0, 128, 128)
#else
  #define IRGB_COLOR_DARKCYAN  IRGBLONG( 0, 146, 170)
#endif
#define IRGB_COLOR_BROWN       IRGBLONG( 128, 128, 0)
#ifdef IC_WIN
  #define IRGB_COLOR_PALEGRAY  IRGBLONG( 192, 192, 192)
#else
  #define IRGB_COLOR_PALEGRAY  IRGBLONG( 204, 204, 204)
#endif

// Values for the bClColor field:
#define  IC_RGB         0       // use RGB values (must be = 0 or false)
#define  IC_COLOR       1       // use Color enum values
#define  IC_SYSCOLOR    2       // use SysColor enum values

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
|                                                                              |
| Default constructor.                                                         |
------------------------------------------------------------------------------*/
IColor::IColor()
   : ucClRed(0), ucClGreen(0), ucClBlue(0), cClColor(black), bClColor(IC_COLOR),
     fColorData( 0 )
{;}

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
|                                                                              |
| Construct an IColor from an RGB mix.                                         |
------------------------------------------------------------------------------*/
IColor::IColor(unsigned char ucRed, unsigned char ucGreen,
               unsigned char ucBlue)
   : ucClRed(ucRed), ucClGreen(ucGreen), ucClBlue(ucBlue), bClColor(IC_RGB),
     fColorData( 0 )
{
   IMODTRACE_DEVELOP("IColor::ctor");
   ITRACE_DEVELOP("From RGB mixes");
}

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
|                                                                              |
| Construct an IColor from a Color enumerator value.                           |
------------------------------------------------------------------------------*/
IColor::IColor(Color clr)
   : ucClRed(0), ucClGreen(0), ucClBlue(0), cClColor(black), bClColor(IC_COLOR),
     fColorData( 0 )

{

   switch (clr) {
   case white:
      ISETRGB( IRGB_COLOR_WHITE );
      cClColor = (long)white;
      break;
   case black:
      ISETRGB( IRGB_COLOR_BLACK );
      cClColor = (long)black;
      break;
   case blue:
      ISETRGB( IRGB_COLOR_BLUE );
      cClColor = (long)blue;
      break;
   case red:
      ISETRGB( IRGB_COLOR_RED );
      cClColor = (long)red;
      break;
   case pink:
      ISETRGB( IRGB_COLOR_PINK );
      cClColor = (long)pink;
      break;
   case green:
      ISETRGB( IRGB_COLOR_GREEN );
      cClColor = (long)green;
      break;
   case cyan:
      ISETRGB( IRGB_COLOR_CYAN );
      cClColor = (long)cyan;
      break;
   case yellow:
      ISETRGB( IRGB_COLOR_YELLOW );
      cClColor = (long)yellow;
      break;
   case darkGray:
      ISETRGB( IRGB_COLOR_DARKGRAY );
      cClColor = (long)darkGray;
      break;
   case darkBlue:
      ISETRGB( IRGB_COLOR_DARKBLUE );
      cClColor = (long)darkBlue;
      break;
   case darkRed:
      ISETRGB( IRGB_COLOR_DARKRED );
      cClColor = (long)darkRed;
      break;
   case darkPink:
      ISETRGB( IRGB_COLOR_DARKPINK );
      cClColor = (long)darkPink;
      break;
   case darkGreen:
      ISETRGB( IRGB_COLOR_DARKGREEN );
      cClColor = (long)darkGreen;
      break;
   case darkCyan:
      ISETRGB( IRGB_COLOR_DARKCYAN );
      cClColor = (long)darkCyan;
      break;
   case brown:
      ISETRGB( IRGB_COLOR_BROWN );
      cClColor = (long)brown;
      break;
   case paleGray:
      ISETRGB( IRGB_COLOR_PALEGRAY );
      cClColor = (long)paleGray;
      break;
   default:
      //if value is not defined set to black.
      ISETRGB( IRGB_COLOR_BLACK );
      cClColor = (long)black;
   } /* endswitch */

}

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IColor::~IColor()
{
  // delete fColorData;
}

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
| index Constructor                                                            |
------------------------------------------------------------------------------*/
IColor::IColor(long index)
   : ucClRed(0), ucClGreen(0), ucClBlue(0), cClColor(black), bClColor(IC_RGB),
     fColorData( 0 )
{
   if (index < -2)
   {
      ITHROWLIBRARYERROR(IC_INVALIDCOLORINDEX,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
   } /* endif */
#ifdef IC_PM
   long rgbColor=0;
   IPresSpaceHandle handle = IWindow::desktopWindow()->presSpace();
   IQUERYRGBCOLOR(handle,index,&rgbColor);
   IWindow::desktopWindow()->releasePresSpace(handle);
   ucClBlue  = (unsigned char)(rgbColor);
   ucClGreen = (unsigned char)(rgbColor >> 8);
   ucClRed   = (unsigned char)(rgbColor >> 16);
#endif
#ifdef IC_WIN
   PALETTEENTRY pe;
   HPALETTE handle = GetStockObject(DEFAULT_PALETTE);
   IQUERYRGBCOLOR(handle,(int)index,&pe);
   DeleteObject(handle);
   ucClBlue  = pe.peBlue;
   ucClGreen = pe.peGreen;
   ucClRed   = pe.peRed;
#endif

}

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
| SysColor Constructor                                                         |
------------------------------------------------------------------------------*/
IColor::IColor(SystemColor scValue)
   : ucClRed(0), ucClGreen(0), ucClBlue(0), cClColor(black),
     bClColor(IC_SYSCOLOR), fColorData( 0 )
{
   IMODTRACE_ALL("IGUIColor::ctor");
   ITRACE_ALL("From SysColor value");
   long lClSysColor;
   switch (scValue) {
   case shadowIconHiliteBgnd:
      lClSysColor = SYSCLR_SHADOWHILITEBGND;
      break;
   case shadowIconHiliteFgnd:
      lClSysColor = SYSCLR_SHADOWHILITEFGND;
      break;
   case shadowIconText:
      lClSysColor = SYSCLR_SHADOWTEXT;
      break;
   case entryFieldBgnd:
   case listBoxBgnd:
      lClSysColor = SYSCLR_ENTRYFIELD;
      break;
   case disableMenuText:
      lClSysColor = SYSCLR_MENUDISABLEDTEXT;
      break;
   case menuHiliteText:
      lClSysColor = SYSCLR_MENUHILITE;
      break;
   case menuHiliteBgnd:
      lClSysColor = SYSCLR_MENUHILITEBGND;
      break;
   case notebookPageBgnd:
      lClSysColor = SYSCLR_PAGEBACKGROUND;
      break;
   case inactiveScrollBar:
   case defaultControl:
      lClSysColor = SYSCLR_FIELDBACKGROUND;
      break;
   case buttonLight:
      lClSysColor = SYSCLR_BUTTONLIGHT;
      break;
   case buttonMiddle:
      lClSysColor = SYSCLR_BUTTONMIDDLE;
      break;
   case buttonDark:
      lClSysColor = SYSCLR_BUTTONDARK;
      break;
   case defaultButton:
#ifdef IC_PM
      lClSysColor = SYSCLR_BUTTONDEFAULT;
#else
      // No real mapping
      lClSysColor = SYSCLR_WINDOWTEXT;
#endif
      break;
   case titleLine:
      lClSysColor = SYSCLR_TITLEBOTTOM;
      break;
   case menuShadow:
   case dialogShadow:
      lClSysColor = SYSCLR_SHADOW;
      break;
   case iconText:
      lClSysColor = SYSCLR_ICONTEXT;
      break;
   case dialogBgnd:
      lClSysColor = SYSCLR_DIALOGBACKGROUND;
      break;
   case hiliteFgnd:
      lClSysColor = SYSCLR_HILITEFOREGROUND;
      break;
   case hiliteBgnd:
      lClSysColor = SYSCLR_HILITEBACKGROUND;
      break;
   case inactiveTitleTextBgnd:
      lClSysColor = SYSCLR_INACTIVETITLETEXTBGND;
      break;
   case activeTitleTextBgnd:
      lClSysColor = SYSCLR_ACTIVETITLETEXTBGND;
      break;
   case inactiveTitleText:
      lClSysColor = SYSCLR_INACTIVETITLETEXT;
      break;
   case activeTitleText:
      lClSysColor = SYSCLR_ACTIVETITLETEXT;
      break;
   case outputText:
      lClSysColor = SYSCLR_OUTPUTTEXT;
      break;
   case windowStaticText:
      lClSysColor = SYSCLR_WINDOWSTATICTEXT;
      break;
   case scrollBar:
      lClSysColor = SYSCLR_SCROLLBAR;
      break;
   case desktopBgnd:
      lClSysColor = SYSCLR_BACKGROUND;
      break;
   case activeTitleBgnd:
      lClSysColor = SYSCLR_ACTIVETITLE;
      break;
   case inactiveTitleBgnd:
      lClSysColor = SYSCLR_INACTIVETITLE;
      break;
   case menuBgnd:
      lClSysColor = SYSCLR_MENU;
      break;
   case windowBgnd:
      lClSysColor = SYSCLR_WINDOW;
      break;
   case frameBorder:
      lClSysColor = SYSCLR_WINDOWFRAME;
      break;
   case menuText:
      lClSysColor = SYSCLR_MENUTEXT;
      break;
   case windowText:
      lClSysColor = SYSCLR_WINDOWTEXT;
      break;
   case titleText:
   case sizeBar:
   case scrollArrow:
      lClSysColor = SYSCLR_TITLETEXT;
      break;
   case activeFrameBorder:
      lClSysColor = SYSCLR_ACTIVEBORDER;
      break;
   case inactiveFrameBorder:
      lClSysColor = SYSCLR_INACTIVEBORDER;
      break;
   case mainWindowBgnd:
      lClSysColor = SYSCLR_APPWORKSPACE;
      break;
   case helpWindowBgnd:
      lClSysColor = SYSCLR_HELPBACKGROUND;
      break;
   case helpText:
      lClSysColor = SYSCLR_HELPTEXT;
      break;
   case helpHiliteText:
      lClSysColor = SYSCLR_HELPHILITE;
      break;
   default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
      break;
   } /* endswitch */

   // store system color index
   cClColor = lClSysColor;
   checkSysRGBValue();

}

/*------------------------------------------------------------------------------
| IColor::IColor                                                               |
| Copy Constructor                                                             |
------------------------------------------------------------------------------*/
IColor::IColor(const IColor& color)
  : ucClRed   ( color.ucClRed ),
    ucClGreen ( color.ucClGreen ),
    ucClBlue  ( color.ucClBlue ),
    bClColor  ( color.bClColor ),
    cClColor  ( color.cClColor ),
    fColorData( 0 )
{
}

/*------------------------------------------------------------------------------
| IColor::operator=                                                            |
| Assignment Operator                                                          |
------------------------------------------------------------------------------*/
IColor& IColor :: operator=(const IColor& color)
{
   IMODTRACE_DEVELOP("IColor::operator=");
   if (this != &color)
   {
      // delete fColorData;
      ucClRed   = color.ucClRed;
      ucClGreen = color.ucClGreen;
      ucClBlue  = color.ucClBlue;
      cClColor  = color.cClColor;
      bClColor  = color.bClColor;
      // fColorData = new IColorData( *(color.fColorData) );
      fColorData = 0;
   } /* endif */
   return *this;
}


/*------------------------------------------------------------------------------
| IColor::index                                                                |
|                                                                              |
| Return the color index closest to the RGB color representation.              |
------------------------------------------------------------------------------*/
long IColor::index() const
{
   ((IColor*)this)->checkSysRGBValue();
#ifdef IC_PM
   //Check and return black or white first, query index API does
   //not return black and white value.
   if (ucClRed == 255 && ucClGreen == 255 && ucClBlue == 255) {
      return CLR_WHITE;
   } else {
      if (ucClRed == 0 && ucClGreen == 0 && ucClBlue == 0) {
         return CLR_BLACK;
      } else {
#endif
         //Query the color index.
         #ifdef IC_PM
          IPresSpaceHandle hps = IWindow::desktopWindow()->presSpace();
          long lIndex = IQUERYCOLORINDEX(hps, asRGBLong());
          IWindow::desktopWindow()->releasePresSpace(hps);
         #endif
         #ifdef IC_WIN
          HPALETTE handle = GetStockObject(DEFAULT_PALETTE);
          long lIndex = IQUERYCOLORINDEX(handle, asRGBLong());
          DeleteObject(handle);
         #endif
         //Check for valid index.
         if (lIndex < -2) {
            ITHROWLIBRARYERROR(IC_INVALIDCOLORINDEX,
                               IBaseErrorInfo::invalidRequest,
                               IException::recoverable);
         } /* endif */
         return lIndex;
#ifdef IC_PM
      } /* endif */
   } /* endif */
#endif
}

/*------------------------------------------------------------------------------
| IColor::asRGBLong                                                            |
------------------------------------------------------------------------------*/
long IColor::asRGBLong() const
{
   ((IColor*)this)->checkSysRGBValue();
#ifdef IC_PM
   return ((ucClRed * 65536) + (ucClGreen * 256) + ucClBlue);
#endif
#ifdef IC_WIN
//   return ((ucClBlue * 65536) + (ucClGreen * 256) + ucClRed);
   return RGB(ucClRed,ucClGreen,ucClBlue);
#endif
}
/*------------------------------------------------------------------------------
| IColor::Color IColor::value                                                  |
|                                                                              |
| Return the Color enumerator value.                                           |
------------------------------------------------------------------------------*/
IColor::Color IColor::value() const
{
   if (bClColor != IC_COLOR)
      {
      Color cColor = black;  //declare & initialize to black;
      switch ( this->asRGBLong() )
         {
         case        IRGB_COLOR_WHITE:
            cColor=white;
            break;
         case        IRGB_COLOR_BLACK:
            cColor = black;
            break;
         case        IRGB_COLOR_BLUE:
            cColor = blue;
            break;
         case        IRGB_COLOR_RED:
            cColor = red;
            break;
         case        IRGB_COLOR_PINK:
            cColor = pink;
            break;
         case        IRGB_COLOR_GREEN:
            cColor = green;
            break;
         case        IRGB_COLOR_CYAN:
            cColor = cyan;
            break;
         case        IRGB_COLOR_YELLOW:
            cColor = yellow;
            break;
         case        IRGB_COLOR_DARKGRAY:
            cColor = darkGray;
            break;
         case        IRGB_COLOR_DARKBLUE:
            cColor = darkBlue;
            break;
         case        IRGB_COLOR_DARKRED:
            cColor = darkRed;
            break;
         case        IRGB_COLOR_DARKPINK:
            cColor = darkPink;
            break;
         case        IRGB_COLOR_DARKGREEN:
            cColor = darkGreen;
            break;
         case        IRGB_COLOR_DARKCYAN:
            cColor = darkCyan;
            break;
         case        IRGB_COLOR_BROWN:
            cColor = brown;
            break;
         case        IRGB_COLOR_PALEGRAY:
            cColor = paleGray;
            break;
         default:
            ITHROWLIBRARYERROR(IC_INVALIDCOLORINDEX,
                               IBaseErrorInfo::invalidRequest,
                               IException::recoverable);
         } /* endswitch */
      // cast away const and update for next time
      IColor* localthis = (IColor*)this;
      localthis->bClColor = IC_COLOR;
      localthis->cClColor = cColor;
      }  // if
   return (IColor::Color)cClColor;
}

/*------------------------------------------------------------------------------
| IColor::SystemColor IColor::systemColor                                      |
|                                                                              |
| Return the SystemColor enumerator value.                                     |
------------------------------------------------------------------------------*/
IColor::SystemColor IColor::systemColor() const
{
   if (bClColor != IC_SYSCOLOR)
     ITHROWLIBRARYERROR(IC_INVALIDCOLORINDEX,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
   return (IColor::SystemColor)cClColor;
}

/*------------------------------------------------------------------------------
| IColor::checkSysRGBValue                                                     |
|                                                                              |
| If this is a system color representation, check and update the RGB values.   |
------------------------------------------------------------------------------*/
IBase::Boolean IColor::checkSysRGBValue( )
{
   Boolean result = false;
   if (bClColor == IC_SYSCOLOR)
      {
      long lRGBColor = IQUERYSYSCOLOR(cClColor);
#ifdef IC_PM
      RGB2* prgb2 = (RGB2*)&lRGBColor;
      this->ucClGreen = prgb2->bGreen;
      this->ucClRed   = prgb2->bRed;
      this->ucClBlue  = prgb2->bBlue;
#endif
#ifdef IC_WIN
      this->ucClGreen = GetGValue(lRGBColor);
      this->ucClRed   = GetRValue(lRGBColor);
      this->ucClBlue  = GetBValue(lRGBColor);
#endif
      result = true;
      }
   return result;
}

/*------------------------------------------------------------------------------
| IDeviceColor::IDeviceColor                                                   |
|                                                                              |
| Construct an IDeviceColor from a DeviceColor enumerator value.               |
------------------------------------------------------------------------------*/
IDeviceColor::IDeviceColor(DeviceColor dclr)
   : IColor()
{
   switch (dclr) {
   case background:
      dcClDeviceColor = background;
#ifdef IC_WIN
      this->setGreen(255);
      this->setRed(255);
      this->setBlue(255);
#endif
      break;
   case neutral:
      dcClDeviceColor = neutral;
#ifdef IC_WIN
      this->setGreen(0);
      this->setRed(0);
      this->setBlue(0);
#endif
      break;
   case defaultColor:
      /************************************************************************/
      /* CLR_DEFAULT is not valid for GpiQueryRGBColor.  It is device         */
      /* dependent and is the default window text color (SYSCLR_WINDOWTEXT)   */
      /* for displays.                                                        */
      /************************************************************************/
      {
      IGUIColor guiColor(IGUIColor::windowText);
      this->setGreen(guiColor.greenMix());
      this->setRed(guiColor.redMix());
      this->setBlue(guiColor.blueMix());
      dcClDeviceColor = defaultColor;
      }
      break;
   default:
     ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                        IBaseErrorInfo::invalidRequest,
                        IException::recoverable);
      break;
   } /* endswitch */

#ifdef IC_PM
   if (dclr != defaultColor)
   {
      HPS hps = IWindow::desktopWindow()->presSpace();
      long lRGBColor = GpiQueryRGBColor(hps, 0,
                        (dclr==background) ? CLR_BACKGROUND : CLR_NEUTRAL);

      IWindow::desktopWindow()->releasePresSpace(hps);

      if (lRGBColor == GPI_ALTERROR)
         ITHROWGUIERROR2("GpiQueryRGBColor",
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);

      RGB2* prgb2 = (RGB2*)&lRGBColor;
      this->setGreen(prgb2->bGreen);
      this->setRed(prgb2->bRed);
      this->setBlue(prgb2->bBlue);
   }
#endif
}

/*------------------------------------------------------------------------------
| IDeviceColor::~IDeviceColor                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IDeviceColor::~IDeviceColor( )
{ }

/*------------------------------------------------------------------------------
| IGUIColor::IGUIColor                                                         |
|                                                                              |
| Construct an IGUIColor from a SysColor enumerator value.                     |
------------------------------------------------------------------------------*/
IGUIColor::IGUIColor(SystemColor scValue)
  : IColor(scValue)
{
}

/*------------------------------------------------------------------------------
| IGUIColor::~IGUIColor                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGUIColor::~IGUIColor( )
{}

/*------------------------------------------------------------------------------
| IGUIColor::setColor                                                          |
|                                                                              |
| Change the GUI component system color.                                       |
------------------------------------------------------------------------------*/
IGUIColor& IGUIColor::setColor(const IColor& newColor)
{
   //Entry structure to API.
#ifdef IC_WIN
   typedef struct {
      int      lIndex;
      COLORREF lColor;
   } Entry;
#else
   typedef struct {
      long lIndex;
      long lColor;
   } Entry;
#endif

   Entry alTable[1];
   alTable[0].lColor = newColor.asRGBLong();
   switch (this->systemColor())
   {
   case shadowIconHiliteBgnd:
      alTable[0].lIndex = SYSCLR_SHADOWHILITEBGND;
      break;
   case shadowIconHiliteFgnd:
      alTable[0].lIndex = SYSCLR_SHADOWHILITEFGND;
      break;
   case shadowIconText:
      alTable[0].lIndex = SYSCLR_SHADOWTEXT;
      break;
   case entryFieldBgnd:
   case listBoxBgnd:
      alTable[0].lIndex = SYSCLR_ENTRYFIELD;
      break;
   case disableMenuText:
      alTable[0].lIndex = SYSCLR_MENUDISABLEDTEXT;
      break;
   case menuHiliteText:
      alTable[0].lIndex = SYSCLR_MENUHILITE;
      break;
   case menuHiliteBgnd:
      alTable[0].lIndex = SYSCLR_MENUHILITEBGND;
      break;
   case notebookPageBgnd:
      alTable[0].lIndex = SYSCLR_PAGEBACKGROUND;
      break;
   case inactiveScrollBar:
   case defaultControl:
      alTable[0].lIndex = SYSCLR_FIELDBACKGROUND;
      break;
   case buttonLight:
      alTable[0].lIndex = SYSCLR_BUTTONLIGHT;
      break;
   case buttonMiddle:
      alTable[0].lIndex = SYSCLR_BUTTONMIDDLE;
      break;
   case buttonDark:
      alTable[0].lIndex = SYSCLR_BUTTONDARK;
      break;
   case titleLine:
      alTable[0].lIndex = SYSCLR_TITLEBOTTOM;
      break;
   case menuShadow:
   case dialogShadow:
      alTable[0].lIndex = SYSCLR_SHADOW;
      break;
   case iconText:
      alTable[0].lIndex = SYSCLR_ICONTEXT;
      break;
   case dialogBgnd:
      alTable[0].lIndex = SYSCLR_DIALOGBACKGROUND;
      break;
   case hiliteFgnd:
      alTable[0].lIndex = SYSCLR_HILITEFOREGROUND;
      break;
   case hiliteBgnd:
      alTable[0].lIndex = SYSCLR_HILITEBACKGROUND;
      break;
   case inactiveTitleTextBgnd:
      alTable[0].lIndex = SYSCLR_INACTIVETITLETEXTBGND;
      break;
   case activeTitleTextBgnd:
      alTable[0].lIndex = SYSCLR_ACTIVETITLETEXTBGND;
      break;
   case inactiveTitleText:
      alTable[0].lIndex = SYSCLR_INACTIVETITLETEXT;
      break;
   case activeTitleText:
      alTable[0].lIndex = SYSCLR_ACTIVETITLETEXT;
      break;
   case outputText:
      alTable[0].lIndex = SYSCLR_OUTPUTTEXT;
      break;
   case windowStaticText:
      alTable[0].lIndex = SYSCLR_WINDOWSTATICTEXT;
      break;
   case scrollBar:
      alTable[0].lIndex = SYSCLR_SCROLLBAR;
      break;
   case desktopBgnd:
      alTable[0].lIndex = SYSCLR_BACKGROUND;
      break;
   case activeTitleBgnd:
      alTable[0].lIndex = SYSCLR_ACTIVETITLE;
      break;
   case inactiveTitleBgnd:
      alTable[0].lIndex = SYSCLR_INACTIVETITLE;
      break;
   case menuBgnd:
      alTable[0].lIndex = SYSCLR_MENU;
      break;
   case windowBgnd:
      alTable[0].lIndex = SYSCLR_WINDOW;
      break;
   case frameBorder:
      alTable[0].lIndex = SYSCLR_WINDOWFRAME;
      break;
   case menuText:
      alTable[0].lIndex = SYSCLR_MENUTEXT;
      break;
   case windowText:
      alTable[0].lIndex = SYSCLR_WINDOWTEXT;
      break;
   case titleText:
   case sizeBar:
   case scrollArrow:
      alTable[0].lIndex = SYSCLR_TITLETEXT;
      break;
   case activeFrameBorder:
      alTable[0].lIndex = SYSCLR_ACTIVEBORDER;
      break;
   case inactiveFrameBorder:
      alTable[0].lIndex = SYSCLR_INACTIVEBORDER;
      break;
   case mainWindowBgnd:
      alTable[0].lIndex = SYSCLR_APPWORKSPACE;
      break;
   case helpWindowBgnd:
      alTable[0].lIndex = SYSCLR_HELPBACKGROUND;
      break;
   case helpText:
      alTable[0].lIndex = SYSCLR_HELPTEXT;
      break;
   case helpHiliteText:
      alTable[0].lIndex = SYSCLR_HELPHILITE;
      break;
   case defaultButton:
#ifdef IC_WIN
      return *this;             // this area not supported
#else
      alTable[0].lIndex = SYSCLR_BUTTONDEFAULT;
      break;
#endif
   default:
      ITHROWLIBRARYERROR(IC_INVALIDENUMVALUE,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable);
      break;
   } /* endswitch */

   if (ISETSYSCOLORS(1, &alTable[0].lIndex, &alTable[0].lColor) == FALSE) {
      ITHROWGUIERROR("WinSetSysColors");
   } /* endif */

   return *this;
}

#if (IC_OBSOLETE <= IC_OBSOLETE_1)
/*------------------------------------------------------------------------------
| IGUIColor::IGUIColor                                                         |
|                                                                              |
| Construct an IGUIColor from a SysColor enumerator value.                     |
------------------------------------------------------------------------------*/
IGUIColor::IGUIColor(SysColor scValue)
  : IColor( (SystemColor)scValue )
{
}
#endif
