/*******************************************************************************
* FILE NAME: ifontdlg.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ifontdlg.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINSTDFONT
  #define INCL_WINWINDOWMGR
  #define INCL_WINSYS
  #define INCL_WINBUTTONS
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #include <iwindefs.h>
  #include <string.h>
}

#include <ifontdlg.hpp>
#ifdef IC_PM
#include <ipmexsta.hpp>
#endif
#include <icoordsy.hpp>
#include <ievent.hpp>
#include <iexcept.hpp>
#include <ifont.hpp>
#include <ihandler.hpp>
#include <iobjwin.hpp>
#include <iplatfrm.hpp>
#include <ireslib.hpp>
#include <ithread.hpp>
#include <itrace.hpp>
#ifdef IC_WIN
#include <icolor.hpp>
#endif // IC_WIN

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFONTDLG_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLStaticConst)
const IFontDialog::Style
  IFontDialog::helpButton        = FNTS_HELPBUTTON,            // 0x00000008
  IFontDialog::applyButton       = FNTS_APPLYBUTTON,           // 0x00000010
  IFontDialog::resetButton       = FNTS_RESETBUTTON,           // 0x00000020
#ifdef IC_PM
  IFontDialog::modeless          = FNTS_MODELESS,              // 0x00000040
#endif
#ifdef IC_WIN
  IFontDialog::modeless          ( 0, IFNTS_MODELESS ),
#endif
  IFontDialog::proportionalOnly  = FNTS_PROPORTIONALONLY,      // 0x00000800
  IFontDialog::bitmapOnly        = FNTS_BITMAPONLY,            // 0x00000100
  IFontDialog::vectorOnly        = FNTS_VECTORONLY,            // 0x00000200
  IFontDialog::fixedWidthOnly    = FNTS_FIXEDWIDTHONLY,        // 0x00000400
  IFontDialog::noSynthesize      = FNTS_NOSYNTHESIZEDFONTS,    // 0x00001000
  IFontDialog::noStyle           = 0,
  IFontDialog::classDefaultStyle = FNTS_RESETBUTTON;

const unsigned long
  IFontDialog::ok                = DID_OK,
  IFontDialog::cancel            = DID_CANCEL;
#pragma data_seg()

#pragma data_seg(ICLNonConst)
  IFontDialog::Style IFontDialog::currentDefaultStyle = FNTS_RESETBUTTON;
#pragma data_seg()

#ifdef IC_PM
#define IFLAGDATA  fData->fl
#endif
#ifdef IC_WIN
#define IFLAGDATA  fData->Flags
#define IC_PREVIEW_SET     0x0001
#define IC_TITLE_SET       0x0002
#define IC_POSITION_SET    0x0004
#endif

#ifdef IC_WIN
class IFontDialogData : public IBase {
/*------------------------------------------------------------------------------
| The IFontDialogData class encapsulates private data and functions            |
| used by the IFontDialog class.  An object of this class is created in        |
| the IFontDialog constructors.  This is the windows version.                  |
------------------------------------------------------------------------------*/
public:
  IFontDialogData ( );

virtual
 ~IFontDialogData ( );

IColor
  clrbackground;

HBRUSH
  bkgrndbrush;

}; // IFontDialogData

/*------------------------------------------------------------------------------
| IFontDialogData::IFontDialogData                                             |
|                                                                              |
------------------------------------------------------------------------------*/
IFontDialogData::IFontDialogData ( )
  : clrbackground ( IColor::dialogBgnd ),
    bkgrndbrush( 0 )
{ }

/*------------------------------------------------------------------------------
| IFontDialogData::~IFontDialogData                                            |
|                                                                              |
------------------------------------------------------------------------------*/
IFontDialogData::~IFontDialogData ( )
{
  if ( bkgrndbrush != 0 )
    DeleteObject(bkgrndbrush);
}
#endif // IC_WIN

//
// Implementation for nested Settings

/*------------------------------------------------------------------------------
| IFontDialog::Settings::Settings                                              |
------------------------------------------------------------------------------*/
IFontDialog::Settings::Settings (IFont* pfont)
{
   fApplyFont = pfont;
#ifdef IC_PM
   fData = new FONTDLG;
   memset(fData, 0, sizeof(FONTDLG) );

   fData->cbSize = sizeof(FONTDLG);
   fData->clrFore = SYSCLR_OUTPUTTEXT;
   fData->clrBack = SYSCLR_ENTRYFIELD;
   fData->usWeight = 5;
   fData->usWidth = 5;
   fData->fAttrs.usRecordLength = sizeof(FATTRS);
#endif
#ifdef IC_WIN
   // Initialize settings data for the new data block
   fSettingsChanged = 0;
   fData = new CHOOSEFONT;
   memset(fData, 0, sizeof(CHOOSEFONT) );
   fData->lStructSize = sizeof(CHOOSEFONT);

   // Copy the font information out of the font passed
   fData->lpLogFont = new LOGFONT;
   fData->Flags |= CF_SCREENFONTS;
   fData->nFontType = SCREEN_FONTTYPE;
#endif
   if (fApplyFont)
   {
#ifdef IC_PM
     memcpy(&(fData->fAttrs), fApplyFont->fattrs(), sizeof(FATTRS));
     fData->fxPointSize = MAKEFIXED(fApplyFont->pointSize(), 0);
#endif
#ifdef IC_WIN
     memcpy( fData->lpLogFont, fApplyFont->logfont(), sizeof(LOGFONT) );
#endif
     IFLAGDATA |= FNTS_INITFROMFATTRS;

     if (fApplyFont->isBitmapOnly())
       IFLAGDATA |= FNTS_BITMAPONLY;

     if (fApplyFont->isVectorOnly())
       IFLAGDATA |= FNTS_VECTORONLY;

     if (fApplyFont->isNonPropOnly())
       IFLAGDATA |= FNTS_FIXEDWIDTHONLY;
   }
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::~Settings                                             |
------------------------------------------------------------------------------*/
IFontDialog::Settings::~Settings ()
{
#ifdef IC_PM
   delete [] fData->pszTitle;
   delete [] fData->pszPreview;
   delete [] fData->pszPtSizeList;
   delete [] fData->pszFamilyname;
#endif
#ifdef IC_WIN
   delete fData->lpLogFont;
#endif
   delete fData;
}

/*------------------------------------------------------------------------------
| IFontDialog::Settings::setFont                                               |
------------------------------------------------------------------------------*/
IFontDialog::Settings& IFontDialog::Settings::setFont( IFont* font )
{
   fApplyFont = font;

   if (fApplyFont)
   {
#ifdef IC_PM
     memcpy(&(fData->fAttrs), fApplyFont->fattrs(), sizeof(FATTRS));
     fData->fxPointSize = MAKEFIXED(fApplyFont->pointSize(), 0);
#endif
#ifdef IC_WIN
     memcpy( fData->lpLogFont, fApplyFont->logfont(), sizeof(LOGFONT) );
#endif
     IFLAGDATA |= FNTS_INITFROMFATTRS;

     if (fApplyFont->isBitmapOnly())
       IFLAGDATA |= FNTS_BITMAPONLY;

     if (fApplyFont->isVectorOnly())
       IFLAGDATA |= FNTS_VECTORONLY;

     if (fApplyFont->isNonPropOnly())
       IFLAGDATA |= FNTS_FIXEDWIDTHONLY;
   }
   return *this;
}

/*------------------------------------------------------------------------------
| IFontDialog::Settings::setTitle                                              |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setTitle (const char* pszTitle)
{
#ifdef IC_PM
   delete [] fData->pszTitle;
   if (pszTitle) {
      fData->pszTitle = (PSZ)new char[strlen(pszTitle)+1];
      strcpy((char*)fData->pszTitle, pszTitle);
   } else {
      fData->pszTitle = 0;
   } /* endif */
#endif
#ifdef IC_WIN
   if (pszTitle)
   {
     fTitle = IString( pszTitle );
     fSettingsChanged |= IC_TITLE_SET;
   }
#endif
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings& IFontDialog::Settings::setTitle                       |
------------------------------------------------------------------------------*/
IFontDialog::Settings& IFontDialog::Settings::setTitle(const IResourceId& id)
{
    IString newText = id.resourceLibrary().loadString(id);
    if (newText.length())
      setTitle(newText);
    return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setPreviewText                                        |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setPreviewText (const char* pszPreview)
{
#ifdef IC_PM
   delete [] fData->pszPreview;
   if (pszPreview) {
      fData->pszPreview = (PSZ)new char[strlen(pszPreview)+1];
      strcpy((char*)fData->pszPreview, pszPreview);
   } else {
      fData->pszPreview = 0;
   } /* endif */
#endif
#ifdef IC_WIN
   if (pszPreview)
   {
     fPreview = IString( pszPreview );
     fSettingsChanged |= IC_PREVIEW_SET;
   }
#endif
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings ::setPosition                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings ::setPosition (const IPoint& position)
{
#ifdef IC_PM
   fData->x = (short)(position.x());
   fData->y = (short)(position.y());
#endif
#ifdef IC_WIN
   fPosition = position;
   fSettingsChanged |= IC_POSITION_SET;
#endif
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setDialogTemplate                                     |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setDialogTemplate(const IResourceId& resId)
{
#ifdef IC_PM
   fData->usDlgId = (unsigned short)(resId.id());
   fData->hMod = resId.resourceLibrary().handle();
#endif
#ifdef IC_WIN
   dlgId = resId.id();
   fData->lpTemplateName = MAKEINTRESOURCE(resId.id());
   fData->hInstance = resId.resourceLibrary().handle();
#endif
   return *this;
}


#ifdef IC_PM
/*------------------------------------------------------------------------------
| IFontDialog::Settings::setSizeList                                           |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setSizeList (const char* pszSizeList)
{
   delete [] fData->pszPtSizeList;
   if (pszSizeList) {
      fData->pszPtSizeList = (PSZ)new char[strlen(pszSizeList)+1];
      strcpy((char*)fData->pszPtSizeList, pszSizeList);
   } else {
      fData->pszPtSizeList = 0;
   } /* endif */
   return *this;
}
#endif // IC_PM


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setDisplayPS                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setDisplayPS (const IPresSpaceHandle& hpsDisplay)
{
#ifdef IC_PM
   fData->hpsScreen = hpsDisplay;
#endif
#ifdef IC_WIN
   fData->hDC = 0;
   fData->Flags &= ~CF_PRINTERFONTS;
#endif
   return *this;
}


/*------------------------------------------------------------------------------
| IFontDialog::Settings::setPrinterPS                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setPrinterPS (const IPresSpaceHandle& hpsPrinter)
{
#ifdef IC_PM
   fData->hpsPrinter = hpsPrinter;
#endif
#ifdef IC_WIN
   fData->hDC = hpsPrinter;
   fData->Flags |= CF_PRINTERFONTS;
#endif
   return *this;
}


#ifdef IC_PM
/*------------------------------------------------------------------------------
| IFontDialog::Settings::setFamily                                             |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setFamily (const char* pszFontFamily)
{
   delete [] fData->pszFamilyname;
   fData->pszFamilyname = new char[strlen(pszFontFamily) + 1];
   strcpy(fData->pszFamilyname, pszFontFamily);
   return *this;
}

/*------------------------------------------------------------------------------
| IFontDialog::Settings::setPointSize                                          |
------------------------------------------------------------------------------*/
IFontDialog::Settings&
  IFontDialog::Settings::setPointSize (unsigned long usPointSize)
{
   fData->fxPointSize = MAKEFIXED(usPointSize, 0);
   return *this;
}
#endif // IC_PM

//
// Implementation for IFontDialog
//
/*------------------------------------------------------------------------------
| IFontDialog::IFontDialog                                                     |
------------------------------------------------------------------------------*/
IFontDialog::IFontDialog(IWindow*     parent,
                         IWindow*     owner,
                         const Style& style )
             : IFrameWindow(IFrameWindow::deferCreation),
#ifdef IC_WIN
               fFontDialogData( new IFontDialogData )
#else
               fFontDialogData( 0 )
#endif // IC_WIN
{
   Settings tempSettings;
   initialize(parent, owner, style, (IHandler*)0, tempSettings );
}

/*------------------------------------------------------------------------------
| IFontDialog::IFontDialog                                                     |
------------------------------------------------------------------------------*/
IFontDialog::IFontDialog(IWindow*        parent,
                         IWindow*        owner,
                         const Settings& settings,
                         const Style&    style )
             : IFrameWindow(IFrameWindow::deferCreation),
#ifdef IC_WIN
               fFontDialogData( new IFontDialogData )
#else
               fFontDialogData( 0 )
#endif // IC_WIN
{
   initialize(parent, owner, style, (IHandler*)0, settings );
}

/*------------------------------------------------------------------------------
| IFontDialog::IFontDialog                                                     |
------------------------------------------------------------------------------*/
IFontDialog::IFontDialog(IWindow*    parent,
                         IWindow*    owner,
                         const Style&    style,
                         const Settings& settings )
             : IFrameWindow(IFrameWindow::deferCreation),
#ifdef IC_WIN
               fFontDialogData( new IFontDialogData )
#else
               fFontDialogData( 0 )
#endif // IC_WIN
{
   initialize(parent, owner, style, (IHandler*)0, settings );

}

/*------------------------------------------------------------------------------
| IFontDialog::IFontDialog                                                     |
------------------------------------------------------------------------------*/
IFontDialog::IFontDialog(IWindow*        parent,
                         IWindow*        owner,
                         IHandler*       handler,
                         const Style&    style,
                         const Settings& settings )
             : IFrameWindow(IFrameWindow::deferCreation),
#ifdef IC_WIN
               fFontDialogData( new IFontDialogData )
#else
               fFontDialogData( 0 )
#endif // IC_WIN
{
   initialize(parent, owner, style, handler, settings );
}


/*------------------------------------------------------------------------------
| IFontDialog::~IFontDialog                                                    |
------------------------------------------------------------------------------*/
IFontDialog::~IFontDialog  ()
{
#ifdef IC_PM
   delete [] fData->pszTitle;
   delete [] fData->pszPreview;
   delete [] fData->pszPtSizeList;
   delete [] fData->pszFamilyname;
#endif
#ifdef IC_WIN
   delete fData->lpLogFont;
   if (fFontDialogData != 0)
      delete fFontDialogData;
#endif
   delete fData;
}


/*------------------------------------------------------------------------------
| IFontDialog::pointSize                                                       |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::pointSize () const
{
#ifdef IC_PM
   return FIXEDINT(fData->fxPointSize);
#endif
#ifdef IC_WIN
   return (unsigned long)(fData->iPointSize/10);
#endif
}


#ifdef IC_PM
/*------------------------------------------------------------------------------
| IFontDialog::fontFamily                                                      |
------------------------------------------------------------------------------*/
IString IFontDialog::fontFamily () const
{
   IString strTemp((char*)fData->pszFamilyname);
   return strTemp;
}


/*------------------------------------------------------------------------------
| IFontDialog::emHeight                                                        |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::emHeight () const
{
   return fData->lEmHeight;
}


/*------------------------------------------------------------------------------
| IFontDialog::xHeight                                                         |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::xHeight () const
{
   return fData->lXHeight;
}


/*------------------------------------------------------------------------------
| IFontDialog::externalLeading                                                 |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::externalLeading () const
{
   return fData->lExternalLeading;
}


/*------------------------------------------------------------------------------
| IFontDialog::nominalPointSize                                                |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::nominalPointSize () const
{
   return fData->sNominalPointSize;
}


/*------------------------------------------------------------------------------
| IFontDialog::fontWeight                                                      |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::fontWeight () const
{
   return fData->usWeight;
}


/*------------------------------------------------------------------------------
| IFontDialog::fontWidth                                                       |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::fontWidth () const
{
   return fData->usWidth;
}
#endif  // IC_PM


/*------------------------------------------------------------------------------
| IFontDialog::isModeless                                                      |
------------------------------------------------------------------------------*/
Boolean IFontDialog::isModeless () const
{
#ifdef IC_PM
   return fData->fl & FNTS_MODELESS;
#endif
#ifdef IC_WIN
   return extendedStyle() & IFNTS_MODELESS;
#endif
}


/*------------------------------------------------------------------------------
| IFontDialog::buttonPressedId                                                 |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::buttonPressedId () const
{
#ifdef IC_PM
   return fData->lReturn;
#endif
#ifdef IC_WIN
   return fReturnVal;
#endif
}

/*------------------------------------------------------------------------------
| IFontDialog::pressedOK                                                       |
------------------------------------------------------------------------------*/
Boolean IFontDialog::pressedOK () const
{
   return buttonPressedId() == DID_OK;
}

/*------------------------------------------------------------------------------
| IFontDialog::returnValue                                                     |
| Returns the system return code from the font                                 |
| dialog.  FNTS_SUCCESSFUL means no error occured.                             |
| A positive return indicates an error.  The codes                             |
| are listed in the fcldlgp.h file.                                            |
------------------------------------------------------------------------------*/
long IFontDialog::returnValue () const
{
#ifdef IC_PM
   return fData->lSRC;
#endif
#ifdef IC_WIN
   return fSRC;
#endif
}

/*------------------------------------------------------------------------------
| IFontDialog::defaultStyle                                                    |
|                                                                              |
| Return the default style for new font dialog objects.                        |
------------------------------------------------------------------------------*/
IFontDialog::Style  IFontDialog::defaultStyle()
{
  return currentDefaultStyle;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IFontDialog::setDefaultStyle                                                 |
|                                                                              |
| Set the default style for new font dialog objects.                           |
------------------------------------------------------------------------------*/
void IFontDialog::setDefaultStyle( const IFontDialog::Style& newDefault )
{
  currentDefaultStyle = newDefault;
}
#endif

/*------------------------------------------------------------------------------
| IFontDialog::convertToGUIStyle                                               |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::convertToGUIStyle(const IBitFlag& guiStyle,
                                             Boolean bExtOnly) const
{
  // Obtain the style from the class (IFrameWindow) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= extendedStyle() & IFNTS_EXTMASK;
  }
  else
  {
    // FNTS_ styles have a one-to-one correspondence to our style bits, and
    // inhabit the lower word of the base GUI style.  Therefore, obtain
    // that portion asis, masking out the upper word.
    ulStyle |= guiStyle.asUnsignedLong() & IFNTS_MASK;
  }

  return( ulStyle );
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IFontDialog::id                                                              |
|                                                                              |
| Get the object identity                                                      |
------------------------------------------------------------------------------*/
unsigned long IFontDialog::id() const
{
  return fDialogId;
}

/*------------------------------------------------------------------------------
| IFontDialog::setId                                                           |
|                                                                              |
| Set the object identifier                                                    |
------------------------------------------------------------------------------*/
IFontDialog& IFontDialog::setId( unsigned long ulId )
{
  fDialogId = ulId;
  return *this;
}

/*------------------------------------------------------------------------------
| IFontDialog::backgroundColor                                                 |
|                                                                              |
| Returns the background color of the font dialog.                             |
------------------------------------------------------------------------------*/
IColor IFontDialog::backgroundColor () const
{
  return fFontDialogData->clrbackground;
}

/*------------------------------------------------------------------------------
| IFontDialog::setBackgroundColor                                              |
|                                                                              |
| Stores the background color of the font dialog.                              |
------------------------------------------------------------------------------*/
IFontDialog& IFontDialog::setBackgroundColor (const IColor &color)
{
  fFontDialogData->clrbackground = color;
  return *this;
}
#endif // IC_WIN

/*------------------------------------------------------------------------------
| IFontDialog::initialize                                                      |
------------------------------------------------------------------------------*/
void IFontDialog::initialize (IWindow*        parent,
                              IWindow*        owner,
                              const Style&    style,
                              IHandler*       handler,
                              const Settings& settings)
{
#ifdef IC_PM
   // It's possible that the first thing an application may do is
   // create a IFontDialog. Since we deferCreation of our parent IFrameWindow
   // we should be sure that PM has been initialized.
   IThread::current().initializeGUI();

   Boolean fMadeHPS = false;

   fApplyFont = settings.fApplyFont;

   _handlerToAdd = handler;

   fData = new FONTDLG;
   memcpy(fData, settings.fData, sizeof(FONTDLG) );

   if (settings.fData->pszTitle) {
      char* src = settings.fData->pszTitle;
      fData->pszTitle = new char[strlen(src) + 1];
      strcpy(fData->pszTitle, src);
   }

   if (settings.fData->pszPreview) {
      char* src = settings.fData->pszPreview;
      fData->pszPreview = new char[strlen(src) + 1];
      strcpy(fData->pszPreview, src);
   }

   if (settings.fData->pszPtSizeList) {
      char* src = settings.fData->pszPtSizeList;
      fData->pszPtSizeList = new char[strlen(src) + 1];
      strcpy(fData->pszPtSizeList, src);
   }

   fData->pszFamilyname = new char[FACESIZE];
   fData->usFamilyBufLen = FACESIZE;
   if (settings.fData->pszFamilyname)
      strncpy(fData->pszFamilyname, settings.fData->pszFamilyname, FACESIZE);

   // Save the extended style to make sure we have a copy of it stored
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   // Convert the style bits into the GUI native style settings
   fData->fl = convertToGUIStyle( style );

   if (fApplyFont) {
      fData->fl |= FNTS_INITFROMFATTRS;
      // Even with FNTS_INITFROMATTRS set, wants the name to be displayed
      // set in pszFamilyname.
      strncpy(fData->pszFamilyname, settings.fData->fAttrs.szFacename, FACESIZE);

   }

   if (fData->x == 0 && fData->y ==0)
      fData->fl |= FNTS_CENTER;

   if (fData->usDlgId) {
      fData->fl |= FNTS_CUSTOM;
   }

   fData->ulUser = (unsigned long)this;

   fData->pfnDlgProc = (FNWP*)_IFontDlgProc;

   if (!fData->hpsScreen && !fData->hpsPrinter) {
      fData->hpsScreen = WinGetPS(HWND_DESKTOP);
      fMadeHPS = TRUE;
   } /* endif */


   // Need to set a variable in IWindow that holds the default procedure
   // so that IFrameHandler will work.
   setDefaultProcedure(IPMCtlStatics::defFontDlgProcAddr());

   if (!IPMCtlStatics::fontDlg(parent == 0 ? desktopWindow()->handle() : parent->handle(),
                   owner  == 0 ? IWindowHandle(0)        : owner->handle(),
                   fData) )
   {
      ITHROWGUIERROR("WinFontDlg");
   }

   if (fMadeHPS)
      WinReleasePS(fData->hpsScreen);
#endif
#ifdef IC_WIN
   /*******************************************************************/
   /* Declare a variable for saving font to apply to the dialog, save */
   /* handler passed into the dialog data (for use in subclass proc)  */
   /*******************************************************************/
   BOOL rc;
   fReturnVal = 0;
   fSRC = 0;
   fApplyFont = settings.fApplyFont;
   _handlerToAdd = handler;

   /*******************************************************************/
   /* Copy the settings data passed into a new settings data block    */
   /*******************************************************************/
   fData = new CHOOSEFONT;
   memcpy( fData, settings.fData, sizeof(CHOOSEFONT) );
   fData->lStructSize = sizeof(CHOOSEFONT);

   /*******************************************************************/
   /* Copy the font information out of the old settings data block    */
   /*******************************************************************/
   fData->lpLogFont = new LOGFONT;
   memcpy( fData->lpLogFont, settings.fData->lpLogFont, sizeof(LOGFONT) );

   /*******************************************************************/
   /* Save the extended style to make sure we have a copy of it stored*/
   /*******************************************************************/
   setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

   /*******************************************************************/
   /* Convert the style bits into the GUI native style settings and   */
   /* default it to display fonts                                     */
   /*******************************************************************/
   fData->Flags = convertToGUIStyle( style );
   fData->Flags |= CF_SCREENFONTS | CF_EFFECTS;
   fData->nFontType = SCREEN_FONTTYPE;

   /*******************************************************************/
   /* Check if an initial font was set and if so, set flag to         */
   /* dialog from that fonts information                              */
   /*******************************************************************/
   if (fApplyFont)
     fData->Flags |= CF_INITTOLOGFONTSTRUCT;

   /*******************************************************************/
   /* Set the dialog id.                                              */
   /*******************************************************************/
   fDialogId = IC_FONT_DIALOG;

   /*******************************************************************/
   /* Check if a custom dialog template was specified                 */
   /*******************************************************************/
   if (fData->lpTemplateName)
   {
     fDialogId = settings.dlgId;
     fData->Flags |= CF_ENABLETEMPLATE;
   }

   /*******************************************************************/
   /* Set the owner of the dialog based on parameters passed          */
   /* If dialog is modeless, create an object window and make it owner*/
   /*******************************************************************/
   IObjectWindow* objWin = 0;
   if (extendedStyle() & IFNTS_MODELESS)
   {
     objWin = new IObjectWindow();
     objWin->setAutoDestroyWindow(true);
     fData->hwndOwner = objWin->handle();
   }
   /*******************************************************************/
   /*  otherwise use the owner handle passed (if any)                 */
   /*******************************************************************/
   else
     fData->hwndOwner = owner == 0 ? IWindowHandle(0) : owner->handle();

   /*******************************************************************/
   /* Save the this pointer in the application data field in template */
   /* to allow access back to IFontDialog object from within hook proc*/
   /*******************************************************************/
   fData->lCustData = (unsigned long)this;

   /*******************************************************************/
   /* Set the hook proc to our dialog procedure since we need to get  */
   /* in to process interactions                                      */
   /*******************************************************************/
   fData->lpfnHook = (LPCFHOOKPROC)_IFontDlgProc;
   fData->Flags |= CF_ENABLEHOOK;

   /*******************************************************************/
   /* Copy settings specific items out of the settings block          */
   /*******************************************************************/
   fPreview = settings.fPreview;
   fTitle = settings.fTitle;
   fSettingsChanged = settings.fSettingsChanged;
   fPosition = settings.fPosition;

   /*******************************************************************/
   /* Everything is now set up, so invoke the dialog                  */
   /*******************************************************************/
   rc = ChooseFont( fData );

   /*******************************************************************/
   /* If complete, then perform any cleanup necessary                 */
   /*******************************************************************/
   if (objWin)
     delete objWin;

   /*******************************************************************/
   /* Check return code to determine correct return value to set      */
   /*******************************************************************/
   if ( rc )
   {
     fReturnVal = DID_OK;
   }
   else
   {
     fReturnVal = DID_CANCEL;
     fSRC = CommDlgExtendedError();
     if (fSRC)
      ITHROWGUIERROR("ChooseFont");
   }
#endif
}

/*------------------------------------------------------------------------------
| _IFontDlgProc                                                                |
|                                                                              |
| This is the custom dialog procedure that is used to initialize the dialog    |
| and perform the ICLUI magic under the covers.  It does the following tasks:  |
|                                                                              |
|  1) Completes the IFrameWindow initialization that had to be deferred in     |
|     the constructor since the window was not yet created                     |
|  2) Attaches any handlers (like an IFontDialogHandler) requested             |
|  3) Processes font information when the dialog is dismissed                  |
|  4) Routes messages in conformance with our general dispatching logic        |
|  5) Performs special case initialization for Windows functions that are      |
|     not provided in the native operating system API                          |
------------------------------------------------------------------------------*/
#ifdef IC_WIN
void* __stdcall _IFontDlgProc( void*
#else
void*   _System _IFontDlgProc( unsigned long
#endif
                                             hwnd,
                               unsigned long msg,
                               void* mp1, void* mp2 )

{
   void* result(FALSE);       // default result
#ifdef IC_PM
   /*****************************************************************/
   /* Get the "this" pointer for window object from the window word */
   /* which was set by WM_CREATE in above.                          */
   /*****************************************************************/
   FONTDLG* pfntstruct = (FONTDLG*)WinQueryWindowPtr(hwnd, QWL_USER);
   IFontDialog* pfntd = (IFontDialog*)pfntstruct->ulUser;

   if (pfntd == 0)                   // No window object pass to defWin.
       return IPMCtlStatics::defFontDlgProc(hwnd, msg, mp1, mp2);
#else
   IFontDialog* pfntd = (IFontDialog*)IQUERYWINDOWULONG( hwnd, DWL_USER );
   if ((pfntd == 0) && (msg != WM_INITDLG))     // No window object to pass
     return FALSE;
#endif

   switch(msg)
   {
#ifdef IC_WIN
      case WM_CTLCOLORDLG:
      case WM_CTLCOLORSTATIC:
      case WM_CTLCOLORBTN:
      {
        IColor clrBgnd = pfntd->backgroundColor();
        IEvent evt(hwnd, msg, mp1, mp2);

        // Set background colors in device context
        SetBkColor( (HDC)evt.parameter1(), clrBgnd.asRGBLong() );

        // Return HBRUSH for painting controls background
        if ( pfntd->fFontDialogData->bkgrndbrush != 0 )
          DeleteObject( pfntd->fFontDialogData->bkgrndbrush );
        pfntd->fFontDialogData->bkgrndbrush =
          CreateSolidBrush (NEARESTPALETTECOLOR(clrBgnd.asRGBLong()));
        return( pfntd->fFontDialogData->bkgrndbrush );
      }
      break;
#endif //IC_WIN

      case WM_INITDLG:
      {
#ifdef IC_WIN
         /*************************************************************/
         /* Get the "this" pointer for font dialog object from lparam */
         /* (saved in custData) and typecast for usage.               */
         /*************************************************************/
         CHOOSEFONT* cfs = (CHOOSEFONT*)mp2;
         pfntd = (IFontDialog*)cfs->lCustData;

         /*************************************************************/
         /* Since this is only passed on WM_INITDIALOG, save in the   */
         /* user window word so that we can retrieve it on other msgs */
         /*************************************************************/
         ISETWINDOWULONG( hwnd, DWL_USER, (LPARAM)pfntd );

         /*************************************************************/
         /* Modify the preview text string, if one was set            */
         /*************************************************************/
         if (pfntd->fSettingsChanged & IC_PREVIEW_SET)
           ISETWINDOWTEXT( IWINDOWFROMID( hwnd, stc5 ), pfntd->fPreview );

         /*************************************************************/
         /* Modify the title text string, if one was set              */
         /*************************************************************/
         if (pfntd->fSettingsChanged & IC_TITLE_SET)
           ISETWINDOWTEXT( hwnd, pfntd->fTitle );

         /*************************************************************/
         /* Hide the color control since we don't support in ifc      */
         /*************************************************************/
         ISHOWWINDOW( IWINDOWFROMID( hwnd, stc4 ), FALSE );
         ISHOWWINDOW( IWINDOWFROMID( hwnd, cmb4 ), FALSE );

         /*************************************************************/
         /* If the dialog position was set, move it to that spot      */
         /*************************************************************/
         if (pfntd->fSettingsChanged & IC_POSITION_SET)
         {
           /***********************************************************/
           /* Get the position requested which is relative to the     */
           /*  owner window and convert it to screen coordinates      */
           /***********************************************************/
           RECT rectOwner;
           IPoint pos = pfntd->fPosition;
           GetWindowRect( cfs->hwndOwner, &rectOwner );
           pos.setX( rectOwner.left + pos.x() );

           /***********************************************************/
           /* If coordinate system is different, recoordinate here    */
           /***********************************************************/
           if (ICoordinateSystem::applicationOrientation() ==
                                      ICoordinateSystem::originLowerLeft )
           {
             RECT rectDlg;
             GetWindowRect( hwnd, &rectDlg );
             pos.setY( rectOwner.bottom - pos.y() -
                       (rectDlg.bottom - rectDlg.top) );
           }
           else
             pos.setY( rectOwner.top + pos.y() );

           /***********************************************************/
           /* Once position is calculated in screen coord,move window */
           /***********************************************************/
           ISETWINDOWPOS( hwnd, NULL, pos.x(), pos.y(), 0, 0,
                          SWP_NOZORDER | SWP_NOSIZE | SWP_NOACTIVATE );
         }
         if (cfs->hwndOwner)
         {
           // Insure that owner is behind this window
           ISETWINDOWPOS( cfs->hwndOwner, hwnd, 0,0,0,0,
                          SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE );
         }
         /*************************************************************/
         /*  Unless there is a user provided handler that processes   */
         /*  WM_INITDIALOG, we want to return true to Windows.  This  */
         /*  tells Windows to set focus to the dialog.                */
         /*************************************************************/
         result = (void*)1;
#endif
         /*************************************************************/
         /* Default IFrameWindow handler needed since we created the  */
         /* Frame with the deferCreation flag.  Also need to add the  */
         /* window to the window list for the same reason             */
         /*************************************************************/
         pfntd->addDefaultHandler();
         pfntd->addToWindowSet(pfntd, IWindowHandle(hwnd));

         /*************************************************************/
         /* If an additional handler is to be used, add it now        */
         /*************************************************************/
         if (pfntd->_handlerToAdd != 0)
            pfntd->_handlerToAdd->handleEventsFor(pfntd);
      }
         break;

      case WM_COMMAND:
      {
         /*************************************************************/
         /* Setup id of apply pushbutton                              */
         /*************************************************************/
         unsigned short applyButton =
#ifdef IC_WIN
                                      psh3;
#else
                                      DID_APPLY_BUTTON;
#endif
         /*************************************************************/
         /* Check if the apply button was hit for a modeless dialog   */
         /*************************************************************/
         if (pfntd->isModeless() && (LOWORD((WPARAM)mp1) == applyButton)
             && (HIWORD((WPARAM)mp1) == BN_CLICKED))
         {
           /***********************************************************/
           /* Get font device context                                 */
           /***********************************************************/
#ifdef IC_WIN
           HDC device = pfntd->fData->hDC;
#else
           HPS device;
           if (pfntd->fData->flFlags & FNTF_SCREENFONTSELECTED)
              device = pfntd->fData->hpsScreen;
           else if (pfntd->fData->flFlags & FNTF_PRINTERFONTSELECTED)
              device = pfntd->fData->hpsPrinter;
           else
              device = 0;
#endif
           /***********************************************************/
           /* If so, then modify any settings font passed             */
           /***********************************************************/
           if (pfntd->fApplyFont)
             pfntd->fApplyFont->setFontFromDialog( *pfntd,
                                                   device );

           /***********************************************************/
           /* then send message to self that font should be applied   */
           /***********************************************************/
           pfntd->handle().sendEvent( IC_UM_DLG_APPLY_PRESSED,
                                      IEventParameter1( pfntd->fApplyFont ),
                                      IEventParameter2( 0 ) );
         }
#ifdef IC_WIN
         /*************************************************************/
         /* Check if the OK button was hit for the dialog             */
         /*************************************************************/
         if (( LOWORD((WPARAM)mp1) == DID_OK ) &&
             ( HIWORD((WPARAM)mp1) == BN_CLICKED ))
         {
           // If Windows 95, copy the logfont structure, since it does
           // not copy it until after the window is destroyed
           if ( (IPlatform::isWin95()) || (IPlatform::isWin32S()) )
             pfntd->handle().sendEvent( WM_CHOOSEFONT_GETLOGFONT,
                           IEventParameter1( 0 ),
                           IEventParameter2( pfntd->fData->lpLogFont ));

           pfntd->fReturnVal = DID_OK;
         }

         /*************************************************************/
         /* If Help button pressed,                                   */
         /*************************************************************/
         if (( LOWORD((WPARAM)mp1) == pshHelp ) &&
             ( HIWORD((WPARAM)mp1) == BN_CLICKED ))
         {

           /***********************************************************/
           /* then post help message to the font dialog window        */
           /***********************************************************/
           pfntd->postEvent( IWindow::help,
                             IEventParameter1( 0 ),
                             IEventParameter2( 0 ) );
         }
#endif
      }
      case WM_SYSCOMMAND:
      case WM_CLOSE:
#ifdef IC_WIN
      /****************************************************************/
      /* Do not dispatch paint messages since doing so causes problems*/
      /*  with painting in the preview window in Windows              */
      /****************************************************************/
      case WM_PAINT:
         return FALSE;
#else
         return IPMCtlStatics::defFontDlgProc(hwnd, msg, mp1, mp2);
#endif

   } /* end switch */

  IEvent newEvt(hwnd, msg, mp1, mp2);

  if (pfntd->dispatch(newEvt))
     result = newEvt.result();
#ifdef IC_PM
  else
     result = IPMCtlStatics::defFontDlgProc(hwnd, msg, mp1, mp2);
#endif

  if(msg==WM_DESTROY)
  {
     pfntd->removeFromWindowSet(pfntd);

     if (pfntd->pressedOK() && pfntd->fApplyFont)
     {
#ifdef IC_WIN
       HDC activeDevice = pfntd->fData->hDC;
#else
       HPS activeDevice;
       if (pfntd->fData->flFlags & FNTF_SCREENFONTSELECTED)
          activeDevice = pfntd->fData->hpsScreen;
       else if (pfntd->fData->flFlags & FNTF_PRINTERFONTSELECTED)
          activeDevice = pfntd->fData->hpsPrinter;
       else
          activeDevice = 0;
#endif
       pfntd->fApplyFont->setFontFromDialog(*pfntd, activeDevice);
     }
  }

  ISETWINDOWULONG(hwnd, DWL_MSGRESULT, (unsigned long)result);
  return result;
}

