#ifndef _IFRAME_
#define _IFRAME_
/*******************************************************************************
* FILE NAME: iframe.hpp                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the declarations of the classes:                        *
*     IFrameWindow                                                             *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <iwindow.hpp>
#include <icconst.h>
#include <icmdevt.hpp>
#include <ihandle.hpp>
#include <irect.hpp>
#include <ireslib.hpp>

class IColor;
class IFont;
class IFrameExtension;
class IFrameExtensions;
class IFrameHandler;
class IFrameWindowData;
class IFrameWindowNotifyHandler;
class IToolBarList;

#pragma pack(4)

class IFrameWindow : public IWindow {
typedef IWindow
  Inherited;
public:
class Style;
enum FrameSource {
  dialogResource,
  noDialogResource,
  tryDialogResource
  };
/*------------------------------- Constructors -------------------------------*/
  IFrameWindow ( unsigned long id = IC_DEFAULT_FRAME_ID,
                 FrameSource   source = tryDialogResource );

  IFrameWindow ( const IResourceId& resId,
                 IWindow*           owner = 0,
                 FrameSource        source = tryDialogResource );

  IFrameWindow ( const IResourceId& resId,
                 IWindow*           parent,
                 IWindow*           owner,
                 FrameSource        source = tryDialogResource );

  IFrameWindow ( const IWindowHandle& hwnd );

  IFrameWindow ( const IResourceId&         resId,
                 const IFrameWindow::Style& style );

  IFrameWindow ( const IFrameWindow::Style& style,
                 const IResourceId&         resId = IC_DEFAULT_FRAME_ID );

  IFrameWindow ( const char*                title,
                 const IResourceId&         resId = IC_DEFAULT_FRAME_ID,
                 const IFrameWindow::Style& style = defaultStyle() );

  IFrameWindow ( const IResourceId&         resId,
                 IWindow*                   parent,
                 IWindow*                   owner ,
                 const IRectangle&          initRect,
                 const IFrameWindow::Style& style = defaultStyle(),
                 const char*                title = 0 );

virtual
 ~IFrameWindow ( );

/*---------------------------------- Styles ----------------------------------*/
INESTEDBITFLAGCLASSDEF1( Style, IFrameWindow, IWindow );

static const Style
  IC_IMPORTU classDefaultStyle,
  IC_IMPORTU accelerator,
  IC_IMPORTU alignNoAdjust,
#ifndef IC_WIN_FLAGNOP
  IC_IMPORTU animated,
  IC_IMPORTU appDBCSStatus,
#endif
  IC_IMPORTU border,
  IC_IMPORTU dialogBackground,
  IC_IMPORTU dialogBorder,
#ifndef IC_WIN_FLAGNOP
  IC_IMPORTU hideButton,
#endif
  IC_IMPORTU horizontalScroll,
  IC_IMPORTU maximizeButton,
  IC_IMPORTU maximized,
  IC_IMPORTU menuBar,
  IC_IMPORTU minimizeButton,
  IC_IMPORTU minimized,
  IC_IMPORTU minimizedIcon,
#ifndef IC_WIN_FLAGNOP
  IC_IMPORTU noMoveWithOwner,
#endif
  IC_IMPORTU shellPosition,
  IC_IMPORTU sizingBorder,
  IC_IMPORTU systemMenu,
#ifndef IC_WIN_FLAGNOP
  IC_IMPORTU systemModal,
#endif
  IC_IMPORTU titleBar,
  IC_IMPORTU verticalScroll,
  IC_IMPORTU windowList;

static Style
  defaultStyle      ( );

static void
  setDefaultStyle   ( const Style&    style );

#ifdef IC_PMWIN
virtual unsigned long
  convertToGUIStyle ( const IBitFlag& style,
                      Boolean         extendedOnly = false ) const;
#endif

/*----------------------- System-Menu-Related Actions ------------------------*/
virtual IFrameWindow
 &close          ( ),
#ifndef IC_MOTIF_FLAGNOP
 &maximize       ( ),
 &restore        ( ),
 &setRestoreRect ( const IRectangle& rect ),
#endif
 &minimize       ( );

#ifndef IC_MOTIF_FLAGNOP
virtual IRectangle
  maximizeRect  ( ) const,
  minimizeRect  ( ) const,
  restoreRect   ( ) const;

static IRectangle
  nextShellRect ( );
#endif

/*---------------------------- Moving and Sizing -----------------------------*/
virtual IFrameWindow
 &moveSizeToClient ( const IRectangle& clientRect );

virtual IRectangle
  clientRectFor    ( const IRectangle& frameRect ) const,
  frameRectFor     ( const IRectangle& clientRect ) const;

/*------------------------------- Border Size --------------------------------*/
#ifndef IC_MOTIF_FLAGNOP
unsigned long
  borderWidth     ( ) const,
  borderHeight    ( ) const;

ISize
  borderSize      ( ) const;
#endif

#ifndef IC_MOTIFWIN_FLAGNOP
IFrameWindow
 &setBorderWidth  ( unsigned long cx ),
 &setBorderHeight ( unsigned long cy ),
 &setBorderSize   ( const ISize&  size ),
 &setBorderSize   ( unsigned long cxcy ),
 &setBorderSize   ( unsigned long cx,
                    unsigned long cy );
#endif

/*------------------------------ Modal Display -------------------------------*/
virtual unsigned long
  showModally ( ),
  result      ( ) const;

virtual IFrameWindow
 &dismiss     ( unsigned long result = 0 ),
 &setResult   ( unsigned long result ),

 &notifyOwner ( unsigned long         id,
                ICommandEvent::Source source = ICommandEvent::unknown,
                Boolean               pointerDevice = false );

/*----------------------------- Application Icon -----------------------------*/
virtual IPointerHandle
  icon    ( ) const;

virtual IFrameWindow
 &setIcon ( const IPointerHandle& icon ),
 &setIcon ( const IResourceId&    iconResId ),
 &setIcon ( unsigned long         iconResId );

#ifdef IC_PMWIN
/*------------------------------ Mouse Pointer -------------------------------*/
virtual IPointerHandle
  mousePointer    ( ) const;

virtual IFrameWindow
 &setMousePointer ( const IPointerHandle& mousePointer );
#endif

/*--------------------------------- Colors -----------------------------------*/
#ifndef IC_WIN_FLAGNOP
virtual IColor
  backgroundColor              ( ) const;
#endif //!IC_WIN_FLAGNOP

#ifndef IC_MOTIFWIN_FLAGNOP
virtual IColor
  disabledBackgroundColor      ( ) const;
#endif //!IC_MOTIFWIN_FLAGNOP

#ifdef IC_PMWIN
#ifndef IC_WIN_FLAGNOP
virtual IFrameWindow
 &resetBackgroundColor         ( ),
 &resetDisabledBackgroundColor ( );
#endif //!IC_WIN_FLAGNOP
#endif //IC_PMWIN

#ifndef IC_MOTIF_FLAGNOP
Boolean
  usesDialogBackground         ( ) const;
#endif

#ifdef IC_WIN
/*------------------------------ Font Functions ------------------------------*/
virtual IWindow
 &setFont       ( const IFont& font );
#endif

/*------------------------------ Client Window -------------------------------*/
virtual IWindowHandle
  clientHandle ( ) const;

virtual IWindow
 *client       ( ) const;

virtual IFrameWindow
 &setClient    ( IWindow* newClient );

/*----------------------------- Frame Extensions -----------------------------*/
enum Location {
#ifndef IC_MOTIFWIN_FLAGNOP
  leftOfTitleBar,
  rightOfTitleBar,
  leftOfMenuBar,
  rightOfMenuBar,
#endif
  leftOfClient,
  rightOfClient,
  aboveClient,
  belowClient
  };

enum SeparatorType {
  none,
  thinLine,
  thickLine
  };

virtual IFrameWindow
 &addExtension     ( IWindow*      newExtension,
                     Location      location,
                     unsigned long widthOrHeight,
                     SeparatorType separator = thinLine ),
#ifdef IC_PMWIN
 &addExtension     ( IWindow*      newExtension,
                     Location      location,
                     int           widthOrHeight,
                     SeparatorType separator = thinLine ),
#endif
 &addExtension     ( IWindow*      newExtension,
                     Location      location,
                     double        percentage,
                     SeparatorType separator = thinLine ),
#ifdef IC_PMWIN
 &addExtension     ( IWindow*      newExtension,
                     Location      location,
                     SeparatorType separator = thinLine ),
#endif

 &removeExtension  ( IWindow*      extension,
                     Boolean       updateDisplay = true ),

 &setExtensionSize ( IWindow*      extension,
                     unsigned long widthOrHeight ),
#ifdef IC_PMWIN
 &setExtensionSize ( IWindow*      extension,
                     int           widthOrHeight ),
#endif
 &setExtensionSize ( IWindow*      extension,
                     double        widthOrHeight ),
#ifdef IC_PMWIN
 &useExtensionMinimumSize ( IWindow* extension ),
#endif

 &update           ( );

virtual Boolean
  isAnExtension    ( const IWindow* window ) const;

/*------------------------------ Window Display ------------------------------*/
virtual IFrameWindow
 &show ( Boolean showWindow = true );

/*---------------------------- Window Destruction ----------------------------*/
virtual IFrameWindow
 &setDestroyOnClose  ( Boolean destroy = true );

Boolean
  willDestroyOnClose ( ) const;

/*--------------------------------- Testing ----------------------------------*/
Boolean
#ifndef IC_MOTIF_FLAGNOP
  isFlashing  ( ) const,
  isMaximized ( ) const,
#endif
  isMinimized ( ) const,
  isModal     ( ) const;

/*------------------------------- Window List --------------------------------*/
#ifndef IC_MOTIF_FLAGNOP
virtual IFrameWindow
 &addToWindowList      ( ),
 &removeFromWindowList ( );
#endif

/*------------------------- Standard Control Access --------------------------*/
virtual IWindowHandle
  handleFor ( const Style& standardControl ) const;

/*----------------------------- Window Flashing ------------------------------*/
#ifndef IC_MOTIF_FLAGNOP
virtual IFrameWindow
 &beginFlashing ( ),
 &endFlashing   ( );

#ifndef IC_WIN
/*------------------------------- DBCS Support -------------------------------*/
virtual IFrameWindow
 &shareParentDBCSStatus ( );
#endif
#endif

/*------------------------------ Sibling Order -------------------------------*/
static void
  setDefaultOrdering ( IWindow::SiblingOrder ordering );
static IWindow::SiblingOrder
  defaultOrdering    ( );

#ifdef IC_PMWIN
/*------------------------------ Window Layout -------------------------------*/
IFrameWindow
 &setLayoutDistorted ( unsigned long layoutAttributesOn,
                       unsigned long layoutAttributesOff );
#endif

/*------------------------- Dialog Behavior Support --------------------------*/
#ifdef IC_WIN
virtual IWindowHandle
  defaultPushButton  ( ) const,
  matchForMnemonic   ( unsigned short character ) const;
#endif

/*--------------------- Notification Event Descriptions ----------------------*/
static INotificationId const
  IC_IMPORTU closeId,
  IC_IMPORTU activateId,
  IC_IMPORTU deactivateId;

/*-------------------------- Observer Notification ---------------------------*/
virtual IFrameWindow
 &enableNotification ( Boolean enable = true );

#ifdef IC_MOTIF
/*------------------------ Compound Control Functions ------------------------*/
virtual IWindowHandle
  topHandle ( ) const;
#endif

#ifndef IC_MOTIF
/*----------------------------- Tool Bar Support -----------------------------*/
IFrameWindow
 &setToolBarList ( IToolBarList* toolBarList );
IToolBarList
 *toolBarList    ( ) const;
#endif

protected:
/*----------------------- Frame Extensions Management ------------------------*/
IFrameExtensions
 *extensions    ( ) const;

unsigned
  findExtension ( IWindow* window );

IFrameWindow
 &setExtensions ( IFrameExtensions* extensions );

/*------------------------------ Implementation ------------------------------*/
#ifdef __BORLANDC__
public:
#endif
static const Style
  IC_IMPORTU deferCreation;
#ifdef __BORLANDC__
protected:
#endif

IFrameWindow
 &tryToLoadDialog ( const IResourceId& resId,
                    IWindow*           parent = 0,
                    IWindow*           owner  = 0,
                    FrameSource        source = tryDialogResource );

#ifndef IC_MOTIF_FLAGNOP
IFrameWindow
 &initialize      ( const IResourceId& resId,
                    const Style&       style,
                    IWindow*           parent   = 0,
                    IWindow*           owner    = 0,
                    const IRectangle&  initRect = nextShellRect(),
                    const char*        title    = 0 );
#endif
#ifdef  IC_MOTIF_FLAGNOP
IFrameWindow
 &initialize      ( const IResourceId& resId,
                    const Style&       style,
                    IWindow*           parent   = 0,
                    IWindow*           owner    = 0,
                    const IRectangle&  initRect = IRectangle(),
                    const char*        title    = 0 );
#endif

#ifdef IC_WIN
unsigned long
  registerFrameClass ( const Style& style,
                       const IResourceId& resId );
#endif

#ifdef IC_PMWIN
IWindowHandle
  create ( unsigned long         id,
           const char*           title,
           unsigned long         style,
           const char*           windowClass,
           const IWindowHandle&  parent,
           const IWindowHandle&  owner,
           const IRectangle&     initRect,
           const void*           ctlData,
           const void*           presParams,
           IWindow::SiblingOrder ordering = defaultOrdering(),
           unsigned long         extendedStyle = 0 );
#endif

#ifdef __BORLANDC__
public:
#endif
IFrameWindow
 &start  ( const IWindowHandle& hwnd );
#ifdef __BORLANDC__
protected:
#endif

IFrameWindow
 &addDefaultHandler    ( ),
 &removeDefaultHandler ( );

#ifdef IC_MOTIF
IWindowHandle
  create ( unsigned long         id,
           const char*           title,
           unsigned long         style,
           IXmCreateFunction     createFunction,
           const IWindowHandle&  parent,
           const IWindowHandle&  owner,
           const IRectangle&     initRect,
           const void*           callerArgList,
           const unsigned int    callerNumberArguments,
           IWindow::SiblingOrder ordering = defaultOrdering() );


/*---------------------- Event-Handling Implementation -----------------------*/
virtual void
  registerCallbacks   ( ),
  unregisterCallbacks ( );

/*------------------------ Compound Control Functions ------------------------*/
virtual Boolean
  isRelatedHandle ( const IWindowHandle& windowHandle ) const;

IFrameWindow
 &attachClient    ( IFrameExtension*  p ),
 &findExtensions  ( IFrameExtension*& topExt,
                    IFrameExtension*& bottomExt,
                    IFrameExtension*& leftExt,
                    IFrameExtension*& rightExt );
virtual Boolean
  isFrameWindow   ( ) const;
#endif

private:
/*------------------------------ Hidden Members ------------------------------*/
  IFrameWindow ( const IFrameWindow& frame );
IFrameWindow
 &operator=    ( const IFrameWindow& frame );

/*--------------------------------- Private ----------------------------------*/
friend class IFrameHandler;
friend class IFrameWindowNotifyHandler;
friend class IMenuBar;
#ifdef IC_MOTIF
friend class ICurrentThread;
static IFrameWindow* currentModalFrameWindow;
#endif //IC_MOTIF
#ifdef IC_WIN
friend class IMenuPrivate;
#endif

IFrameWindow
 &addExtension ( IFrameExtension* extension );

static const Style
  IC_IMPORTU styleBits;
static Style
  currentDefault;

#ifdef IC_PMWIN
static IWindow::SiblingOrder
  siblingCreateOrder;
#endif

IFrameWindowData
 *fFrameWindowData;

}; // class IFrame

INESTEDBITFLAGCLASSFUNCS(Style, IFrameWindow);

#pragma pack()

#endif // _IFRAME_
