/*******************************************************************************
* FILE NAME: igbitmap.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igbitmap.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

extern "C"
  {
  #include <iwindefs.h>
  #include <wingdi.h>
  #include <windef.h>
  #include <math.h>
  #include <igbmodel.hpp>
  #undef word
  }

#include <igbitmap.hpp>
#include <iapp.hpp>
#include <ibmpdata.hpp>
#include <icconst.h>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <igbundle.hpp>
#include <igrafctx.hpp>
#include <igrect.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <ireslib.hpp>
#include <iresstat.hpp>
#include <istring.hpp>
#include <ithread.hpp>
#include <itrace.hpp>
#include <itrnsfrm.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGBITMAP_CPP_
  #include <ipagetun.h>
#endif

class IGBitmapData : public IBase {
typedef IBase
  Inherited;

public:
  IGBitmapData( );

 ~IGBitmapData( );

  HPALETTE hpal;
  Boolean  fUsePal;
static Boolean
  skipMaskBlt;
};

IGBitmapData::IGBitmapData( )
  : hpal(0)
{
}

IGBitmapData::~IGBitmapData( )
{
  if (hpal)
  {
    DeleteObject(hpal);
  }
}

#pragma data_seg(ICLNonConst)
Boolean IGBitmapData::skipMaskBlt(0);
#pragma data_seg()

/***************************************************************/
/* raster opetarors.                                           */
/***************************************************************/
#pragma data_seg(ICLStaticConst)
const long IGBitmap::black            (BLACKNESS          );
const long IGBitmap::notSourceErase   (NOTSRCERASE        );
const long IGBitmap::halftone         (0X00FC008A         );
const long IGBitmap::invert           (NOTSRCCOPY         );
const long IGBitmap::sourceErase      (NOTSRCERASE        );
const long IGBitmap::destInvert       (DSTINVERT          );
const long IGBitmap::patternInvert    (PATINVERT          );
const long IGBitmap::sourceInvert     (SRCINVERT          );
const long IGBitmap::sourceAnd        (SRCAND             );
const long IGBitmap::mergePaint       (MERGEPAINT         );
const long IGBitmap::mergeCopy        (MERGECOPY          );
const long IGBitmap::normal           (SRCCOPY            );
const long IGBitmap::sourcePaint      (SRCPAINT           );
const long IGBitmap::patternCopy      (PATCOPY            );
const long IGBitmap::patternPaint     (PATPAINT           );
const long IGBitmap::white            (WHITENESS          );
#pragma data_seg()

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( const IBitmapHandle& bitmapHandle )
  : fBitmap(bitmapHandle), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fIf(IGBitmap::Bitmap), fMod(0), fGBitmapData(new IGBitmapData())
{
}

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( unsigned long bitmapId, IBase::Boolean useColorData)

  : fBitmap(0), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fIf(IGBitmap::Bitmap), fMod(0), fGBitmapData(new IGBitmapData())
{
  IBitmapData *pBmpData(0);
  IResourceId resourceId(bitmapId);  //create a object wrapper for resource ID.

  // Check if the bitmap has already been loaded in the resource lib.
  // returns a bitmap handle if it is already loaded.
  // bitmapSet() is a class member function of private class IResLibStaticPtr
  // which tracks all loaded bitmaps.
  pBmpData = IResLibStaticPtr::bitmapSet().find( bitmapId,
                                                 resourceId.resourceLibrary().fileName(),
                                                 ISize(0,0) );
  if ( pBmpData )  //bitmap has already been loaded
  {
    fBitmap = IBitmapHandle((void*)pBmpData->handle());   //assign bitmap handle as data member.
  }
  else //load bitmap.
  {
    HBITMAP hBmp;
    HMODULE hMod = IApplication::current().userResourceLibrary().handle();
    if (useColorData)
    {
      // The following function loads the bitmap with the color table if
      // the bitmap resource has one.
      hBmp = LoadBitmapResource( hMod, (LPSTR)MAKEINTRESOURCE(bitmapId),
                                 &(fGBitmapData->hpal) );
    }
    else
    {
      hBmp = LoadBitmap( hMod, (LPSTR)MAKEINTRESOURCE(bitmapId));
    }

    //create a new IBitmapData object.
    pBmpData = new IBitmapData (bitmapId, (unsigned long)hBmp, IString());

    // register bmp with IResLibStaticPtr
    IResLibStaticPtr::bitmapSet().add(pBmpData);
    fBitmap = IBitmapHandle( hBmp );
  }
}

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( const IResourceId& resourceId, IBase::Boolean useColorData)
  : fBitmap(0), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fIf(IGBitmap::Bitmap), fMod(0), fGBitmapData(new IGBitmapData())
{
  IBitmapData *pBmpData(0);

  // Check if the bitmap has already been loaded from file into the resource lib.
  // returns a bitmap handle if it is already loaded.
  // bitmapSet() is a class member function of private class IResLibStaticPtr
  // which tracks all loaded bitmaps.
  pBmpData = IResLibStaticPtr::bitmapSet().find( resourceId,
                                                 resourceId.resourceLibrary().fileName(),
                                                 ISize(0,0) );
  if ( pBmpData )  //bitmap has already been loaded
  {
    fBitmap = IBitmapHandle((void*)pBmpData->handle());   //assign bitmap handle as data member.
  }
  else
  {
    HBITMAP hBmp;
    HMODULE hMod = resourceId.resourceLibrary().handle();
    if (useColorData)
    {
      // The following function loads the bitmap with the color table if
      // the bitmap resource has one.
      hBmp = LoadBitmapResource( hMod, (LPSTR)MAKEINTRESOURCE(resourceId),
                                 &(fGBitmapData->hpal) );
    }
    else
    {
      hBmp = LoadBitmap( hMod, (LPSTR)MAKEINTRESOURCE(resourceId));
    }

    //create a new IBitmapData object.
    pBmpData = new IBitmapData (resourceId, (unsigned long)hBmp, resourceId.resourceLibrary().fileName());

    // register bmp with IResLibStaticPtr
    IResLibStaticPtr::bitmapSet().add(pBmpData);
    fBitmap = IBitmapHandle( hBmp );
  }
}

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( const IString& ImageFilename )
  : fBitmap(0), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fMod(0), fGBitmapData(new IGBitmapData())
{
  loadFromFile( ImageFilename, UNKNOWN_IMAGETYPE );
}

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( const IString& ImageFilename, ImageFormat imageFormat )
  : fBitmap(0), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fMod(0), fGBitmapData(new IGBitmapData())
{
  loadFromFile( ImageFilename, ft(imageFormat));
}

/*------------------------------------------------------------------------------
| IGBitmap::loadFromFile                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::loadFromFile( const IString& filename, int ft )
{
  MOD_ERR me;
  long fgColor;
  long bgColor;
  unsigned long bmp;

  fMod = new MOD;
  me   = ModCreateFromFile(filename, &ft, (char*)"", fMod);

  // map from gbm filetype to IGBitmap enum
  switch (ft)
  {
    case 0:
      fIf = Bitmap;
      break;
    case 1:
      fIf = GIF;
      break;
    case 2:
      fIf = PCX;
      break;
    case 3:
      fIf = TIFF;
      break;
    case 4:
      fIf = Targa;
      break;
    case 5:
      fIf = Amiga;
      break;
    case 11:
      fIf = XBM;
      break;
    case 12:
      fIf = PSEG;
      break;
  } /* endswitch */

  if (me != MOD_ERR_OK)
  {
    ITHROWLIBRARYERROR(ModError(me),
                       IBaseErrorInfo::accessError,
                       IException::recoverable);
  }

  if (!ModMapVisual( fMod, 0,
                     (HBITMAP*)&bmp, &bgColor, &fgColor, &(fGBitmapData->hpal)))
    ITHROWGUIERROR("Error mapping visual.");

  IBitmapData *pBmpData(new IBitmapData (999, bmp, filename ));
  IResLibStaticPtr::bitmapSet().add(pBmpData);
  fBitmap = IBitmapHandle((void*)bmp );
  // remove cast

  if ( fMod->gbm.bpp == 1 )
  {
    IGraphicBundle currentBundle(this->graphicBundle());
    currentBundle.setPenColor( IColor((unsigned char)(fgColor >> 16),
                                      (unsigned char)(fgColor >> 8),
                                      (unsigned char)fgColor ));
    currentBundle.setBackgroundColor( IColor((unsigned char)(bgColor >> 16),
                                             (unsigned char)(bgColor >> 8),
                                             (unsigned char)bgColor ));
    this->setGraphicBundle( currentBundle );
  }


  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::writeToFile                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::writeToFile( const IString& imageFilename,
                                 ImageFormat imageFormat )
{
  MOD_ERR me;
  Boolean fDelete(false);

  if (!fMod)
  {
    fDelete = true;
    fMod = new MOD;
    me = ModCreateFromHBitmap( fMod, fBitmap, fGBitmapData->hpal );
    if (me != MOD_ERR_OK)
    {
      ITHROWLIBRARYERROR(ModError(me),
                         IBaseErrorInfo::accessError,
                         IException::recoverable);
    } /* endif */
  } /* endif */


  me = ModWriteToFile( fMod, ft(imageFormat), imageFilename, (char*)"");
  if (me != MOD_ERR_OK)
  {
    ITHROWLIBRARYERROR(ModError(me),
                       IBaseErrorInfo::accessError,
                       IException::recoverable);
  } /* endif */

  if (fDelete)
  {
    ModDelete(fMod);
    delete fMod;
    fMod = 0;
  }

  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::imageFormat                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::ImageFormat IGBitmap::imageFormat( ) const
{
  return fIf;
}

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( const IGraphicContext& gc,
                    const IRectangle& windowRectangle )
  : fBitmap(0), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fIf(IGBitmap::Bitmap), fMod(0), fGBitmapData(new IGBitmapData())
{
  HBITMAP hBmp;
  HBITMAP hOldBmp;
  IGraphicContext memoryGC;
//POINT           bitmapPoints[3] = { 0,
//                                    0,
//                                    windowRectangle.width(),
//                                    windowRectangle.height(),
//                                    windowRectangle.left(),
//                                    windowRectangle.minY() };

//DPtoLP( gc.handle(), bitmapPoints, 3 );

  hBmp = CreateCompatibleBitmap( gc.handle(), (int)windowRectangle.width(),
                                              (int)windowRectangle.height() );
  if (!hBmp)
    ITHROWGUIERROR("CreateCompatibleBitmap");

  hOldBmp = SelectObject( memoryGC.handle(), hBmp );

  if (!BitBlt( memoryGC.handle(),
               0,0,
               (int)windowRectangle.width(),
               (int)windowRectangle.height(),
               gc.handle(),
               (int)windowRectangle.left(),
               (int)windowRectangle.minY(),
               SRCCOPY ))
    ITHROWGUIERROR("BitBlt");

  SelectObject( memoryGC.handle(), hOldBmp );

  // remove cast
  IBitmapData *pBmpData(new IBitmapData (999, (unsigned long)hBmp, IString()));
  IResLibStaticPtr::bitmapSet().add(pBmpData);
  fBitmap = IBitmapHandle( hBmp );
}

/*------------------------------------------------------------------------------
| IGBitmap::IGBitmap                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::IGBitmap( const IGBitmap&        bitmap,
                    const IRectangle&      rectangle,
                    long                   rasterOperation )
  : fBitmap(0), fMask(0), fTransparent(0), fCheckMaskNeeded(true),
    fVo(IGBitmap::raw), fIf(IGBitmap::Bitmap), fMod(0), fGBitmapData(new IGBitmapData())
{
  HBITMAP hBmp;
  HBITMAP oldBmp1;
  HBITMAP oldBmp2;
  HDC     hdcMem;
  HDC     hdcScreen;
  int     newWidth((int)rectangle.width());
  int     newHeight((int)rectangle.height());
  ISize   bitmapSize(bitmap.size());

  hdcMem    = CreateCompatibleDC( 0 );

  // Create a screen device context to create the bitmap.
  // A memory device context cannot be used because a compatible
  // bitmap is a monochrome bitmap.
  hdcScreen = GetDC( IWindow::desktopWindow()->handle() );

  hBmp = CreateCompatibleBitmap( hdcScreen, newWidth, newHeight );

  ReleaseDC( IWindow::desktopWindow()->handle(), hdcScreen );

  // Now create a memory dc to select the bitmap into.
  hdcScreen = CreateCompatibleDC( 0 );

  if ( !hBmp )
    ITHROWGUIERROR("CreateCompatibleBitmap");

  oldBmp1 = SelectObject( hdcMem,    bitmap.handle() );
  oldBmp2 = SelectObject( hdcScreen, hBmp );

  SetStretchBltMode( hdcMem,    COLORONCOLOR );
  SetStretchBltMode( hdcScreen, COLORONCOLOR );

  if (bitmap.fGBitmapData->hpal)
  {
    SelectPalette(hdcMem, bitmap.fGBitmapData->hpal, false);
    RealizePalette(hdcMem);
  }

  int startY;

  if (ICoordinateSystem::applicationOrientation() ==
      ICoordinateSystem::originUpperLeft)
  {
    startY = (int)rectangle.minY();
  }
  else
  {
    startY = (int)(bitmapSize.height() - rectangle.maxY());
  }

  Boolean fSuccess = StretchBlt( hdcScreen,
                                 0,0,
                                 newWidth,
                                 newHeight,
                                 hdcMem,
                                 (int)rectangle.left(),
                                 startY,
                                 newWidth,
                                 newHeight,
                                 rasterOperation );
  if (!fSuccess)
    ITHROWGUIERROR("StretchBlt");

  SelectObject( hdcMem,    oldBmp1 );
  SelectObject( hdcScreen, oldBmp2 );

  DeleteDC( hdcMem );
  DeleteDC( hdcScreen );

  IBitmapData *pBmpData(new IBitmapData (999, (unsigned long)hBmp, IString()));
  IResLibStaticPtr::bitmapSet().add(pBmpData);
  fBitmap = IBitmapHandle( hBmp );
}

/*------------------------------------------------------------------------------
| IGBitmap::~IGBitmap                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::~IGBitmap()
{
  if (fMask)
    DeleteObject(fMask);

  if (fTransparent)
    delete fTransparent;

  if (fMod)
  {
    ModDelete(fMod);
    delete fMod;
  }
  if (fGBitmapData)
    delete fGBitmapData;
}

/*------------------------------------------------------------------------------
| IGBitmap::loadBitmap                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::loadBitmap( unsigned long bitmapId,
                                   const IModuleHandle& moduleHandle,
                                   const ISize&  bitmapSize )
{
  unsigned long   hbmReturn(0);

  hbmReturn = tryToLoadBitmap(bitmapId, moduleHandle, bitmapSize);

  if (!hbmReturn)
     ITHROWGUIERROR(IString("LoadBitmap")+"Id: "+ IString(bitmapId)+" ");

  return hbmReturn;
}


/*------------------------------------------------------------------------------
| IGBitmap::tryToLoadBitmap                                                    |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::tryToLoadBitmap( unsigned long bitmapId,
                                         const IModuleHandle& moduleHandle,
                                         const ISize&  bitmapSize )
{
  IGraphicContext gc;
  unsigned long   hbmReturn(0);
  HBITMAP         hbmOld(LoadBitmap(moduleHandle,
                                    (LPCSTR)MAKEINTRESOURCE(bitmapId)));

  // If load succeeded, make a copy of the bitmap and delete
  if (hbmOld)
  {
     // remove cast
     hbmReturn = copyBitmap(gc, (unsigned long)hbmOld, bitmapSize);
     if(!DeleteObject(hbmOld))
        ITHROWGUIERROR(IString("DeleteObject")+"Id: "+ IString(bitmapId)+" ");
  }
  return hbmReturn;
}

/*------------------------------------------------------------------------------
| IGBitmap::handle                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IBitmapHandle IGBitmap::handle( ) const
{
  return IBitmapHandle( fBitmap );
}

/*------------------------------------------------------------------------------
| IGBitmap::handle                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPointerHandle IGBitmap::asPointerHandle( ) const
{
//ISize   bitmapSize(this->size());
//HDC     hdcScreen;
//HBITMAP hBmp;

//hdcScreen = GetDC( IWindow::desktopWindow()->handle() );
//hBmp = CreateCompatibleBitmap( hdcScreen, newWidth, newHeight );
//ReleaseDC( IWindow::desktopWindow()->handle(), hdcScreen );


  return IPointerHandle(0);
}

/*------------------------------------------------------------------------------
| IGBitmap::size                                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
ISize IGBitmap::size( ) const
{
  BITMAP bm;

  if (!GetObject(fBitmap, sizeof(bm),(LPSTR)&bm))
    ITHROWGUIERROR("GetObject");

  return ISize( bm.bmWidth, bm.bmHeight );
}

/*------------------------------------------------------------------------------
| IGBitmap::drawOn                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::drawOn( IGraphicContext& gc )
{
  drawOn( gc, IGBitmap::normal );
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::drawOn                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::drawOn( IGraphicContext& gc, long rasterOperation )
{
  POINTL aPtl[4];
  ISize  sizeBitmap(size());
  IPoint newPoint(fPos);

  if (ICoordinateSystem::isConversionNeeded() &&
      (gc.recoordinationHeight() != 0)        &&
      (!gc.isHitTesting())                      &&
      (!gc.isAccumulatingBoundingRect()))
  {
    newPoint.setY( gc.recoordinationHeight() - newPoint.y() - sizeBitmap.height() );
  }

  aPtl[0]   = newPoint.asPOINTL();
  aPtl[2].x = 0;
  aPtl[2].y = 0;
  aPtl[3].x = sizeBitmap.width();
  aPtl[3].y = sizeBitmap.height();
  aPtl[1].x = aPtl[0].x + aPtl[3].x;
  aPtl[1].y = aPtl[0].y + aPtl[3].y;

  return draw( gc, rasterOperation, (void*)aPtl );
}

/*------------------------------------------------------------------------------
| IGBitmap::drawOn                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::drawOn( IGraphicContext& gc,
                            const IPoint&    targetBottomLeft,
                            const IPoint&    targetTopRight,
                            const IPoint&    sourceBottomLeft,
                            const IPoint&    sourceTopRight,
                            long             rasterOperation,
                            CompressMode     compressMode )
{
  POINTL aPtl[4];

  aPtl[0] = targetBottomLeft.asPOINTL();
  aPtl[1] = targetTopRight.asPOINTL();
  aPtl[2] = sourceBottomLeft.asPOINTL();
  aPtl[3] = sourceTopRight.asPOINTL();

  if (ICoordinateSystem::isConversionNeeded() &&
      (gc.recoordinationHeight() != 0)        &&
      (!gc.isHitTesting())                      &&
      (!gc.isAccumulatingBoundingRect()))
  {
    for (int i=0;i<3;i++)
    {
      aPtl[i].y = gc.recoordinationHeight() - aPtl[i].y;
    }
  }
  return draw( gc, rasterOperation, (void*)aPtl );
}

/*------------------------------------------------------------------------------
| IGBitmap::sizeTo                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::sizeTo( const ISize& newSize )
{
  unsigned long id(999);

  if (newSize != this->size())
  {
    Boolean fDelete(false);
    unsigned long bmp;
    MOD modNew;
    MOD_ERR me;
    long bgColor,
         fgColor;

    if (!fMod)
    {
      fDelete = true;
      fMod = new MOD;
      me = ModCreateFromHBitmap( fMod, fBitmap, fGBitmapData->hpal );
      if (me != MOD_ERR_OK)
      {
        ITHROWLIBRARYERROR(ModError(me),
                           IBaseErrorInfo::accessError,
                           IException::recoverable);
      }
    }

    if (fMask)
    {
      DeleteObject(fMask);
      fCheckMaskNeeded = true;
      fMask = 0;
    }

    me = ModResize(fMod, (int)newSize.width(), (int)newSize.height(), &modNew);
    if (me != MOD_ERR_OK)
    {
      ITHROWLIBRARYERROR(ModError(me),
                         IBaseErrorInfo::accessError,
                         IException::recoverable);
    } /* endif */

    if (!ModMapVisual( &modNew, (int)fVo,
                       (HBITMAP*)&bmp, &bgColor, &fgColor, &(fGBitmapData->hpal) ))
    {
      ITHROWGUIERROR("Error mapping visual.");
    }
    // Instead of using 999, lets try and use the id of the original bitmap.
    IBitmapData* pBmpData = IResLibStaticPtr::bitmapSet().find( fBitmap );
    if (pBmpData)
      id = pBmpData->id();

    // Put newly sized bitmap into bitmap set.
    pBmpData = new IBitmapData( id, bmp, IString() );
    IResLibStaticPtr::bitmapSet().add(pBmpData);
    fBitmap = (IBitmapHandle)(void*)bmp;

    if (fDelete)
    {
      ModDelete(&modNew);
      ModDelete(fMod);
      delete fMod;
      fMod = 0;
    }
    else
    {
      ModDelete(fMod);
      ModMove(&modNew, fMod);
    }
  } /* endif */
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::reflectHorizontally                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::reflectHorizontally( )
{
  modReflect(ModReflectHorz);
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::reflectVertically                                                  |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::reflectVertically( )
{
  modReflect(ModReflectVert);
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::transposeXForY                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::transposeXForY( )
{
  modReflect(ModTranspose);
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::rotateBy90                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::rotateBy90( )
{
  modReflect(ModRotate90);
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::rotateBy180                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::rotateBy180( )
{
  modReflect(ModRotate180);
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::rotateBy270                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::rotateBy270( )
{
  modReflect(ModRotate270);
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::setViewOption                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::setViewOption( ViewOption viewOption )
{
  if (fVo != viewOption)
  {
    MOD_ERR me;
    long fgColor;
    long bgColor;
    unsigned long bmp;

    if (!fMod)
    {
      fMod = new MOD;
      me = ModCreateFromHBitmap( fMod, fBitmap, fGBitmapData->hpal );
      if (me != MOD_ERR_OK)
      {
        ITHROWLIBRARYERROR(ModError(me),
                           IBaseErrorInfo::accessError,
                           IException::recoverable);
      } /* endif */
    }

    if (fMask)
    {
      DeleteObject(fMask);
      fCheckMaskNeeded = true;
      fMask = 0;
    }

    if (!ModMapVisual( fMod, (int)viewOption,
                       (HBITMAP*)&bmp, &bgColor, &fgColor, &(fGBitmapData->hpal) ))
      ITHROWGUIERROR("Error mapping visual.");

    // put fbitmap into bitmap set.
    IBitmapData *pBmpData(new IBitmapData (999, (unsigned long)(void*)bmp, IString()));
    IResLibStaticPtr::bitmapSet().add(pBmpData);
    // remove cast
    fBitmap = (IBitmapHandle)(void*)bmp;

    fVo = viewOption;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::viewOption                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap::ViewOption IGBitmap::viewOption( ) const
{
  return fVo;
}

/*------------------------------------------------------------------------------
| IGBitmap::moveTo                                                             |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::moveTo ( const IPoint& point )
{
  fPos = point;
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::position                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IPoint IGBitmap::position ( ) const
{
  return fPos;
}

/*------------------------------------------------------------------------------
| IGBitmap::copy                                                               |
|                                                                              |
| Creates a copy of a bitmap.  It is the caller's responsibility to delete     |
| the bitmap.  This function is useful when you need to create a bitmap and    |
| "give it away" as in placing a bitmap on the clipboard.                      |
------------------------------------------------------------------------------*/
IBitmapHandle IGBitmap::copy(const IBitmapHandle& bitmapHandle )
{
  IGraphicContext gc;
  // remove cast
  return IBitmapHandle( (void*)copyBitmap( gc, (unsigned long)
                                           (void*)bitmapHandle, ISize(0,0)) );
}

/*------------------------------------------------------------------------------
| IGBitmap::copyBitmap                                                         |
|                                                                              |
| Private helper function for copy call above.                                 |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::copyBitmap( IGraphicContext& gc,
                                    unsigned long bmp, const ISize& newSize )
{
  HBITMAP           hbmReturn(0);
  HBITMAP           hbmOld;
  HBITMAP           hbmOld2;
  IGraphicContext   memGc;
  int               oldWidth, oldHeight, newWidth, newHeight;
  BITMAP            bm;
  Boolean           fEnlarge(newSize != ISize(0,0));

  // Query the size of the original bitmap
  if (!GetObject((HBITMAP)bmp, sizeof(bm),(LPSTR)&bm))
    ITHROWGUIERROR("GetObject");

  oldWidth  = (int)bm.bmWidth;
  oldHeight = (int)bm.bmHeight;

  if (fEnlarge)
  {
    newWidth  = (int)newSize.width();
    newHeight = (int)newSize.height();
  }
  else
  {
    newWidth  = oldWidth;
    newHeight = oldHeight;
  }

  hbmOld2 = SelectObject( gc.handle(), (HBITMAP)bmp );

  SetStretchBltMode( memGc.handle(), COLORONCOLOR );
  SetStretchBltMode( gc.handle(), COLORONCOLOR );

  hbmReturn = CreateCompatibleBitmap( gc.handle(), newWidth, newHeight );
  if (!hbmReturn)
    ITHROWGUIERROR("CreateCompatibleBitmap");

  hbmOld = SelectObject(memGc.handle(), hbmReturn );

  Boolean fSuccess = StretchBlt( memGc.handle(),
                                 0,0,
                                 newWidth,
                                 newHeight,
                                 gc.handle(),
                                 0,0,
                                 oldWidth,
                                 oldHeight,
                                 SRCCOPY );

  if (!fSuccess)
    ITHROWGUIERROR("StretchBlt");

  SelectObject( memGc.handle(), hbmOld );
  SelectObject( gc.handle(), hbmOld2 );

  return (unsigned long)hbmReturn;
}

/*------------------------------------------------------------------------------
| IGBitmap::setTransparentColor                                                |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::setTransparentColor(const IColor& aColor)
{
  if (fTransparent)
  {
    delete fTransparent;
    fTransparent = 0;
  }
  fTransparent = new IColor(aColor);

  // If there is an old mask bitmap, delete it

  if (fMask)
  {
    DeleteObject(fMask);
    fMask = 0;
  }

  // Reset flag so we will generate the new mask bitmap if needed

  fCheckMaskNeeded = true;

  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::resetTransparentColor                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::resetTransparentColor()
{
  if (fTransparent)
  {
    delete fTransparent;
    fTransparent = 0;
    if (fMask)
    {
      DeleteObject(fMask);
      fMask = 0;
    }
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::transparentColor                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IColor IGBitmap::transparentColor() const
{
  if (fTransparent)
    return *fTransparent;
  else
  {
    // Need to talk about this and decide if we should throw an exception
    // or just return a color with index of 0
    return IColor(0);
  }
}

/*------------------------------------------------------------------------------
| IGBitmap::hasTransparentColor                                                |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IGBitmap::hasTransparentColor() const
{
  if (fTransparent)
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| IGBitmap::setPalette                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::setPalette( unsigned long palette )
{
  unsigned long oldPal((unsigned long)fGBitmapData->hpal);
  fGBitmapData->hpal = (HPALETTE)palette;
  return oldPal;
}

/*------------------------------------------------------------------------------
| IGBitmap::palette                                                            |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGBitmap::palette( ) const
{
  return (unsigned long)fGBitmapData->hpal;
}

/*------------------------------------------------------------------------------
| IGBitmap::createMask                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
void IGBitmap::createMask ()
{
  ISize         bitmapSize(this->size());
  HDC           hdcMask(CreateCompatibleDC(0));
  HDC           hdcBitmap(CreateCompatibleDC(0));
  HBITMAP       hbmMask(CreateCompatibleBitmap(hdcMask,
                                               (int)bitmapSize.width(),
                                               (int)bitmapSize.height()));
  HBITMAP       hbmOldMask;
  HBITMAP       hbmOldBitmap;
  HPALETTE      oldPal;

  hbmOldMask   = SelectObject( hdcMask, hbmMask );
  hbmOldBitmap = SelectObject( hdcBitmap, fBitmap );

  if (fGBitmapData->hpal)
    oldPal = SelectPalette( hdcBitmap, fGBitmapData->hpal, false );

  SetBkColor(hdcBitmap,NEARESTPALETTECOLOR(fTransparent->asRGBLong()));
  if (!BitBlt( hdcMask,
               0,
               0,
               (int)bitmapSize.width(),
               (int)bitmapSize.height(),
               hdcBitmap,
               0,
               0,
               normal ))
  ITHROWGUIERROR("BitBlt");

  hbmOldMask   = SelectObject( hdcMask, hbmOldMask );
  hbmOldBitmap = SelectObject( hdcBitmap, hbmOldBitmap );
  if (fGBitmapData->hpal)
    SelectPalette( hdcBitmap, oldPal, false );
  DeleteDC( hdcMask );
  DeleteDC( hdcBitmap );

  fMask = IBitmapHandle( hbmMask );
}


/*------------------------------------------------------------------------------
| IGBitmap::draw                                                               |
|                                                                              |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::draw( IGraphicContext& gc, long lRasterOp, void *pPoints )
{
  ITransformMatrix currentTransform;
  IGraphicBundle   currentBundle;
  POINT*           bitmapPoints((POINT*)pPoints);
  long x0, y0, x1, y1;

  // Check if device supports the source transparent mix mode and
  // create the mask bitmap if it does not
  if (hasGraphicBundle())
  {
    currentBundle = gc.graphicBundle();
    gc.setGraphicBundle(graphicBundle());
  }

  if ( fCheckMaskNeeded && fTransparent && !fMask )
  {
    createMask();
    fCheckMaskNeeded = false;
  }

  if (hasTransformMatrix())
  {
    currentTransform = gc.worldTransformMatrix();
    gc.setWorldTransformMatrix(transformMatrix(), transformMethod());
  }

  x0 = bitmapPoints[0].x;
  y0 = bitmapPoints[0].y;
  x1 = bitmapPoints[1].x;
  y1 = bitmapPoints[1].y;

  if (!gc.isHitTesting() && !gc.isAccumulatingBoundingRect())
  {
    IGraphicContext memoryGc;
    HPALETTE        oldPal;

    if (!SetGraphicsMode( gc.handle(), GM_ADVANCED ) &&
        gc.worldTransformMatrix() != ITransformMatrix()) // Windows95, win32s
    {
      long x, y;
      x = bitmapPoints[0].x * gc.worldTransformMatrix().element11() +
          bitmapPoints[0].y * gc.worldTransformMatrix().element21() +
          (long)gc.worldTransformMatrix().element31();
      y = bitmapPoints[0].x * gc.worldTransformMatrix().element12() +
          bitmapPoints[0].y * gc.worldTransformMatrix().element22() +
          (long)gc.worldTransformMatrix().element32();
      bitmapPoints[0].x = x;
      bitmapPoints[0].y = y;

      x = bitmapPoints[1].x * gc.worldTransformMatrix().element11() +
          bitmapPoints[1].y * gc.worldTransformMatrix().element21() +
          (long)gc.worldTransformMatrix().element31();
      y = bitmapPoints[1].x * gc.worldTransformMatrix().element12() +
          bitmapPoints[1].y * gc.worldTransformMatrix().element22() +
          (long)gc.worldTransformMatrix().element32();
      bitmapPoints[1].x = x;
      bitmapPoints[1].y = y;
    }

    if (fGBitmapData->hpal)
    {
      oldPal = SelectPalette(gc.handle(), fGBitmapData->hpal, false);
      RealizePalette(gc.handle());
    }

    SetStretchBltMode( gc.handle(), COLORONCOLOR );

    if (fMask )
    {
      SelectObject( memoryGc.handle(), fBitmap );
      if ( IGBitmapData::skipMaskBlt ||
           !MaskBlt( gc.handle(),
                     (int)bitmapPoints[0].x,
                     (int)bitmapPoints[0].y,
                     (int)(bitmapPoints[1].x - bitmapPoints[0].x),
                     (int)(bitmapPoints[1].y - bitmapPoints[0].y),
                     memoryGc.handle(),
                     (int)bitmapPoints[2].x,
                     (int)bitmapPoints[2].y,
                     fMask,
                     0,0,
                     MAKEROP4(0x00AA0029,(unsigned long)normal) ))
      {

        IGBitmapData::skipMaskBlt = true;

        if (!StretchBlt( gc.handle(),
                         (int)bitmapPoints[0].x,
                         (int)bitmapPoints[0].y,
                         (int)(bitmapPoints[1].x - bitmapPoints[0].x),
                         (int)(bitmapPoints[1].y - bitmapPoints[0].y),
                         memoryGc.handle(),
                         (int)bitmapPoints[2].x,
                         (int)bitmapPoints[2].y,
                         (int)(bitmapPoints[3].x - bitmapPoints[2].x),
                         (int)(bitmapPoints[3].y - bitmapPoints[2].y),
                         sourceInvert ))
          ITHROWGUIERROR("StretchBlt");

        SelectObject( memoryGc.handle(), fMask );

        COLORREF origTextColor = GetTextColor(gc.handle());
        COLORREF origBgndColor = GetBkColor  (gc.handle());

        SetTextColor( gc.handle(), 0x00000000);
        SetBkColor  ( gc.handle(), 0x00FFFFFF);

        if (!StretchBlt( gc.handle(),
                         (int)bitmapPoints[0].x,
                         (int)bitmapPoints[0].y,
                         (int)(bitmapPoints[1].x - bitmapPoints[0].x),
                         (int)(bitmapPoints[1].y - bitmapPoints[0].y),
                         memoryGc.handle(),
                         (int)bitmapPoints[2].x,
                         (int)bitmapPoints[2].y,
                         (int)(bitmapPoints[3].x - bitmapPoints[2].x),
                         (int)(bitmapPoints[3].y - bitmapPoints[2].y),
                         sourceAnd ))
          ITHROWGUIERROR("StretchBlt");

        SetTextColor( gc.handle(), origTextColor );
        SetBkColor  ( gc.handle(), origBgndColor );

        SelectObject( memoryGc.handle(), fBitmap );

        if (!StretchBlt( gc.handle(),
                         (int)bitmapPoints[0].x,
                         (int)bitmapPoints[0].y,
                         (int)(bitmapPoints[1].x - bitmapPoints[0].x),
                         (int)(bitmapPoints[1].y - bitmapPoints[0].y),
                         memoryGc.handle(),
                         (int)bitmapPoints[2].x,
                         (int)bitmapPoints[2].y,
                         (int)(bitmapPoints[3].x - bitmapPoints[2].x),
                         (int)(bitmapPoints[3].y - bitmapPoints[2].y),
                         sourceInvert ))
          ITHROWGUIERROR("StretchBlt");
      }
    }
    else
    {
      SelectObject( memoryGc.handle(), fBitmap );

      IGraphicBundle temp;

      // If the raster-op is halftone, set the pattern to a checker-board
      // and save the current bundle attributes.
      if (lRasterOp == IGBitmap::halftone)
      {
        IGraphicBundle halftone;
        temp = gc.graphicBundle();
        halftone.setFillPattern( IGraphicBundle::dense5 )
                .setPenColor( IColor::white )
                .setBackgroundColor( IColor::black );
        gc.setGraphicBundle( halftone );
      }

      if (!StretchBlt( gc.handle(),
                       (int)bitmapPoints[0].x,
                       (int)bitmapPoints[0].y,
                       (int)(bitmapPoints[1].x - bitmapPoints[0].x),
                       (int)(bitmapPoints[1].y - bitmapPoints[0].y),
                       memoryGc.handle(),
                       (int)bitmapPoints[2].x,
                       (int)bitmapPoints[2].y,
                       (int)(bitmapPoints[3].x - bitmapPoints[2].x),
                       (int)(bitmapPoints[3].y - bitmapPoints[2].y),
                       lRasterOp ))
        ITHROWGUIERROR("StretchBlt");

      // if the raster op is halftone, restore the gc's bundle attributes
      // only if the bitmap does not contain a graphic bundle.  If the
      // bitmap contains a bundle, then it will be restored.
      if (lRasterOp == IGBitmap::halftone && !hasGraphicBundle())
      {
        gc.setGraphicBundle(temp);
      }
    }

    if (fGBitmapData->hpal)
    {
      SelectPalette(gc.handle(), oldPal, false);
      RealizePalette(gc.handle());
    }
  }
  else  // for hit testing and accumlating bounding rectangle
  {
    // Create a temporary IGRectangle with the same dimensions
    // of the bitmap.
    IGraphicBundle bundle;
    bundle.setDrawOperation( IGraphicBundle::fill );
    IGRectangle tempRect( IRectangle( IPoint( x0, y0 ), IPoint( x1, y1 )));
    tempRect.setGraphicBundle( bundle );

    tempRect.drawOn( gc );
    if (tempRect.isHitSelected())
      this->setHitSelected( true );
  }

  if (hasTransformMatrix())
  {
    gc.setWorldTransformMatrix(currentTransform, IGraphic::replace );
  }

  if (hasGraphicBundle())
  {
    gc.setGraphicBundle(currentBundle);
  }

  bitmapPoints[0].x = x0;
  bitmapPoints[0].y = y0;
  bitmapPoints[1].x = x1;
  bitmapPoints[1].y = y1;

  return *this;
}

/*------------------------------------------------------------------------------
| IGBitmap::ft                                                                 |
|                                                                              |
| Return the file type based on the image format.                              |
------------------------------------------------------------------------------*/
int IGBitmap::ft( const ImageFormat& imageFormat )
{
  int imageFt(0);

  if (imageFormat == Bitmap)
    imageFt = 0;
  if (imageFormat == GIF)
    imageFt = 1;
  if (imageFormat == PCX)
    imageFt = 2;
  if (imageFormat == TIFF)
    imageFt = 3;
  if (imageFormat == Targa)
    imageFt = 4;
  if (imageFormat == Amiga)
    imageFt = 5;
  if (imageFormat == XBM)
    imageFt = 11;
  if (imageFormat == PSEG)
    imageFt = 12;
  return imageFt;
}

/*------------------------------------------------------------------------------
| IGBitmap::modReflect                                                         |
|                                                                              |
| General purpose reflector function.                                          |
------------------------------------------------------------------------------*/
IGBitmap& IGBitmap::modReflect(int (* _Optlink reflector)(_MOD*, _MOD*))
{
  Boolean fDelete(false);
  MOD modNew;
  long bgColor,
       fgColor;
  unsigned long bmp;
  MOD_ERR me;

  if (!fMod)
  {
    fDelete = true;
    fMod = new MOD;
    me = ModCreateFromHBitmap( fMod, fBitmap, fGBitmapData->hpal );
    if (me != MOD_ERR_OK)
    {
      ITHROWLIBRARYERROR(ModError(me),
                         IBaseErrorInfo::accessError,
                         IException::recoverable);
    } /* endif */
  }

  if (fMask)
  {
    DeleteObject(fMask);
    fCheckMaskNeeded = true;
    fMask = 0;
  }

  me = (*reflector)(fMod, &modNew);
  if (me != MOD_ERR_OK)
  {
    ITHROWLIBRARYERROR(ModError(me),
                       IBaseErrorInfo::accessError,
                       IException::recoverable);
  } /* endif */

  if (!ModMapVisual( &modNew, (int)fVo,
                     (HBITMAP*)&bmp, &bgColor, &fgColor, &(fGBitmapData->hpal) ))
  {
    ITHROWLIBRARYERROR1(ModError(me),
                        IBaseErrorInfo::accessError,
                        IException::recoverable,
                        "Error mapping visual." );
  }

  // put fbitmap into bitmap set.
  IBitmapData *pBmpData(new IBitmapData (999, bmp, IString()));
  IResLibStaticPtr::bitmapSet().add(pBmpData);
  fBitmap = (IBitmapHandle)(void*)bmp;

  if (fDelete)
  {
    ModDelete(fMod);
    ModDelete(&modNew);
    delete fMod;
    fMod = 0;
  }
  else
  {
    ModDelete( fMod );
    ModMove(&modNew, fMod);
  }
  return *this;
}
