/*******************************************************************************
* FILE NAME: igrszhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igrszhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINFRAMEMGR          // for WM_WINDOWPOSCHANGED
#define INCL_WINSYS
extern "C" {
   #include <iwindefs.h>
}

#include <igrszhdr.hpp>
#include <iwindow.hpp>
#include <iiconctl.hpp>
#include <igraphbt.hpp>
#include <itrace.hpp>
#include <istring.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRSZHDR_CPP_
  #include <ipagetun.h>
#endif


/*******************************************************************/
/* Default constructor here for page tuning.                       */
/*******************************************************************/
IGraphicPushButtonResizeHandler :: IGraphicPushButtonResizeHandler ( )
{}


/*******************************************************************/
/* Empty destructor here for page tuning.                          */
/*******************************************************************/
IGraphicPushButtonResizeHandler :: ~IGraphicPushButtonResizeHandler ( )
{}

/*********************************************************************/
/* IGraphicPushButtonResizeHandler :: windowResize                   */
/*                                                                   */
/* Sizes the graphic window portion of the IGraphicPushButton control*/
/* to be within the border of the push button.                       */
/*********************************************************************/
Boolean IGraphicPushButtonResizeHandler :: windowResize( IResizeEvent& evt )
{
  IGraphicPushButton* button = (IGraphicPushButton*)evt.window();
  ITRACE_ALL( IString("IGraphicPushButtonResizeHandler hwnd=") +
              IString( evt.handle().asUnsigned() ).d2x() );
  if ((evt.oldSize() != evt.newSize()) && !(button->isSizeToGraphic()))
  {                               // Size change occurred
     ISize border=button->marginSize();
     long inith  = border.height(),
          initw  = border.width(),
          width  = -initw,
          height = -inith;
     if ((width += evt.newSize().width() ) < 0)
        width = 0;
     if ((height+= evt.newSize().height()) < 0)
        height = 0;
     ITRACE_ALL( IString("IGraphicPushButtonResizeHandler hwnd=") +
                 IString( evt.handle().asUnsigned() ).d2x() +
                 IString(" newRect=") +
                 IRectangle(initw,inith,width,height).asString() );
     button->graphicWindow().moveSizeTo(IRectangle(initw,inith,width,height));
  }                               // End size change occurred
#ifdef IC_WIN
  else if (button->isSizeToGraphic())
  {
     // need to center the graphic in the button
     IPoint center =
        IRectangle( IPoint(0,0), evt.newSize() ).centerXCenterY();
     IPoint newOrigin =
        button->graphicWindow().rect().centerAt( center ).minXMinY();
     button->graphicWindow().moveTo( newOrigin );
  }
#endif
  return true;
}

