/*******************************************************************************
* FILE NAME: iiconhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iiconhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C"
  {
  #define INCL_GPIBITMAPS
  #define INCL_GPIPRIMITIVES
  #define INCL_WINPOINTERS
  #define INCL_GPILOGCOLORTABLE
  #include <iwindefs.h>
  }

#include <iiconhdr.hpp>

#include <ipainevt.hpp>
#include <iswp.hpp>
#include <itrace.hpp>
#include <icontrol.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <iiconctl.hpp>
#include <igrafctx.hpp>
#include <igrect.hpp>
#include <igbitmap.hpp>
#include <iiconctp.hpp>
#include <iplatfrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IICONHDR_CPP_
  #include <ipagetun.h>
#endif

IIconHandler :: IIconHandler() {;}
IIconHandler :: ~IIconHandler() {;}

/*-------------------------------- Overrides -----------------------------------
| This inherited function is overridden:                                       |
|   paintWindow - Process the paint events for a Icon control window.          |
------------------------------------------------------------------------------*/
Boolean IIconHandler::paintWindow( IPaintEvent& evt )
{
  IMODTRACE_ALL("IIconHandler::paintWindow");
  IIconControl* icon = (IIconControl*)evt.window();

  if (icon->icon())
  {

    IRectangle      winrect(icon->size());
    IGraphicContext memoryGC( winrect.size() );
    IGraphicContext gc      ( evt.presSpaceHandle() );

    // Prepare the background of the memory GC
    if (icon->hasFillBackground())
    {
      // fill background ... fill with the specified color
#ifndef IC_WIN
      ICREATELOGCOLORTABLE( memoryGC.handle(), 0L,
                            LCOLF_RGB, 0L, 0L, 0 );
#endif
      memoryGC.setFillColor( icon->fillColor() );
      memoryGC.setDrawOperation( IGraphicBundle::fill );
      memoryGC.draw( IGRectangle( winrect ) );
    }
    else
    {
      // copy the image currently in the window to the memory GC
      IGBitmap( gc, winrect ).drawOn( memoryGC );
    }
#ifdef IC_WIN
    if ( (icon->bitmap() != icon->fIconControlData->fbmColor)  &&
         (icon->bitmap() != icon->fIconControlData->fbmMask ) )
    {
       return false;                 //Let the bitmap painthandler
                                     //have this since must have called
                                     //setBitmap on this IIconControl
    }

    if (!IPlatform::isWin32S())
    {
      // 32 bit Windows
      DrawIconEx(memoryGC.handle(),
                 0,0,
                 icon->icon(),
                 (int)winrect.width(),
                 (int)winrect.height(),
                 0,0, DI_NORMAL );
    }
    else
#endif
    {
      // 32 bit Windows processing is all done above...PM and Win32s use
      // the code below.
#ifdef IC_WIN
       ITRACE_ALL( IString(" bitmap()=") +
                   IString( icon->bitmap().asUnsigned() ).d2x() +
                   IString(" fbmColor=") +
                   IString( icon->fIconControlData->fbmColor.asUnsigned() ).d2x() +
                   IString(" fbmMask=") +
                   IString( icon->fIconControlData->fbmMask.asUnsigned() ).d2x() );

       IGBitmap        bmMask ( icon->fIconControlData->fbmMask );
#else
       POINTERINFO
          pointerinfo;
       IQUERYPOINTERINFO(icon->icon(),&pointerinfo);
       if ( (icon->bitmap() != pointerinfo.hbmColor) &&
            (icon->bitmap() != pointerinfo.hbmPointer))
          return false;                 //Let the bitmap painthandler
                                        //have this since must have called
                                        //setBitmap on this IIconControl
       IGBitmap        bmMask ( IBitmapHandle(pointerinfo.hbmPointer) );
#endif
       POINTL          aptlPoints[4];

       ISize bmSize  = bmMask.size();
       // set up location of AND mask (upper part of bitmap)
#ifdef IC_WIN
       // Windows ... XOR mask only present in B/W icon
       long bmHeight = bmSize.height();
       if ( icon->fIconControlData->fbmColor == 0)
          {
          // black and white icon
          bmHeight *= .5;
          }
       aptlPoints[2].y = 0;
#else
       // PM... XOR mask present in all icons
       long bmHeight = bmSize.height() * .5;
       aptlPoints[2].y = bmHeight;
#endif
       aptlPoints[0].x = 0;
       aptlPoints[0].y = 0;
       aptlPoints[1].x = winrect.width();
       aptlPoints[1].y = winrect.height();
       aptlPoints[2].x = 0L;
       aptlPoints[3].x = bmSize.width();
       //The following values are special for the AND MASK
       aptlPoints[3].y = aptlPoints[2].y + bmHeight ;

#ifdef IC_PM
       //Set up the AND MASK colors
       IColor lcolor  = gc.penColor();
       IColor lbcolor = gc.backgroundColor();
       memoryGC.setPenColor( IColor( IColor::white ) );
       memoryGC.setBackgroundColor( lbcolor );
#endif

       bmMask.drawOn( memoryGC,
                      IPoint(aptlPoints[0]),   // target corner
                      IPoint(aptlPoints[1]),   // target corner
                      IPoint(aptlPoints[2]),   // source corner
                      IPoint(aptlPoints[3]),   // source corner
                      IGBitmap::sourceAnd );   // operation


       //The following values are for the color bitmap and the XOR MASK
#ifdef IC_WIN
       if ( icon->fIconControlData->fbmColor == 0)
          aptlPoints[2].y = bmHeight;
       else
#endif
          aptlPoints[2].y = 0L;
       aptlPoints[3].y = aptlPoints[2].y + bmHeight;


       // Paint the color bitmap if there is one.
#ifdef IC_WIN
       if ( icon->fIconControlData->fbmColor != 0 )
          {
          // draw the color bitmap
          IGBitmap  bmColor(
             icon->fIconControlData->fbmColor );
          bmColor.drawOn( memoryGC,
                          IPoint(aptlPoints[0]),   // target corner
                          IPoint(aptlPoints[1]),   // target corner
                          IPoint(aptlPoints[2]),   // source corner
                          IPoint(aptlPoints[3]),   // source corner
                          IGBitmap::sourceInvert );   // operation
          } // Color
       else
          {
          // draw the XOR mask for B/W bitmap.  Always done in PM,
          // only done for B/W bitmap in Windows
          bmMask.drawOn( memoryGC,
                         IPoint(aptlPoints[0]),   // target corner
                         IPoint(aptlPoints[1]),   // target corner
                         IPoint(aptlPoints[2]),   // source corner
                         IPoint(aptlPoints[3]),   // source corner
                         IGBitmap::sourceInvert );   // operation
          }  // mono
#else
       if ( pointerinfo.hbmColor != 0 )
          {
          // draw the color bitmap
          IGBitmap  bmColor(
             IBitmapHandle(pointerinfo.hbmColor) );
          bmColor.drawOn( memoryGC,
                          IPoint(aptlPoints[0]),   // target corner
                          IPoint(aptlPoints[1]),   // target corner
                          IPoint(aptlPoints[2]),   // source corner
                          IPoint(aptlPoints[3]),   // source corner
                          IGBitmap::sourcePaint  );   // operation
          } // Color
        // draw the XOR mask for B/W bitmap.  Always done in PM,
        bmMask.drawOn( memoryGC,
                       IPoint(aptlPoints[0]),   // target corner
                       IPoint(aptlPoints[1]),   // target corner
                       IPoint(aptlPoints[2]),   // source corner
                       IPoint(aptlPoints[3]),   // source corner
                       IGBitmap::sourceInvert );   // operation
#endif
     }  // PM or Win32s

     // We have painted to the memoryGC.  Put the results into the window,
     // halftoning if the control is disabled
     IGBitmap( memoryGC, winrect)
        .drawOn( gc,
                 winrect.minXMinY(),
                 winrect.maxXMaxY(),
                 winrect.minXMinY(),
                 winrect.maxXMaxY(),
                 (!icon->isEnabled() || icon->isHalftone()) ?
                    IGBitmap::halftone : IGBitmap::normal );

  }   // has icon
  else
  {
     // no icon set
     if (icon->hasFillBackground())
     {
#ifndef IC_WIN
       ICREATELOGCOLORTABLE( evt.presSpaceHandle(), 0L,
                             LCOLF_RGB, 0L, 0L, 0 );
#endif
       evt.clearBackground(icon->fillColor());
     }
     return false;
  } /* endif */

  unsigned long length(icon->textLength());
  if (length)
  {
    if (icon->hasFillBackground())
    {
      // Turn off fill background so that we do not overpaint icon
      icon->disableUpdate();
      icon->disableFillBackground();
      icon->enableUpdate();
      Inherited::paintWindow(evt);
      icon->disableUpdate();
      icon->enableFillBackground();
      icon->enableUpdate();
    }
    else
    {
      Inherited::paintWindow(evt);
    }
  }
  return true;
}
