/*******************************************************************************
* FILE NAME: ilistbas.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ilistbas.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C"
{
  #define INCL_WINLISTBOXES      // LIT_xxx, LM_xxx, LS_xxx
  #define INCL_WINSYS            // SV_xxx, PP_xxx
  #include <iwindefs.h>
}

#include <inotifev.hpp>
#include <ilistbas.hpp>
#include <istring.hpp>
#include <ireslib.hpp>
#include <ihandle.hpp>
#include <ihandler.hpp>
#include <iexcept.hpp>
#include <icconst.h>
#include <itrace.hpp>
#include <icolor.hpp>
#include <ifont.hpp>
#include <iplatfrm.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ILISTBAS_CPP_
  #include <ipagetun.h>
#endif

/***************************************************************/
/* Public list box styles.                                     */
/***************************************************************/
#pragma data_seg(ICLStaticConst)
const IBaseListBox::Style
  IBaseListBox::multipleSelect    = LS_MULTIPLESEL,            // 0x00000001
  IBaseListBox::drawItem          = LS_OWNERDRAW,              // 0x00000002
  IBaseListBox::noAdjustPosition  = LS_NOADJUSTPOS,            // 0x00000004
  IBaseListBox::horizontalScroll  = LS_HORZSCROLL,             // 0x00000008
  IBaseListBox::extendedSelect    = LS_EXTENDEDSEL,            // 0x00000010
#ifdef IC_WIN
  IBaseListBox::border3D          ( 0, WS_EX_CLIENTEDGE ),
  IBaseListBox::classDefaultStyle ( LS_HORZSCROLL  |
                                    LS_NOADJUSTPOS |
                                    WS_VISIBLE,
                                    WS_EX_CLIENTEDGE );
#else
  IBaseListBox::border3D          ( 0, 0 ),
  IBaseListBox::classDefaultStyle ( LS_HORZSCROLL  |
                                    LS_NOADJUSTPOS |
                                    WS_VISIBLE,
                                    0 );
#endif

const unsigned long
  IBaseListBox::notFound = (unsigned long)ULONG_MAX,
  IBaseListBox::first    = (unsigned long)ULONG_MAX;
#pragma data_seg()

/***************************************************************/
/* Default style for new objects (initial value)               */
/***************************************************************/
#pragma data_seg(ICLNonConst)
#ifdef IC_WIN
  IBaseListBox::Style IBaseListBox::currentDefaultStyle ( LS_HORZSCROLL  |
                                                  LS_NOADJUSTPOS |
                                                  WS_VISIBLE,
                                                  WS_EX_CLIENTEDGE );
#else
  IBaseListBox::Style IBaseListBox::currentDefaultStyle ( LS_HORZSCROLL  |
                                                  LS_NOADJUSTPOS |
                                                  WS_VISIBLE,
                                                  0 );
#endif
#pragma data_seg()

class IBaseListBoxData : public IBase {
/*******************************************************************************
* The IBaseListBoxData class encapsulates private data and functions used by the *
* IBaseListBox class.  An object of this class is created in the IBaseListBox  *
* constructors.                                                                *
*******************************************************************************/
public:
  IBaseListBoxData ( ) ;

virtual
 ~IBaseListBoxData ( );

unsigned long
  minimumRows,
  minimumCharacters;

#ifdef IC_WIN
HBRUSH
  bkgrndbrush;

#endif // IC_WIN

private:
  IBaseListBoxData ( const IBaseListBoxData& );
IBaseListBoxData
 &operator=    ( const IBaseListBoxData& );

}; // IBaseListBoxData

/*------------------------------------------------------------------------------
| IBaseListBoxData::IBaseListBoxData                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IBaseListBoxData::IBaseListBoxData ( )
    : minimumRows( 4 ),
      minimumCharacters( 25 )
#ifdef IC_WIN
      ,
      bkgrndbrush( 0 )
#endif // IC_WIN
{ }

/*------------------------------------------------------------------------------
| IBaseListBoxData::~IBaseListBoxData                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IBaseListBoxData::~IBaseListBoxData ( )
{
#ifdef IC_WIN
  if ( bkgrndbrush != 0 )
    DeleteObject(bkgrndbrush);
#endif // IC_WIN
}

#ifdef IC_WIN

class IBaseListHandler : public IHandler {
typedef IHandler
  Inherited;
public:

/*------------------------------- Constructors -------------------------------*/
  IBaseListHandler ( );

virtual
  ~IBaseListHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

}; // class IBaseListHandler

/*------------------------------------------------------------------------------
| IBaseListHandler::IBaseListHandler                                           |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
IBaseListHandler::IBaseListHandler()
{ }

/*------------------------------------------------------------------------------
| IBaseListHandler::~IBaseListHandler                                          |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IBaseListHandler::~IBaseListHandler()
{ }

/*------------------------------------------------------------------------------
| IBaseListHandler::dispatchHandlerEvent                                       |
|                                                                              |
| Intercepts control color event                                               |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
IBase::Boolean IBaseListHandler::dispatchHandlerEvent( IEvent& event )
{
  IMODTRACE_ALL("IBaseListHandler::dispatchHandlerEvent");
  Boolean
    result = false;

  switch( event.eventId() )
  {
    case WM_KEYDOWN:
      // If this is an owner draw listbox on the oldshell, we need to handle
      // page down processing due to a Windows bug.
      if (event.parameter1() == VK_PAGEDOWN)
      {
         IBaseListBox *listBox = (IBaseListBox*)(event.controlWindow());
         if (listBox && listBox->isDrawItem() &&
             !(IPlatform::isWin95() || IPlatform::isNTNewShell()))
         {
            IWindow *owner = listBox->owner();

            // Get number of items in the list box.
            unsigned long itemCount = listBox->count();
            if (!itemCount || !listBox->numberOfSelections())
               break;

            // Get the index for the item which is to be at the top.
            unsigned long topIndex = listBox->selection();
            if (topIndex == itemCount - 1)
               break;

            // Get the height of the client area of the list box.
            RECT rect;
            GetClientRect( listBox->handle(), &rect );
            unsigned long lbHeight = rect.bottom - rect.top;

            // Get the index for the last visible item in the list
            // box based on the new top item.
            unsigned long bottomIndex = 0,
                          itemHeights = 0;

            // Determine the last item in the listbox which should be
            // visible after the page down.
            MEASUREITEMSTRUCT mis;
            mis.CtlType    = ODT_LISTBOX;
            mis.CtlID      = (UINT)listBox->id();
            mis.itemID     = (UINT)topIndex;
            mis.itemWidth  = 0;
            mis.itemHeight = 0;
            while (!bottomIndex)
            {
               // Get the item height of the next item.
               mis.itemData = listBox->sendEvent( LB_GETITEMDATA,
                                       (unsigned long)mis.itemID,
                                       0 );
               owner->sendEvent( WM_MEASUREITEM,
                                 (unsigned long)mis.CtlID,
                                 (void*)&mis );
               itemHeights += mis.itemHeight;

               if (itemHeights <= lbHeight )
                  if (mis.itemID == (itemCount - 1))
                     bottomIndex = mis.itemID;
                  else
                     mis.itemID++;
               else
                  bottomIndex = --mis.itemID;
            }

            // Special case: When the height of the list box is less than or
            // equal to the height of an item, the list box displays only one
            // item or part of an item at a time. So, when paging down we need
            // to display the next item in the list.
            if (bottomIndex <= topIndex)
               bottomIndex = ++topIndex;

            // Move the item that had the focus to the top, then set the
            // selection to the last visible item in the list box.
            listBox->disableUpdate();
            listBox->select( bottomIndex );
            listBox->setTop( topIndex );
            listBox->enableUpdate();
            listBox->refresh();

            // Disabling update prevents the scroll bar from being updated.
            // Set the new position explicitly.
            unsigned long pos = GetScrollPos(listBox->handle(), SB_VERT);
            SetScrollPos(listBox->handle(), SB_VERT, (int)pos, TRUE);

            result = true;
         }
      }
      break;

    case WM_CTLCOLORLISTBOX:
         {
         IWindow* ctlwin =
           IWindow::windowWithHandle(IWindowHandle(event.parameter2()) );
         IColor clrFgnd =
           ((IBaseListBox *) ctlwin)->foregroundColor();
         IColor clrBgnd =
           ((IBaseListBox *) ctlwin)->backgroundColor();

         // Set foreground and background text colors in device context
         SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
         SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

         // Return HBRUSH for painting controls background
         if ( ((IBaseListBox *) ctlwin)->fBaseListBoxData->bkgrndbrush != 0 )
           DeleteObject( ((IBaseListBox *) ctlwin)->fBaseListBoxData->bkgrndbrush);
         ((IBaseListBox *) ctlwin)->fBaseListBoxData->bkgrndbrush =
           CreateSolidBrush (NEARESTPALETTECOLOR(clrBgnd.asRGBLong()));
         event.setResult( ((IBaseListBox *) ctlwin)->fBaseListBoxData->bkgrndbrush );

         result = true;
         }
      break;

     // Adding code to trap accelerator/mnemonics and send directly to
     // owner (allowing it to naturally progress through dispatcher results
     // in an audible beep caused by the default window procedure because
     // it cannot find a match for a mnemonic that we process in
     // ICanvas or IFrameWindow), eating original event.
     case WM_SYSCHAR:
     {
       IWindow* window = event.window();
        if (window)
          window->owner()->sendEvent( event );
        result = true;
     }
     break;

    default:
      break;
  }
  return result;
}

/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointer to the object |
| when the wrapper is destructed.                                              |
------------------------------------------------------------------------------*/
static struct DefaultHandler
{
  operator IBaseListHandler* ();
IBaseListHandler
 *operator -> ()
    {
    return *this;
    }
 ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
IBaseListHandler
 *ptr;
} defaultHandler;

DefaultHandler::operator IBaseListHandler* ( )
  {
  if ( !ptr )
    ptr = new IBaseListHandler;
  return ptr;
  }
#endif //IC_WIN

/*------------------------------------------------------------------------------
| IBaseListBox::IBaseListBox                                                           |
|                                                                              |
| Construct an IBaseListBox on an IWindow.                                         |
------------------------------------------------------------------------------*/
IBaseListBox::IBaseListBox ( unsigned long     Id,
                     IWindow*          pParent,
                     IWindow*          pOwner,
                     const IRectangle& rectInit,
                     const Style&      style )
  : fBaseListBoxData(new IBaseListBoxData)
{
  // assertions on input parms
  IASSERTPARM(pParent!=0);

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

#ifdef __BORLANDC__
  IWindowHandle owner;
  if (pOwner == 0)
    owner = IWindowHandle(0);
  else
    owner = pOwner->handle();
#else
  IWindowHandle owner = (pOwner == 0) ? IWindowHandle(0) : pOwner->handle();
#endif

  IWindowHandle whListBox =
      this -> create( Id,
                      0,
                      convertToGUIStyle( style ),
                      WC_LISTBOX,
                      pParent->handle(),
                      owner,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( style, true ) );

#ifdef IC_WIN
   this->addHandler( defaultHandler );
#endif
   startHandlingEventsFor(whListBox);

#ifdef IC_WIN
   // IC_NOTYET...  The horizontal extend must be set for Windows to
   // display the horizontal scroll bar.  Temp fix is to set it to the
   // screen width.  Real fix could be to monitor adds/deletes and set
   // it to the widest string in current font.
   if ( (style & IBaseListBox::horizontalScroll) &&
        (!this->isDrawItem()) )
      {
      whListBox.sendEvent( LB_SETHORIZONTALEXTENT,
                           (unsigned long)IWindow::desktopWindow()->size().width(),
                           0 );
      }
#endif

}

/*------------------------------------------------------------------------------
| IBaseListBox::IBaseListBox                                                   |
|                                                                              |
| Construct an IBaseListBox on a dialog.                                       |
------------------------------------------------------------------------------*/
IBaseListBox::IBaseListBox ( unsigned long id,
                     IWindow* parent )
  : fBaseListBoxData (new IBaseListBoxData)
{
  setAutoDestroyWindow(false);
#ifdef IC_WIN
   this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(id, parent);
}

/*------------------------------------------------------------------------------
| IBaseListBox::IBaseListBox                                                   |
|                                                                              |
| Wrapper an existing ListBox.                                                 |
------------------------------------------------------------------------------*/
IBaseListBox::IBaseListBox ( const IWindowHandle& handle )
  : fBaseListBoxData (new IBaseListBoxData)
{
   setAutoDestroyWindow(false);
#ifdef IC_WIN
   this->addHandler( defaultHandler );
#endif
   startHandlingEventsFor(handle);
}

/*------------------------------------------------------------------------------
| IBaseListBox::~IBaseListBox                                                  |
|                                                                              |
| Default destructor here for page tuning.                                     |
------------------------------------------------------------------------------*/
IBaseListBox::~IBaseListBox()
{
#ifdef IC_WIN
  removeHandler( defaultHandler );
#endif
  if ( fBaseListBoxData != 0 )
    delete fBaseListBoxData;
}

/*------------------------------------------------------------------------------
| IBaseListBox::isMultipleSelect                                               |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IBaseListBox::isMultipleSelect() const
{
  return( (style() & multipleSelect.asUnsignedLong()) ? true : false );
}

/*------------------------------------------------------------------------------
| IBaseListBox::isExtendedSelect                                               |
------------------------------------------------------------------------------*/
Boolean IBaseListBox::isExtendedSelect()  const
{
  return( (style() & extendedSelect.asUnsignedLong()) ? true : false );
}

/*------------------------------------------------------------------------------
| IBaseListBox::isDrawItem                                                     |
------------------------------------------------------------------------------*/
Boolean IBaseListBox::isDrawItem()  const
{
  return( (style() & drawItem.asUnsignedLong()) ? true : false );
}

/*------------------------------------------------------------------------------
| IBaseListBox::isNoAdjustPosition                                             |
------------------------------------------------------------------------------*/
Boolean IBaseListBox::isNoAdjustPosition()  const
{
  return( (style() & noAdjustPosition.asUnsignedLong()) ? true : false );
}

/*------------------------------------------------------------------------------
| IBaseListBox::enableMultipleSelect                                           |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::enableMultipleSelect ( Boolean enable )
{
#ifndef IC_WIN
  unsigned long ulStyle(style());
  unsigned long ulOldStyle(ulStyle);
  Boolean       refreshListBox(false);

  if (enable)
  {
    ulStyle |= multipleSelect.asUnsignedLong();
  }
  else
  {
    // Since extended and multiple select can be set at the
    // same time, do not deselect all items if extended select
    // is still set.
    if (!(ulStyle & extendedSelect.asUnsignedLong()))
    {
      deselectAll();
      refreshListBox = true;
    }
    ulStyle &= ~multipleSelect.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    if (refreshListBox)
      refresh();
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::disableMultipleSelect                                          |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::disableMultipleSelect()
{
  enableMultipleSelect(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::enableExtendedSelect                                           |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::enableExtendedSelect ( Boolean enable )
{
#ifndef IC_WIN
  unsigned long ulStyle(style());
  unsigned long ulOldStyle(ulStyle);
  Boolean       refreshListBox(false);

  if (enable)
  {
    ulStyle |= extendedSelect.asUnsignedLong();
  }
  else
  {
    // Since extended and multiple select can be set at the
    // same time, do not deselect all items if multiple select
    // is still set.
    if (!(ulStyle & multipleSelect.asUnsignedLong()))
    {
      deselectAll();
      refreshListBox = true;
    }
    ulStyle &= ~extendedSelect.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    if (refreshListBox)
      refresh();
  }
#endif
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::disableExtendedSelect                                          |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::disableExtendedSelect()
{
  enableExtendedSelect(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::enableDrawItem                                                 |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::enableDrawItem ( Boolean enable )
{
  unsigned long ulStyle(style());
  unsigned long ulOldStyle(ulStyle);

  if (enable)
  {
    ulStyle |= drawItem.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~drawItem.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);

    if ( enable )
    {         // Force draw-item handlers to be called.
       IWindow* owner = this->owner();
       if (owner)
       {
#ifdef IC_PM
          owner->sendEvent( WM_MEASUREITEM,
                            IEventParameter1( this->id() ),
                            0 );
#endif
#ifdef IC_WIN
          MEASUREITEMSTRUCT mis;
          mis.CtlType    = ODT_LISTBOX;
          mis.CtlID      = (UINT)this->id();
          mis.itemID     = 0;
          mis.itemWidth  = 0;
          mis.itemHeight = 0;
          // get the value previously set if any
          mis.itemData =
             this->sendEvent( LB_GETITEMDATA,
                              IEventParameter1( (unsigned long)mis.itemID ),
                              IEventParameter2( 0 ) );
          owner->sendEvent( WM_MEASUREITEM,
                            IEventParameter1( (unsigned long)mis.CtlID ),
                            IEventParameter2( (void*) &mis ) );
#endif
       }
    }

    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::disableDrawItem                                                |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::disableDrawItem()
{
  enableDrawItem(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::enableNoAdjustPosition                                         |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::enableNoAdjustPosition ( Boolean enable )
{
  unsigned long ulStyle(style());
  unsigned long ulOldStyle(ulStyle);

  if (enable)
  {
    ulStyle |= noAdjustPosition.asUnsignedLong();
  }
  else
  {
    ulStyle &= ~noAdjustPosition.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::disableNoAdjustPosition                                        |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::disableNoAdjustPosition()
{
  enableNoAdjustPosition(false);
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::Style IBaseListBox::defaultStyle                               |
------------------------------------------------------------------------------*/
IBaseListBox::Style IBaseListBox::defaultStyle()
{
   return currentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IBaseListBox::setDefaultStyle                                                |
------------------------------------------------------------------------------*/
void IBaseListBox::setDefaultStyle(const Style& style)
{
  currentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IBaseListBox::convertToGUIStyle                                              |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IBaseListBox::convertToGUIStyle(const IBitFlag& guiStyle,
                                          Boolean bExtOnly) const
{
  // Obtain the style from the class (IControl) that we inherit from
  unsigned long ulStyle( Inherited::convertToGUIStyle( guiStyle, bExtOnly ));

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & ILS_EXTGUIMASK;
  }
  else
  {
    // LS_ styles have a one-to-one correspondence to our style bits, and
    // inhabit the lower word of the base GUI style.  Therefore, obtain
    // that portion asis, masking out the upper word.
    ulStyle |= guiStyle.asUnsignedLong() & ILS_MASK;

#ifdef IC_WIN
    ulStyle |= WS_CHILD | LBS_NOTIFY | WS_BORDER | LBS_HASSTRINGS | WS_VSCROLL;
#endif
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| IBaseListBox::selectAll                                                      |
|                                                                              |
| Sequentially select all items in listbox.                                    |
------------------------------------------------------------------------------*/
IBaseListBox&  IBaseListBox::selectAll()
{
   // Verify that the list contains items and if not, fail call
   if (isEmpty())
     ITHROWLIBRARYERROR( IC_EMPTY_LIST,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );

   // Verify that the list supports selection of multiple items
   // i.e. it is not a single select list box
   if ( isMultipleSelect() || isExtendedSelect() )
   {
     unsigned long
       index,
       lastIndex;
     for (index = 0, lastIndex = count(); index < lastIndex; index++)
     {
       IBaseListBox::select(index);
     }
   }
   else
     ITHROWLIBRARYERROR( IC_SINGLE_SELECT_LIST,
                         IBaseErrorInfo::invalidRequest,
                         IException::recoverable );

   return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::setItemHeight                                                  |
|                                                                              |
| Set height of the items in a listbox.                                        |
------------------------------------------------------------------------------*/
IBaseListBox& IBaseListBox::setItemHeight(unsigned long lNewHeight)
{
   IEventResult evt;
#ifdef IC_WIN
   if (this->isDrawItem())
   {
      // For owner draw listboxes, we have to send the LB_SETITEMHEIGHT to
      // each item in the list.
      unsigned long index,
                    itemCount = this->count();
      for (index = 0; index < itemCount; index++)
      {
         evt = handle().sendEvent( LM_SETITEMHEIGHT,
                                   IEventParameter1( index ),
                                   IEventParameter2( lNewHeight ));
         if (evt.asUnsignedLong() == LIT_ERROR)
            ITHROWGUIERROR( "LB_SETITEMHEIGHT" );
      }
   }
   else
   {
      evt = handle().sendEvent( LM_SETITEMHEIGHT,
                                IEventParameter1( 0 ),
                                IEventParameter2( lNewHeight ));
      if (evt.asUnsignedLong() == LIT_ERROR)
         ITHROWGUIERROR( "LB_SETITEMHEIGHT" );
   }
   refresh();
#else
   evt = handle().sendEvent( LM_SETITEMHEIGHT,
                             IEventParameter1((unsigned long)lNewHeight),
                             IEventParameter2(0));
   if (!(evt.asUnsignedLong()))
      ITHROWGUIERROR("LM_SETITEMHEIGHT");
#endif
   return *this;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IBaseListBox::itemHeight                                                     |
|                                                                              |
| Queries the item height in a listbox.                                        |
------------------------------------------------------------------------------*/
unsigned long IBaseListBox::itemHeight () const
{
  IEventResult evt = handle().sendEvent(LB_GETITEMHEIGHT,
                                        IEventParameter1(0),
                                        IEventParameter2(0));
  if (evt.asLong() == LB_ERR)
    ITHROWSYSTEMERROR( (unsigned long)LB_ERR, "LB_GETITEMHEIGHT",
                       IBaseErrorInfo::accessError,
                       IException::recoverable );
  return evt.asUnsignedLong();
}
#endif // IC_WIN

/*------------------------------------------------------------------------------
| IBaseListBox::backgroundColor                                                |
|                                                                              |
| Returns the background color of the IBaseListBox.                            |
------------------------------------------------------------------------------*/
IColor IBaseListBox::backgroundColor () const
{
  return (IWindow::color(PP_BACKGROUNDCOLOR,
                         IGUIColor(IGUIColor::listBoxBgnd)));
}

/*------------------------------------------------------------------------------
| IBaseListBox::calcMinimumSize                                                |
|                                                                              |
| Calculate the minimum size needed by the control.                            |
------------------------------------------------------------------------------*/
ISize  IBaseListBox :: calcMinimumSize() const
{
  ISize sizMin;
  IFont fnt(this);

  sizMin.setHeight ( fBaseListBoxData->minimumRows * fnt.maxCharHeight() +
#ifdef IC_WIN
                     2 * IQUERYSYSVALUE(SV_CYBORDER) );
#else
                     2 * IQUERYSYSVALUE(SV_CYBORDER) + 2);
#endif

  sizMin.setWidth  ( fBaseListBoxData->minimumCharacters * fnt.avgCharWidth() +
                     2 * IQUERYSYSVALUE(SV_CXBORDER) +
                     IQUERYSYSVALUE(SV_CXVSCROLL));

  if (isHorizontalScroll())
  {
#ifdef IC_WIN
     sizMin.setHeight (sizMin.height() + IQUERYSYSVALUE(SV_CYHSCROLL) - 1);
#else
     sizMin.setHeight (sizMin.height() + IQUERYSYSVALUE(SV_CYHSCROLL));
#endif
  }

  if ( ( extendedStyle() & border3D.asExtendedUnsignedLong() ) &&
       ( IPlatform::isWin95() || IPlatform::isNTNewShell() ))
  {
     sizMin += ISize(4,4);
  }

  return sizMin;
}

/*------------------------------------------------------------------------------
| IBaseListBox::setLayoutDistorted                                             |
|                                                                              |
| Cause a font change to result in a new minimum size.                         |
------------------------------------------------------------------------------*/
IBaseListBox&
  IBaseListBox::setLayoutDistorted ( unsigned long layoutAttributesOn,
                                     unsigned long layoutAttributesOff )
{
  unsigned long flagsOn = layoutAttributesOn;
  if ( layoutAttributesOn & IWindow::fontChanged )
  {                // New font means a new minimum size.
     flagsOn |= IWindow::minimumSizeChanged;

#ifdef IC_WIN
     // If this is an owner draw listbox, the listbox items need to be redrawn.
     // Since Windows doesn't send a WM_MEASUREITEM with font changes, we need
     // to force the draw handler's setItemHeight to be called for each item
     // in the list.
     unsigned long itemId,
                   itemCount = this->count();
     if (isDrawItem())
     {
        for (itemId = 0; itemId < itemCount; itemId++)
        {
           MEASUREITEMSTRUCT mis;
           mis.CtlType    = ODT_LISTBOX;
           mis.CtlID      = (UINT)this->id();
           mis.itemID     = (UINT)itemId;
           mis.itemWidth  = 0;
           mis.itemHeight = 0;
           // get the value previously set if any
           mis.itemData =
              this->sendEvent( LB_GETITEMDATA,
                               IEventParameter1( (unsigned long)itemId ),
                               IEventParameter2( 0 ) );
           owner()->sendEvent( WM_MEASUREITEM,
                             IEventParameter1( (unsigned long)mis.CtlID ),
                             IEventParameter2( (void*) &mis ) );
        }
     }
#endif
  }
  Inherited::setLayoutDistorted( flagsOn, layoutAttributesOff );
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::setMinimumCharacters                                           |
|                                                                              |
| Sets the number of characters in an item of a minimum size listbox.          |
------------------------------------------------------------------------------*/
IBaseListBox&  IBaseListBox :: setMinimumCharacters (unsigned long minimumCharacters)
{
  fBaseListBoxData->minimumCharacters = minimumCharacters;
  setLayoutDistorted (minimumSizeChanged, 0);
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::minimumCharacters                                              |
|                                                                              |
| Returns the number of characters in an item in a minimum size listbox.       |
------------------------------------------------------------------------------*/
unsigned long  IBaseListBox ::  minimumCharacters() const
{
  return fBaseListBoxData->minimumCharacters;
}

/*------------------------------------------------------------------------------
| IBaseListBox::setMinimumRows                                                 |
|                                                                              |
| Sets the number of rows in a minimum size listbox.                           |
------------------------------------------------------------------------------*/
IBaseListBox&  IBaseListBox :: setMinimumRows (unsigned long minimumRows)
{
  fBaseListBoxData->minimumRows = minimumRows;
  setLayoutDistorted (minimumSizeChanged, 0);
  return *this;
}

/*------------------------------------------------------------------------------
| IBaseListBox::minimumRows                                                    |
|                                                                              |
| Returns the number of rows in a minimum size listbox.                        |
------------------------------------------------------------------------------*/
unsigned long  IBaseListBox :: minimumRows ( ) const
{
  return fBaseListBoxData->minimumRows;
}


#include <ilistbx3.hpp>
