/*******************************************************************************
* FILE NAME: iradiobt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in iradiobt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINBUTTONS
  #define INCL_WINERRORS
  #define INCL_GPIBITMAPS
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #define INCL_GPIPRIMITIVES
  #include <iwindefs.h>
}

#ifndef _IRADIOBT_
  #include <iradiobt.hpp>
#endif
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif
#ifndef _ISTRING_
  #include <istring.hpp>
#endif
#ifndef _IFONT_
    #include <ifont.hpp>
#endif
#ifndef _ICOLOR_
  #include <icolor.hpp>
#endif
#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif
#ifndef _IHANDLER_
  #include <ihandler.hpp>
#endif
#ifndef _IPOINT_
  #include <ipoint.hpp>
#endif
#ifndef _IRECT_
  #include <irect.hpp>
#endif
#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif


// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRADIOBT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public radio button styles.                                                  |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IRadioButton::Style
#ifdef IC_PM
  IRadioButton::noCursorSelect    = BS_NOCURSORSELECT,
#endif
#ifdef IC_WIN
  IRadioButton::noCursorSelect    = 0,    // Style ignored
#endif
  IRadioButton::autoSelect        ( 0, IBS_AUTORADIOBUTTON ),
  IRadioButton::classDefaultStyle ( WS_VISIBLE, IBS_AUTORADIOBUTTON );

const unsigned long
  IRadioButton::noSelections      = ULONG_MAX;
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
  IRadioButton::Style
        IRadioButton::currentDefaultStyle ( WS_VISIBLE, IBS_AUTORADIOBUTTON );
#pragma data_seg()

#ifdef IC_WIN
class IRadioButtonData : public IBase {
/*------------------------------------------------------------------------------
| The IRadioButtonData class encapsulates private data and functions           |
| used by the IRadioButton class.  An object of this class is created in       |
| the IRadioButton constructors.  This is the os2/windows version.             |
------------------------------------------------------------------------------*/
public:
  IRadioButtonData ( );

virtual
 ~IRadioButtonData ( );

HBRUSH
  bkgrndbrush;

}; // IRadioButtonData

/*------------------------------------------------------------------------------
| IRadioButtonData::IRadioButtonData                                           |
|                                                                              |
------------------------------------------------------------------------------*/
IRadioButtonData::IRadioButtonData ( )
  : bkgrndbrush( 0 )
{ }

/*------------------------------------------------------------------------------
| IRadioButtonData::~IRadioButtonData                                          |
|                                                                              |
------------------------------------------------------------------------------*/
IRadioButtonData::~IRadioButtonData ( )
{
  if ( bkgrndbrush != 0 )
    DeleteObject(bkgrndbrush);
}

class IRadioHandler : public IHandler {
typedef IHandler
  Inherited;
public:

/*------------------------------- Constructors -------------------------------*/
  IRadioHandler ( );

virtual
  ~IRadioHandler ( );

/*----------------------------- Event Processing -----------------------------*/
virtual Boolean
  dispatchHandlerEvent ( IEvent& event );

}; // class IRadioHandler

/*------------------------------------------------------------------------------
| IRadioHandler::IRadioHandler                                                 |
|                                                                              |
| Empty constructor here for page tuning.                                      |
------------------------------------------------------------------------------*/
IRadioHandler::IRadioHandler()
{ }

/*------------------------------------------------------------------------------
| IRadioHandler::~IRadioHandler                                                |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
IRadioHandler::~IRadioHandler()
{ }

/*------------------------------------------------------------------------------
| IRadioHandler::dispatchHandlerEvent                                          |
|                                                                              |
| Intercepts control color event                                               |
|    This handler returns false for all other events.                          |
------------------------------------------------------------------------------*/
IBase::Boolean IRadioHandler::dispatchHandlerEvent( IEvent& event )
{
  Boolean
    result = false;

  switch( event.eventId() )
  {
    case BM_SETCHECK:
      if (event.parameter1().asUnsignedLong())
         {
         // This radio button is being checked.  We have to find any disabled
         // radio buttons in the group ourselves and uncheck them, since
         // Windows does not do this.
         IWindowHandle
            hwnd    ( event.handle() ),
            current ( event.handle() );

         // Find the beginning of the group.  We don't use GetNextDlgGroupItem
         // since it ignores disabled controls...
         while ( (current) && ((ISTYLEOF(current) & WS_GROUP) == 0) )
            {
            current = GetWindow( current, GW_HWNDPREV );
            }

         // handle case where WS_GROUP is not set...Windows does this when
         // you click
         if (!current)
            current = GetWindow( hwnd, GW_HWNDFIRST );

         // Loop through the group, unchecking any disabled radio buttons
         do
            {
            unsigned long style = ISTYLEOF(current);
            if ((current != hwnd) &&
                (style & WS_DISABLED) &&
                (current.sendEvent(WM_QUERYDLGCODE, 0, 0) & DLGC_RADIOBUTTON))
               {
               // Uncheck the button.
               current.sendEvent(BM_SETCHECK,
                                 IEventParameter1(0),
                                 IEventParameter2(0));
               // If tabstop is set turn it off.  Otherwise we have
               // key navigation problems.
               if (style & WS_TABSTOP)
                  {
                  style &= (unsigned long)~WS_TABSTOP;
                  ISETWINDOWSTYLE(current, style);
                  }
               }
            current = GetWindow( current, GW_HWNDNEXT);
            }
         while ( (current) && ((ISTYLEOF(current) & WS_GROUP) == 0) );

         }
      break;
    case WM_CTLCOLORBTN:
    case WM_CTLCOLORSTATIC:
         {
         IWindow* ctlwin =
           IWindow::windowWithHandle(IWindowHandle(event.parameter2()) );
         IColor clrFgnd =
           ((IRadioButton *) ctlwin)->foregroundColor();
         IColor clrBgnd =
           ((IRadioButton *) ctlwin)->backgroundColor();

         // Set foreground and background text colors in device context
         SetBkColor( (HDC)event.parameter1(), clrBgnd.asRGBLong() );
         SetTextColor( (HDC)event.parameter1(), clrFgnd.asRGBLong() );

         // Return HBRUSH for painting controls background
         if ( ((IRadioButton *) ctlwin)->fRadioButtonData->bkgrndbrush != 0 )
           DeleteObject( ((IRadioButton *) ctlwin)->fRadioButtonData->bkgrndbrush);
         ((IRadioButton *) ctlwin)->fRadioButtonData->bkgrndbrush =
           CreateSolidBrush (NEARESTPALETTECOLOR(clrBgnd.asRGBLong()));
         event.setResult( ((IRadioButton *) ctlwin)->fRadioButtonData->bkgrndbrush );

         result = true;
         }
      break;

    default:
      break;
  }
  return result;
}

/*------------------------------------------------------------------------------
| DefaultHandler                                                               |
|                                                                              |
| This class wrappers an object pointer and destroys the pointer to the object |
| when the wrapper is destructed.                                              |
------------------------------------------------------------------------------*/
static struct DefaultHandler {
  operator IRadioHandler* ();
IRadioHandler
 *operator -> ()
    {
    return *this;
    }
 ~DefaultHandler ( )
    {
    if (ptr)
      {
      delete ptr;
      ptr = 0;
      }
    }
IRadioHandler
 *ptr;
} defaultHandler;

DefaultHandler::operator IRadioHandler* ( )
  {
  if ( !ptr )
    ptr = new IRadioHandler;
  return ptr;
  }
#endif //IC_WIN

/*------------------------------------------------------------------------------
| IRadioButton::IRadioButton                                                   |
|                                                                              |
| Constructor to create a radio button on a standard window.                   |
------------------------------------------------------------------------------*/
IRadioButton :: IRadioButton ( unsigned long ulId,
                               IWindow* parent,
                               IWindow* pwndOwner,
                               const IRectangle& rectInit,
                               const Style& rbsStyle )
#ifdef IC_WIN
  : fRadioButtonData( new IRadioButtonData )
#else
  : fRadioButtonData( 0 )
#endif // IC_WIN
{
  IASSERTPARM(parent!=0);             //Assertions on input parms

  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | rbsStyle.asExtendedUnsignedLong() );

  //Create radio button control
  IWindowHandle ownerHandle = IWindowHandle(0);
  if (pwndOwner)
     ownerHandle = pwndOwner->handle();
  IWindowHandle whRadioButton =
      this -> create( ulId,
                      0,
                      convertToGUIStyle( rbsStyle ),
                      WC_BUTTON,
                      parent->handle(),
                      ownerHandle,
                      rectInit,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( rbsStyle, true ) );

#ifdef IC_WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(whRadioButton);
}


/*------------------------------------------------------------------------------
| IRadioButton::IRadioButton                                                   |
|                                                                              |
| Constructor to instantiate an object for a dialog template.                  |
------------------------------------------------------------------------------*/
IRadioButton :: IRadioButton ( unsigned long id,
                               IWindow* parentDialog)
#ifdef IC_WIN
                               : fRadioButtonData( new IRadioButtonData )
#else
                               : fRadioButtonData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow( false );
#ifdef IC_WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor(id, parentDialog);
}


/*------------------------------------------------------------------------------
| IRadioButton::IRadioButton                                                   |
|                                                                              |
| Constructor to instantiate from an existing radio button control.            |
------------------------------------------------------------------------------*/
IRadioButton :: IRadioButton ( const IWindowHandle& wh )
#ifdef IC_WIN
                               : fRadioButtonData( new IRadioButtonData )
#else
                               : fRadioButtonData( 0 )
#endif // IC_WIN
{
  setAutoDestroyWindow( false );
#ifdef IC_WIN
  this->addHandler( defaultHandler );
#endif
  startHandlingEventsFor( wh );
}


/*------------------------------------------------------------------------------
| IRadioButton:~IRadioButton                                                   |
------------------------------------------------------------------------------*/
IRadioButton :: ~IRadioButton ( )
{
#ifdef IC_WIN
  removeHandler(defaultHandler);
  if ( fRadioButtonData != 0 )
    delete fRadioButtonData;
#endif //IC_WIN
}


/*------------------------------------------------------------------------------
| IRadioButton:defaultStyle                                                    |
|                                                                              |
| Return the default style for new radio button objects.                       |
------------------------------------------------------------------------------*/
IRadioButton::Style  IRadioButton :: defaultStyle ( )
{
  return currentDefaultStyle;
}


/*------------------------------------------------------------------------------
| IRadioButton:setDefaultStyle                                                 |
|                                                                              |
| Replace the default style for new radio button objects.                      |
------------------------------------------------------------------------------*/
void IRadioButton :: setDefaultStyle ( const Style& rbsStyle )
{
  currentDefaultStyle = rbsStyle;
}


/*------------------------------------------------------------------------------
| IRadioButton::convertToGUIStyle                                              |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IRadioButton::convertToGUIStyle(const IBitFlag& guiStyle,
                                              Boolean bExtOnly) const
{
  // Obtain the style from the class (ISettingButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    // Let the clean BS_ styles, upper 12 bits of the lower word of the
    // style, flow thru
    ulStyle |= guiStyle.asUnsignedLong() & IBS_MASK;

    // BS_ styles in the lower 4 bits of the style overlap.  Therefore, map
    // our extended style bits, back to these values.
    unsigned long ulTmp = guiStyle.asExtendedUnsignedLong() & IBS_EXTRADIOBTN;

    // Handle overlapping styles
    if (ulTmp & IBS_AUTORADIOBUTTON)
      ulStyle |= BS_AUTORADIOBUTTON;
    else
      ulStyle |= BS_RADIOBUTTON;

#ifdef IC_WIN
    ulStyle |= WS_CHILD;
#endif

  }


  return( ulStyle );
}


/*------------------------------------------------------------------------------
| IRadioButton:enableAutoSelect                                                |
|                                                                              |
| Sets or removes the autoSelect style for the radio button control.           |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton :: enableAutoSelect ( Boolean bTurnOn )
{
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;     //Use masking flag from pmwin.h
    ulStyle |= BS_AUTORADIOBUTTON;
  }
  else
  {
    ulStyle &= (unsigned long)~BS_PRIMARYSTYLES;     //Use masking flag from pmwin.h
    ulStyle |= BS_RADIOBUTTON;
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
    refresh();                //Force refresh
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:disableAutoSelect                                               |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton :: disableAutoSelect()
{
  enableAutoSelect(false);
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:isAutoSelect                                                    |
------------------------------------------------------------------------------*/
Boolean  IRadioButton :: isAutoSelect() const
{
  return( (style() & BS_AUTORADIOBUTTON & (unsigned long)~(BS_RADIOBUTTON)) ?
          true : false );
}


/*------------------------------------------------------------------------------
| IRadioButton:enableCursorSelect                                              |
|                                                                              |
| Removes or sets the noCursorSelect style.  The noCursorSelect style causes   |
| the radio button not to select itself when given the focus as the result     |
| of an arrow key or tab key.  This style only applies to autoSelect radio     |
| buttons.                                                                     |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton :: enableCursorSelect(Boolean bTurnOn)
{
#ifdef IC_PM
  unsigned long ulStyle = style();
  unsigned long ulOldStyle = ulStyle;

  if (bTurnOn)
  {
    ulStyle &= ~noCursorSelect.asUnsignedLong();
  }
  else
  {
    ulStyle |= noCursorSelect.asUnsignedLong();
  }

  if (ulStyle != ulOldStyle)
  {
    setStyle(ulStyle);
  }
#endif
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:disableCursorSelect                                             |
------------------------------------------------------------------------------*/
IRadioButton&  IRadioButton :: disableCursorSelect()
{
  enableCursorSelect(false);
  return *this;
}


/*------------------------------------------------------------------------------
| IRadioButton:isCursorSelect                                                  |
------------------------------------------------------------------------------*/
Boolean  IRadioButton :: isCursorSelect() const
{
#ifdef IC_PM
  unsigned long ulStyle = style();

  if ((ulStyle & BS_AUTORADIOBUTTON & ~(BS_RADIOBUTTON)) &&
      (ulStyle & noCursorSelect.asUnsignedLong()))
    return false;
  else
#endif
    return true;
}

/*------------------------------------------------------------------------------
| IRadioButton:selectedIndex                                                   |
|                                                                              |
| Returns the zero-based index of a selected radio button in a group.          |
| NOTE:  noSelections (-1) is returned if nothing is selected.                 |
------------------------------------------------------------------------------*/
unsigned long  IRadioButton :: selectedIndex() const
{
  unsigned long index = noSelections;
#ifdef IC_PM
  IEventResult evt = handle().sendEvent(BM_QUERYCHECKINDEX,
                             IEventParameter1(0),
                             IEventParameter2(0));
  if (evt.asLong() != -1)
  {                // A button is selected, so return its index.
     index = evt.asUnsignedLong();
  }
#endif
#ifdef IC_WIN
   // Find the beginning of the group
   IWindowHandle hwnd = this->handle();
   while ( (hwnd) && ((ISTYLEOF(hwnd) & WS_GROUP) == 0) )
      {
      hwnd = GetWindow( hwnd, GW_HWNDPREV );
      }
   // find the checked button
   unsigned long count = 0;
   while ( (hwnd) &&
     ( hwnd.sendEvent( BM_GETCHECK, 0 ,0 ).asUnsignedLong() != 1) )
      {
      hwnd = GetWindow( hwnd, GW_HWNDNEXT );
      count++;
      }
   if (hwnd)
      {
      index = count;
      }
#endif

  return index;
}


/*------------------------------------------------------------------------------
| IRadioButton:calcMinimumSize                                                 |
|                                                                              |
| Calculate the minimum screen size needed by the control.  The minimum size   |
| is based on the maximum height and width of the font of the radio button     |
| text (plus the size of the radio button itself).                             |
------------------------------------------------------------------------------*/
ISize  IRadioButton :: calcMinimumSize() const
{
#ifdef IC_PM
   BITMAPINFOHEADER pbmpData;
   pbmpData.cbFix=sizeof(BITMAPINFOHEADER);
#endif
#ifdef IC_WIN
   BITMAP           pbmpData;
#endif
   HBITMAP hbit = (HBITMAP) IGETSYSBITMAP(SBMP_CHECKBOXES);
   IQUERYBITMAPPARAMETERS(hbit, &pbmpData);
   IDELETEBITMAP(hbit);

   ISize size = displaySize(text());
   unsigned long ulWidth = size.width();
   unsigned long i=textLength();
#ifdef IC_PM
   long bmpWidth = pbmpData.cx;
#endif
#ifdef IC_WIN
   long bmpWidth = pbmpData.bmWidth;
#endif

   if (i==0)
     ulWidth += ((bmpWidth/4) + 2 + (2 * IQUERYSYSVALUE(SV_CXBORDER)));
   else
     ulWidth += (ulWidth/i + (bmpWidth/4) + 2 +
                                    (2 * IQUERYSYSVALUE(SV_CXBORDER)));
   unsigned long ulHeight = size.height() +
                                     (2 * IQUERYSYSVALUE(SV_CYBORDER));
   return ISize(ulWidth, ulHeight);
}
