/*******************************************************************************
* FILE NAME: islidhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in islidhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#define INCL_WINSTDSLIDER
extern "C" {
  #include <iwindefs.h>
}

#ifdef IC_WIN
  #ifndef _ICLSLIDERW_
    #include <iclsldw.h>
  #endif
#endif

#ifndef _IDIEVT_
  #include <idievt.hpp>
#endif
#ifndef _ISLIDHDR_
  #include <islidhdr.hpp>
#endif
#ifndef _IWCNAME_
  #include <iwcname.hpp>
#endif

// Segment definitions.
#ifdef IC_PAGETUNE
  #define _ISLIDHDR_CPP_
  #include <ipagetun.h>
#endif


/*------------------------------------------------------------------------------
| ISliderDrawHandler::ISliderDrawHandler                                       |
|                                                                              |
| Default constructor here for page tuning.                                    |
------------------------------------------------------------------------------*/
ISliderDrawHandler::ISliderDrawHandler ( )
  : ISliderDrawHandler::Inherited( )
{ }

/*------------------------------------------------------------------------------
| ISliderDrawHandler::~ISliderDrawHandler                                      |
|                                                                              |
| Empty destructor here for page tuning.                                       |
------------------------------------------------------------------------------*/
ISliderDrawHandler::~ISliderDrawHandler ( )
{ }

/*------------------------------------------------------------------------------
| ISliderDrawHandler::dispatchHandlerEvent                                     |
|                                                                              |
| Dispatch command functions for this handler.                                 |
------------------------------------------------------------------------------*/
Boolean ISliderDrawHandler::dispatchHandlerEvent ( IEvent& event )
{
  Boolean bRc = false;

  if ( event.eventId() == WM_DRAWITEM )
  {
     IDrawItemEvent drawevt( event );
     IWindowHandle hwndControl = drawevt.controlHandle();
     if ( hwndControl  &&
          IWindowClassName( hwndControl ) == WC_SLIDER )
     {                                 // Slider case.
        switch ( drawevt.itemId() )
        {
           case SDA_SLIDERSHAFT:
           {
              bRc = this->drawShaft( drawevt );
              break;
           }
           case SDA_RIBBONSTRIP:
           {
              bRc = this->drawRibbonStrip( drawevt );
              break;
           }
           case SDA_SLIDERARM:
           {
              bRc = this->drawArm( drawevt );
              break;
           }
           case SDA_BACKGROUND:
           {
              bRc = this->drawBackground( drawevt );
              break;
           }
        }

        event.setResult( bRc );
     }
  }

  return bRc;
}

/*------------------------------------------------------------------------------
| ISliderDrawHandler::drawShaft                                                |
------------------------------------------------------------------------------*/
Boolean ISliderDrawHandler::drawShaft ( IDrawItemEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISliderDrawHandler::drawRibbonStrip                                          |
------------------------------------------------------------------------------*/
Boolean ISliderDrawHandler::drawRibbonStrip ( IDrawItemEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISliderDrawHandler::drawArm                                                  |
------------------------------------------------------------------------------*/
Boolean ISliderDrawHandler::drawArm ( IDrawItemEvent& event )
{
  return false;
}

/*------------------------------------------------------------------------------
| ISliderDrawHandler::drawBackground                                           |
------------------------------------------------------------------------------*/
Boolean ISliderDrawHandler::drawBackground ( IDrawItemEvent& event )
{
  return false;
}

