#ifndef _ISPINBAS_
  #define _ISPINBAS_
/*******************************************************************************
* FILE NAME: ispinbas.hpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IBaseSpinButton                                                          *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#include <icontrol.hpp>
#include <irect.hpp>
#include <ihandle.hpp>

class IBaseSpinButtonData;

#pragma pack(4)

class IBaseSpinButton : public IControl {
typedef IControl
  Inherited;

public :

/*----------------------------- Constructors ---------------------------------*/
virtual
  ~IBaseSpinButton ( );

/*----------------------------- Styles ---------------------------------------*/
INESTEDBITFLAGCLASSDEF2(Style, IBaseSpinButton, IWindow, IControl);

static const Style
  IC_IMPORTU master,
  IC_IMPORTU servant,
  IC_IMPORTU readOnly,
  IC_IMPORTU noBorder,
  IC_IMPORTU pmCompatible,
  IC_IMPORTU fastSpin;
#ifndef IC_MOTIFPM_FLAGNOP
static const Style
  IC_IMPORTU border3D;
#endif
#ifndef IC_MOTIF_FLAGNOP
static const Style
  IC_IMPORTU leftAlign,
  IC_IMPORTU centerAlign,
  IC_IMPORTU rightAlign;
#endif

#ifdef IC_PMWIN
  virtual unsigned long
    convertToGUIStyle ( const IBitFlag& style,
                        Boolean         extendedOnly = false ) const;
#endif

/*---------------------------- Style Functions -------------------------------*/
#ifndef IC_MOTIF_FLAGNOP
enum Alignment {
  left,
  right,
  center
};
#endif

virtual IBaseSpinButton
  &enableDataUpdate  ( Boolean   writeable = true ),
  &disableDataUpdate ( ),
  &enableFastSpin    ( Boolean   fast      = true ),
  &disableFastSpin   ( ),
  &addBorder         ( Boolean   add       = true ),
  &removeBorder      ( );
#ifndef IC_MOTIFWIN_FLAGNOP
virtual IBaseSpinButton
  &setAlignment      ( Alignment alignment = left );
#endif

Boolean
  isMaster          ( ) const,
  isServant         ( ) const,
  isWriteable       ( ) const,
  isFastSpinEnabled ( ) const,
  isPMCompatible    ( ) const,
  hasBorder         ( ) const;

#ifndef IC_MOTIF_FLAGNOP
Alignment
  alignment  ( ) const;
#endif

/*----------------------------- Limit and Spin -------------------------------*/
unsigned long
  limit ( ) const;

virtual IBaseSpinButton
  &setLimit  ( unsigned long    aNumber = 255 ),
  &setMaster ( IBaseSpinButton& master        ),
  &spinDown  ( unsigned long    spinBy  = 1 ),
  &spinUp    ( unsigned long    spinBy  = 1 );

virtual Boolean
  isSpinFieldValid ( Boolean caseSensitive ) const = 0;

#ifdef IC_MOTIF
/*------------------------- Compound Control Method --------------------------*/
virtual IWindowHandle
  topHandle() const;
#endif  //IC_MOTIF

/*--------------------------------- Overrides --------------------------------*/
#ifdef IC_PMWIN
virtual IBaseSpinButton
 &enable ( Boolean enable = true );
#endif

#ifdef IC_PMWIN
virtual IColor
  foregroundColor () const;
#endif // IC_PMWIN

#ifdef IC_WIN
virtual IColor
  backgroundColor () const;

virtual IBaseSpinButton
 &setForegroundColor ( const IColor& color ),
 &setBackgroundColor ( const IColor& color ),
 &resetForegroundColor ( ),
 &resetBackgroundColor ( );
#endif //IC_WIN

/*------------------------- Attributes ---------------------------------------*/
#ifdef  IC_WIN
virtual Boolean
  hasFocus            ( ) const;
#endif

#ifdef IC_MOTIF
/*-------------------------------- Window Positioning ------------------------*/
virtual IBaseSpinButton
 &moveSizeTo ( const IRectangle& aRectangle );
#endif  //IC_MOTIF

/*----------------------- Notification Event Descriptions --------------------*/
static INotificationId const
  IC_IMPORTU valueId,
  IC_IMPORTU textId;


protected:
/*------------------------------ Implementation ------------------------------*/
virtual ISize
  calcMinimumSize ( ) const;

void
  initialize ( unsigned long        windowId,
               const IWindowHandle& parent,
               const IWindowHandle& owner,
               unsigned long        style,
               const IRectangle&    initial );
#ifdef IC_PMWIN
virtual void
  initialize ( );
#endif

/*-------------------------- Protected Constructors --------------------------*/
  IBaseSpinButton ( );

/*---------------------- Event-Handling Implementation -----------------------*/
#ifdef IC_MOTIF
virtual void
  registerCallbacks (),
  unregisterCallbacks ();
#endif   //IC_MOTIF


private:
/*------------------------------- Hidden Members -----------------------------*/
  IBaseSpinButton ( const IBaseSpinButton& spinButton );
IBaseSpinButton
 &operator=       ( const IBaseSpinButton& spinButton );

/*--------------------------------- Private ----------------------------------*/
friend class IBaseSpinDefaultHandler;

IBaseSpinButtonData
 *fBaseSpinButtonData;
};  // IBaseSpinButton

INESTEDBITFLAGCLASSFUNCS( Style, IBaseSpinButton);

#pragma pack()

#endif // _ISPINBAS_
