/* NOSHIP */
#ifndef _IWCNAME_
#define _IWCNAME_
/*******************************************************************************
* FILE NAME: iwcname.hpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the classes:                                                *
*     IWindowClassName                                                         *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#include <ivbase.hpp>
#include <istring.hpp>

#pragma pack(4)

class IWindowHandle;
class ostream;

class IWindowClassName : public IVBase {
typedef IVBase
  Inherited;
/*******************************************************************************
* Objects of this class are used to represent Presentation Manager             *
* window class names.  Such names are passed to IWindow::create at window      *
* construction time and returned by IWindow::windowClassName.                  *
*******************************************************************************/
public:
/*------------------------- Constructors/Destructor ----------------------------
| This class provides the following constructors:                              |
|                                                                              |
|   o From a char * argument:                                                  |
|                                                                              |
|     This constructor is used to construct a window class name from a         |
|     character string such as "MyClassName" or WC_FRAME.  It is required      |
|     so that the WC_* pseudo-pointers are handled appropriately.              |
|                                                                              |
|   o From a window handle:                                                    |
|                                                                              |
|     This will construct a class name for the window whose handle is          |
|     passed as argument.                                                      |
|                                                                              |
|   o Copy constructor                                                         |
|                                                                              |
|     This is required so that objects of this class can be used as the        |
|     return type of functions such as IWindow::windowClassName.               |
------------------------------------------------------------------------------*/
  IWindowClassName ( const char *className );
  IWindowClassName ( const IWindowHandle &handle );
  IWindowClassName ( const IWindowClassName &className );

  ~IWindowClassName ( );

/*-------------------------------- Conversion ----------------------------------
| This class provides the following functions to convert objects to other      |
| representations:                                                             |
|   asString              - Returns string representation of the class name;   |
|                           this will convert WC_* names to "#1", etc.         |
|   operator const char * - Returns the appropriate class name as a char       |
|                           pointer; note that WC_* class names are returned   |
|                           as (const char *)(0xFFFF0001L), etc.               |
------------------------------------------------------------------------------*/
IString
  asString ( ) const;

  operator const char * ( ) const;

/*-------------------------------- Comparison ----------------------------------
| This class provides the following comparison operators to permit comparison  |
| to various representations of window class names:                            |
|   operator == - Compares the name to either a char* or window class name.    |
|   operator != - Tests if arguments are different.                            |
------------------------------------------------------------------------------*/
Boolean
  operator == ( const char *className ) const,
  operator != ( const char *className ) const,
  operator == ( const IWindowClassName &className ) const,
  operator != ( const IWindowClassName &className ) const;

friend Boolean
  operator == ( const char *className1,
                const IWindowClassName &className2 ),
  operator != ( const char *className1,
                const IWindowClassName &className2 );

/*-------------------------------- Stream I/O ----------------------------------
| operator << - Puts the asString representation of the class name to the      |
|               output stream.                                                 |
------------------------------------------------------------------------------*/
friend ostream
 &operator << ( ostream &aStream, const IWindowClassName &className );

protected:
/*------------------------------ Implementation --------------------------------
| The following functions provide utilities for implementing this class:       |
|   initialize - Sets the class's name from const char * value.                |
------------------------------------------------------------------------------*/
IWindowClassName
 &initialize ( const char *className );

private:
/*--------------------------------- Private ----------------------------------*/
IString
  name;
}; // class IWindowClassName

#pragma pack()

#ifndef I_NO_INLINES
//#include <iwcname.inl>
#endif

#endif // _IWCNAME_
