/*******************************************************************************
* FILE NAME: immcdhdr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in immcdhdr.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )
extern "C"
  {
  #include <string.h>
  #define INCL_MCIOS2
  #define INCL_WINATOM
  #define INCL_WINMESSAGEMGR
  #define IC_MULTIMEDIA
  #include <immdefs.h>
  }

#include <immcdhdr.hpp>

#ifndef _IMMCDDA_
  #include <immcdda.hpp>
#endif

#ifndef _INOTIFEV_
  #include <inotifev.hpp>
#endif

#ifndef _IMMEVT_
   #include <immevt.hpp>
#endif
#ifndef _ITIMER_
  #include <itimer.hpp>
#endif
#ifndef _IMMTIME_
  #include <immtime.hpp>
#endif

#ifndef _IMMOWIN_
   #include <immowin.hpp>
#endif

#ifndef _ITRACE_
   #include <itrace.hpp>
#endif

#ifndef _IMMEXCPT_
   #include <immexcpt.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IFRAME_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMMAudioCD::passDevice                                                       |
| Process pass device events.                                                  |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCDHandler::passDevice(const IMMPassDeviceEvent &evt)
{
  IMODTRACE_DEVELOP("IMMAudioCDHandler::passDevice") ;
  Boolean result = false;
  IMMAudioCD* device = (IMMAudioCD*)evt.device();
  if (device)
  {
     //If haven't seen the first pass device message yet, means
     //we are just now finished opening the device, so we have to
     //initialize everything at this point
     if (!device->wasPassDeviceRequested())
     {
        IMMDeviceHandler::passDevice(evt);
        if (device->fDelay && device->isOpen())
        {
           if (device->isMediaPresent())
           {
              device->setTimeFormat(IMMTime::trackMinSecFrame);
              device->setTableOfContents(device->getTableFromDisc());
           }
        } /* endif */
     }
     //Causes the media timer to start
     result = IMMRemovableMediaHandler::passDevice(evt);
     if (device->isOpen())
     {
        if (evt.isGainingUse() && device->isMediaPresent())
        {
           IMMAudioCDContents currentToc = device->contents();
           //If the disc has changed then we get the new
           //table of contents
           if (currentToc.discId() != device->uniqueDiscIdentifier())
           {
              mediaUnloaded();
              mediaLoaded();
           }
           if (device->playing())
              device->putInPlayingState();
        }
        else
        {
            //Causes the update timers to stop, we don't need them
            //running since we gave up access to the device
            device->stopT();
        }
     } /* endif */
  }
  return result;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::~IMMAudioCDHandler                                        |
| Destructor                                                                   |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler::~IMMAudioCDHandler()
{
}
/*------------------------------------------------------------------------------
| IMMAudioCDHandler::IMMAudioCDHandler                                         |
| Construct a handler.                                                         |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler::IMMAudioCDHandler()
                         : IMMRemovableMediaHandler(),
                           fDevice(0)
{
   IMODTRACE_DEVELOP("IMMAudioCDHandler::IMMAudioCDHandler") ;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::dispatchHandlerEvent                                      |
| Dispatch the event.                                                          |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IMMAudioCDHandler::dispatchHandlerEvent(IEvent &evt)
{
  IMODTRACE_DEVELOP("IMMAudioCDHandler::dispatchHandlerEvent") ;
  Boolean ret = Inherited::dispatchHandlerEvent(evt);

#ifdef IC_WIN
  if (evt.eventId() == IC_UM_MM_MCI_CMDNOTIFY |   // handle the no-command-in-event
      evt.eventId() == MM_MCINOTIFY             ) // workaround in IMMDevice
#endif
#ifdef IC_PM
  if (evt.eventId() == MM_MCINOTIFY)
#endif
  {
     IMMNotifyEvent notifyEvent(evt);
     IMMAudioCD* device = (IMMAudioCD*)notifyEvent.device();
     if (device)
        //does special processing when the play finishes, for example,
        //to play the next track
        device->processInternal(evt);
  }
  return ret;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::handleEventsFor                                           |
| Start handling events for the device passed.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler& IMMAudioCDHandler::handleEventsFor (IMMAudioCD* device )
{
   IMODTRACE_DEVELOP("IMMAudioCDHandler::handleEventsFor") ;
   fDevice = device;
   Inherited::handleEventsFor(device);
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::mediaLoaded                                               |
| Handle case that media has been loaded.                                      |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler& IMMAudioCDHandler::mediaLoaded()
{
   IMODTRACE_DEVELOP("IMMAudioCDHandler::mediaLoaded") ;
   if (fDevice)
   {
      //Since a new disc was loaded need to load the new table of contents
      fDevice->setTimeFormat(IMMTime::trackMinSecFrame);
      fDevice->setTableOfContents(fDevice->getTableFromDisc());
      Inherited::mediaLoaded();
      if (fDevice->isAutoPlayEnabled())
         fDevice->play();
   } else {
      Inherited::mediaLoaded();
   } /* endif */
   return *this;
}
/*------------------------------------------------------------------------------
| IMMAudioCDHandler::mediaUnloaded                                             |
| Handle case that media has been unloaded.                                    |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler& IMMAudioCDHandler::mediaUnloaded()
{
   IMODTRACE_DEVELOP("IMMAudioCDHandler::mediaUnloaded") ;
   if (fDevice)
      //Changes a state in the device, so that it knows the table of
      //contents that is currently set is invalid
      fDevice->changeTable();
   Inherited::mediaUnloaded();
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::handleEventsFor                  **aj**                   |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler& IMMAudioCDHandler::handleEventsFor ( IMMRemovableMedia* device )
{
   ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable );
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::handleEventsFor                                           |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler& IMMAudioCDHandler::handleEventsFor ( IMMDevice* device )
{
   ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable );
   return *this;
}

/*------------------------------------------------------------------------------
| IMMAudioCDHandler::handleEventsFor                                           |
| Process illegal use of this member function.                                 |
|                                                                              |
------------------------------------------------------------------------------*/
IMMAudioCDHandler& IMMAudioCDHandler::handleEventsFor ( IWindow* window )
{
   ITHROWLIBRARYERROR( IC_MEMBER_ACCESS_ERROR,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable );
   return *this;
}
