/*******************************************************************************
* FILE NAME: icoordsy.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in icoordsy.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/

#define INCL_WINMESSAGEMGR          // for WM_CALCVALIDRECTS
extern "C" {
  #include <iwindefs.h>
}

#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <ihandle.hpp>
#include <ipoint.hpp>
#include <irect.hpp>
#include <iwindow.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IRECOHDR_CPP_
  #include <ipagetun.h>
#endif


#pragma data_seg(ICLNonConst)
// The default application coordinate system is the native coordinate system.
#ifdef IC_PM
ICoordinateSystem::Orientation
   ICoordinateSystem::fsapplication = ICoordinateSystem::originLowerLeft ;
#else
ICoordinateSystem::Orientation
   ICoordinateSystem::fsapplication = ICoordinateSystem::originUpperLeft ;
#endif
#pragma data_seg()


/*------------------------------------------------------------------------------
| ICoordinateSystem::ICoordinateSystem   (hidden ... all static class)         |
------------------------------------------------------------------------------*/
ICoordinateSystem::ICoordinateSystem   ( )
{ }

/*------------------------------------------------------------------------------
| ICoordinateSystem::convertToApplication                                      |
------------------------------------------------------------------------------*/
IRectangle ICoordinateSystem::convertToApplication(
                                 const IRectangle& nativeRectangle,
                                 const ISize&      referenceSize )
{
   IMODTRACE_ALL("ICoordinateSystem::convertToApplication");
   // With the 2 coordinate systems supported, the
   // conversion algorithm is the same in either direction.
   return convertRectangle( nativeRectangle, referenceSize );
}


/*------------------------------------------------------------------------------
| ICoordinateSystem::convertToNative                                           |
------------------------------------------------------------------------------*/
IRectangle ICoordinateSystem::convertToNative(
                                 const IRectangle& applicationRectangle,
                                 const ISize&      referenceSize )
{
   IMODTRACE_ALL("ICoordinateSystem::convertToNative");
   // With the 2 coordinate systems supported, the
   // conversion algorithm is the same in either direction.
   return convertRectangle( applicationRectangle, referenceSize );
}

/*------------------------------------------------------------------------------
| ICoordinateSystem::convertRectangle                                          |
------------------------------------------------------------------------------*/
IRectangle  ICoordinateSystem::convertRectangle(
                                   const IRectangle& rectangle,
                                   const ISize&      referenceSize )
{
   ITRACE_ALL(IString("rectangle=") + rectangle.asString() );
   IRectangle result = rectangle;
   if ( isConversionNeeded() )
      {
      // since there are only 2 coordinate systems supported we assume
      // that application is the opposite of native.  The desired y coordinate
      // for the origin is the distance between the far corner of the
      // reference rectangle and the corner of the rectangle to be converted.
      // IRectangle::maxY gives the "far corner" y coordinate, which we
      // subtract from the reference size height to give our desired new
      // coordinate.
      result.moveTo( IPoint(rectangle.minX(),
                            referenceSize.height() -
                            rectangle.maxY() ) );
      }
   ITRACE_ALL(IString("result=") + result.asString() );
   return result;
}


/*------------------------------------------------------------------------------
| ICoordinateSystem::convertToApplication                                      |
------------------------------------------------------------------------------*/
IPoint ICoordinateSystem::convertToApplication( const IPoint&  nativePoint,
                                                const ISize&   referenceSize )
{
   IPoint   result = nativePoint;
   if ( isConversionNeeded( ) )
     result.setY( referenceSize.height() - result.y() );
   return result;
}

/*------------------------------------------------------------------------------
| ICoordinateSystem::convertToNative                                           |
------------------------------------------------------------------------------*/
IPoint ICoordinateSystem::convertToNative ( const IPoint&     applicationPoint,
                                            const ISize&      referenceSize )
{
   IPoint   result = applicationPoint;
   if ( isConversionNeeded( ) )
     result.setY( referenceSize.height() - result.y() );
   return result;
}

// Note: These functions are inlined on all platforms except Windows due to
//       variables they reference not being exported.
#ifdef IC_WIN
/*------------------------------------------------------------------------------
| ICoordinateSystem::applicationOrientation                                    |
------------------------------------------------------------------------------*/
ICoordinateSystem::Orientation
   ICoordinateSystem::applicationOrientation  ( )
{
   return fsapplication;
}

/*------------------------------------------------------------------------------
| ICoordinateSystem::setApplicationOrientation                                 |
------------------------------------------------------------------------------*/
ICoordinateSystem::Orientation
   ICoordinateSystem::setApplicationOrientation ( Orientation coordsystem )
{
   fsapplication = coordsystem;
   return fsapplication;
}

/*------------------------------------------------------------------------------
| ICoordinateSystem::isConversionNeeded                                        |
------------------------------------------------------------------------------*/
IBase::Boolean ICoordinateSystem::isConversionNeeded ( )
{
   return (fsapplication != nativeOrientation()) ;
}
#endif //IC_WIN
