/*******************************************************************************
* FILE NAME: idmcmdhr.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMPopUpMenuHandler                                                      *
*     IDMPopUpCommandHandler                                                   *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
#ifndef _IDMCMDHR_
  #include <idmcmdhr.hpp>
#endif

// Trace Facility
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif
#ifndef _IDMTGTOP_
  #include <idmtgtop.hpp>
#endif

#include <icconst.h>

// Segment definitions

#ifdef IC_PAGETUNE
  #define _IDMCMDHR_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::makePopUpMenu                                           |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IDMPopUpMenuHandler :: makePopUpMenu(IMenuEvent& menuEvent)
  {
  IMODTRACE_DEVELOP("IDMPopUpMenuHandler::makePopUpMenu");
  IPopUpMenu* popUp;

  popUp = new IPopUpMenu(menuEvent.window(), 0);

  IDMTargetOperation::Handle pTgtOp;
  pTgtOp = IDMTargetOperation::targetOperation();
  unsigned short
    supportedOperations = 0;

  if ( pTgtOp )
    {
    IDMItem::Handle pItemH;
    pItemH = pTgtOp->item(1);

     supportedOperations = pItemH->supportedOperations();
    }

  ITRACE_DEVELOP("Supported operations: " + IString(supportedOperations));
  // Create the popup menu based on the supported operations
  if ( supportedOperations & IDMItem::moveable )
    (*popUp).addText(IC_DM_MI_MOVE,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_MOVE));
  if ( supportedOperations & IDMItem::copyable )
    (*popUp).addText(IC_DM_MI_COPY,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_COPY));
  if ( supportedOperations & IDMItem::linkable )
    (*popUp).addText(IC_DM_MI_SHORTCUT,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_SHORTCUT));
  (*popUp)
    .addSeparator()
    .addText(IC_DM_MI_CANCEL,
       IApplication::current().resourceLibrary().loadString(IC_DM_MI_CANCEL));

  (*popUp)
    .show(menuEvent.mousePosition())
    .setAutoDeleteObject();

  return true;
  }

/*------------------------------------------------------------------------------
| IDMPopUpMenuHandler::dispatchHandlerEvent                                    |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IDMPopUpMenuHandler::dispatchHandlerEvent(IEvent& evt)
  {
  Boolean bProcessed = false;

  switch(evt.eventId())
    {
    case WM_RBUTTONUP:
      {
      if (fNonDefaultDrag)
        {
        ::ShowCursor(true);
        IMenuEvent mnEvt(evt);
        if (makePopUpMenu(mnEvt))
          {
          evt.setResult(true);
          }
        bProcessed = true;
        }
      break;
      }
    }
   return bProcessed;
  }

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::command                                              |
|                                                                              |
------------------------------------------------------------------------------*/
Boolean IDMPopUpCommandHandler::command( ICommandEvent& cmdEvent)
  {
  IMODTRACE_DEVELOP("IDMPopUpCommandHandler::command");

  Boolean brc = false;
  DWORD dwOperation = 0;
  IBase::Boolean fValidOperation = false;

  IDMTargetOperation::Handle pTgtOp =
    IDMTargetOperation::targetOperation();

  switch (cmdEvent.commandId())
    {
    case IC_DM_MI_MOVE :
      {
      ITRACE_DEVELOP("Move selected");
      dwOperation = DROPEFFECT_MOVE;
      fValidOperation = true;
      brc = true;
      break;
      }
    case IC_DM_MI_COPY :
      {
      ITRACE_DEVELOP("Copy selected");
      dwOperation = DROPEFFECT_COPY;
      fValidOperation = true;
      brc = true;
      break;
      }
    case IC_DM_MI_SHORTCUT :
      {
      ITRACE_DEVELOP("Create Shortcut selected");
      dwOperation = DROPEFFECT_LINK;
      fValidOperation = true;
      brc = true;
      break;
      }
    case IC_DM_MI_CANCEL :
      {
      ITRACE_DEVELOP("Cancel selected");
      cmdEvent.window()->sendEvent( DM_DRAGLEAVE, 0, 0);
      dataObject->Release();
      brc = true;
      break;
      }
    }

  if ( fValidOperation )
    {
    if ( pTgtOp )
      pTgtOp->setOperation( dwOperation );

    IEventResult evt =  cmdEvent.window()->sendEvent( DM_DROP,
                        IEventData( point.x, point.y ),
                        IEventData( (void *) dataObject ));

    dataObject->Release();
    }

  return brc;
  }

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::setPoint                                             |
|                                                                              |
------------------------------------------------------------------------------*/
void IDMPopUpCommandHandler::setPoint( POINTL pt)
  {
  point = pt;
  }

/*------------------------------------------------------------------------------
| IDMPopUpCommandHandler::setDataObject                                        |
|                                                                              |
------------------------------------------------------------------------------*/
void IDMPopUpCommandHandler::setDataObject( IDataObject* dataObj)
  {
  dataObject = dataObj;
  }
