/*******************************************************************************
* FILE NAME: idmcomm.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     Enumerations -                                                           *
*       Source                                                                 *
*       DropIndicator                                                          *
*       RenderCompletion                                                       *
*       DragImageStyle                                                         *
*       DropStyle                                                              *
*       DiscardIndicator                                                       *
*       PrintIndicator                                                         *
*                                                                              *
*     Drag Item Types -                                                        *
*       any                                                                    *
*       binary                                                                 *
*       binaryData                                                             *
*       bitmap                                                                 *
*       container                                                              *
*       containerObject                                                        *
*       file                                                                   *
*       icon                                                                   *
*       plainText                                                              *
*       text                                                                   *
*       unknown                                                                *
*                                                                              *
*     Rendering Mechanisms (RM) -                                              *
*       rmFile                                                                 *
*       rmObject                                                               *
*       rmDiscard                                                              *
*       rmPrint                                                                *
*       rmLibrary                                                              *
*       rmAny                                                                  *
*                                                                              *
*     Rendering Formats (RF) -                                                 *
*       rfUnknown                                                              *
*       rfText                                                                 *
*       rfProcess                                                              *
*       rfSharedMem                                                            *
*       rfObject                                                               *
*       rfAny                                                                  *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  /* include PM headers */
  #define INCL_WINSTDDRAG
  #include <iwindefs.h>
}

#ifndef _IDMCOMM_
  #include <idmcomm.hpp>
#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMCOMM_CPP_
  #include <ipagetun.h>
#endif

#pragma data_seg(ICLNonConst)
/*------------------------------------------------------------------------------
| Static type definitions.                                                     |
|                                                                              |
| These values exploit some of the specific values used for the PM toolkit     |
| DRT_* values.  Item creation must support the operations corresponding       |
| to the DO_* values.  Additionally, we are defining new types that are        |
| NOT included in the PM Toolkit.                                              |
------------------------------------------------------------------------------*/
IDM::Type IDM::any              ( "Any"              ),
          IDM::binary           ( "Binary"           ),
          IDM::binaryData       ( DRT_BINDATA        ),
          IDM::bitmap           ( DRT_BITMAP         ),
          IDM::container        ( "Container"        ),
          IDM::containerObject  ( "Container Object" ),
          IDM::file             ( "File"             ),
          IDM::icon             ( DRT_ICON           ),
          IDM::menuItem         ( "Menuitem"         ),
          IDM::plainText        ( DRT_TEXT           ),
          IDM::text             ( "Text"             ),
          IDM::toolBarButton    ( "ToolBarButton"    ),
          IDM::unknown          ( DRT_UNKNOWN        );


/*------------------------------------------------------------------------------
| Static rendering mechanisms definitions.                                     |
|                                                                              |
| These values exploit the specific names documented in the PM Programming     |
| Reference and Guide as DRM_* values, including the private rendering         |
| mechanism that has been defined:  DRM_CSet++UI.                              |
------------------------------------------------------------------------------*/
IDM::RM IDM::rmFile     ( "DRM_OS2FILE"  ),
        IDM::rmObject   ( "DRM_OBJECT"   ),
        IDM::rmDiscard  ( "DRM_DISCARD"  ),
        IDM::rmPrint    ( "DRM_PRINT"    ),
        IDM::rmLibrary  ( "DRM_CSet++UI" ),
        IDM::rmAny      ( "" );

/*------------------------------------------------------------------------------
| Static rendering formats definitions.                                        |
|                                                                              |
| These values define the specific names of the DRF_* values that are          |
| supported in the UI Class Library.                                           |
------------------------------------------------------------------------------*/
IDM::RF IDM::rfUnknown    ( "DRF_UNKNOWN"   ),
        IDM::rfText       ( "DRF_TEXT"      ),
        IDM::rfProcess    ( "DRF_PROCESS"   ),
        IDM::rfSharedMem  ( "DRF_SHAREDMEM" ),
        IDM::rfObject     ( "DRF_OBJECT"    ),
        IDM::rfAny        ( "" );

#pragma data_seg()

