/*******************************************************************************
* FILE NAME: idmoper.cpp                                                       *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Implementation of the class(es):                                           *
*     IDMOperation                                                             *
*       IDMSourceOperation                                                     *
*       IDMTargetOperation                                                     *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1995       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
  #define IUSING_OLE
  #ifdef WIN32_LEAN_AND_MEAN
    #undef WIN32_LEAN_AND_MEAN
  #endif
  #ifdef _OLE2_H_
    #undef _OLE2_H_
  #endif
  /* define what subsections of PM headers to include */
  #define INCL_WINPOINTERS
  #define INCL_WINSYS
  #define INCL_WININPUT
  #define INCL_GPIBITMAPS
  #define INCL_WINSTDDRAG       // Standard drag stuff
  #define INCL_WINSTDCNR        // Standard container stuff
  #include <iwindefs.h>
#ifdef IC_WIN
  #include <shlobj.h>
#endif
}

// Exception handling
#ifndef _IEXCEPT_
  #include <iexcept.hpp>
#endif

// Trace Facility
#ifndef _ITRACE_
  #include <itrace.hpp>
#endif

#ifndef _IPOINT_
  #include <ipoint.hpp>
#endif

#ifndef _IDMSRCOP_
  #include <idmsrcop.hpp>
#endif

#ifndef _IDMTGTOP_
  #include <idmtgtop.hpp>
#endif

#ifndef _IDMITEM_
  #include <idmitem.hpp>
#endif

#ifndef _IDMPROV_
  #include <idmprov.hpp>
#endif

#ifndef _IDMEVENT_
  #include <idmevent.hpp>
#endif

#ifndef _IDMCOMM_
  #include <idmcomm.hpp>
#endif

#ifndef _ICTLEVT_
  #include <ictlevt.hpp>
#endif

#ifndef _ICNRCTL_
  #include <icnrctl.hpp>
#endif

#ifndef _ICNROBJ_
  #include <icnrobj.hpp>
#endif

#ifndef _ICNRREC_
  #include <icnrrec.hpp>
#endif

#ifndef _IWINDOW_
  #include <iwindow.hpp>
#endif

#ifndef _IHANDLE_
  #include <ihandle.hpp>
#endif

#ifndef _IDMIMAGE_
  #include <idmimage.hpp>
#endif

#ifndef _IDMSEQ_                           //PRIVATE
  #include <idmseq.hpp>
#endif

#ifndef _IGBITMAP_
  #include <igbitmap.hpp>
#endif

#ifndef _ICOORDSY_
  #include <icoordsy.hpp>
#endif

#ifdef IC_WIN
#ifndef _IDMDATOB_
  #include <idmdatob.hpp>
#endif

#ifndef IDMDRSRC_
  #include <idmdrsrc.hpp>
#endif

#ifndef _IDMFMTEN_
  #include <idmfmten.hpp>
#endif

#ifndef _IDMFORMAT_
  #include <idmformt.hpp>
#endif

#ifdef IC_WIN

  #ifndef _IDMDNFO_
    #include <idmdnfo.hpp>
  #endif

  #ifndef  _IDMDITM_
    #include <idmditm.hpp>
  #endif

  #ifndef _IDMFORMT_
    #include <idmformt.hpp>
  #endif

  #ifndef _IPLATFRM_
    #include <iplatfrm.hpp>
  #endif

#endif

#endif

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IDMOPER_CPP_
  #include <ipagetun.h>
#endif

#ifdef IC_WIN

class IDMSourceOperationData
{
  public:
    IDMSourceOperationData() : imageListFlags(ILC_COLOR  | ILC_MASK)
    {;}
      ~IDMSourceOperationData()
    {;}
  void doDrag(IDMSourceOperation *srcOper);

  private:
    IStream* marshall(IDragInfoWrapper* pDInfoWrapper);

    unsigned stackImages(                  // Result: number of images used
              IDMImage& anImage,           // Output: image of stack
              IPoint& stackHotspot,        // Output: size of member icons
              IDMSourceOperation* pSrcOp); // Input: source drag operation

    unsigned stackImagesPass1(             // Result: number of images used
              IDMImage& anImage,           // I/O: image of stack
              IDMSourceOperation* pSrcOp,  // Input: source drag operation
              const ISize& offset,         // Input: stagger offset in pixels
              HIMAGELIST& himagelist,      // Output: list of stackable images
              IRectangle& rect,            // Output: bounding rect of stack
              IPoint& hotspot);            // Output: working hotspot

    unsigned stackImagesPass2(         // Result: number of images used
              IDMImage& anImage,       // Output: image of stack
              const ISize& offset,     // Input: stagger offet in pixels
              const HIMAGELIST himagelist, // Input: list of images
              const IRectangle& rect); // Input: stack image bounds

    const unsigned imageListFlags;
};


class IDMTargetOperationData
{
  public:
    IDMTargetOperationData() : pDInfo(NULL)
      {;}
      ~IDMTargetOperationData()
      {delete pDInfo;}
    void buildItems(IDMTargetOperation *targetOper,
                    IDMTargetEnterEvent &event );
    Boolean isContainerControl( const IWindow *win );
  private:
    _DRAGINFO *pDInfo;
};

#endif

#define DRG_DEBUG        ((PVOID) 0x80000000L)    /* prevents screen locking */

typedef IDM::DragImageStyle DragImageStyle;
typedef IDM::DropStyle      DropStyle;

/* Initialize static */
#pragma data_seg(ICLNonConst)
IDMTargetOperation::Handle IDMTargetOperation::pDMTgtOpHandle(0);
Boolean IDMOperation::fgDebugSupport = false;
#ifdef IC_WIN
IDMSourceOperation::Handle IDMSourceOperation::pDMSrcOpHandle(0);
#endif
#pragma data_seg()

/*------------------------------------------------------------------------------
| Static operation definitions.                                                |
|                                                                              |
| These values exploit the specific values used for the PM toolkit             |
| DO_* values.  Item creation must support the operations corresponding        |
| to the DO_* values.                                                          |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const unsigned long IDMOperation::drag    (DO_DEFAULT);
const unsigned long IDMOperation::move    (DO_MOVE);
const unsigned long IDMOperation::copy    (DO_COPY);
const unsigned long IDMOperation::link    (DO_LINK);
const unsigned long IDMOperation::unknown (DO_UNKNOWN);
#pragma data_seg()

/*------------------------------------------------------------------------------
| IDMOperation::IDMOperation                                                   |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMOperation :: IDMOperation(IDMSourceBeginEvent &event)
  : src(event.source())
  , pos(event.position())
  , op(drag)
  , srcWindowHandle(0)
  , tgtWindowHandle(0)
  , pCnrObj(0)
  , pCnrRefresh(0)
#ifdef IC_PM
  , pPMDragInfo(0)
#endif
  , bDragResult(true)
  , pDMItemSeqCl(0)

{
}

/*------------------------------------------------------------------------------
| IDMOperation::IDMOperation                                                   |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMOperation :: IDMOperation(IDMTargetEnterEvent &event)  :
                             src(IDM::pointingDevice),
                             pos(IPoint(0,0)),
                             op(drag),
                             srcWindowHandle(0),
                             tgtWindowHandle(0),
                             pCnrObj(0),
                             pCnrRefresh(0),
#ifdef IC_PM
                             pPMDragInfo(0),
#endif
                             bDragResult(true),
                             pDMItemSeqCl(0)
{
}

/*------------------------------------------------------------------------------
| IDMOperation::~IDMOperation                                                  |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMOperation :: ~IDMOperation ( )
{
  delete this->pDMItemSeqCl;
}

/*------------------------------------------------------------------------------
| IDMOperation::operation                                                      |
|                                                                              |
| Returns the current direct manipulation operation.                           |
------------------------------------------------------------------------------*/
unsigned long IDMOperation :: operation () const
{
  return (this->op);
}

/*------------------------------------------------------------------------------
| IDMOperation::setOperation                                                   |
|                                                                              |
| Sets the operation of the Direct Manipulation operation.                     |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setOperation (unsigned long Operation)
{
  this->op = Operation;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::position                                                       |
|                                                                              |
| Returns the drag or drop position within the source or target window.        |
------------------------------------------------------------------------------*/
IPoint IDMOperation :: position () const
{
  return (this->pos);
}

/*------------------------------------------------------------------------------
| IDMOperation::source                                                         |
|                                                                              |
| Returns the source of the direct manipulation operation                      |
| (i.e. always pointing device)                                                |
------------------------------------------------------------------------------*/
IDM::Source IDMOperation :: source() const
{
  return (this->src);
}

/*------------------------------------------------------------------------------
| IDMOperation::sourceWindowHandle                                             |
|                                                                              |
| Returns the handle of the source window for the direct manipulation          |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindowHandle IDMOperation :: sourceWindowHandle () const
{
  return (this->srcWindowHandle);
}

/*------------------------------------------------------------------------------
| IDMOperation::targetWindowHandle                                             |
|                                                                              |
| Returns the handle of the target window for the direct manipulation          |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindowHandle IDMOperation :: targetWindowHandle () const
{
  return (this->tgtWindowHandle);
}

/*------------------------------------------------------------------------------
| IDMOperation::sourceWindow                                                   |
|                                                                              |
| Returns a pointer to the source window for the direct manipulation           |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindow* IDMOperation :: sourceWindow () const
{
  return (IWindow::windowWithHandle(this->srcWindowHandle));
}

/*------------------------------------------------------------------------------
| IDMOperation::targetWindow                                                   |
|                                                                              |
| Returns a pointer to the target window for the direct manipulation           |
| operation.                                                                   |
------------------------------------------------------------------------------*/
IWindow* IDMOperation :: targetWindow () const
{
  return (IWindow::windowWithHandle(this->tgtWindowHandle));
}

/*------------------------------------------------------------------------------
| IDMOperation::numberOfItems                                                  |
|                                                                              |
| Returns the number of items in the collection.                               |
------------------------------------------------------------------------------*/
unsigned IDMOperation :: numberOfItems ()
{
  return unsigned(itemCollection()->numberOfElements());
}

/*------------------------------------------------------------------------------
| IDMOperation::item                                                           |
|                                                                              |
| Returns the drag item with a given index.  0 is returned if the index is     |
| invalid.                                                                     |
------------------------------------------------------------------------------*/
IDMItem::Handle IDMOperation :: item(unsigned index)
{
  /*------------------------------------------------------------------------
    Get number of items
  ------------------------------------------------------------------------*/
  unsigned count = numberOfItems();

  /*------------------------------------------------------------------------
    Verify index
  ------------------------------------------------------------------------*/
  if ((index < 1)  ||  (index > count))
  {
  //Throw exception
  ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
      IBaseErrorInfo::invalidRequest,
      IException::recoverable);
  return(0);
  }

  /*------------------------------------------------------------------------
     Return item handle at index
  ------------------------------------------------------------------------*/
  return(itemCollection()->elementAtPosition(index));
}

/*------------------------------------------------------------------------------
| IDMOperation::replaceItem                                                    |
|                                                                              |
| Replace a given drag item with another.                                      |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: replaceItem (unsigned index,
                                           const IDMItem::Handle &pDI)
{
  /*------------------------------------------------------------------------
     Get number of items
  ------------------------------------------------------------------------*/
  unsigned count = numberOfItems();

  /*------------------------------------------------------------------------
    Verify index
  ------------------------------------------------------------------------*/
  if ((index < 1)  ||  (index > count))
  {
    //Throw exception
    ITHROWLIBRARYERROR(IC_INDEX_OUT_OF_RANGE,
                       IBaseErrorInfo::invalidRequest,
                       IException::recoverable);
    return(*this);
  }

  /*------------------------------------------------------------------------
    Replace the item
  ------------------------------------------------------------------------*/
  removeItem(index);
  itemCollection()->addAtPosition(index, pDI);

  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::addItem                                                        |
|                                                                              |
| Associate another drag item with this operation.                             |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: addItem (const IDMItem::Handle &pDI)
{
  IMODTRACE_DEVELOP( "IDMOperation::addItem" );

  /*-------------------------------------------------------------------
     Store item in collection and bump the operation's use count.
   -------------------------------------------------------------------*/
  itemCollection()->add(pDI);
  ITRACE_DEVELOP( "IDMOperation::addItem - addRef" );
  this->addRef();
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::removeItem                                                     |
|                                                                              |
| Remove a drag item from the operation.                                       |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: removeItem (unsigned index)
{
  /*------------------------------------------------------------------------
    Define a cursor for the drag item sequence and set to the proper
    position.
  ------------------------------------------------------------------------*/
  IDMItemSeq::Cursor cursor(*(itemCollection()));
  itemCollection()->setToPosition(index, cursor);

  /*-----------------------------------------------------------------------
      Set the item handle to 0 and remove item from the collection
   -----------------------------------------------------------------------*/
  itemCollection()->elementAt(cursor) = 0;
  itemCollection()->removeAtPosition(index);
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::removeItem                                                     |
|                                                                              |
| Remove a drag item from the operation.                                       |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: removeItem (const IDMItem::Handle &pDI)
{
  /*------------------------------------------------------------------------
    Get number of items
  ------------------------------------------------------------------------*/
  unsigned count = numberOfItems();

  /*------------------------------------------------------------------------
    Loop thru the collection and remove item if a match is found
  ------------------------------------------------------------------------*/
  for (unsigned index = 1; (index <= count); index++)
  {
    if ((itemCollection()->elementAtPosition(index)) == pDI)
    {
      removeItem(index);
      break;
    }
  }

  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::containerObject                                                |
|                                                                              |
| Returns a container object.                                                  |
------------------------------------------------------------------------------*/
IContainerObject* IDMOperation :: containerObject() const
{
  return (this->pCnrObj);
}

/*------------------------------------------------------------------------------
| IDMOperation::setContainerObject                                             |
|                                                                              |
| Sets a pointer to a container object.                                        |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setContainerObject(IContainerObject *object)
{
  this->pCnrObj = object;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setContainerRefreshOff                                         |
|                                                                              |
| Disables the container refreshes, will be re-enable it after processing.     |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setContainerRefreshOff(IContainerControl* pCnr)
{
  IMODTRACE_DEVELOP("IDMOperation::setContainerRefreshOff");

  this->pCnrRefresh = pCnr;
  pCnr->setRefreshOff();

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setContainerRefreshOn                                          |
|                                                                              |
| Re-enables container refreshes.                                              |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setContainerRefreshOn()
{
  IMODTRACE_DEVELOP("IDMOperation::setContainerRefreshOn");

  if (this->pCnrRefresh)
  {
    if (targetWindowHandle())
    this->pCnrRefresh->setRefreshOn().refresh(true);
#ifdef IC_PM
    else
    {
      DRAGINFO*  pPMDragInfo = dragInfo();
      if (!pPMDragInfo ||
         ((unsigned long)pPMDragInfo->usOperation == IDMOperation::copy) ||
         ((unsigned long)pPMDragInfo->usOperation == IDMOperation::link) ||
          dragWasInterrupted())
      {
        this->pCnrRefresh->setRefreshOn();
      }
      else
      {
        this->pCnrRefresh->setRefreshOn().refresh();
      }
    }
#endif
#ifdef IC_WIN
    else
    {
      _DRAGINFO*  pPMDragInfo = dragInfo();
      unsigned long dragOp = 0;
      if (!pPMDragInfo ||
         (dragOp=((unsigned long)IDragInfoWrapper(*pPMDragInfo).dragOperation())
          == IDMOperation::copy) ||
             (dragOp == IDMOperation::link) ||
             dragWasInterrupted())
      {
        this->pCnrRefresh->setRefreshOn();
      }
      else
      {
        this->pCnrRefresh->setRefreshOn().refresh(true);
      }
    }
#endif
  }

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setSourceWindowHandle                                          |
|                                                                              |
| Sets the source window of the direct manipulation operation during a source  |
| begin event.                                                                 |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation ::
              setSourceWindowHandle (IWindowHandle srcWindow)
{
  this->srcWindowHandle = srcWindow;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setTargetWindowHandle                                          |
|                                                                              |
| Sets the target window of the direct manipulation operation during a target  |
| enter event.                                                                 |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation ::
              setTargetWindowHandle (IWindowHandle tgtWindow)
{
  this->tgtWindowHandle = tgtWindow;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setSource                                                      |
|                                                                              |
| Sets the source of the Direct Manipulation operation.                        |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setSource (Source source)
{
  this->src = source;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::setPosition                                                    |
|                                                                              |
| Sets the pointing device position for the Direct Manipulation operation.     |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setPosition (IPoint position)
{
  this->pos = position;
  return (*this);
}

#if defined(IC_PM) || defined(IC_WIN)
/*------------------------------------------------------------------------------
| IDMOperation::setDragInfo                                                    |
|                                                                              |
| Sets the pointer to the PM DRAGINFO structure.                               |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setDragInfo (_DRAGINFO *pDragInfo)
{
  this->pPMDragInfo = pDragInfo;
  return (*this);
}
#endif

/*------------------------------------------------------------------------------
| IDMOperation::setDragResult                                                  |
|                                                                              |
| Sets the result of the drag operation.                                       |
------------------------------------------------------------------------------*/
IDMOperation& IDMOperation :: setDragResult (Boolean result)
{
  this->bDragResult = result;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMOperation::itemCollection                                                 |
|                                                                              |
| Returns a pointer to the source collection of IDMItem::Handles.              |
| Notes:                                                                       |
|   1) Private                                                                 |
------------------------------------------------------------------------------*/
IDMItemSeq* IDMOperation :: itemCollection()
{
  if (!this->pDMItemSeqCl)
    this->pDMItemSeqCl = new IDMItemSeq;

  return (this->pDMItemSeqCl);
}


/*------------------------------ Source Operation ----------------------------*/


/*------------------------------------------------------------------------------
| IDMSourceOperation::IDMSourceOperation                                       |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMSourceOperation :: IDMSourceOperation(IDMSourceBeginEvent &event,
                                         DragImageStyle dIStyle)
  : IDMOperation(event),
    drgImgStyle(dIStyle),
    totalImgOffset(0,0),
    ptrOffset(0,0),
    stackingPct(IPair(50,50))
#ifdef IC_WIN
  , fDMSourceOperationData( new IDMSourceOperationData() )
#endif
{
  IMODTRACE_DEVELOP("IDMSourceOperation::ctor");

  if (event.eventId() == WM_BEGINDRAG)
    setSourceWindowHandle(event.handle());
#ifdef IC_PM
  else
  {
    /*-----------------------------------------------------------------
       Verify that this is the correct message and notification code
     -----------------------------------------------------------------*/
    IASSERT(event.eventId() == WM_CONTROL &&
            event.parameter1().number2() == CN_INITDRAG);

    IControlEvent ctlevt(event);
    setSourceWindowHandle(ctlevt.controlWindow()->handle());
  }
#endif
}

/*------------------------------------------------------------------------------
| IDMSourceOperation:: ~IDMSourceOperation                                     |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMSourceOperation :: ~IDMSourceOperation ()
{
  IMODTRACE_DEVELOP("IDMSourceOperation::dtor");

  /*-------------------------------------------------------------------
    Enable container refresh (this function can detect whether or
    not we're processing a container) ...
   -------------------------------------------------------------------*/
  setContainerRefreshOn();

#ifdef IC_PM
  DRAGINFO*  pPMDragInfo = dragInfo();
  DrgFreeDraginfo (pPMDragInfo);
#endif

#ifdef IC_WIN
  _DRAGINFO*  pPMDragInfo = dragInfo();
  IDragInfoWrapper dInfoWrapper( *pPMDragInfo );
  dInfoWrapper.deleteSourceResources();

  if (fDMSourceOperationData )
    delete fDMSourceOperationData;
#endif

}

/*------------------------------------------------------------------------------
| IDMSourceOperation::begin                                                    |
|                                                                              |
| Begin the drag operation.                                                    |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::begin ()
{
  IMODTRACE_DEVELOP("IDMSourceOperation::begin");

#ifdef IC_PM
  _DRAGINFO       *pDInfo;
  _DRAGIMAGE      DragImage;
  _DRAGIMAGE      *pDrgImg;
  _DRAGIMAGE      *pDragImage = 0;
  Boolean         done = false;
  unsigned long   arraySize = 0;
  unsigned short  numberOfImagesFound = 0;
  unsigned long   ulItemCnt = numberOfItems();
  DragImageStyle  diStyle   = imageStyle();

  /*-------------------------------------------------------------------
     Make sure we have some items to drag...
   -------------------------------------------------------------------*/
  if (ulItemCnt)
  {
    pDInfo = DrgAllocDraginfo (ulItemCnt);
    if (!pDInfo)
      ITHROWGUIERROR("DrgAllocDraginfo");

    /*-------------------------------------------------------------------
       Set the window handle of the source and the drag operation
       in DRAGINFO.   All other fields in DRAGINFO are initialized
       by PM.
     -------------------------------------------------------------------*/
    pDInfo->hwndSource = sourceWindowHandle();
    pDInfo->usOperation = operation();

    /*-------------------------------------------------------------------
       Set up the drag image array if needed.
     -------------------------------------------------------------------*/
    if (diStyle == IDM::stack3AndFade)
    {
      arraySize = 4;
    }
    else if (diStyle == IDM::allStacked)
    {
      arraySize = ulItemCnt;
    }

    if (arraySize)
    {
      pDrgImg = pDragImage = new _DRAGIMAGE[arraySize];
      IASSERT(pDrgImg!=0);
    }

    /*-----------------------------------------------------------------
       Store "handle" to our drag information
     -----------------------------------------------------------------*/
    setDragInfo (pDInfo);

    /*-----------------------------------------------------------------
       Loop through the collection of IDMItems.  For each, copy
       the corresponding data, and set each one into the PM DragInfo
       structure.  Also, build the array of drag images to be displayed.
     -----------------------------------------------------------------*/
    for (unsigned long ulItem = 0; (ulItem < ulItemCnt); ulItem++)
    {
      IDMItem::Handle pItemHandle;
      pItemHandle = item((ulItem+1));
      _DRAGITEM      PMDragItem;

      /*---------------------------------------------------------------
         Create a PM DragItem from the corresponding IDMItem.
         Then, set the PM DragItem into the PM DragInfo structure.
       ---------------------------------------------------------------*/
      pItemHandle->asPMDragItem(&PMDragItem);
      if (!DrgSetDragitem (pDInfo, &PMDragItem, sizeof(_DRAGITEM), ulItem))
        ITHROWGUIERROR("DrgSetDragitem");

      /*---------------------------------------------------------------
         Query the PM DragItem pointer from shared memory, and set
         its value in the corresponding ICLUI drag item.
       ---------------------------------------------------------------*/
      _DRAGITEM* pdi = DrgQueryDragitemPtr (pDInfo, ulItem);
      if (pdi)
      {
        pItemHandle->setPMDragItem(pdi);
      }
      else
      {
        ITHROWGUIERROR("DrgQueryDragitemPtr");
      }

      /*---------------------------------------------------------------
         Set up the dragimage for this drag item.
       ---------------------------------------------------------------*/
      if ((diStyle == IDM::allStacked) ||
          (diStyle == IDM::stack3AndFade))
      {
        IDMImage &image = pItemHandle->image();

        /*-------------------------------------------------------------
           Determine if we need to add the fade icon.
         -------------------------------------------------------------*/
        if ((diStyle == IDM::stack3AndFade) && (!done) &&
            (numberOfImagesFound == 3))
        {
          /*-----------------------------------------------------------
             Add the special fade icon if we have already found the
             3 we stack.
           -----------------------------------------------------------*/
          image.asPMDragImage(pDrgImg, false, true);
          ISize imageOffset = totalImageOffset();
          pDrgImg->cxOffset = (short)imageOffset.width();
          pDrgImg->cyOffset = (short)imageOffset.height();
          done = true;
          numberOfImagesFound++;
        }
        else if (!done)
        {
          image.asPMDragImage(pDrgImg);

          /*-----------------------------------------------------------
             If we got an image, increase the count.
           -----------------------------------------------------------*/
          if (pDrgImg->hImage)
          {
            ISize imageOffset;
            ISize  ptrOffset;
            numberOfImagesFound++;
            ptrOffset = image.pointerOffset();
            if (ptrOffset == 0)
            {
              imageOffset = totalImageOffset();
              pDrgImg->cxOffset = (short)imageOffset.width();
              pDrgImg->cyOffset = (short)imageOffset.height();
                storeImageOffset (&image);
            }
            else
            {
              pDrgImg->cxOffset = (short)ptrOffset.width();
              pDrgImg->cyOffset = (short)ptrOffset.height();
            }
            pDrgImg++;
          }
        }
      }
    }

    /*-----------------------------------------------------------------
       Give them system icons if they ask for them, or if they did not
       supply any images.
     -----------------------------------------------------------------*/
    if ((diStyle == IDM::systemImages) || (!numberOfImagesFound))
    {
      IDMImage image = IDMImage();
      image.asPMDragImage(&DragImage, true);

      /*---------------------------------------------------------------
         If we are dragging more than 1 item, give them the multiple
         file icon.  Otherwise, give them the single file icon.
       ---------------------------------------------------------------*/
      if (ulItemCnt > 1)
      {
        DragImage.hImage = (unsigned long)*(new ISystemPointerHandle (
                           ISystemPointerHandle::multipleFile));
      }
      else
      {
        DragImage.hImage = (unsigned long)*(new ISystemPointerHandle (
                           ISystemPointerHandle::singleFile));
      }
      numberOfImagesFound = 1;
      pDragImage = &DragImage;
    }

    /*-----------------------------------------------------------------
       Start the drag - DrgDrag executes in a synchronous manner

       Note:  Check the debug flag in order to fully enable debug
              support.
     -----------------------------------------------------------------*/
    unsigned rc = DrgDrag (pDInfo->hwndSource,
                           pDInfo,
                           pDragImage,
                           numberOfImagesFound,
                           VK_ENDDRAG,
                           ((debugSupport()) ? DRG_DEBUG
                                                : 0));

    /*-----------------------------------------------------------------
       Delete the array if we created one.
      -----------------------------------------------------------------*/
    if ((diStyle != IDM::systemImages) && (pDragImage))
    {
      delete [] pDragImage;
    }
    ITRACE_DEVELOP("IDMSourceOperation:  DrgDrag rc = "+IString(rc));

    /*-----------------------------------------------------------------
       Remove any remaining items from the source operation's
       collection.  There will be remaining items if one of the
       following conditions occurs:
          - the operation was cancelled
          - help was requested
          - the drop was not allowed
        All of these conditions return NULL
     -----------------------------------------------------------------*/
    if (!rc)
    {
      setDragResult( false );
      unsigned count = numberOfItems();
      for (unsigned i = count; i; i--)
        removeItem(i);
    }
  } /* end if */
#endif

#ifdef IC_WIN
  /*-------------------------------------------------------------------
     Start the drag (Windows)
   -------------------------------------------------------------------*/
  fDMSourceOperationData->doDrag(this);
#endif

  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::imageStyle                                               |
|                                                                              |
| Returns the source operation drag image style.                               |
------------------------------------------------------------------------------*/
DragImageStyle IDMSourceOperation::imageStyle () const
{
  return (this->drgImgStyle);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::pointerOffset                                            |
|                                                                              |
| Returns the position of the pointing device relative to the source container |
| object's origin.                                                             |
------------------------------------------------------------------------------*/
ISize IDMSourceOperation::pointerOffset () const
{
  return(this->ptrOffset);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::operation                                                |
|                                                                              |
| Returns the current direct manipulation operation.                           |
------------------------------------------------------------------------------*/
unsigned long IDMSourceOperation :: operation () const
{
#ifdef IC_WIN
  /*-------------------------------------------------------------------
     This is where we add the operation support to determine
     copy, move, link, etc...
   -------------------------------------------------------------------*/
  return( Inherited::operation() );
#endif

#ifdef IC_PM
  _DRAGINFO* di = dragInfo();
  if (di)
  {
    return((unsigned long)di->usOperation);
  }
  else
  {
    return( Inherited::operation() );
  }
#endif
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::stackingPercentage                                       |
|                                                                              |
| Returns the stacking percentage.                                             |
------------------------------------------------------------------------------*/
IPair IDMSourceOperation::stackingPercentage () const
{
  return (this->stackingPct);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setImageStyle                                            |
|                                                                              |
| Sets the source operation drag image style.                                  |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::setImageStyle(DragImageStyle dIStyle)
{
  this->drgImgStyle = dIStyle;
  return (*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setPointerOffset                                         |
|                                                                              |
| Sets the position of the pointing device relative to the source container    |
| object's origin.                                                             |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::setPointerOffset (const ISize &pOff)
{
  this->ptrOffset = pOff;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setStackingPercentage                                    |
|                                                                              |
| Sets the stacking percentage.                                                |
------------------------------------------------------------------------------*/
IDMSourceOperation& IDMSourceOperation::
                    setStackingPercentage (const IPair& stackingPercent)
{
  this->stackingPct = stackingPercent;
  return (*this);
}

#ifdef IC_PM
/*------------------------------------------------------------------------------
| IDMSourceOperation::operationFrom                                            |
|                                                                              |
| Retrieves a handle to the source drag operation from the drag information    |
| structure.                                                                   |
------------------------------------------------------------------------------*/
IDMSourceOperation::Handle IDMSourceOperation ::
                           operationFrom(_DRAGINFO *pDInfo)
{
  /*-------------------------------------------------------------------
     Access the PM DRAGINFO structure which was passed to us.
   -------------------------------------------------------------------*/
  if (!DrgAccessDraginfo (pDInfo))
   ITHROWGUIERROR("DrgAccessDraginfo");

  /*-------------------------------------------------------------------
     Get count of PM drag items
   -------------------------------------------------------------------*/
  unsigned count = pDInfo->cditem;

  /*-------------------------------------------------------------------
     Loop thru PM Drag Items until we find one we can construct
     source operation form ...
   -------------------------------------------------------------------*/
  IDMSourceOperation::Handle pSrcOpH = 0;
  for (unsigned i=0; i < count; i++)
  {
    /*-----------------------------------------------------------------
       Get PM drag item
     -----------------------------------------------------------------*/
    _DRAGITEM *pPMDItem = DrgQueryDragitemPtr (pDInfo, i);

    /*-----------------------------------------------------------------
       If the pointer and ulItemID are not NULL, assume ulItemID
       contains a pointer to the source item which in turn, can
       give us the pointer to the source operation object,
       which we can construct an IDMSourceOperation::Handle from.
     -----------------------------------------------------------------*/
    if (pPMDItem)
    {
      if (pPMDItem->ulItemID)
      {
        pSrcOpH = ((IDMItem *)(pPMDItem->ulItemID))->sourceOperation();
        break;
      }
    }
  }

  return(pSrcOpH);
}
#endif

/*------------------------------------------------------------------------------
| IDMSourceOperation::storeImageOffset                                         |
|                                                                              |
| Store the offset at which to place the next image.                           |
| Notes:                                                                       |
|   1) Private                                                                 |
------------------------------------------------------------------------------*/
void IDMSourceOperation::storeImageOffset(IDMImage *pIDMImage)
{
  unsigned long style = (pIDMImage->style()).asUnsignedLong();
  ISize stretchSize = pIDMImage->stretchSize();
  IPointerHandle ptr;
  IBitmapHandle  bmp;
#ifdef IC_PM
  POINTERINFO    PointerInfo;
#endif
  long           cxPtr,
                 cyPtr;

  /*-------------------------------------------------------------------
     Calculate the percentage that is used to position the next
     image in relation to the current image.  Default is (50,50)
     which means the nth image (where n=2, 3, or 4) is positioned
     with its lower left corner in the middle of the current image.
   -------------------------------------------------------------------*/
#ifdef IC_PM
  if (style & DRG_ICON)
  {
    ptr = pIDMImage->pointer();
    if ((unsigned long)ptr)
    {
      if (WinQueryPointerInfo ((HPOINTER)ptr, &PointerInfo))
      {
        if (PointerInfo.fPointer)
        {
          cxPtr = WinQuerySysValue (HWND_DESKTOP, SV_CXPOINTER);
          cyPtr = WinQuerySysValue (HWND_DESKTOP, SV_CYPOINTER);
          if ((!cxPtr) || (!cyPtr))
            ITHROWGUIERROR("WinQuerySysValue");
        }
        else
        {
          cxPtr = WinQuerySysValue (HWND_DESKTOP, SV_CXICON);
          cyPtr = WinQuerySysValue (HWND_DESKTOP, SV_CYICON);
          if ((!cxPtr) || (!cyPtr))
            ITHROWGUIERROR("WinQuerySysValue");
        }
        this->totalImgOffset.setWidth(this->totalImgOffset.width() +
                             (stackingPercentage().coord1() *
                             (cxPtr + stretchSize.width())) / 100);
        this->totalImgOffset.setHeight(this->totalImgOffset.height() +
                             (stackingPercentage().coord2() *
                             (cyPtr + stretchSize.height())) / 100);
      }
      else
        ITHROWGUIERROR("WinQueryPointerInfo");
    }
  }
  else
  {
#endif
    bmp = pIDMImage->bitmap();
    if ((unsigned long)bmp.asUnsigned() ) // new
    {
      IGBitmap bitmap(bmp);
      ISize    bmpSize = bitmap.size();
        this->totalImgOffset.setWidth(this->totalImgOffset.width() +
                             (stackingPercentage().coord1() *
                             (bmpSize.width() + stretchSize.width())) / 100);
        this->totalImgOffset.setHeight(this->totalImgOffset.height() +
                             (stackingPercentage().coord2() *
                             (bmpSize.height() + stretchSize.height())) / 100);
#ifdef IC_PM
    }
#endif
  }
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::totalImageOffset                                         |
|                                                                              |
| Return the total image offset.                                               |
| Notes:                                                                       |
|   1) Private                                                                 |
------------------------------------------------------------------------------*/
ISize IDMSourceOperation::totalImageOffset()
{
  return (this->totalImgOffset);
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| IDMSourceOperation::sourceOperation                                          |
|                                                                              |
| Returns the source operation object handle.                                  |
------------------------------------------------------------------------------*/
IDMSourceOperation::Handle IDMSourceOperation :: sourceOperation()
{
  return (pDMSrcOpHandle);
}

/*------------------------------------------------------------------------------
| IDMSourceOperation::setSourceOperation                                       |
|                                                                              |
| Sets the source operation object handle.                                     |
------------------------------------------------------------------------------*/
void IDMSourceOperation ::
    setSourceOperation(IDMSourceOperation::Handle sourceOp)
{
  pDMSrcOpHandle = sourceOp;
}
#endif


/*------------------------------ Target Operation ----------------------------*/

/*------------------------------------------------------------------------------
| IDMTargetOperation::IDMTargetOperation                                       |
|                                                                              |
| Constructor                                                                  |
------------------------------------------------------------------------------*/
IDMTargetOperation :: IDMTargetOperation(IDMTargetEnterEvent &event)  :
  IDMOperation(event),
  bWasDragAfter(0),
  sizeDropOffset(0,0),
  dropStyle(IDM::notContainer),
  bNotDragLeave(true),
  bDrop(false)
#ifdef IC_WIN
 ,fDMTargetOperationData( new IDMTargetOperationData() )
#endif
#ifdef IC_PMWIN
 ,bFirstTimeEntered(true)
#endif
{
  IMODTRACE_DEVELOP("IDMTargetOperation::ctor");

#ifdef IC_WIN
  /*-------------------------------------------------------------------
    Build a list of the target drag items
    -------------------------------------------------------------------*/
  fDMTargetOperationData->buildItems(this, event);

  if ( fDMTargetOperationData->isContainerControl( event.window() ))
  {
    /*-----------------------------------------------------------------
      Set the target window handle
      -----------------------------------------------------------------*/
    setTargetWindowHandle(event.window()->handle());

    /*-----------------------------------------------------------------
       Get flag for over/after event and set our default drop style ...
      -----------------------------------------------------------------*/
    this->bWasDragAfter = event.isDragAfter();
    if (this->bWasDragAfter)
      setStyle(IDM::alignFlow);
    else
      setStyle(IDM::dropAtPosition);

    /*-----------------------------------------------------------------
       Set the container object pointer for the object that the over
       or after event occurred over.  0 if over container white space.
     -----------------------------------------------------------------*/
    setContainerObject(event.object());
  }
#endif

#ifdef IC_PM
  _DRAGINFO *pDInfo;

  IDMItemProvider *pProvider;

  if (event.eventId() == DM_DRAGOVER)
  {
    /*-----------------------------------------------------------------
      Set the target window handle
      -----------------------------------------------------------------*/
    setTargetWindowHandle(event.handle());

    /*-----------------------------------------------------------------
      Get the PM DRAGINFO structure
     -----------------------------------------------------------------*/
    pDInfo = (_DRAGINFO *)event.parameter1().asUnsignedLong();

    /*-----------------------------------------------------------------
      Get pointer to the item provider
     -----------------------------------------------------------------*/
    pProvider = event.window()->itemProvider();
  }
  else
  {
    /*-----------------------------------------------------------------
       Verify that this is the correct message and notification code
     -----------------------------------------------------------------*/
    IASSERT(event.eventId() == WM_CONTROL &&
            (event.parameter1().number2() == CN_DRAGOVER ||
             event.parameter1().number2() == CN_DRAGAFTER));

    /*-----------------------------------------------------------------
       Set the target window handle
     -----------------------------------------------------------------*/
    IControlEvent ctlevt(event);
    setTargetWindowHandle(ctlevt.controlWindow()->handle());

    /*-----------------------------------------------------------------
       Get flag for over/after event and set our default drop style ...
      -----------------------------------------------------------------*/
    this->bWasDragAfter = event.isDragAfter();
    if (this->bWasDragAfter)
      setStyle(IDM::alignFlow);
    else
      setStyle(IDM::dropAtPosition);

    /*-----------------------------------------------------------------
       Set the container object pointer for the object that the over
       or after event occurred over.  0 if over container white space.
     -----------------------------------------------------------------*/
    setContainerObject(event.object());

    /*-----------------------------------------------------------------
       Get the PM DRAGINFO structure
     -----------------------------------------------------------------*/
    _CNRDRAGINFO *pCnrDInfo =
                 (_CNRDRAGINFO *)event.parameter2().asUnsignedLong();
    pDInfo = pCnrDInfo->pDragInfo;

    /*-----------------------------------------------------------------
      Get pointer to the item provider
     -----------------------------------------------------------------*/
    pProvider = ctlevt.controlWindow()->itemProvider();
  }
  /*-------------------------------------------------------------------
     Access the PM DRAGINFO structure which was passed to us.
   -------------------------------------------------------------------*/
  if (!DrgAccessDraginfo (pDInfo))
    ITHROWGUIERROR("DrgAccessDraginfo");

  /*-------------------------------------------------------------------
     Store "handle" to our drag information
   -------------------------------------------------------------------*/
  setDragInfo (pDInfo);

  /*-------------------------------------------------------------------
     Set the source window handle
   -------------------------------------------------------------------*/
  setSourceWindowHandle(pDInfo->hwndSource);

  /*-------------------------------------------------------------------
      Set the target enter (i.e. drag over) position of the pointing
     device.
    -------------------------------------------------------------------*/
  setPosition (IPoint(pDInfo->xDrop, pDInfo->yDrop));

  /*-------------------------------------------------------------------
      Set the current drag operation.
    -------------------------------------------------------------------*/
  setOperation (pDInfo->usOperation);

  /*-------------------------------------------------------------------
      Get first PM drag item
    -------------------------------------------------------------------*/
  unsigned long ulItem = 0;
  _DRAGITEM *pPMDItem = DrgQueryDragitemPtr (pDInfo, ulItem);

  /*-------------------------------------------------------------------
      For each dragitem in the draginfo, we need to create a
      corresponding IDMItem.  We construct each based upon the
      information in the PM DRAGINFO structure.
    -------------------------------------------------------------------*/
  while (pPMDItem)
  {
    /*-----------------------------------------------------------------
       Get the target operation handle.
       -----------------------------------------------------------------*/
    Handle pTgtOpH;
    pTgtOpH = this;

    /*-----------------------------------------------------------------
      Create the generic drag item based upon the PM DRAGITEM ...
      -----------------------------------------------------------------*/
    IDMItem tempItem(pTgtOpH, pPMDItem);
    IDMItem::Handle pDIH;
    pDIH = &tempItem;

    /*-----------------------------------------------------------------
      Obtain a derived target item based upon the generic drag
      item via the item provider  - Throw an exception if the
      provider fails ...
      -----------------------------------------------------------------*/
    IDMItem::Handle pTgtItemH;
    pTgtItemH = pProvider->provideTargetItemFor(pDIH);
    if (!pTgtItemH)
    {
      ITRACE_DEVELOP("IDMTargetOperation: target item provider for failed");
      //Throw exception here
      ITHROWLIBRARYERROR(IC_TARGET_ITEM_PROVIDER_FAILED,
          IBaseErrorInfo::invalidRequest,
          IException::recoverable);
      break;
    }

    /*-----------------------------------------------------------------
    Add the target item to the collection ...
      -----------------------------------------------------------------*/
    addItem (pTgtItemH);

    /*-----------------------------------------------------------------
     Get the next PM DRAGITEM, it it exists ...
     -----------------------------------------------------------------*/
    pPMDItem = DrgQueryDragitemPtr (pDInfo, ++ulItem);
  }

  /*-------------------------------------------------------------------
     Free the PM DRAGINFO structure
    -------------------------------------------------------------------*/
  DrgFreeDraginfo (pDInfo);
#endif
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::~IDMTargetOperation                                      |
|                                                                              |
| Destructor                                                                   |
------------------------------------------------------------------------------*/
IDMTargetOperation :: ~IDMTargetOperation ()
{
  IMODTRACE_DEVELOP("IDMTargetOperation::dtor");

  /*-------------------------------------------------------------------
     Enable container refresh (this function can detect whether or
     not we're processing a container) ...
   -------------------------------------------------------------------*/
  setContainerRefreshOn();

#ifdef IC_PM
  DRAGINFO*  pPMDragInfo = dragInfo();
  if (this->bNotDragLeave)
    DrgDeleteDraginfoStrHandles (pPMDragInfo);

  if (this->bDrop)
    DrgFreeDraginfo (pPMDragInfo);
#endif

#ifdef IC_WIN
  if (fDMTargetOperationData )
    delete fDMTargetOperationData;
#endif

}

/*------------------------------------------------------------------------------
| IDMTargetOperation::instanceFor                                              |
|                                                                              |
| Returns the IDMTargetOperation object for the given target enter event.      |
------------------------------------------------------------------------------*/
IDMTargetOperation::Handle IDMTargetOperation ::
                           instanceFor (IDMTargetEnterEvent &event)
{
  /*-------------------------------------------------------------------
     If the target operation has not been created yet, create it and
     store it as part of the IDMTargetOperation class.
   -------------------------------------------------------------------*/
  IDMTargetOperation::Handle pTgtOpH = targetOperation();
  if (!pTgtOpH)
    return (0);

#ifdef IC_WIN
  /*-------------------------------------------------------------------
     If the target is a container ...
   -------------------------------------------------------------------*/
  if ( pTgtOpH->fDMTargetOperationData->isContainerControl( event.window() ))
  {
    pTgtOpH->bWasDragAfter = event.isDragAfter();
    if (pTgtOpH->bWasDragAfter)
      pTgtOpH->setStyle(IDM::alignFlow);
    else
      pTgtOpH->setStyle(IDM::dropAtPosition);

    /*-------------------------------------------------------------------
       Set the container object pointer for the object that the over
       or after event occurred over.  0 if over container white space.
     -------------------------------------------------------------------*/
    pTgtOpH->setContainerObject(event.object());

    /*-------------------------------------------------------------------
       Set the target enter (i.e. drag over) position of the pointing
       device.
      -------------------------------------------------------------------*/
    pTgtOpH->setPosition (event.position());
  }
#endif

#ifdef IC_PM
  /*------------------------------------------------------------------------
    Extract pointer to PM DRAGINFO struct from event parms
  ------------------------------------------------------------------------*/
  _DRAGINFO *pDInfo;
  if (event.eventId() == DM_DRAGOVER)
  {
    /*-----------------------------------------------------------------
       Get the PM DRAGINFO structure
     -----------------------------------------------------------------*/
    pDInfo = (_DRAGINFO *)event.parameter1().asUnsignedLong();
  }
  else
  {
    /*-----------------------------------------------------------------
       Verify that this is the correct message and notification code
     -----------------------------------------------------------------*/
    IASSERT(event.eventId() == WM_CONTROL &&
            (event.parameter1().number2() == CN_DRAGOVER ||
             event.parameter1().number2() == CN_DRAGAFTER));

    /*-----------------------------------------------------------------
       Get flag for over/after event and set our default drop style ...
     -----------------------------------------------------------------*/
    pTgtOpH->bWasDragAfter = event.isDragAfter();
    if (pTgtOpH->bWasDragAfter)
      pTgtOpH->setStyle(IDM::alignFlow);
    else
      pTgtOpH->setStyle(IDM::dropAtPosition);

    /*-------------------------------------------------------------------
       Set the container object pointer for the object that the over
       or after event occurred over.  0 if over container white space.
     -------------------------------------------------------------------*/
    pTgtOpH->setContainerObject(event.object());

    pDInfo = ((_CNRDRAGINFO *)event.parameter2().asUnsignedLong())->pDragInfo;
  }

  /*-------------------------------------------------------------------
     Access the PM DRAGINFO structure which was passed in the event.
   -------------------------------------------------------------------*/
  if (!DrgAccessDraginfo (pDInfo))
    ITHROWGUIERROR("DrgAccessDraginfo");

  /*-------------------------------------------------------------------
     Set the target enter (i.e. drag over) position of the pointing
     device.
   -------------------------------------------------------------------*/
  pTgtOpH->setPosition (IPoint(pDInfo->xDrop, pDInfo->yDrop));

  /*-------------------------------------------------------------------
     Set the current drag operation.
   -------------------------------------------------------------------*/
  pTgtOpH->setOperation (pDInfo->usOperation);

  /*-------------------------------------------------------------------
     Free the PM DRAGINFO structure
   -------------------------------------------------------------------*/
  DrgFreeDraginfo (pDInfo);
#endif

  return (pTgtOpH);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::targetOperation                                          |
|                                                                              |
| Returns the target operation object handle.                                  |
------------------------------------------------------------------------------*/
IDMTargetOperation::Handle IDMTargetOperation :: targetOperation()
{
  return (pDMTgtOpHandle);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setTargetOperation                                       |
|                                                                              |
| Sets the target operation object handle.                                     |
------------------------------------------------------------------------------*/
void IDMTargetOperation ::
        setTargetOperation(IDMTargetOperation::Handle targetOp)
{
  pDMTgtOpHandle = targetOp;
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::wasDragAfter                                             |
|                                                                              |
| Returns the state of the latest DRAGOVER or DRAGAFTER.                       |
------------------------------------------------------------------------------*/
Boolean IDMTargetOperation :: wasDragAfter() const
{
  return(this->bWasDragAfter);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::dropOffset                                               |
|                                                                              |
| Queries the current drop offset.                                             |
------------------------------------------------------------------------------*/
ISize IDMTargetOperation :: dropOffset() const
{
  return(this->sizeDropOffset);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::style                                                    |
|                                                                              |
| Returns the drop style.                                                      |
------------------------------------------------------------------------------*/
DropStyle IDMTargetOperation :: style() const
{
  return(this->dropStyle);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setDropOffset                                            |
|                                                                              |
| Sets the current drop offset.                                                |
------------------------------------------------------------------------------*/
IDMTargetOperation& IDMTargetOperation ::
                       setDropOffset(const ISize &offset)
{
  this->sizeDropOffset = offset;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setStyle                                                 |
|                                                                              |
| Sets the drop style.                                                         |
------------------------------------------------------------------------------*/
IDMTargetOperation& IDMTargetOperation ::
       setStyle(DropStyle drpStyle)
{
  this->dropStyle = drpStyle;
  return(*this);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::isStyle                                                  |
|                                                                              |
| Queries whether a specified style flag is set.                               |
------------------------------------------------------------------------------*/
IBase::Boolean IDMTargetOperation :: isStyle(DropStyle drpStyle)
{
  return((style() & drpStyle) == drpStyle);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::dropPosition                                             |
|                                                                              |
| Computes the drop position for this item, in container workspace             |
| coordinates, and sets the offset for the next item, according to             |
| the operation's drop style.                                                  |
------------------------------------------------------------------------------*/
IPoint IDMTargetOperation :: dropPosition(const IDMItem::Handle &pDIH,
                                          IDMTargetDropEvent &event)
{
  IMODTRACE_DEVELOP("IDMTargetOperation dropPosition");

  IPoint ptDrop;

  if (isStyle(IDM::dropAtPosition))
  {
    /*-----------------------------------------------------------------
       Drop at position
     -----------------------------------------------------------------*/
    ptDrop = positionRelativeTo(pDIH, event);
  }
  else
  {
    /*-----------------------------------------------------------------
       Drop-align
     -----------------------------------------------------------------*/
    ISize sizOff = dropOffset();

    IPoint ptMouse = positionRelativeTo(0, event);

     /*-----------------------------------------------------------------
       Initial drop guess is mouse position plus current offset
     -----------------------------------------------------------------*/
    ptDrop = ptMouse + sizOff;

    /*-----------------------------------------------------------------
       If flow, check that we are not out of screen
     -----------------------------------------------------------------*/
    IRectangle rectObject;
    if (event.object())
      rectObject = event.container()->iconRectangle(event.object(), true);
    else
      rectObject = IRectangle();


    if (isStyle(IDM::alignFlow))
    {
      IRectangle rectWorkspace = event.container()->viewPortOnWorkspace();

      /*---------------------------------------------------------------
         Correct next position if out of container window
       ---------------------------------------------------------------*/
      if (isStyle(IDM::alignVertical))
      {
        /*-------------------------------------------------------------
           Vertical alignment
          -------------------------------------------------------------*/
        if (rectWorkspace.bottom() > ptDrop.y())
        {
          sizOff = ISize((sizOff.width() + rectObject.width()), 0);
          if (!isStyle(IDM::flowBeside))
          {
            sizOff.setHeight(rectWorkspace.top() - ptMouse.y() -
                             rectObject.height());
          }

          /*-----------------------------------------------------------
           Re-calc drop point
           -----------------------------------------------------------*/
          ptDrop = ptMouse + sizOff;
        }
      }
      else
      {
        /*-------------------------------------------------------------
           Horizontal alignment
         -------------------------------------------------------------*/
        if (rectWorkspace.right() < (ptDrop.x() + rectObject.width()))
        {
          sizOff = ISize(0, (sizOff.height() - rectObject.height()));
          if (!isStyle(IDM::flowBeside))
            sizOff.setWidth(rectWorkspace.left() - ptMouse.x());

          /*-----------------------------------------------------------
             Re-calc drop point
           -----------------------------------------------------------*/
          ptDrop = ptMouse + sizOff;
        }
      }
    }

    /*-----------------------------------------------------------------
       Compute next drop offset
     -----------------------------------------------------------------*/
    if (isStyle(IDM::alignVertical))
    {
      /*---------------------------------------------------------------
         Align vertically (-y)
         ---------------------------------------------------------------*/
      sizOff.setHeight(sizOff.height() - rectObject.height());
    }
    else
    {
      /*---------------------------------------------------------------
         Align horizontally (+x)
       ---------------------------------------------------------------*/
      sizOff.setWidth(sizOff.width() + rectObject.width());
    }

    setDropOffset(sizOff);
  }

  return(ptDrop);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::positionRelativeTo                                       |
|                                                                              |
| Returns a drag item's position relative to the target container.  If the     |
| item pointer is 0, the drop point relative to the container is returned.     |
------------------------------------------------------------------------------*/
IPoint IDMTargetOperation ::
       positionRelativeTo(const IDMItem::Handle &pDIH,
                          IDMTargetDropEvent &event)
{
  IMODTRACE_DEVELOP("IDMTargetOperation::positionRelativeTo");

  IPoint ptDrop;
  IContainerControl *pCnrTgt = event.container();

  if (pCnrTgt)
  {
    /*-----------------------------------------------------------------
       Map to container window position
     -----------------------------------------------------------------*/
    IPoint dropPoint = event.dropPosition();
    ITRACE_DEVELOP( "dropPoint is " + dropPoint.asString() );

    ptDrop = IWindow::mapPoint( event.dropPosition(),
                                IWindow::desktopWindow()->handle(),
                                pCnrTgt->handle() );

    ITRACE_DEVELOP( "mapped ptDrop is " + ptDrop.asString() );
    /*-----------------------------------------------------------------
       Adjustment for workspace displacement relative to viewPort
       ---------------------------------------------------------------*/
    ptDrop += pCnrTgt->viewPortOnWorkspace().bottomLeft();
    ITRACE_DEVELOP( "vpOnWorkspace ptDrop is " + ptDrop.asString() );

    ptDrop -= pCnrTgt->viewPortOnWindow().bottomLeft();
    ITRACE_DEVELOP( "vpOnWindow ptDrop is " + ptDrop.asString() );

    /*-----------------------------------------------------------------
       Adjustment for icon offset
     -----------------------------------------------------------------*/
    if (pDIH)
      ptDrop += pDIH->imageOffset();
    ITRACE_DEVELOP( "and the is ptDrop is " + ptDrop.asString() );

  }

  return(ptDrop);
}


#ifdef IC_PMWIN
/*------------------------------------------------------------------------------
| IDMTargetOperation::firstTimeEntered                                         |
|                                                                              |
| Returns a flag indicating if first time entered                              |
------------------------------------------------------------------------------*/
Boolean IDMTargetOperation :: firstTimeEntered() const
{
  return (this->bFirstTimeEntered);
}

/*------------------------------------------------------------------------------
| IDMTargetOperation::setFirstTimeEntered                                      |
|                                                                              |
| Sets a flag indicating if first time entered                                 |
------------------------------------------------------------------------------*/
IDMTargetOperation &IDMTargetOperation :: setFirstTimeEntered(Boolean flag)
{
  this->bFirstTimeEntered = flag;
  return (*this);
}
#endif


#ifdef IC_WIN
/*-------------------------------------------------------------------
| Return truth of parameter *IWindow begin a container control
-------------------------------------------------------------------*/
Boolean IDMTargetOperationData::isContainerControl( const IWindow *win )
{
  char pszClassName[30];
  long lLen = IQUERYCLASSNAME(win->handle(),
                              sizeof(pszClassName),
                              (PSZ)pszClassName);

  /*------------------------------------------
  | If window class is edit control...
  ------------------------------------------*/
  IString className(pszClassName);
  return ( lLen > 0 &&
           (( className == IString(WC_CONTAINER)) ||
            (className == IString(WC_NATIVECONTAINER)) ));
}


/*------------------------------------------------------------------------------
| IDMTargetOperationData::buildItems                                (PRIVATE)  |
|                                                                              |
| Builds a list of the target drag items                                       |
------------------------------------------------------------------------------*/
void  IDMTargetOperationData::buildItems(IDMTargetOperation* pTgtOp,
                                         IDMTargetEnterEvent& event )
{

  IMODTRACE_DEVELOP("IDMTargetOperation::buildItems");

  ULONG ulItemCnt = 1;
  IDMItemProvider *pProvider;

  IDataObject *dataObject =
      (IDataObject *) event.parameter2().asUnsignedLong();


  /*------------------------------------------------------------
  | Set window handle of target operation so it will be
  | picked up in DRAGINFO instance..
  ------------------------------------------------------------*/
  pTgtOp->setTargetWindowHandle(event.window()->handle());

 /*------------------------------------------------------------
  | Build a DRAGINFO wrapper from the OLE IDataObject sent to
  | us in the event.If drag from non-OpenClass window, the
  | pDInfoWrapper will default to one item with types set
  | from the formats advertised in the dataObject.
  ------------------------------------------------------------*/
  IDragInfoWrapper* pDInfoWrapper = IDragInfoWrapper::dragInfoWrapper(dataObject);
  if (pDInfoWrapper)
    ulItemCnt = pDInfoWrapper->numDragItems();

  if (event.eventId() == DM_DRAGOVER)
  {
    /*-----------------------------------------------------------------
       Set the target window handle
      ----------------------------------------------------------------*/
    pTgtOp->setTargetWindowHandle(event.handle());

    /*-----------------------------------------------------------------
       Get pointer to the item provider
      ----------------------------------------------------------------*/
    pProvider = event.window()->itemProvider();
  }

  if (pDInfoWrapper)
  {
    /*----------------------------------------------------------------
    |   Set source window handle
    ----------------------------------------------------------------*/
    pTgtOp->setSourceWindowHandle(pDInfoWrapper->sourceWindowHandle());
  }

  /*------------------------------------------------------------------
     Set the target enter (i.e. drag over) position of the pointing
     device.
  ------------------------------------------------------------------*/
  IPoint pt( event.parameter1().number1(), event.parameter2().number2());
  pTgtOp->setPosition(pt);

  /*------------------------------------------------------------------
     Set the current drag operation.
  ------------------------------------------------------------------*/
  pTgtOp->setOperation( IDMOperation::move );

  IDMTargetOperation::Handle pTgtOpH;
  pTgtOpH = pTgtOp;

  /*--------------------------------------------------------------------
    Create the generic drag item from IDataObject ...
    Get number of objects in source op.
    Iterate over the objects in source op:
      For each, do the following, through the addItem
  --------------------------------------------------------------------*/
  ITRACE_DEVELOP(IString("Target buildItems see's ") + IString(ulItemCnt) + " Items");

  /*--------------------------------------------------------------------
    Iterate over the source items adding them to the target operation
  --------------------------------------------------------------------*/
  const IString aSpace(' ');
  for (unsigned nItem = 1; nItem <= ulItemCnt; ++nItem)
  {
          _DRAGITEM& dragItem = (*pDInfoWrapper)[nItem - 1]; // ie. IDragInfo&
    IDMItem tempItem( pTgtOpH, &dragItem );
    IDragItemWrapper dragItemWrapper(dragItem);

    tempItem.setPMDragItem(&dragItem);
    tempItem.addRMF( dragItemWrapper.rmfs() );
    tempItem.addRMF( IDM::rmLibrary, IDM::rfSharedMem );

    ITRACE_DEVELOP(IString("buildItems processed item ") +
                 IString(dragItemWrapper.itemID() ));

    IDMItem::Handle pDIH;
    pDIH = &tempItem;

    /*-----------------------------------------------------------------
      Obtain a derived target item based upon the generic drag
      item via the item provider  - Throw an exception if the
      provider fails ...
    -----------------------------------------------------------------*/
    IDMItem::Handle pTgtItemH;
    pTgtItemH = pProvider->provideTargetItemFor(pDIH);
    if (!pTgtItemH)
    {
      ITRACE_DEVELOP("IDMTargetOperation: target item provider for failed");
    }

    /*-----------------------------------------------------------------
      Add the target item to the collection ...
    -----------------------------------------------------------------*/
    pTgtOpH->addItem (pTgtItemH);
  }

  //---------------------------------------------------------
  // Save the _DRAGINFO because items have references to it
  // Let our destructor delete it because we destruct after
  // items are done with ref's to the contained _DRAGITEM's.
  //---------------------------------------------------------
  if (pDInfo)
    delete pDInfo;
  pDInfo = pDInfoWrapper->asDragInfoPtr ( );

  delete pDInfoWrapper;
}

/*---------------------------------------------------------------
| Marshall DRAGINFO and DRAGITEMs to byte array:
---------------------------------------------------------------*/
IStream* IDMSourceOperationData::marshall(
                    IDragInfoWrapper* pDInfoWrapper)
{
  IStream* pIStream = NULL;

  /*---------------------------------------------------------------
  | 1. Get length of marshall byte array.
  | 2. Marshall draginfo object instance to byte array.
  ---------------------------------------------------------------*/
  unsigned cbDragInfo = pDInfoWrapper->asDragInfoPtr()->marshall();

  char* pMarshalledData = new char[cbDragInfo];
  if (pMarshalledData != NULL)
  {
    ITRACE_DEVELOP("Marshalling dInfoWrapper for " +
          IString(cbDragInfo) + IString(" bytes"));

    pDInfoWrapper->asDragInfoPtr()->marshall(0, pMarshalledData);

    /*---------------------------------------------------------------
    | 3. Create hGlobal IStream of DRAGINFO and DRAGITEMs' byte array
    ---------------------------------------------------------------*/
    ITRACE_DEVELOP("Creating IStream of marshalled dInfoWrapper");

    HRESULT hr = CreateStreamOnHGlobal(0, True, &pIStream);
    ULONG cbResult = 0;

    /*---------------------------------------------------------------
    | 4. Write marshalled byte array (flattened DRAGINFO) to IStream
    ---------------------------------------------------------------*/
    if (hr == S_OK)
      hr = pIStream->Write(pMarshalledData, cbDragInfo, &cbResult);   // Write marshalled byte-array

    IASSERT(cbResult == cbDragInfo);

    delete [] pMarshalledData;
    pMarshalledData = NULL;
  }
  return pIStream;
}


/*------------------------------------------------------------------------------
| IDMSourceOperationData::doDrag                                    (PRIVATE)  |
|                                                                              |
| Starts the direct manipulation                                               |
------------------------------------------------------------------------------*/
void IDMSourceOperationData::doDrag(IDMSourceOperation* pSrcOp)
{
  IMODTRACE_DEVELOP("IDMSourceOperationData::doDrag");

  /*----------------------------------------------------------------------------
  | Make sure we have items to drag...
  ----------------------------------------------------------------------------*/
  const unsigned long   ulItemCnt = pSrcOp->numberOfItems();
  ITRACE_DEVELOP( "Number of Items is " + IString(ulItemCnt) );

  Boolean doTheDrag = Boolean(ulItemCnt);

  /*--------------------------------------------------------------
  | Count the types of the first item. We must assume that all
  | items of the drag have these types because OLE assumes
  | all advertised formats (types) apply to everything getting
  | transfered by OLE Uniform Data Transfer.
  --------------------------------------------------------------*/
  IDMItem::Handle pItemHandle;
  pItemHandle = pSrcOp->item( 1 );
  const IString& itemTypes(pItemHandle->types());
  const unsigned countTypes = itemTypes.occurrencesOf(',') + 1;

  Boolean filesSupported(false);
  unsigned fileFormats = 0;
  if ( pItemHandle->supportsRMF( IDM::rmFile, IDM::rfText ) )
  {
    ITRACE_DEVELOP("File Format supported");
    filesSupported = True;
    fileFormats = 3;

    // CF_HROP  // already added
    // FileName
    // ShellIDList
    // ShellIDListArray
  }

  /*--------------------------------------------------------------
  | Reserve space for all formats of first item, plus one
  | IOC_DRAGINFO format.
  --------------------------------------------------------------*/
  FORMATETC *prgfe = new FORMATETC[countTypes + 1 + fileFormats];

  /*-----------------------------------------------------------------
  | Add all types of first item to the format list in FIFO order.
  -----------------------------------------------------------------*/
  unsigned startPos = 1;
  for ( unsigned typeIndex = 0; typeIndex < countTypes; typeIndex++ )
  {
    const unsigned index = itemTypes.indexOf(',', startPos);
    const IString type = (index ?
                    itemTypes.subString(startPos, index - startPos) :
                    itemTypes.subString(startPos));

    startPos = index + 1;

    /*----------------------------------------------------------------
    | Convert type to corresponding clipboard format
    ----------------------------------------------------------------*/
    const CLIPFORMAT cfFormat = CLIPFORMAT(IDMFormat::formatFromString(type));
    ITRACE_DEVELOP("converted type " + IString(cfFormat));

    /*----------------------------------------------------------------
    | Add clipboard format (type) to array in FIFO order
    ----------------------------------------------------------------*/
    SETDefFormatEtc( prgfe[ typeIndex ], cfFormat, TYMED_HGLOBAL );

   }

  /*-----------------------------------------------------------------
  | Create an empty _DRAGINFO object for our number of items
  -----------------------------------------------------------------*/
  _DRAGINFO *pDInfo = new IDragInfo (
            (unsigned short)(unsigned long)ulItemCnt);

  /*-----------------------------------------------------------------
  | Populate _DRAGINFO and encapsulate it in a draginfo wrapper.
  -----------------------------------------------------------------*/
  IDragInfoWrapper* pDInfoWrapper =
      IDragInfoWrapper::setDragInfoWrapper( pSrcOp, pDInfo );

  /*---------------------------------------------------------------
  | Marshall draginfo wrapper to a global IStream
  | Note: we own this IStream. The IDataObject will clone
  | it to any target requesting it. Each target owns the clone
  | given to it. Each owner, including us, must release any
  | owned IStream interface.
  ---------------------------------------------------------------*/
  IStream* pIStream = marshall( pDInfoWrapper);

  /*---------------------------------------------------------------
  | Set private clipboard format for the IStream containing
  | marshalled DRAGINFO and DRAGITEMs
  ---------------------------------------------------------------*/
  unsigned restOfFormats = countTypes;
  if (pIStream != NULL)
  {
    ITRACE_DEVELOP("Registering private clipboard format DRAGINFO");
    const CLIPFORMAT clipFmt =
        CLIPFORMAT(::RegisterClipboardFormat(IDMDataObject::formatDragInfo()));

    SETDefFormatEtc( prgfe[restOfFormats], clipFmt, TYMED_ISTREAM );
    restOfFormats++;
    doTheDrag = True;
  }


  if ( filesSupported )
  {
    ITRACE_DEVELOP("Files supported");

    CLIPFORMAT clipFmt = IDMFormat::formatFromString( CFSTR_SHELLIDLIST);
    ITRACE_DEVELOP("Added format " + IString(clipFmt) );
    SETDefFormatEtc( prgfe[restOfFormats], clipFmt, TYMED_HGLOBAL );
    restOfFormats++;

    clipFmt = IDMFormat::formatFromString( CFSTR_SHELLIDLISTOFFSET);
    ITRACE_DEVELOP("Added format " + IString(clipFmt) );
    SETDefFormatEtc( prgfe[restOfFormats], clipFmt, TYMED_HGLOBAL );
    restOfFormats++;

//    ITRACE_DEVELOP("Added format " + IString(CF_HDROP) );
//    SETDefFormatEtc( prgfe[restOfFormats], CF_HDROP, TYMED_HGLOBAL );
//    restOfFormats++;

    clipFmt = IDMFormat::formatFromString( CFSTR_FILENAME);
    ITRACE_DEVELOP("Added format " + IString(clipFmt) );
    SETDefFormatEtc( prgfe[restOfFormats], clipFmt, TYMED_HGLOBAL );
    restOfFormats++;
  }

  /*---------------------------------------------------------------
  | Create IDataObject from generated formats, source window,
  | and the IStream containing simulated PM DRAGINFO, DRAGITEMs.
  ---------------------------------------------------------------*/
  // + 1 is for IOCDRAGINFO
  IDMEnumFormat formats( countTypes + 1 + fileFormats, prgfe );

  delete [] prgfe;

  IDataObject* pIDataObject = NULL;
  IDMDataObject* pIDMDataObject = NULL;

  if (doTheDrag)
  {
    pIDMDataObject = new IDMDataObject(
           IWindow::windowWithHandle(pSrcOp->sourceWindowHandle()),
           &formats, pIStream);
    if (FAILED(pIDMDataObject->QueryInterface(IID_IDataObject,
             (void **) &pIDataObject)))
    {
      doTheDrag = false;
    }
  }

  IDropSource* pIDropSource = NULL;
  IDMDropSource* pIDMDropSource = NULL;
      if (doTheDrag)
  {
    /*-------------------------------------------------------------------
    | Store "handle" to our drag information
    -------------------------------------------------------------------*/
    pDInfoWrapper->setSourceWindowHandle( pSrcOp->sourceWindowHandle() );
    pSrcOp->setDragInfo (pDInfo);

    pIDMDropSource = new IDMDropSource;

    if (FAILED(pIDMDropSource->QueryInterface(IID_IDropSource,
              (void **) &pIDropSource)))
    {
      doTheDrag = false;
    }
  }

  if ( doTheDrag )
  {
    unsigned long supportedOperations = 0,
                  operations = 0;
    IDMItem::Handle anItem;
    anItem = pSrcOp->item(1);

    if ( (IDM::allStacked == pSrcOp->imageStyle() ) ||
         (IDM::stack3AndFade == pSrcOp->imageStyle() ))
    {
      IDMImage image;
      IPoint stackHotspot;
      if ( stackImages(image, stackHotspot, pSrcOp) )
        pIDMDropSource->setImage( image, stackHotspot );
      ITRACE_DEVELOP("stackHotspot is " + stackHotspot.asString());
    }

    supportedOperations = anItem->supportedOperations();
    if ( supportedOperations & IDMItem::moveable )
    {
      operations |= DROPEFFECT_MOVE;
      ITRACE_DEVELOP( "Move supported");
    }

    if ( supportedOperations & IDMItem::copyable )
    {
      operations |= DROPEFFECT_COPY;
      ITRACE_DEVELOP( "Copy supported");
    }

    if ( supportedOperations & IDMItem::linkable )
    {
      operations |= DROPEFFECT_LINK;
      ITRACE_DEVELOP( "Link supported");
    }

    /*----------------------------------------------------------------
       Perform modal drag loop through OLE
    ----------------------------------------------------------------*/
    ITRACE_DEVELOP("DoDragDrop called");
    DWORD dwEffect;
    HRESULT hResult = DoDragDrop(
       (LPDATAOBJECT) pIDataObject,
       (LPDROPSOURCE) pIDropSource,
       operations,
       &dwEffect );

  } /* end if */

  ITRACE_DEVELOP("DoDragDrop ended" );

  if (pIDMDataObject)
    pIDMDataObject->Release();

  if (pIDMDropSource)
    pIDMDropSource->Release();

  delete pDInfo;

  // We are done with the source operation, set it back to 0.
//  IDMSourceOperation::setSourceOperation( 0 );
}
#endif


#ifdef IC_WIN


/*------------------------------------------------------------------------------
| IDMSourceOperationData::stackImages                                  PRIVATE
|
| Build a masked image of the stacked images. Set result as drag image.
| NOTE: Very Win32-specific. Needs to be abstracted (2D graphics,
| class IImageList, etc.
|
------------------------------------------------------------------------------*/
unsigned IDMSourceOperationData::stackImages(
              IDMImage& anImage,
              IPoint& stackHotspot,
              IDMSourceOperation* pSrcOp)
{
  IMODTRACE_DEVELOP("IDMSourceOperationData::stackImages");

  //--------------------------------------------
  // Hotspot will be adjusted for larger Win32
  // icon retangle, if necessary.
  //--------------------------------------------
  IPoint hotspot(IPoint(0,0) - pSrcOp->pointerOffset());
  ITRACE_DEVELOP("Initial hot spot is " + hotspot.asString() );

  //--------------------------------------------
  // Hold individual input images in image list.
  //--------------------------------------------
  HIMAGELIST himagelist = NULL;

  //--------------------------------------------
  // Const stagger offset.
  //--------------------------------------------
  const ISize offset(16, 16);  // Need stacking percentage

  //--------------------------------------------
  // Bounding rectangle of stack. Will grow
  // this during pass 1. Adjust hotspot.
  //--------------------------------------------
  IRectangle rect(IPoint(0, 0), ISize(0, 0));

  //--------------------------------------------
  // Pass 1: compute number of images, bounding
  // rectangle of stack, and hotspot.
  // Return value: number of images stacked.
  //--------------------------------------------
  unsigned cImagesUsed = stackImagesPass1(
            anImage,      // Image to receive stacked images
            pSrcOp,       // Drag source operation
            offset,       // Stack stagger in pixels
            himagelist,   // Receives images to be stacked
            rect,         // Receives computed bounding rect of stack
            hotspot       // Receives intermediate hotspot
            );

  //--------------------------------------------
  // If pass 1 found stackable images, carry out
  // pass 2: stack the images.
  //--------------------------------------------
  if (cImagesUsed)
    stackImagesPass2(
                anImage,
                offset,
                himagelist,
                rect);

  //--------------------------------------------
  // If success, set stack hotspot size, then
  // free work image list.
  //--------------------------------------------
  if (himagelist)
  {
    int cx, cy;
    ::ImageList_GetIconSize(himagelist, &cx, &cy);
    ISize cellSize(ISize(cx, cy));

    stackHotspot = IPoint(hotspot.x(),
                          rect.height() - cellSize.height() + hotspot.y() );

    ::ImageList_Destroy(himagelist);
  }

  return cImagesUsed;
}

//-----------------------------------------------------------------
// PASS ONE of stack images:
// Iterate over collection of IDMItems.  For each, set its image
// into a work image list. Compute the bounding rectangle of the
// images when they're stacked, using the stagger offset.
//-----------------------------------------------------------------
unsigned IDMSourceOperationData::stackImagesPass1(
              IDMImage& anImage,           // I/O: image of stack
              IDMSourceOperation* pSrcOp,  // Input: source drag operation
              const ISize& offset,         // Input: stagger offset in pixels
              HIMAGELIST& himagelist,      // Output: list of stackable images
              IRectangle& rect,            // Output: bounding rect of stack
              IPoint& hotspot)             // Output: working hotspot
{
  IMODTRACE_DEVELOP("IDMSourceOperationData::stackImagesPass1");

  const unsigned       itemCount  = pSrcOp->numberOfItems();
  const DragImageStyle diStyle    = pSrcOp->imageStyle();

  //--------------------------------------------
  // Return value: number of images stacked
  //--------------------------------------------
  unsigned cImagesUsed = 0;

  //-------------------------------------------------------------------
  // Wer haben Item, nicht whar? Throw exception if no drag items.
  //-------------------------------------------------------------------
  if ( itemCount )
  {
    //-----------------------------------------------------------------
    // Iterate over collection of IDMItems.  For each, set its image
    // into a work image list. Compute the bounding rectangle of the
    // images when they're stacked, using the supplied offsets.
    //-----------------------------------------------------------------
    for (unsigned itemIndex = 0; itemIndex < itemCount; itemIndex++)
    {
      //-------------------------------------
      // Finished if we cached the fade icon
      //-------------------------------------
      if ( itemIndex >= 4 && diStyle == IDM::stack3AndFade )
        break;

      HICON hicon = NULL;

      IDMItem::Handle pItemHandle;  // Caution: don't construct and assign.
      pItemHandle = pSrcOp->item(itemIndex + 1); // Now okay to initialize it.

      //--------------------------------------------
      // Get image
      // It can't be a ref because we may substitute
      // the fade icon later.
      //--------------------------------------------
      IDMImage image(pItemHandle->image());

      ISize sizeBmp(0, 0);

      IGBitmap* pbmImage = NULL; // Image bitmap

      //----------------------------------
      // Using fade and we've got three?
      // Then substitute the fade icon.
      //----------------------------------
      if ( itemIndex >= 3 && diStyle == IDM::stack3AndFade )
        image = IDMImage(ISystemPointerHandle(
                              ISystemPointerHandle::dragFade));

      //-------------------
      // If icon...
      //-------------------
      if ( image.pointer() )
      {
        hicon = image.pointer();

        //--------------------------------------------
        // Disect icon. If it's cursor, we can't
        // stack it. Many programs routinely set
        // StackAll or Stack3AndFade for cursor
        // images. We use OLE default cursors in
        // that case, so we don't want to stack
        // cursors.
        //--------------------------------------------
        ICONINFO iconInfo;

        if (!::GetIconInfo(hicon, &iconInfo) || !iconInfo.fIcon )
        {
          ITRACE_DEVELOP("bailing out of stackImage - 2 " + IString(itemIndex));
          return 0; // Found a cursor: use OLE default instead.
        }

        //---------------------------------------------------------
        // Win32s reports cursors as being icons. Cursors are
        // always monchrome. We consider all Win32s monochrome
        // icons to be cursors. This means we'll never stack true
        // monochrome icons in Win32s -- probably not a problem.
        //---------------------------------------------------------
        if (IPlatform::isWin32S())
        {
          BITMAP bmpInfo;
          ::GetObject(iconInfo.hbmColor, sizeof bmpInfo, &bmpInfo);

          if (bmpInfo.bmBitsPixel == 1 && bmpInfo.bmPlanes == 1)
            return 0; // Found a cursor: use OLE default instead.
        }

        pbmImage = new IGBitmap(iconInfo.hbmColor);
      }
      else
      //--------------------------------------------
      // Else if Bitmap...
      //--------------------------------------------
      if ( image.bitmap() )
      {
        pbmImage = new IGBitmap(image.bitmap());
      }
      else
      {
        return 0; // We only stack bitmaps or icons
      }

      //--------------------------------------------
      // Bitmap information
      //--------------------------------------------
      BITMAP bmpInfo;
      ::GetObject(pbmImage->handle(), sizeof bmpInfo, &bmpInfo);

      //--------------------------------------------
      // Get bounding rectangle of this image
      //--------------------------------------------
      ISize sizeIcon(bmpInfo.bmWidth, bmpInfo.bmHeight);
      IRectangle rectBmp(IPoint(0, 0), sizeIcon);

      //----------------------------------------------------------
      // Maintain the hotspot of the first image.
      // Later, after the images are stacked, we'll compute
      // another hotspot for the entire stack.
      //----------------------------------------------------------
      if (itemIndex == 0)
      {
        //---------------------------------------------------------------
        // If the hot spot is relative to the lower left than we need to
        // convert it to the upper left. ImageLists understand only
        // native window MM_TEXT mapping mode coordinates.
        //---------------------------------------------------------------
        if ( ICoordinateSystem::applicationOrientation() ==
               ICoordinateSystem::originLowerLeft )
        {
          hotspot = ICoordinateSystem::convertToNative(
              hotspot,
              sizeIcon );
          ITRACE_DEVELOP("Converted hot spot is " + hotspot.asString() );
        }
      }

      //----------------------------------------
      // Create image list sized to first icon.
      //----------------------------------------
      if (himagelist == NULL)
        himagelist = ::ImageList_Create(
                           int(bmpInfo.bmWidth),
                           int(bmpInfo.bmHeight),
                           imageListFlags,
                           4,
                           4  // May need more for StackAll
                           );

      //--------------------------------------------
      // Add icon to image list
      //--------------------------------------------
      if ( image.pointer() )
        ::ImageList_AddIcon(himagelist, hicon);
      else
        ::ImageList_AddMasked(himagelist, pbmImage->handle(),
                              RGB(255,255,255));

      //--------------------------------------------
      // Grow rect to bound icon if necessary
      //    rect.UnionRect(&rect, &rectBmp);
      //--------------------------------------------
      rect |= rectBmp;

      //--------------------------------------------
      // Grow rect by offset * position in sequence
      //--------------------------------------------
      if (itemIndex != 0)
        rect = IRectangle(IPoint(0, 0),
      ISize(ISize(rect.width(),rect.height()) + offset));

      delete pbmImage; // Carry out GDI DeleteObject

      //-------------------------------
      // Count number of icons used
      //-------------------------------
      cImagesUsed++;

    }  // end pass 1

    //-----------------------------------------------
    // We know the final hotspot so set it in result
    //-----------------------------------------------
    anImage.setPointerOffset(hotspot);

  }
  else
      ITHROWGUIERROR("stackImagesPass1");

  return cImagesUsed++;
}

//-----------------------------------------------------------------
// PASS TWO:
// Given an imagelist of icons, stack them, last-to-first, from
// the NE to SW corners of the bounding rectangle that was
// computed in pass one. The last icons stacked is the top-most
// icon and is the icon under the mouse.
//
// Result: number of images used
// Parameters:  IDMImage& anImage,       // Output: image of stack
//              const ISize& offset,     // Input: stack stager in pixels
//              HIMAGELIST himagelist,   // Input: list of images
//              const IRectangle& rect   // Input: stack image bounds
//
//-----------------------------------------------------------------
unsigned IDMSourceOperationData::stackImagesPass2(
                IDMImage& anImage,       // Output: image of stack
                const ISize& offset,     // Input: stagger offet in pixels
                const HIMAGELIST himagelist, // Input: list of images
                const IRectangle& rect)  // Input: stack image bounds
{
  IMODTRACE_DEVELOP("IDMSourceOperationData::stackImagesPass1");

  const unsigned cImagesUsed = ::ImageList_GetImageCount(himagelist);

  ITRACE_DEVELOP(IString(cImagesUsed) + IString(" = images used"));
  ITRACE_DEVELOP("rect == " + rect.asString());

  //-------------------------------------------------------
  // Build imagelist for stack image
  //-------------------------------------------------------
  HIMAGELIST hImageStack = ::ImageList_Create(
                              int(rect.width()),
                              int(rect.height()),
                              imageListFlags,
                              1,
                              0      // Allow  one only
                              );

  if (hImageStack == NULL)
    ITHROWGUIERROR("stackImagesPass2");

  //-------------------------------------------------------
  // Enumerate icons last-to-first.
  // Lay each into the image, offsetting from NE to SW.
  // Lay each image's mask in the same position.
  //-------------------------------------------------------
  for (int index = cImagesUsed; index > 0; --index)
  {
    const int position = index - 1;
    HIMAGELIST himagelist2 = ::ImageList_Merge(
                                      hImageStack,
                                      0,
                                      himagelist,
                                      position,
                                      int(offset.width() * position),
                                      int(offset.height() * (cImagesUsed - index))
                                 );

    if (himagelist2 == NULL)
      ITHROWGUIERROR("stackImagesPass2");

    ::ImageList_Destroy(hImageStack);
    hImageStack = himagelist2;
  }

  //--------------------------------------------
  // Set drag image
  //--------------------------------------------
  HICON hiconStack = ::ImageList_GetIcon(hImageStack, 0, ILD_TRANSPARENT );
  if (hiconStack == NULL)
    ITHROWGUIERROR("stackImagesPass2");

  ::ImageList_Destroy(hImageStack);

  //--------------------------------------------
  // Set result: picture
  //--------------------------------------------
  anImage.setPointer(IPointerHandle (hiconStack ));


  return cImagesUsed;
}

#endif

/*------------------------------------------------------------------------------
| IDMOperation::debugSupport                                                   |
|                                                                              |
| Returns the static to indicate if debug support is set for DrgDrag           |
------------------------------------------------------------------------------*/
Boolean IDMOperation::debugSupport()
{
  return IDMOperation::fgDebugSupport;
}

/*------------------------------------------------------------------------------
| IDMOperation::setdebugSupport                                                |
|                                                                              |
| Sets the debugSupport flag used for DrgDrag                                  |
------------------------------------------------------------------------------*/
void IDMOperation::setDebugSupport(Boolean debugSupport)
{
  IDMOperation::fgDebugSupport = debugSupport;
}

