/*******************************************************************************
* FILE NAME: imenuevt.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in imenuevt.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
extern "C" {
   #define INCL_WINMESSAGEMGR
   #define INCL_WINWINDOWMGR
   #define INCL_WININPUT
   #define INCL_WINPOINTERS
   #include <iwindefs.h>
}

#include <iapp.hpp>
#include <icoordsy.hpp>
#include <imnitem.hpp>
#include <imenuevt.hpp>
#include <imenuprv.hpp>
#include <iplatfrm.hpp>
#include <ipoint.hpp>
#include <isubmenu.hpp>
#include <ithread.hpp>
#include <iwindow.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IMENUEVT_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IMenuEvent::IMenuEvent                                                       |
------------------------------------------------------------------------------*/
IMenuEvent :: IMenuEvent(IEvent& evt)
  : IEvent(evt) {;}

/*------------------------------------------------------------------------------
| IMenuEvent::~IMenuEvent                                                      |
------------------------------------------------------------------------------*/
IMenuEvent :: ~IMenuEvent( ) {}

/*------------------------------------------------------------------------------
| IMenuEvent::menuItemId                                                       |
------------------------------------------------------------------------------*/
unsigned long IMenuEvent::menuItemId() const
{
  // Check for menu start or end and return id info
#ifdef IC_PM
  if (eventId() == WM_INITMENU || eventId() == WM_MENUEND)
  {
    return parameter1();
  }
#endif
#ifdef IC_WIN
  MENUITEMINFO miItem;
  long id = 0;
  if (eventId() == WM_INITMENUPOPUP)
  {
    if (IPlatform::isWin95() || IPlatform::isNTNewShell())
    {
      miItem.cbSize = sizeof(MENUITEMINFO);
      miItem.fMask = MIIM_ID;

      if (GetMenuItemInfo( (HMENU)fPrevMenuParent, parameter2().number1(),
                           TRUE, &miItem ))
        id = miItem.wID;
    }
    else
    {
      // Query for the id of the item at the position specified
      id = GetMenuItemID( (HMENU)fPrevMenuParent, parameter2().number1() );

      // If error, the item might be a submenu so query for a menu handle
      if ( id == 0xFFFFFFFFul )
      {
        HMENU hPop = GetSubMenu( (HMENU)fPrevMenuParent,
                                 parameter2().number1() );

        // If a menu handle is returned, then get the id from the lookup table
        if (hPop)
          id = IMenuPrivate::getItemIdFromLookUpTable(
                          IMenuPrivate::locateFrame( window() ), hPop );
      }
    }
    return id;
  }
  else if (eventId() == WM_MENUEND)
  {
    return parameter1();
  }
#endif
  else
  {
    // Otherwise menu select action, id info in different location
    if (eventId() == WM_MENUSELECT)
    {
#ifdef IC_WIN
      // If pop-up in Windows, check platform for how to get id
      if ( parameter1().number2() & MF_POPUP )
      {
        // If new Windows 32, then use extended menu support
        if (IPlatform::isWin95() || IPlatform::isNTNewShell())
        {
          miItem.cbSize = sizeof(MENUITEMINFO);
          miItem.fMask = MIIM_ID;
          if (GetMenuItemInfo( (HMENU)parameter2(), parameter1().number1(),
                           TRUE, &miItem ))
            id = miItem.wID;
        }
        // If pop-up in Win32s or old shell, check lookup table for id
        else
        {
          HMENU hPop = GetSubMenu( (HMENU)parameter2(),
                                   parameter1().number1() );
          if (hPop)
            id = IMenuPrivate::getItemIdFromLookUpTable(
                          IMenuPrivate::locateFrame( window() ), hPop );
        }
        return id;
      }
#endif
      return parameter1().number1();
    }
  } /* endif */
  return 0;
}

/*------------------------------------------------------------------------------
| IMenuEvent::mousePosition                                                    |
|  Returns mouse point mapped to the menu control owner window.                |
------------------------------------------------------------------------------*/
IPoint IMenuEvent::mousePosition() const
{
   POINTL ptl;
   IQUERYMSGPOS(IThread::current().anchorBlock(), &ptl);
   return IWindow::mapPoint(
              ICoordinateSystem::convertToApplication(
                 ptl,
                 IWindow::desktopWindow()->size() ),
              HWND_DESKTOP,
              window()->handle() );
}

/*------------------------------------------------------------------------------
| IMenuEvent::menuItem                                                         |
------------------------------------------------------------------------------*/
IMenuItem IMenuEvent::menuItem() const
{
   // Only have the id of an actual MenuItem for WM_MENUSELECT
   // All other msgs pertain to an entire menu
   if (eventId() == WM_MENUSELECT ) {
#ifdef IC_WIN
      ISubmenu submenu((void*)parameter2());
#else
      ISubmenu submenu((unsigned long)parameter2());
#endif
      return submenu.menuItem(menuItemId());
   } /* endif */
   return IMenuItem(0);
}

