/*******************************************************************************
* FILE NAME: ispinnum.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in ispinnum.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINSTDSPIN
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #include <iwindefs.h>
}

#ifdef IC_WIN
  #include <iclspbw.h>
  #include <commctrl.h>
#endif
#include <ispinnum.hpp>
#include <ispinszh.hpp>
#include <icconst.h>
#include <iexcept.hpp>
#include <istring.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ISPINNUM_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| Public spin button styles.                                                   |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
#ifdef IC_WIN
const INumericSpinButton::Style
  INumericSpinButton::padWithZeros      (0, ISPBS_PADWITHZEROS),
  INumericSpinButton::classDefaultStyle ( WS_VISIBLE,
                                          ISPBS_MASTER   |
                                          ISPBS_JUSTLEFT |
                                          ISPBS_NUMERICONLY |
                                          WS_EX_CLIENTEDGE );
#endif
#ifndef IC_WIN
const INumericSpinButton::Style
  INumericSpinButton::padWithZeros      (0, ISPBS_PADWITHZEROS),
  INumericSpinButton::classDefaultStyle ( WS_VISIBLE,
                                          ISPBS_MASTER   |
                                          ISPBS_JUSTLEFT |
                                          ISPBS_NUMERICONLY );
#endif
#pragma data_seg()

/*------------------------------------------------------------------------------
| Default style for new objects (initial value).                               |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLNonConst)
#ifdef IC_WIN
INumericSpinButton::Style
  INumericSpinButton::currentDefaultStyle ( WS_VISIBLE,
                                            ISPBS_MASTER   |
                                            ISPBS_JUSTLEFT |
                                            ISPBS_NUMERICONLY |
                                            WS_EX_CLIENTEDGE );
#endif
#ifndef IC_WIN
INumericSpinButton::Style
  INumericSpinButton::currentDefaultStyle ( WS_VISIBLE,
                                            ISPBS_MASTER   |
                                            ISPBS_JUSTLEFT |
                                            ISPBS_NUMERICONLY );
#endif
#pragma data_seg()

/*------------------------------------------------------------------------------
| INumericSpinButton::INumericSpinButton                                       |
|                                                                              |
| Constructor to create a spin button control on a standard window.            |
------------------------------------------------------------------------------*/
INumericSpinButton :: INumericSpinButton ( unsigned long id,
                                           IWindow* parent,
                                           IWindow* pwndOwner,
                                           const IRectangle& initial,
                                           const Style& style )
                          : IBaseSpinButton()
{
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  /****************************************************************/
  /* Borland has bug with embedded ? operator so below is needed. */
  /****************************************************************/
#ifdef __BORLANDC__
  IWindowHandle owner;
  if (pwndOwner == 0)
    owner = IWindowHandle(0);
  else
    owner = pwndOwner->handle();
#else
  IWindowHandle owner = (pwndOwner == 0) ? IWindowHandle(0)
                                         : pwndOwner->handle();
#endif
  initialize ( id,
               parent->handle(),
               owner,
               convertToGUIStyle( style ),
               initial );
}

/*------------------------------------------------------------------------------
| INumericSpinButton::INumericSpinButton                                       |
|                                                                              |
| Constructor to create a spin button control on a standard window.            |
------------------------------------------------------------------------------*/
INumericSpinButton :: INumericSpinButton ( unsigned long id, IWindow* parent )
      :  IBaseSpinButton()
{
   IWindowHandle
     hwnd = IWindow::handleWithParent( id, parent->handle() );

   if ( !(ISTYLEOF(hwnd) & SPBS_NUMERICONLY) )
      ITHROWLIBRARYERROR(IC_NUMSPIN_WRAP_INVALID,
                         IBaseErrorInfo::invalidRequest, IException::recoverable);
   setAutoDestroyWindow(false);
   setExtendedStyle(
     ISpinButtonStatics::convertToExtendedStyle(ISTYLEOF(hwnd)));
   initialize();
   startHandlingEventsFor( hwnd );
}

/*------------------------------------------------------------------------------
| INumericSpinButton::INumericSpinButton                                       |
|                                                                              |
| Constructor to instantiate spin button object from window handle.            |
------------------------------------------------------------------------------*/
INumericSpinButton :: INumericSpinButton ( const IWindowHandle& handle )
     :  IBaseSpinButton()
{
   if ( !(ISTYLEOF(handle) & SPBS_NUMERICONLY) )
      ITHROWLIBRARYERROR(IC_NUMSPIN_WRAP_INVALID,
                         IBaseErrorInfo::invalidRequest, IException::recoverable);
   setAutoDestroyWindow(false);
   setExtendedStyle(
     ISpinButtonStatics::convertToExtendedStyle(ISTYLEOF(handle)));
   initialize();
   startHandlingEventsFor(handle);
}

/*------------------------------------------------------------------------------
| INumericSpinButton::~INumericSpinButton                                      |
------------------------------------------------------------------------------*/
INumericSpinButton::~INumericSpinButton ( )
{
}

/*------------------------------------------------------------------------------
| INumericTextSpinButton::defaultStyle                                         |
|                                                                              |
| Return the default style for new spin button objects.                        |
------------------------------------------------------------------------------*/
INumericSpinButton::Style INumericSpinButton::defaultStyle()
{
  return currentDefaultStyle;
}

#ifdef IC_WIN
/*------------------------------------------------------------------------------
| INumericTextSpinButton::setDefaultStyle                                      |
|                                                                              |
| Sets the default style for new spin button objects.                          |
------------------------------------------------------------------------------*/
void INumericSpinButton::setDefaultStyle( const INumericSpinButton::Style& stsStyle )
{
  currentDefaultStyle = stsStyle;
}
#endif

/*------------------------------------------------------------------------------
| INumericSpinButton::convertToGUIStyle                                        |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long INumericSpinButton::convertToGUIStyle( const IBitFlag& guiStyle,
                                                     Boolean bExtOnly ) const
{
  // Obtain the style from the class that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }
  else
  {
    unsigned long guiExtUL = guiStyle.asExtendedUnsignedLong();
#ifdef IC_WIN
    if ( guiExtUL & IBaseSpinButton::pmCompatible.asExtendedUnsignedLong() )
#endif
    {
       // PM Compatible spinbutton
       ulStyle |= SPBS_NUMERICONLY;
       if ( guiExtUL & INumericSpinButton::padWithZeros.asExtendedUnsignedLong())
          ulStyle |= SPBS_PADWITHZEROS;

    }
#ifdef IC_WIN
    else
    {
       // In native Windows up/down control, we use these flags in
       // our wrapper control.
       ulStyle |= INSPBS_NUMERICONLY;
       if ( guiExtUL & INumericSpinButton::padWithZeros.asExtendedUnsignedLong())
          ulStyle |= INSPBS_PADWITHZEROS;
    }
#endif
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
| INumericSpinButton::setRange                                                 |
|                                                                              |
| Sets the lower and upper range of the spin button numbers.  If the value in  |
| the spin field does not match the range, PM sets the spin field to the       |
| closest limit.  If the range is empty, PM puts 0 in the spin field.  If      |
| override is set and the current value in the spin field is outside the       |
| new range, the current value in the spin field will not change.              |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: setRange ( const IRange& aNewRange,
                                                     Boolean override )
{
   IASSERTPARM(aNewRange.lowerBound() <= aNewRange.upperBound());
                         //If lower is greater than upper, operation will fail.

   if (this->isPMCompatible())
   {
     if (override)
        handle().sendEvent(SPBM_OVERRIDESETLIMITS,
                           IEventParameter1((int)aNewRange.upperBound()),
                           IEventParameter2((int)aNewRange.lowerBound()));
     else
        handle().sendEvent(SPBM_SETLIMITS,
                           IEventParameter1((int)aNewRange.upperBound()),
                           IEventParameter2((int)aNewRange.lowerBound()));
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     long currentValue = this->value();
     IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
     IEventData newRange((short)aNewRange.upperBound(),
                         (short)aNewRange.lowerBound());
     ISETRANGE(this->handle(), newRange.asLong() );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
           ISpinButtonStatics::buddyHandle( this->handle() ) ) )
     {
       updown.sendEvent( UDM_SETRANGE,
                         IEventParameter1(0),
                         newRange );
     }
     if (!override)
     {
       if (currentValue < aNewRange.lowerBound() )
          this->spinTo( aNewRange.lowerBound() );
       else if (currentValue > aNewRange.upperBound() )
          this->spinTo( aNewRange.upperBound() );
     }
     this->refresh();
   }
#endif

   return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::range                                                    |
|                                                                              |
| Query lower and upper bounds of the numeric range.                           |
------------------------------------------------------------------------------*/
IRange INumericSpinButton :: range ( ) const
{
   long lLow = 0;
   long lHigh = 0;
   if (this->isPMCompatible())
   {
     unsigned long ulOk = handle().sendEvent(SPBM_QUERYLIMITS,
                                             IEventParameter1((void*)&lHigh),
                                             IEventParameter2((void*)&lLow));
     if (!ulOk)
        ITHROWGUIERROR2("SPBM_QUERYLIMITS",
                        IBaseErrorInfo::accessError,
                        IException::recoverable);
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IEventResult result = IQUERYRANGE( this->handle() );
     lLow = (short)result.highNumber();
     lHigh = (short)result.lowNumber();
   }
#endif
   return IRange(lLow, lHigh);
}

/*------------------------------------------------------------------------------
| INumericSpinButton::value                                                    |
|                                                                              |
| Return the number in the spin field.                                         |
------------------------------------------------------------------------------*/
long INumericSpinButton :: value ( ) const
{
   IMODTRACE_DEVELOP("INumericSpinButton::value");

   IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
   unsigned long ulClBufLen =
         (unsigned long)IQUERYWINDOWTEXTLENGTH(ef) + 1;
   char* pszClBuf = new char[ulClBufLen];
   IQUERYWINDOWTEXT(ef, ulClBufLen, pszClBuf);
   IString str(pszClBuf);
   delete [] pszClBuf;
   return str.asInt();
}

/*------------------------------------------------------------------------------
| INumericSpinButton::setValue                                                 |
|                                                                              |
| Set the spin field with the specified value.                                 |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: setValue ( long aValue )
{
  IMODTRACE_DEVELOP("INumericSpinButton::setValue");

  if (aValue != value())
  {
    IWindowHandle ef = ISpinButtonStatics::buddyHandle( this->handle() );
    ITRACE_DEVELOP(IString("Value to be set is ") + IString(aValue));
    ISETWINDOWTEXT(ef, (const char*)IString(aValue));
#ifdef IC_WIN
    if (!this->isPMCompatible())
    {
      if ( this->range().includes( aValue ) )
      {
         IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
         // If current spinbutton not active one don't mess with updown
         if ( (updown.asUnsigned()) &&
              (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
               ISpinButtonStatics::buddyHandle( this->handle() ) ) )
            updown.sendEvent( UDM_SETPOS,
                              IEventParameter1(0),
                              IEventParameter2((short)aValue, 0) );
         ISETLASTPOS(this->handle(), aValue);
      }
    }

    // send the edit field changed message.
    IWindowHandle parent = IPARENTOF( this->handle() );
    parent.sendEvent(
          WM_COMMAND,
          IEventParameter1( (short)this->id() , SPBN_CHANGE),
          IEventParameter2( this->handle().asUnsigned() ) );
#endif
  }
  return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::spinTo                                                   |
|                                                                              |
| Set the spin field to the specified numeric value.                           |
------------------------------------------------------------------------------*/
INumericSpinButton& INumericSpinButton :: spinTo ( long    aValue,
                                                   Boolean spinToClosest )
{
   if (this->isPMCompatible())
   {
     unsigned long ulOk =
          handle().sendEvent(SPBM_SETCURRENTVALUE,
                             IEventParameter1((int)aValue),
                             IEventParameter2(0));
     if (!ulOk)
     {
        if (spinToClosest)
        {
           long lUpper, lLower, lValue;
           ulOk = handle().sendEvent( SPBM_QUERYLIMITS,
                                      IEventParameter1( &lUpper ),
                                      IEventParameter2( &lLower ));
           if (!ulOk)
              ITHROWGUIERROR2( "SPBM_QUERYLIMITS",
                               IBaseErrorInfo::invalidParameter,
                               IException::recoverable );
           if (aValue < lLower)
              lValue = lLower;
           else
              lValue = lUpper;
           ulOk = handle().sendEvent( SPBM_SETCURRENTVALUE,
                                      IEventParameter1( (int)lValue ),
                                      IEventParameter2( 0 ));
        }
     }
     if (!ulOk)
     {
        ITHROWGUIERROR2("SPBM_SETCURRENTVALUE",
                        IBaseErrorInfo::invalidParameter,
                        IException::recoverable);
     }
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IRange range = this->range();
     if ( !range.includes( aValue ) )
        {
        if (spinToClosest)
           {
           if ( aValue > range.upperBound() )
              aValue = range.upperBound();
           else
              aValue = range.lowerBound();
           }
        else
           {
           ITHROWGUIERROR2("UDM_SETPOS",
                           IBaseErrorInfo::invalidParameter,
                           IException::recoverable);
           }
        }
     IWindowHandle updown = IQUERYUPDOWNHANDLE( this->handle() );
     // If current spinbutton not active one don't mess with updown
     if ( (updown.asUnsigned()) &&
          (IQUERYACTIVEBUDDY( IPARENTOF( updown )) ==
           ISpinButtonStatics::buddyHandle( this->handle() ) ) )
        updown.sendEvent( UDM_SETPOS,
                          IEventParameter1(0),
                          IEventParameter2((short)aValue, 0) );
     // Generate a message to be sent.  This simulates the message that
     // is sent when the user clicks on the arrow buttons.
     this->sendEvent( IC_UM_VSCROLL,
                      IEventParameter1( SB_THUMBPOSITION, (short)aValue ),
                      IEventParameter2( updown.asUnsigned() ) );
   }
#endif
   return *this;
}

/*------------------------------------------------------------------------------
| INumericSpinButton::isSpinFieldValid                                         |
|                                                                              |
| Validate that the contents of the spin field are in the spin field number    |
| range.                                                                       |
------------------------------------------------------------------------------*/
IBase::Boolean INumericSpinButton :: isSpinFieldValid (
                                         Boolean /*caseSensitive*/) const
{
   unsigned long ulOk;
   if (this->isPMCompatible())
   {
     long lValue;
     ulOk = handle().sendEvent(SPBM_QUERYVALUE,
                               IEventParameter1((void*)&lValue),
                               IEventParameter2(0,
                                                SPBQ_DONOTUPDATE));
   }
#ifdef IC_WIN
   else
   {
     // Native Up/Down control
     IRange range = this->range();
     ulOk = range.includes( this->value() );
   }
#endif

   return (ulOk) ? true : false;
}
