/*******************************************************************************
* FILE NAME: itbarbut.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in itbarbut.hpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1996       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Priority INT_MIN (-2147483647 - 1) + 1024 + 512
#pragma priority( -2147482112 )

extern "C" {
  #define INCL_WINBUTTONS
  #define INCL_WINSYS
  #define INCL_WINWINDOWMGR
  #define INCL_WINCOUNTRY
  #define INCL_WINHOOKS
  #include <iwindefs.h>
}

#pragma info(none)
  #include <iseq.h>
#pragma info(restore)

#include <igrafctx.hpp>
#include <igbitmap.hpp>
#include <icconst.h>
#include <itbarbut.hpp>
#include <itbar.hpp>
#include <icustbhd.hpp>
#include <icustbev.hpp>
#include <icolor.hpp>
#include <icoordsy.hpp>
#include <iexcept.hpp>
#include <ireslib.hpp>
#include <ifont.hpp>
#include <ireslock.hpp>
#include <idmhndlr.hpp>
#include <itbpriv.hpp>
#include <itrace.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _ITBARBUT_CPP_
  #include <ipagetun.h>
#endif

#define DRAW_STANDARD_BUTTON  1  // undefine to get Win95 toolbar button appearance

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
class IStandardButtonList : public ISequence<IToolBarButton*>
{
public:
  IStandardButtonList ();
 ~IStandardButtonList ();

static void
  addButton    ( IToolBarButton* button );

static void
  removeButton ( IToolBarButton* button );

static void
  bitmapFormatChanged ();

static void
  textFormatChanged ();

private:
static IPrivateResource
  listLock;
static IStandardButtonList
  *list;
};

IStandardButtonList::IStandardButtonList ()
{
}

IStandardButtonList::~IStandardButtonList ()
{
}

#pragma data_seg(ICLNonConst)
IStandardButtonList* IStandardButtonList::list(0);
IPrivateResource     IStandardButtonList::listLock;
#pragma data_seg()

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

void IStandardButtonList::addButton ( IToolBarButton *button )
{
  IResourceLock resLock(listLock);

  if ( list == 0 )
    list = new IStandardButtonList;

  list->addAsLast(button);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

void IStandardButtonList::removeButton ( IToolBarButton *button )
{
  IResourceLock resLock(listLock);
  if ( list )
  {
    IStandardButtonList::Cursor cursor(*list);
    Boolean finished = false;
    cursor.setToFirst();
    while (cursor.isValid() && !finished)
    {
      IToolBarButton* currentButton = (IToolBarButton*)
                                      list->elementAt(cursor);
      if ( currentButton == button )
      {
        list->removeAt(cursor);
        finished = true;
      }
      else
        cursor.setToNext();
    }
  }
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

void IStandardButtonList::bitmapFormatChanged ()
{
  IResourceLock resLock(listLock);
  if ( list )
  {
    IStandardButtonList::Cursor cursor(*list);
    for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
    {
      IToolBarButton* button = (IToolBarButton*)list->elementAt(cursor);
      if ( button->isBitmapVisible() )
        button->setLayoutDistorted(IWindow::minimumSizeChanged,0);
    }
  }
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

void IStandardButtonList::textFormatChanged ()
{
  IResourceLock resLock(listLock);
  if ( list )
  {
    IStandardButtonList::Cursor cursor(*list);
    for (cursor.setToFirst(); cursor.isValid(); cursor.setToNext())
    {
      IToolBarButton* button = (IToolBarButton*)list->elementAt(cursor);
      if ( button->isTextVisible() )
        button->setLayoutDistorted(IWindow::minimumSizeChanged,0);
    }
  }
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

class IToolBarButtonData : public ICustomButtonDrawHandler {
typedef ICustomButtonDrawHandler
  Inherited;
public:
  IToolBarButtonData ();
  ~IToolBarButtonData ();
IBitmapHandle
  bitmapHandle,
  latchedBitmapHandle;
IColor
  *transparentColor;
IGBitmap
  *bmp,
  *lbmp;
ISize
  minSize;
enum ToolBarButtonFlags {     // values for fflags
  invalid    = 0,
  sizeValid  = 1,
  imageValid = 2
};
unsigned long
  fflags;
IGBitmap
  *fpaintedBMP;         // already drawn button image
static IGBitmap
  *standardBitmapOnly,
  *standardTextOnly,
  *standardBitmapAndText;
static Boolean
  *optimizeStandardButtons;
static long
  fgOptRGBDark,           // colors used in background bitmaps...used to
  fgOptRGBHighlight,      // force redraw if system colors changed.
  fgOptRGBMiddle,
  fgOptRGBBorder;
static Boolean
  fgOptImageValid;        // cached background image valid flag
protected:
Boolean
  dispatchHandlerEvent ( IEvent& event );
Boolean
  drawButton ( ICustomButtonDrawEvent& event );
void
  drawForeground ( ICustomButtonDrawEvent& event );
};

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IToolBarButtonData :: IToolBarButtonData ()
                    : bitmapHandle ( 0 ),
                      latchedBitmapHandle ( 0 ),
                      transparentColor ( 0 ),
                      bmp ( 0 ),
                      lbmp ( 0 ),
                      fflags ( IToolBarButtonData::invalid ),
                      fpaintedBMP( 0 )
{
  if ( IToolBarButton::isDefaultTransparentColorSet() )
  {
    transparentColor = new IColor(0,0,0);
    *transparentColor = IToolBarButton::defaultTransparentColor();
  }
}


IToolBarButtonData :: ~IToolBarButtonData ()
{
}

/*------------------------------------------------------------------------------
| IToolBarButtonData::dispatchHandlerEvent                                     |
------------------------------------------------------------------------------*/
IBase::Boolean IToolBarButtonData :: dispatchHandlerEvent ( IEvent &event )
{
  IMODTRACE_ALL("IToolBarButtonData::dispatchHandlerEvent");
  Boolean rc = true;
  switch ( event.eventId() )
  {
#ifndef IC_WIN
    case WM_SETFOCUS:
         if ( event.parameter2().number1() &&
              !((IToolBarButton *)event.window())->allowsMouseClickFocus() )
         {
            event.setResult(0);
            return true;
         }
         break;
#endif
    case IC_UM_BMP_VIEW:
         ( (IToolBarButton *)event.window() )->setView(
                                      IToolBarButton::bitmapView );
         fflags = invalid;
         break;
    case IC_UM_TXT_VIEW:
         ( (IToolBarButton *)event.window() )->setView(
                                      IToolBarButton::textView );
         fflags = invalid;
         break;
    case IC_UM_BMP_TXT_VIEW:
         ( (IToolBarButton *)event.window() )->setView(
                                      IToolBarButton::bitmapAndTextView );
         fflags = invalid;
         break;
    case IC_UM_QRY_BMP_VISIBLE:
         event.setResult(
                  ( (IToolBarButton *)event.window() )->isBitmapVisible() );
         break;
    case IC_UM_QRY_TXT_VISIBLE:
         event.setResult(
                  ( (IToolBarButton *)event.window() )->isTextVisible() );
         break;

    case IC_UM_TOOLBAR:
    {
      switch ( event.parameter1().asUnsignedLong() )
      {
        case IC_UM_TBAR_WIN_DRAG_ENABLE:
          if (!event.window()->itemProvider())
          {
            IDMHandler::enableDragDropFor( (IToolBarButton *)event.window() );
          }
          break;

        case IC_UM_TBAR_TARGET_ENTER:
          event.setResult( true );
          break;
      }
    }
    break;

    default:
         rc = Inherited::dispatchHandlerEvent ( event );
         break;
  }
  return rc;
}

/*------------------------------------------------------------------------------
| IToolBarButtonData::drawButton                                               |
------------------------------------------------------------------------------*/
IBase::Boolean IToolBarButtonData :: drawButton ( ICustomButtonDrawEvent& event )
{
  IMODTRACE_ALL("IToolBarButtonData::drawButton");
  // Get the toolbar button
  IToolBarButton* pTBarBtn = (IToolBarButton *)event.customButton();

  // Check if optimization is turned off
  if ( !optimizeStandardButtons )
  {
     optimizeStandardButtons = new Boolean;

     IString opt(getenv( "ICLUI OPTSTDBUTTONS" ));
     if ( opt == "OFF" )
       *optimizeStandardButtons = false;
     else
       *optimizeStandardButtons = true;
  }

  // There are 2 levels of optimization.  Each toolbar button saves a
  // bitmap image of the entire button in its current up, enabled position.
  // If valid, this image is used to paint.
  // If the image is not available, one is created when the button is drawn
  // in an up position for the first time with the current font, color, bitmap
  // and text values.  The fflags member, imageValid bit is set when this
  // is done and reset by setLayoutDistorted.
  // The second level of optimization stores bitmaps of standard button
  // border/backgrounds.  These images are shared across buttons and primarily
  // serve to speed up the initial paint of a toolbar.

  // Clean up any invalid cached image.  We rely upon setLayoutDistorted
  // being called for font, color, text or bitmap changes which affect the
  // appearance of the button.  That will cause imageValid to become false.
  // We handle disabled or draw down buttons in this routine.
  if ( this->fpaintedBMP && !(fflags & imageValid) )
     {
     // image is not valid...either we havent drawn it yet or a
     // setLayoutDistorted has invalidated it.
     // delete any saved image for this button
     delete this->fpaintedBMP;
     this->fpaintedBMP = 0;
     }

     // check the color settings on static cached bitmaps.  Delete bitmaps
     // if there was a color change which invalidates them.
  if ( (!fgOptImageValid) &&
       (standardBitmapOnly || standardTextOnly || standardBitmapAndText) )
     {
     if ( (fgOptRGBDark       != IColor(IColor::buttonDark   ).asRGBLong()) ||
          (fgOptRGBHighlight  != IColor(IColor::buttonLight  ).asRGBLong()) ||
          (fgOptRGBMiddle     != IColor(IColor::buttonMiddle ).asRGBLong()) ||
          (fgOptRGBBorder     != IColor(IColor::defaultButton).asRGBLong()))
        {
        // color changes.  Delete all cached bitmaps.
        if (standardBitmapOnly) delete standardBitmapOnly;
        standardBitmapOnly = 0;
        if (standardTextOnly) delete standardTextOnly;
        standardTextOnly = 0;
        if (standardBitmapAndText) delete standardBitmapAndText;
        standardBitmapAndText = 0;
        }

     // set image valid flag to bypass further color checking until next
     // setLayoutDistorted.
     fgOptImageValid = true;
     }

  IRectangle rect      ( event.drawingArea() );
  IGBitmap*  pOptimize (0);


  // See if we have a valid cached image of the button.
  if ( *optimizeStandardButtons &&
       this->fpaintedBMP        &&
       event.drawUp()             )
     {
     // use cached image
     pOptimize = this->fpaintedBMP;
     ITRACE_ALL("Image cached");
     }
  else
     {
     // Need to paint.  We do painting into a memory graphics context, which
     // is then drawn on the screen later.

     // Create a new event with a memory GC
     ICustomButtonDrawEvent memoryEvent( event, rect.size() );

     // For standard buttons, we save static bitmaps of the border and background
     // shared across all toolbar buttons
     IGBitmap* pBorderBMP( 0 );
     Boolean   borderOptPossible =
       ( *optimizeStandardButtons &&
         pTBarBtn->isStandardFormat() &&
         event.drawUp() &&
         (fflags & sizeValid) && (minSize == rect.size() ) &&
         ( pTBarBtn->parent()->backgroundColor() == IColor(IColor::menuBgnd) ) &&
         ( pTBarBtn->backgroundColor() == IColor(IColor::buttonMiddle) ));

     // Get the static bitmap for current view
     if ( borderOptPossible )
        {
        // Yes... we can optimize border/background painting
        // Figure out which standard bitmap we want to use
        switch ( pTBarBtn->view() )
           {
           case IToolBarButton::bitmapView:
             pBorderBMP = standardBitmapOnly;
             break;
           case IToolBarButton::textView:
             pBorderBMP = standardTextOnly;
             break;
           case IToolBarButton::bitmapAndTextView:
             pBorderBMP = standardBitmapAndText;
             break;
           }
        }

     // If the standard bitmap does not exist need to paint...this could be
     // because we cannot optimize or because the saved image is invalid.
     if ( !pBorderBMP )
        {

        ITRACE_ALL(IString("Border not optimized - rect=") + rect.asString() );
        // Draw the border and background for the button normally
        drawBorder ( memoryEvent );
        drawBackground ( memoryEvent );

        // If optimization possible save off the bitmap for next time
        if (borderOptPossible)
           {
           // save colors used in bitmap
           fgOptRGBDark       = IColor(IColor::buttonDark   ).asRGBLong();
           fgOptRGBHighlight  = IColor(IColor::buttonLight  ).asRGBLong();
           fgOptRGBMiddle     = IColor(IColor::buttonMiddle ).asRGBLong();
           fgOptRGBBorder     = IColor(IColor::defaultButton).asRGBLong();
           switch ( pTBarBtn->view() )
             {
             case IToolBarButton::bitmapView:
               standardBitmapOnly = new IGBitmap ( memoryEvent.graphicContext(),
                                                   rect );
               break;
             case IToolBarButton::textView:
               standardTextOnly = new IGBitmap( memoryEvent.graphicContext(),
                                                rect );
               break;
             case IToolBarButton::bitmapAndTextView:
               standardBitmapAndText = new IGBitmap( memoryEvent.graphicContext(),
                                                      rect );
               break;
             }  // switch

           }   // optimization possible
        } // no saved border bitmap
     else
        {
        // Use the saved border bitmap image
        ITRACE_ALL("Optimizing border painting");

        pBorderBMP->drawOn(memoryEvent.graphicContext());

        // Reduce the drawing area so the foreground will paint correctly
        unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
        unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);
        // Buttons painted differently in PM and Windows
        IRectangle rect2 = rect.shrunkBy ( IPair ( cxBorder, cyBorder ));
#ifdef IC_ORIGINLOWERLEFT
        rect2 = IRectangle ( rect2.minX() + cxBorder,
                             rect2.minY() + cyBorder + cyBorder,
                             rect2.maxX() - cxBorder - cxBorder,
                             rect2.maxY() - cyBorder );
#else

  #ifdef DRAW_STANDARD_BUTTON
        // 1 additional border top/left, 2 bottom/right
        rect2 = IRectangle ( rect2.minX() + cxBorder,
                             rect2.minY() + cyBorder,
                             rect2.maxX() - cxBorder*2,
                             rect2.maxY() - cyBorder*2);

  #else
        // Win95 native style buttons  (1 additional border on all sides
        rect2.shrinkBy ( IPair ( cxBorder, cyBorder ) );
  #endif   //DRAW_STANDARD_BUTTON

#endif
        ITRACE_ALL(IString("Optimize border - setDrawingArea=") + rect2.asString() );
        memoryEvent.setDrawingArea ( rect2 );
        }   // else use saved border image


     // Draw the foreground
     drawForeground ( memoryEvent );

     // save the bitmap just painted of the whole button
     pOptimize = new IGBitmap( memoryEvent.graphicContext(), rect);

     // save the image for future use
     if ( *optimizeStandardButtons &&
          event.drawUp()             )
        {
        fpaintedBMP = pOptimize;
        fflags |= imageValid;
        }
     }  // else need to paint


  // At this point pOptimize contains the image of the entire button.
  // Paint the image to the screen.
  pOptimize->drawOn( event.graphicContext(),
                     rect.minXMinY(),
                     rect.maxXMaxY(),
                     rect.minXMinY(),
                     rect.maxXMaxY(),
                     event.isButtonEnabled() ?
                          IGBitmap::normal : IGBitmap::halftone );


  // Clean up bitmap if not needed anymore
  if ( pOptimize != this->fpaintedBMP )
     delete pOptimize;

  return true;
}

/*------------------------------------------------------------------------------
| IToolBarButtonData::drawForeground                                           |
------------------------------------------------------------------------------*/
void IToolBarButtonData :: drawForeground ( ICustomButtonDrawEvent& event )
{
  IMODTRACE_ALL("IToolBarButtonData::drawForeground");
  IRectangle rect = event.drawingArea();
  IRectangle newArea = rect;

  IToolBarButton* pTBarBtn = (IToolBarButton *)event.customButton();

  if ( pTBarBtn->isBitmapVisible() && ( bitmapHandle != 0 ))
  {
    IPoint bitmapPos;
    ISize  bitmapSize;

    bitmapSize = bmp->size();

    // Center bitmap

    bitmapPos.setX(rect.minX()+((rect.width()-bitmapSize.width())/2));
    bitmapPos.setY(rect.minY()+((rect.height()-bitmapSize.height())/2));

    // If text is also shown, do not center vertically
    if ( pTBarBtn->isTextVisible() )
    {
      if ( pTBarBtn->isStandardFormat() )
      {
#ifdef IC_ORIGINLOWERLEFT
        bitmapPos.setY(rect.maxY() -
                       IToolBarButton::standardBitmapSize().height()-1+
                      (IToolBarButton::standardBitmapSize().height()-
                       bitmapSize.height())/2);
#else
        bitmapPos.setY(rect.minY() + 1 +
                      (IToolBarButton::standardBitmapSize().height() -
                       bitmapSize.height())/2);
#endif
      }
      else
      {
#ifdef IC_ORIGINLOWERLEFT
        bitmapPos.setY(rect.maxY()-bitmapSize.height()-1);
#else
        bitmapPos.setY(rect.minY() + 1);
#endif
      }

      if ( event.drawUp() == false )
#ifdef IC_ORIGINLOWERLEFT
        bitmapPos.setY(bitmapPos.y() - 1);
#else
        bitmapPos.setY(bitmapPos.y() + 1);
#endif
    }

    // Draw the bitmap
    if ( event.drawLatched() && ( latchedBitmapHandle != 0 ))
      lbmp->moveTo(bitmapPos).drawOn(event.graphicContext());
    else
      bmp->moveTo(bitmapPos).drawOn(event.graphicContext());

    // Remove the bitmap area from the drawing area
#ifdef IC_ORIGINLOWERLEFT
    if ( pTBarBtn->isStandardFormat() )
    {
        bitmapPos.setY(rect.maxY()-
                       IToolBarButton::standardBitmapSize().height() - 1);
    }
    newArea = IRectangle ( rect.minX(),  rect.minY(),
                           rect.maxX(), bitmapPos.y() + 1);
#else
    if ( pTBarBtn->isStandardFormat() )
    {
       newArea = IRectangle ( rect.minX(),
                              bitmapPos.y() +
                              IToolBarButton::standardBitmapSize().height() - 1,
                              rect.maxX(),
                              rect.maxY());
    }
    else
    {
       newArea = IRectangle ( rect.minX(),
                              bitmapPos.y() + bitmapSize.height() - 1 ,
                              rect.maxX(),
                              rect.maxY());
    }
#endif
    if ( event.drawUp() )
      {
      newArea.sizeTo(ISize(newArea.size().width(), newArea.size().height()-1));
#ifdef IC_ORIGINUPPERLEFT
      newArea.moveBy( IPair(0, 1 ) );    // down 1
#endif
      }
    ITRACE_ALL(IString("new drawing area  newArea=") + newArea.asString() );
    event.setDrawingArea ( newArea );

  }

  if ( pTBarBtn->isTextVisible() )
  {
    if ( pTBarBtn->isStandardFormat() )
    {
      // Remove border area from drawing area
      event.setDrawingArea ( IRectangle ( newArea.minX() + 4,
                                          newArea.minY(),
                                          newArea.minX() + 4 +
                                          IToolBarButton::standardTextWidth(),
                                          newArea.maxY() ));

      Inherited::drawForeground(event);
    }
    else
      Inherited::drawForeground(event);
  }

  // Restore drawing area rectangle (since we may have changed it)

  event.setDrawingArea ( rect );
}

/*------------------------------------------------------------------------------
| IToolBarButton styles                                                        |
------------------------------------------------------------------------------*/
#pragma data_seg(ICLStaticConst)
const IToolBarButton::Style
  IToolBarButton::bitmapVisible        ( 0, IBS_TBARBMPVISIBLE ),  //0x00000004
  IToolBarButton::textVisible          ( 0, IBS_TBARTEXTVISIBLE ), //0x00000008
  IToolBarButton::bitmapAndTextVisible ( 0, IBS_TBARBOTHVISIBLE ), //0x00000010
  IToolBarButton::useIdForBitmap       ( 0, IBS_TBARUSEBMPID ),    //0x00000020
  IToolBarButton::useIdForText         ( 0, IBS_TBARUSETEXTID ),   //0x00000040
  IToolBarButton::standardFormat       ( 0, IBS_TBARSTDFORMAT ),   //0x00000080
  IToolBarButton::noDragDelete         ( 0, IBS_TBARNODRAGDEL ),   //0x00000100
#ifdef IC_WIN
  IToolBarButton::classDefaultStyle    ( (WS_VISIBLE ),
                                                          IBS_NOPOINTERFOCUS |
#else
  IToolBarButton::classDefaultStyle    ( (WS_VISIBLE |
                                          BS_NOPOINTERFOCUS),
#endif
                                                         (IBS_TBARBMPVISIBLE |
                                                          IBS_TBARUSEBMPID   |
                                                          IBS_TBARUSETEXTID  |
                                                          IBS_TBARSTDFORMAT) );
#pragma data_seg()

#pragma data_seg(ICLNonConst)
IToolBarButton::Style
#ifdef IC_WIN
  IToolBarButton::fCurrentDefaultStyle ( (WS_VISIBLE) ,
                                                         IBS_NOPOINTERFOCUS |
#else
  IToolBarButton::fCurrentDefaultStyle ( (WS_VISIBLE |
                                         BS_NOPOINTERFOCUS),
#endif
                                                        (IBS_TBARBMPVISIBLE |
                                                         IBS_TBARUSEBMPID   |
                                                         IBS_TBARUSETEXTID  |
                                                         IBS_TBARSTDFORMAT) );
  IColor* IToolBarButton::fCurrentDefaultTransparent(new IColor(255,0,255));
  unsigned long IToolBarButton::fgStandardWidth(50);
  unsigned long IToolBarButton::fgStandardLines(1);
  ISize         IToolBarButton::fgStandardSize(20,17);
  IGBitmap*     IToolBarButtonData::standardBitmapOnly      (0);
  IGBitmap*     IToolBarButtonData::standardTextOnly        (0);
  IGBitmap*     IToolBarButtonData::standardBitmapAndText   (0);
  Boolean*      IToolBarButtonData::optimizeStandardButtons (0);
  long          IToolBarButtonData::fgOptRGBDark      (0);
  long          IToolBarButtonData::fgOptRGBHighlight (0);
  long          IToolBarButtonData::fgOptRGBMiddle    (0);
  long          IToolBarButtonData::fgOptRGBBorder    (0);
  Boolean       IToolBarButtonData::fgOptImageValid   (false);


#pragma data_seg()

/*------------------------------------------------------------------------------
| IToolBarButton ctor                                                          |
------------------------------------------------------------------------------*/
IToolBarButton :: IToolBarButton (unsigned long id,
                                  IWindow* parent,
                                  IWindow* owner,
                                  const IRectangle& initial,
                                  const Style& style )
                : fToolBarButtonData(0)
{
  // Save the extended style to make sure we have a copy of it stored
  setExtendedStyle( extendedStyle() | style.asExtendedUnsignedLong() );

  // Check for bad style combinations

  if ((( style & bitmapVisible ) && ( style & textVisible )) ||
      (( style & bitmapVisible ) && ( style & bitmapAndTextVisible )) ||
      (( style & textVisible )   && ( style & bitmapAndTextVisible )))
  {
     ITHROWLIBRARYERROR( IC_INVALIDSTYLE,
                         IBaseErrorInfo::invalidParameter,
                         IException::recoverable );
  }

  IASSERTPARM(parent!=0);
  fToolBarButtonData = new IToolBarButtonData();

  /****************************************************************/
  /* Borland has bug with embedded ? operator so below is needed. */
  /****************************************************************/
#ifdef __BORLANDC__
  IWindowHandle hOwner;
  if (owner == 0)
    hOwner = IWindowHandle(0);
  else
    hOwner = owner->handle();
#else
  IWindowHandle hOwner = (owner == 0) ? IWindowHandle(0)
                                       : owner->handle();
#endif

  IWindowHandle whButton =
      this -> create( id,
                      0,
                      convertToGUIStyle( style ),
                      WC_BUTTON,
                      parent->handle(),
                      hOwner,
                      initial,
                      0,
                      0,
                      defaultOrdering(),
                      convertToGUIStyle( style, true ) );

  startHandlingEventsFor(whButton);

  // Add handler
  fToolBarButtonData->handleEventsFor(this);

  // Handle style information
  if ( style & useIdForBitmap )
  {
    if ( id < IC_ID_BASE )
    {
      // Load from user resource
      fToolBarButtonData->bitmapHandle = IApplication::current().userResourceLibrary().
                           tryToLoadBitmap(id);
    }
    else
    {
      // Load from class library resource
      fToolBarButtonData->bitmapHandle = IApplication::current().resourceLibrary().
                           tryToLoadBitmap(id);
    }

    // Could not load bitmap, so try to get the default bitmap
    if ( !fToolBarButtonData->bitmapHandle )
    {
      fToolBarButtonData->bitmapHandle = IApplication::current().resourceLibrary().
                           tryToLoadBitmap(IC_ID_DEFAULT);
    }

    // If we could load a bitmap, finish setup
    if ( fToolBarButtonData->bitmapHandle )
    {
      fToolBarButtonData->bmp = new IGBitmap(fToolBarButtonData->bitmapHandle);
      if ( fToolBarButtonData->transparentColor )
        fToolBarButtonData->bmp->setTransparentColor ( *fToolBarButtonData->transparentColor );
    }
  }

  if ( style & useIdForText )
  {
    if ( id < IC_ID_BASE )
    {
      // Load from user resource
      setText(IApplication::current().userResourceLibrary().
              tryToLoadString(id));
    }
    else
    {
      // Load from class library resource
      setText(IApplication::current().resourceLibrary().
              tryToLoadString(id));
    }
  }

  if ( style & standardFormat )
  {
    IStandardButtonList::addButton(this);
    if (fToolBarButtonData->bmp)
    {
      ISize bmpSize = fToolBarButtonData->bmp->size();
      if ( bmpSize.width() > fgStandardSize.width() )
        bmpSize.setWidth(fgStandardSize.width());
      if ( bmpSize.height() > fgStandardSize.height() )
        bmpSize.setHeight(fgStandardSize.height());
      if ( bmpSize != fToolBarButtonData->bmp->size() )
      {
        // The bitmap is too big, so shrink it
        fToolBarButtonData->bmp->sizeTo(bmpSize);
      }
    }
  }

  // Set the default button font to be Helv.8
  // Note: DBCS OS/2 systems should use the default font instead of Helv
  //       since the double-byte characters do not exist fot this font

#ifdef IC_PM
  unsigned long codepage = WinQueryCp(HMQ_CURRENT);
  switch (codepage)
  {
    // Note: The following list represents the DBCS code pages supported
    //       by OS/2 2.1 and Warp as of June 1995

    case  949: // Korean                      (replaces older 934/944)
    case  944: // Korean
    case  934: // Korean
    case 1381: // People's Republic of China  (replaces older 936/946)
    case  936: // People's Republic of China
    case  946: // People's Republic of China
    case  950: // Taiwan                      (replaces older 938/948)
    case  938: // Taiwan
    case  948: // Taiwan
    case  932: // Japanese
    case  942: // Japanese
    {
      IFont fnt(this);
      fnt.setPointSize(8);
      fnt.setWindowFont(this);
    }
    break;
    default:
    {
      IFont fnt("Helv",8);
      fnt.setWindowFont(this);
    }
    break;
  }
#endif
#ifdef IC_WIN
  IFont fnt("Arial",9);
  fnt.setWindowFont(this);
#endif
}

/*------------------------------------------------------------------------------
| IToolBarButton dtor                                                          |
------------------------------------------------------------------------------*/
IToolBarButton :: ~IToolBarButton()
{
  if (fToolBarButtonData)
  {
    fToolBarButtonData->stopHandlingEventsFor(this);
    if ( isStandardFormat() )
      IStandardButtonList::removeButton(this);
    if ( fToolBarButtonData->fpaintedBMP )
       delete fToolBarButtonData->fpaintedBMP;
    if ( fToolBarButtonData->bmp )
      delete fToolBarButtonData->bmp;
    if ( fToolBarButtonData->lbmp )
      delete fToolBarButtonData->lbmp;
    if ( fToolBarButtonData->transparentColor)
      delete fToolBarButtonData->transparentColor;
    delete fToolBarButtonData;
  }
}

/*------------------------------------------------------------------------------
| IToolBarButton::defaultStyle                                                 |
------------------------------------------------------------------------------*/
IToolBarButton::Style IToolBarButton :: defaultStyle()
{
  return fCurrentDefaultStyle;
}

/*------------------------------------------------------------------------------
| IToolBarButton::setDefaultStyle                                              |
------------------------------------------------------------------------------*/
void IToolBarButton :: setDefaultStyle(const Style& style)
{
  fCurrentDefaultStyle = style;
}

/*------------------------------------------------------------------------------
| IToolBarButton::convertToGUIStyle                                            |
|                                                                              |
| Returns base style for the control by default, or extended style if          |
| extended flag (bExtOnly) is set.                                             |
------------------------------------------------------------------------------*/
unsigned long IToolBarButton :: convertToGUIStyle(const IBitFlag& guiStyle,
                                                  Boolean bExtOnly) const
{
  // Obtain the style from the class (ICustomButton) that we inherit from
  unsigned long ulStyle = Inherited::convertToGUIStyle( guiStyle, bExtOnly );

  if (bExtOnly)
  {
    // Use mask to only return extended styles in the user defined range
    ulStyle |= guiStyle.asExtendedUnsignedLong() & IS_EXTMASK;
  }

  return( ulStyle );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IColor IToolBarButton :: defaultTransparentColor()
{
  if ( fCurrentDefaultTransparent )
    return *fCurrentDefaultTransparent;
  else
    return IColor(0,0,0);
}

void IToolBarButton :: clearDefaultTransparentColor ( )
{
  if (fCurrentDefaultTransparent)
    delete fCurrentDefaultTransparent;
  fCurrentDefaultTransparent = 0;
}

void IToolBarButton :: setDefaultTransparentColor (const IColor& aColor )
{
  if (!fCurrentDefaultTransparent)
    fCurrentDefaultTransparent = new IColor(0,0,0);
  *fCurrentDefaultTransparent = aColor;
}

Boolean IToolBarButton :: isDefaultTransparentColorSet ()
{
  if (fCurrentDefaultTransparent)
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| IToolBarButton::setBitmap                                                    |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setBitmap(const IResourceId& bitmapId )
{
  fToolBarButtonData->bitmapHandle = 0;
  if ( fToolBarButtonData->bmp )
  {
    delete fToolBarButtonData->bmp;
    fToolBarButtonData->bmp = 0;
  }

  if ( bitmapId != 0 )
  {
    if ( bitmapId < IC_ID_BASE )
    {
      // Load from user resource
      fToolBarButtonData->bitmapHandle = bitmapId.resourceLibrary().
                           tryToLoadBitmap(bitmapId);
    }
    else
    {
      // Load from class library resource
      fToolBarButtonData->bitmapHandle = IApplication::current().resourceLibrary().
                           tryToLoadBitmap(bitmapId);
    }

    // Could not load bitmap, so try to get the default bitmap
    if ( !fToolBarButtonData->bitmapHandle )
    {
      fToolBarButtonData->bitmapHandle = IApplication::current().resourceLibrary().
                           tryToLoadBitmap(IC_ID_DEFAULT);
    }
  }

  // If we have a bitmap continue processing
  if ( fToolBarButtonData->bitmapHandle )
  {
    fToolBarButtonData->bmp = new IGBitmap(fToolBarButtonData->bitmapHandle);
    if ( fToolBarButtonData->transparentColor )
      fToolBarButtonData->bmp->setTransparentColor ( *fToolBarButtonData->transparentColor );

    // Check if bitmap needs to be resized to fit in standard button
    if ( isStandardFormat() )
    {
      ISize bmpSize = fToolBarButtonData->bmp->size();
      if ( bmpSize.width() > fgStandardSize.width() )
        bmpSize.setWidth(fgStandardSize.width());
      if ( bmpSize.height() > fgStandardSize.height() )
        bmpSize.setHeight(fgStandardSize.height());
      if ( bmpSize != fToolBarButtonData->bmp->size() )
      {
        // The bitmap is too big, so shrink it
        fToolBarButtonData->bmp->sizeTo(bmpSize);
      }
    }
  }
  setLayoutDistorted(IWindow::minimumSizeChanged,0);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarButton::setLatchedBitmap                                             |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setLatchedBitmap
                                  ( const IResourceId& latchedBitmapId)
{
  fToolBarButtonData->latchedBitmapHandle = 0;
  if ( fToolBarButtonData->lbmp )
  {
    delete fToolBarButtonData->lbmp;
    fToolBarButtonData->lbmp = 0;
  }

  if ( latchedBitmapId != 0 )
  {
    fToolBarButtonData->latchedBitmapHandle = latchedBitmapId.resourceLibrary().
                                loadBitmap(latchedBitmapId);
    fToolBarButtonData->lbmp = new IGBitmap(fToolBarButtonData->latchedBitmapHandle);
    if ( fToolBarButtonData->transparentColor )
      fToolBarButtonData->lbmp->setTransparentColor ( *fToolBarButtonData->transparentColor );

    // Check if latched bitmap needs to be resized to fit in standard button
    if ( isStandardFormat() )
    {
      ISize bmpSize = fToolBarButtonData->lbmp->size();
      if ( bmpSize.width() > fgStandardSize.width() )
        bmpSize.setWidth(fgStandardSize.width());
      if ( bmpSize.height() > fgStandardSize.height() )
        bmpSize.setHeight(fgStandardSize.height());
      if ( bmpSize != fToolBarButtonData->lbmp->size() )
      {
        // The bitmap is too big, so shrink it
        fToolBarButtonData->lbmp->sizeTo(bmpSize);
      }
    }
  }
  setLayoutDistorted(IWindow::minimumSizeChanged,0);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarButton::setBitmap                                                    |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setBitmap(unsigned long id )
{
  return setBitmap(IResourceId(id));
}

/*------------------------------------------------------------------------------
| IToolBarButton::setLatchedBitmap                                             |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setLatchedBitmap(unsigned long id)
{
  return setLatchedBitmap(IResourceId(id));
}

/*------------------------------------------------------------------------------
| IToolBarButton::setBitmap                                                    |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setBitmap (const IBitmapHandle& handle)
{
  fToolBarButtonData->bitmapHandle = handle;

  if ( fToolBarButtonData->bmp )
  {
    delete fToolBarButtonData->bmp;
    fToolBarButtonData->bmp = 0;
  }

  if ( handle )
  {
    fToolBarButtonData->bmp = new IGBitmap(fToolBarButtonData->bitmapHandle);
    if ( fToolBarButtonData->transparentColor )
      fToolBarButtonData->bmp->setTransparentColor ( *fToolBarButtonData->transparentColor );

    // Check if bitmap needs to be resized to fit in standard button
    if ( isStandardFormat() )
    {
      ISize bmpSize = fToolBarButtonData->bmp->size();
      if ( bmpSize.width() > fgStandardSize.width() )
        bmpSize.setWidth(fgStandardSize.width());
      if ( bmpSize.height() > fgStandardSize.height() )
        bmpSize.setHeight(fgStandardSize.height());
      if ( bmpSize != fToolBarButtonData->bmp->size() )
      {
        // The bitmap is too big, so shrink it
        fToolBarButtonData->bmp->sizeTo(bmpSize);
        fToolBarButtonData->bitmapHandle = fToolBarButtonData->bmp->handle();
      }
    }
  }

  setLayoutDistorted(IWindow::minimumSizeChanged,0);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarButton::setLatchedBitmap                                             |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setLatchedBitmap (const IBitmapHandle& handle)
{
  fToolBarButtonData->latchedBitmapHandle = handle;

  if ( fToolBarButtonData->lbmp )
  {
    delete fToolBarButtonData->lbmp;
    fToolBarButtonData->lbmp = 0;
  }

  if ( handle )
  {
    fToolBarButtonData->lbmp = new IGBitmap(fToolBarButtonData->latchedBitmapHandle);
    if ( fToolBarButtonData->transparentColor )
      fToolBarButtonData->lbmp->setTransparentColor ( *fToolBarButtonData->transparentColor );

    // Check if latched bitmap needs to be resized to fit in standard button
    if ( isStandardFormat() )
    {
      ISize bmpSize = fToolBarButtonData->lbmp->size();
      if ( bmpSize.width() > fgStandardSize.width() )
        bmpSize.setWidth(fgStandardSize.width());
      if ( bmpSize.height() > fgStandardSize.height() )
        bmpSize.setHeight(fgStandardSize.height());
      if ( bmpSize != fToolBarButtonData->lbmp->size() )
      {
        // The bitmap is too big, so shrink it
        fToolBarButtonData->lbmp->sizeTo(bmpSize);
        fToolBarButtonData->latchedBitmapHandle = fToolBarButtonData->lbmp->handle();
      }
    }
  }

  setLayoutDistorted(IWindow::minimumSizeChanged,0);
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IBitmapHandle IToolBarButton :: bitmap () const
{
  return fToolBarButtonData->bitmapHandle;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IBitmapHandle IToolBarButton :: latchedBitmap () const
{
  return fToolBarButtonData->latchedBitmapHandle;
}

/*------------------------------------------------------------------------------
| IToolBarButton::setView                                                      |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: setView (IToolBarButton::View buttonView)
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  switch ( buttonView )
  {
    case bitmapView:
         ulExtStyle |= bitmapVisible.asExtendedUnsignedLong();
         ulExtStyle &= ~textVisible.asExtendedUnsignedLong();
         ulExtStyle &= ~bitmapAndTextVisible.asExtendedUnsignedLong();
         setLayoutDistorted(IWindow::minimumSizeChanged,0);
         break;
    case textView:
         ulExtStyle &= ~bitmapVisible.asExtendedUnsignedLong();
         ulExtStyle |= textVisible.asExtendedUnsignedLong();
         ulExtStyle &= ~bitmapAndTextVisible.asExtendedUnsignedLong();
         setLayoutDistorted(IWindow::minimumSizeChanged,0);
         break;
    case bitmapAndTextView:
         ulExtStyle &= ~bitmapVisible.asExtendedUnsignedLong();
         ulExtStyle &= ~textVisible.asExtendedUnsignedLong();
         ulExtStyle |= bitmapAndTextVisible.asExtendedUnsignedLong();
         setLayoutDistorted(IWindow::minimumSizeChanged,0);
         break;
  }

  if ( ulExtStyle != ulOldExtStyle )
  {
    setExtendedStyle(ulExtStyle);
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarButton::view                                                         |
------------------------------------------------------------------------------*/
IToolBarButton::View IToolBarButton :: view () const
{
  if ( isBitmapVisible() )
  {
    if ( isTextVisible() )
      return bitmapAndTextView;
    else
      return bitmapView;
  }
  else
    return textView;
}

/*------------------------------------------------------------------------------
| IToolBarButton::isBitmapVisible                                              |
------------------------------------------------------------------------------*/
Boolean IToolBarButton :: isBitmapVisible () const
{
  unsigned long ulMask = bitmapVisible.asExtendedUnsignedLong() |
                         bitmapAndTextVisible.asExtendedUnsignedLong();
  return( (extendedStyle() & ulMask) ?  true : false );
}

/*------------------------------------------------------------------------------
| IToolBarButton::isTextVisible                                                |
------------------------------------------------------------------------------*/
Boolean IToolBarButton :: isTextVisible () const
{
  unsigned long ulMask = textVisible.asExtendedUnsignedLong() |
                         bitmapAndTextVisible.asExtendedUnsignedLong();
  return( (extendedStyle() & ulMask) ?  true : false );
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

ISize IToolBarButton :: bitmapSize () const
{
  ISize sz;
  if ( fToolBarButtonData->bitmapHandle != 0 )
  {
    sz = IGBitmap(fToolBarButtonData->bitmapHandle).size();
  }
  return sz;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IColor IToolBarButton :: transparentColor () const
{
  if (fToolBarButtonData->transparentColor)
    return (*fToolBarButtonData->transparentColor);
  else
    return IColor(0,0,0);
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

Boolean IToolBarButton :: hasTransparentColor () const
{
  if (fToolBarButtonData->transparentColor)
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IToolBarButton& IToolBarButton :: resetTransparentColor ()
{
  if (fToolBarButtonData->transparentColor)
  {
    delete fToolBarButtonData->transparentColor;
    fToolBarButtonData->transparentColor = 0;
    if ( fToolBarButtonData->bmp )
      fToolBarButtonData->bmp->resetTransparentColor();
    if ( fToolBarButtonData->lbmp )
      fToolBarButtonData->lbmp->resetTransparentColor();
    refresh();
  }
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IToolBarButton& IToolBarButton :: setTransparentColor (const IColor& aColor)
{
  if (fToolBarButtonData->transparentColor)
  {
    delete fToolBarButtonData->transparentColor;
    fToolBarButtonData->transparentColor = 0;
  }
  fToolBarButtonData->transparentColor = new IColor(aColor);
  if ( fToolBarButtonData->bmp )
    fToolBarButtonData->bmp->setTransparentColor(aColor);
  if ( fToolBarButtonData->lbmp )
    fToolBarButtonData->lbmp->setTransparentColor(aColor);
  refresh();
  return *this;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/

IToolBarButton& IToolBarButton :: setLayoutDistorted
                                ( unsigned long layoutAttributeOn,
                                  unsigned long layoutAttributeOff )
{
  // If minimum size changed invalidate all caches.
  // If color changed invalidate per-button bitmap image and static
  // background bitmaps
  // If font changed invalidate per-button bitmap image.if text visible.
  if (layoutAttributeOn & IWindow::minimumSizeChanged )
    fToolBarButtonData->fflags = IToolBarButtonData::invalid;
  if (layoutAttributeOn & IWindow::colorChanged )
    {
    fToolBarButtonData->fflags &= ~IToolBarButtonData::imageValid;
    IToolBarButtonData::fgOptImageValid = false;
    }
  if ((layoutAttributeOn & IWindow::fontChanged ) && this->isTextVisible() )
    fToolBarButtonData->fflags &= ~IToolBarButtonData::imageValid;

  Inherited::setLayoutDistorted(layoutAttributeOn,layoutAttributeOff);
  return *this;
}

/*------------------------------------------------------------------------------
| IToolBarButton::enableDragDelete                                             |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: enableDragDelete ( Boolean bTurnOn )
{
  unsigned long ulExtStyle = extendedStyle();
  unsigned long ulOldExtStyle = ulExtStyle;

  if (bTurnOn)
  {
    ulExtStyle &= ~noDragDelete.asExtendedUnsignedLong();
  }
  else
  {
    ulExtStyle |= noDragDelete.asExtendedUnsignedLong();
  }

  if (ulExtStyle != ulOldExtStyle)
  {
    setExtendedStyle( ulExtStyle );
  }

  return( *this );
}

/*------------------------------------------------------------------------------
| IToolBarButton::disableDragDelete                                            |
------------------------------------------------------------------------------*/
IToolBarButton& IToolBarButton :: disableDragDelete ( )
{
  enableDragDelete( false );
  return( *this );
}


/*------------------------------------------------------------------------------
| IToolBarButton::allowsDragDelete                                             |
------------------------------------------------------------------------------*/
Boolean IToolBarButton :: allowsDragDelete ( ) const
{
  return( (extendedStyle() & noDragDelete.asExtendedUnsignedLong()) ? false
                                                                    : true );
}

/*------------------------------------------------------------------------------
| IToolBarButton::calcMinimumSize                                              |
------------------------------------------------------------------------------*/
ISize IToolBarButton :: calcMinimumSize () const
{

  // Get border sizes to be used later
  unsigned long cxBorder = IQUERYSYSVALUE(SV_CXBORDER);
  unsigned long cyBorder = IQUERYSYSVALUE(SV_CYBORDER);

  ISize  oldSize( fToolBarButtonData->minSize );
  // Check for standard formatting
  if ( isStandardFormat() )
  {
    // The minimum size includes 5 pels of border and 2 of pad
    fToolBarButtonData->minSize.setWidth  ( cxBorder * 5 + 2);
    fToolBarButtonData->minSize.setHeight ( cyBorder * 5 + 2);

    if ( isBitmapVisible() )
    {
      // Add room for bitmap
      fToolBarButtonData->minSize.setWidth ( fToolBarButtonData->minSize.width() +
                               fgStandardSize.width() );
      fToolBarButtonData->minSize.setHeight ( fToolBarButtonData->minSize.height() +
                                fgStandardSize.height() );
    }

    if ( isTextVisible() )
    {
      IFont fnt(this);
      // Check if the button is already wide enough for text
      if ( !(isBitmapVisible()) ||
         ( fgStandardWidth + 6 ) > fgStandardSize.width() )
      {
        fToolBarButtonData->minSize.setWidth ( cxBorder * 5 + 8 + fgStandardWidth );
      }

      // Add room for the lines of text
      fToolBarButtonData->minSize.setHeight ( fToolBarButtonData->minSize.height() +
                              ( fgStandardLines * fnt.maxCharHeight() ));
    }

    fToolBarButtonData->fflags |= IToolBarButtonData::sizeValid;
    if ( fToolBarButtonData->minSize != oldSize )
       fToolBarButtonData->fflags &= ~IToolBarButtonData::imageValid;

  }

  // Normal sizing (based on the size of bitmap and text)
  if ( fToolBarButtonData->fflags & IToolBarButtonData::sizeValid )
  {
    return fToolBarButtonData->minSize;
  }
  else
  {
    if ( isTextVisible() )
    {
      fToolBarButtonData->minSize = Inherited::calcMinimumSize();
    }
    else
    {
      fToolBarButtonData->minSize.setWidth ( cxBorder * 5 );
      fToolBarButtonData->minSize.setHeight ( cyBorder * 5 );
    }

    if ( isBitmapVisible() )
    {
      // The minimum size when the bitmap is visible should include room
      // for the bitmap and 5 pels of border (for width)
      ISize minBitmapSize;
      if ( fToolBarButtonData->bmp )
         minBitmapSize = fToolBarButtonData->bmp->size() + ISize ( cxBorder * 5, 0 );
      else
         minBitmapSize = ISize ( cxBorder * 5, 0 );

      if ( fToolBarButtonData->minSize.width() <  minBitmapSize.width() )
      {
        fToolBarButtonData->minSize.setWidth ( minBitmapSize.width() );
      }

      fToolBarButtonData->minSize.setHeight ( fToolBarButtonData->minSize.height() +
                                minBitmapSize.height() );
    }

    fToolBarButtonData->fflags |= IToolBarButtonData::sizeValid;
    if ( fToolBarButtonData->minSize != oldSize )
       fToolBarButtonData->fflags &= ~IToolBarButtonData::imageValid;
    return fToolBarButtonData->minSize;
  }
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IToolBarButton::standardTextWidth ()
{
  return fgStandardWidth;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IToolBarButton::standardTextLines ()
{
  return fgStandardLines;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
ISize IToolBarButton::standardBitmapSize ()
{
  return fgStandardSize;
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
void IToolBarButton::setStandardTextWidth ( unsigned long newWidth )
{
  fgStandardWidth = newWidth;
  if ( IToolBarButtonData::standardTextOnly )
  {
     delete IToolBarButtonData::standardTextOnly;
     IToolBarButtonData::standardTextOnly = 0;
  }
  if ( IToolBarButtonData::standardBitmapAndText )
  {
     delete IToolBarButtonData::standardBitmapAndText;
     IToolBarButtonData::standardBitmapAndText = 0;
  }
  IStandardButtonList::textFormatChanged();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
void IToolBarButton::setStandardTextLines ( unsigned long newLines )
{
  fgStandardLines = newLines;
  if ( IToolBarButtonData::standardTextOnly )
  {
     delete IToolBarButtonData::standardTextOnly;
     IToolBarButtonData::standardTextOnly = 0;
  }
  if ( IToolBarButtonData::standardBitmapAndText )
  {
     delete IToolBarButtonData::standardBitmapAndText;
     IToolBarButtonData::standardBitmapAndText = 0;
  }
  IStandardButtonList::textFormatChanged();
}

/*------------------------------------------------------------------------------
|                                                                              |
------------------------------------------------------------------------------*/
void IToolBarButton::setStandardBitmapSize ( const ISize& newSize )
{
  fgStandardSize = newSize;
  if ( IToolBarButtonData::standardBitmapOnly )
  {
     delete IToolBarButtonData::standardBitmapOnly;
     IToolBarButtonData::standardBitmapOnly = 0;
  }
  if ( IToolBarButtonData::standardBitmapAndText )
  {
     delete IToolBarButtonData::standardBitmapAndText;
     IToolBarButtonData::standardBitmapAndText = 0;
  }
  IStandardButtonList::bitmapFormatChanged();
}

/*------------------------------------------------------------------------------
| IToolBarButton::isStandardFormat                                             |
------------------------------------------------------------------------------*/
Boolean IToolBarButton :: isStandardFormat () const
{
  return( (extendedStyle() & standardFormat.asExtendedUnsignedLong()) ? true
                                                                      : false );
}

